#region License
// Copyright  2004 - 2010 All Right Reserved, Derek Goslin
// 
// 1. Definitions
// The terms "reproduce," "reproduction," "derivative works," and "distribution" have the same meaning here as under U.S. copyright law.
// A "contribution" is the original software, or any additions or changes to the software.
// A "contributor" is any person that distributes its contribution under this license.
// "Licensed patents" are a contributor's patent claims that read directly on its contribution.
// 
// 2. Grant of Rights
// (A) Copyright Grant- Subject to the terms of this license, including the license
//     conditions and limitations in section 3, each contributor grants you a non-exclusive,
//     worldwide, royalty-free copyright license to reproduce its contribution, prepare derivative 
//     works of its contribution, and distribute its contribution or any derivative works that you create.
// (B) Patent Grant- Subject to the terms of this license, including the license conditions and limitations
//     in section 3, each contributor grants you a non-exclusive, worldwide, royalty-free license under its
//     licensed patents to make, have made, use, sell, offer for sale, import, and/or otherwise dispose of
//     its contribution in the software or derivative works of the contribution in the software.
// 
// 3. Conditions and Limitations
// (A) Reciprocal Grants- For any file you distribute that contains code from the software (in source code or binary format), 
//     you must provide recipients the source code to that file along with a copy of this license, which license will govern that file. 
//     You may license other files that are entirely your own work and do not contain code from the software under any terms you choose.
// (B) No Trademark License- This license does not grant you rights to use any contributors' name, logo, or trademarks.
// (C) If you bring a patent claim against any contributor over patents that you claim are infringed by the software,
//     your patent license from such contributor to the software ends automatically.
// (D) If you distribute any portion of the software, you must retain all copyright, patent, trademark,
//     and attribution notices that are present in the software.
// (E) If you distribute any portion of the software in source code form, you may do so only under this license by including a complete copy 
//     of this license with your distribution. If you distribute any portion of the software in compiled or object code form, you may only do 
//     so under a license that complies with this license.
// (F) The software is licensed "as-is." You bear the risk of using it. The contributors give no express warranties, guarantees or conditions.
//    You may have additional consumer rights under your local laws which this license cannot change. To the extent permitted under your
//    local laws, the contributors exclude the implied warranties of merchantability, fitness for a particular purpose and non-infringement.
#endregion

using System;
using DnEcuDiag.Ecus.Kwp;
using DnEcuDiag.Factories;
using NUnit.Framework;
using DnEcuDiag.Ecus.Generic;
using DnEcuDiag.Ecus.Enums;

namespace DnEcuDiag.Tests
{
    [TestFixture]
    public class CFC218Tests
    {
        private KwpEcu _ecu;
        private KwpTestDevice _device;

        [TestFixtureSetUp]
        public void Init()
        {
            try
            {
                _device = new KwpTestDevice();

                _ecu = EcuFactory.CreateEcu(EcuFactory.EcuList.Find(obj => obj.Name.Contains("SeleSpeed CFC218F"))) as KwpEcu;
                _ecu.DiagnosticInterface = _device;
                _ecu.Connect();
            }
            catch (Exception e)
            {
                Console.WriteLine(e.ToString());
            }
        }

        [Test]
        public void TestConnect()
        {
            Assert.AreEqual("SeleSpeed CFC218F", _ecu.Model);
            Assert.AreEqual("0123456789 ", _ecu.DrawingNumber);
            Assert.AreEqual("0123456789 ", _ecu.HardwareNumber);
            Assert.AreEqual("0x10", _ecu.HardwareVersionNumber);
            Assert.AreEqual("0123456789 ", _ecu.SoftwareNumber);
            Assert.AreEqual("0x85170", _ecu.SoftwareVersionNumber);
            Assert.AreEqual("01234 ", _ecu.OmologationNumber);
            Assert.AreEqual("55 AA 55 AA 55 ", _ecu.IsoCode);
            Assert.AreEqual("19 97 03 31 ", _ecu.ProductionDate);
            Assert.AreEqual("19 97 03 31 ", _ecu.ProgrammingDate);
            Assert.IsNull(_ecu.RpmSensor);
            Assert.IsNull(_ecu.SpeedSensor);
        }

        [Test]
        public void TestSensors_SSCFC218F()
        {
            TestSensorValues_SSCFC218F(_ecu);
        }

        [Test]
        public void TestFaults_SSCFC218F()
        {
            Assert.AreNotEqual(0, _ecu.ErrorMemoryStructureDefinition.ErrorDefinitions.Count,
                               "m_ssCFC218F.ErrorMemoryStructureDefinition.Count == 0 for SSCFC218F");
            Assert.AreEqual(33, _ecu.ErrorMemoryStructureDefinition.ErrorDefinitions.Count,
                            "m_ssCFC218F.ErrorMemoryStructureDefinition.Count != 32 for SSCFC218F");

            //////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
            // Three P0715
            //////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////			
            _device.FaultcodeList = new byte[] {0x58, 01, 07, 0x15, 0xE1};
            _device.Faultcodeinfo = new byte[] {0x57, 1, 0x07, 0x15, 0x08, 10, 10, 100, 0x33, 10, 10, 1};
            _ecu.ReadFaultCodes();

            Assert.AreEqual(1, _ecu.ActiveFaultCodes.Count, "m_ssCFC218F.ActiveFaultCodes.Count == 1 for SSCFC218F");
            Console.WriteLine(_ecu.ActiveFaultCodes[0].FaultDescription);
            Assert.AreEqual("P0715", (_ecu.ActiveFaultCodes[0]).FaultComponent);
            Assert.AreEqual(
                "Failed Component Code: Clutch Speed\r\n" +
                "Fault Symptom: Signal Not Plausible\r\n" +
                "Warning Lamp Status: Warning Lamp not Illuminated\r\n" +
                "Fault Test State: Test complete for this Error or not applicable.\r\n" +
                "Fault Code Storage State: No Error Detected at time of request\r\n" +
                "Event Count: 1 count\r\n" +
                "Environmental Condition One: Hydraulic Pressure: 10 bar\r\n" +
                "Environmental Condition Two: Engine Speed : 320 rpm\r\n" +
                "Environmental Condition Three: Vehicle Speed: 141 km/h\r\n" +
                "Environmental Condition Four: Gearbox Oil Temperature: 32 C\r\n" +
                "Environmental Condition Five: Requested Gear: Third\r\n" +
                "Environmental Condition Five: Inserted Gear: Third\r\n" +
                "Environmental Condition Six: Battery Voltage: 8.99 volts\r\n",
                _ecu.ActiveFaultCodes[0].FaultDescription);

            //////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
            // one P1772
            //////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

            _device.FaultcodeList = new byte[] {0x58, 0x1, 0x17, 0x72, 0xE1};
            _device.Faultcodeinfo = new byte[] {0x57, 1, 0x17, 0x72, 0x08, 10, 10, 100, 0x33, 10, 10, 1};
            _ecu.ReadFaultCodes();

            Assert.AreEqual(1, _ecu.ActiveFaultCodes.Count, "m_ssCFC218F.ActiveFaultCodes.Count == 1 for SSCFC218F");
            Console.WriteLine(_ecu.ActiveFaultCodes[0].FaultDescription);
            Assert.AreEqual("P1772", (_ecu.ActiveFaultCodes[0]).FaultComponent);
            Assert.AreEqual(
                "Failed Component Code: Switch lever stuck\r\n" +
                "Fault Symptom: Signal Not Plausible\r\n" +
                "Warning Lamp Status: Warning Lamp not Illuminated\r\n" +
                "Fault Test State: Test complete for this Error or not applicable.\r\n" +
                "Fault Code Storage State: No Error Detected at time of request\r\n" +
                "Event Count: 1 count\r\n" +
                "Environmental Condition One: Hydraulic Pressure: 10 bar\r\n" +
                "Environmental Condition Two: Engine Speed : 320 rpm\r\n" +
                "Environmental Condition Three: Vehicle Speed: 141 km/h\r\n" +
                "Environmental Condition Four: Gearbox Oil Temperature: 32 C\r\n" +
                "Environmental Condition Five: Requested Gear: Third\r\n" +
                "Environmental Condition Five: Inserted Gear: Third\r\n" +
                "Environmental Condition Six: Battery Voltage: 8.99 volts\r\n",
                _ecu.ActiveFaultCodes[0].FaultDescription);

            //////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
            // one P0725
            //////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

            _device.FaultcodeList = new byte[] {0x58, 0x1, 0x07, 0x25, 0xE1};
            _device.Faultcodeinfo = new byte[] {0x57, 1, 0x07, 0x25, 0x08, 10, 10, 100, 0x33, 10, 10, 1};
            _ecu.ReadFaultCodes();

            Assert.AreEqual(1, _ecu.ActiveFaultCodes.Count, "m_ssCFC218F.ActiveFaultCodes.Count == 1 for SSCFC218F");
            Console.WriteLine(_ecu.ActiveFaultCodes[0].FaultDescription);
            Assert.AreEqual("P0725", (_ecu.ActiveFaultCodes[0]).FaultComponent);
            Assert.AreEqual(
                "Failed Component Code: Engine Speed\r\n" +
                "Fault Symptom: Signal Not Plausible\r\n" +
                "Warning Lamp Status: Warning Lamp not Illuminated\r\n" +
                "Fault Test State: Test complete for this Error or not applicable.\r\n" +
                "Fault Code Storage State: No Error Detected at time of request\r\n" +
                "Event Count: 1 count\r\n" +
                "Environmental Condition One: Hydraulic Pressure: 10 bar\r\n" +
                "Environmental Condition Two: Engine Speed : 320 rpm\r\n" +
                "Environmental Condition Three: Vehicle Speed: 141 km/h\r\n" +
                "Environmental Condition Four: Gearbox Oil Temperature: 32 C\r\n" +
                "Environmental Condition Five: Requested Gear: Third\r\n" +
                "Environmental Condition Five: Inserted Gear: Third\r\n" +
                "Environmental Condition Six: Battery Voltage: 8.99 volts\r\n",
                _ecu.ActiveFaultCodes[0].FaultDescription);
        }

        [Test]
        public void TestPerformanceMeasurment_SSCFC218F()
        {
            Assert.AreEqual(null, _ecu.SpeedSensor);
            Assert.AreEqual(null, _ecu.RpmSensor);
        }

        [Test]
        public void TestRoutines_SSCFC218F()
        {
            Assert.AreEqual(6, _ecu.Routines.Count);

            Assert.AreEqual("Bleed Clutch", _ecu.Routines[0].Name);
            Assert.AreEqual(RoutineType.Calibration, _ecu.Routines[0].RoutineType);
            Assert.AreEqual(0, _ecu.Routines[0].Result);

            Assert.AreEqual("Depressurise Accumulator", _ecu.Routines[1].Name);
            Assert.AreEqual(RoutineType.Calibration, _ecu.Routines[1].RoutineType);
            Assert.AreEqual(0, _ecu.Routines[1].Result);

            Assert.AreEqual("Enable AutoAdjustment of Clutch", _ecu.Routines[2].Name);
            Assert.AreEqual(RoutineType.Activation, _ecu.Routines[2].RoutineType);
            Assert.AreEqual(0, _ecu.Routines[2].Result);
            Assert.AreEqual(0, _ecu.Routines[2].Arguments.Count);

            Assert.AreEqual("AutoAdjustment EOL/Service", _ecu.Routines[3].Name);
            Assert.AreEqual(RoutineType.Calibration, _ecu.Routines[3].RoutineType);
            Assert.AreEqual(0, _ecu.Routines[3].Result);

            Assert.AreEqual("New Actuator", _ecu.Routines[4].Name);
            Assert.AreEqual(RoutineType.Activation, _ecu.Routines[4].RoutineType);
            Assert.AreEqual(0, _ecu.Routines[4].Result);

            Assert.AreEqual("Calculate the clutch degredation index", _ecu.Routines[5].Name);
            Assert.AreEqual(RoutineType.Calibration, _ecu.Routines[5].RoutineType);
            Assert.AreEqual(0, _ecu.Routines[5].Result);

            _device.LocalidentifierData = new byte[] {0x73, 0x10, 0};

            Assert.AreEqual(0, _ecu.ExecuteRoutine(_ecu.Routines[0]));

            _device.LocalidentifierData = new byte[] {0x73, 0x10, 0xFF};

            Assert.AreEqual(0xFF, _ecu.ExecuteRoutine(_ecu.Routines[0]));
        }

        private void TestSensorValues_SSCFC218F(Ecu ecu)
        {
            TimeStampedMeasurement tsm;
            LocalIdentifier li;
            int i = 0;

            Console.WriteLine(string.Format("Testing Sensor: [{0}] {1}", i, ecu.Sensors[i].Name));
            li = ecu.Sensors[i] as LocalIdentifier;
            _device.LocalidentifierData = new byte[] {0x61, (Byte) li.Id, 0xff, 0xff};
            tsm = ecu.ReadSensor(ecu.Sensors[i]);

            Assert.AreEqual(0xffff*0.036f, tsm.Measurement);

            i++;

            Console.WriteLine(string.Format("Testing Sensor: [{0}] {1}", i, ecu.Sensors[i].Name));
            li = ecu.Sensors[i] as LocalIdentifier;
            _device.LocalidentifierData = new byte[] {0x61, (Byte) li.Id, 0xff, 0xff};
            tsm = ecu.ReadSensor(ecu.Sensors[i]);

            Assert.AreEqual(0xffff*0.032f, tsm.Measurement);

            i++;

            Console.WriteLine(string.Format("Testing Sensor: [{0}] {1}", i, ecu.Sensors[i].Name));
            li = ecu.Sensors[i] as LocalIdentifier;
            _device.LocalidentifierData = new byte[] {0x61, (Byte) li.Id, 0xff, 0xff};
            tsm = ecu.ReadSensor(ecu.Sensors[i]);

            Assert.AreEqual(0xffff*0.026f, tsm.Measurement);

            i++;

            Console.WriteLine(string.Format("Testing Sensor: [{0}] {1}", i, ecu.Sensors[i].Name));
            li = ecu.Sensors[i] as LocalIdentifier;
            _device.LocalidentifierData = new byte[] {0x61, (Byte) li.Id, 0xff, 0xff};
            tsm = ecu.ReadSensor(ecu.Sensors[i]);

            Assert.AreEqual(0xffff*0.026f, tsm.Measurement);

            i++;

            Console.WriteLine(string.Format("Testing Sensor: [{0}] {1}", i, ecu.Sensors[i].Name));
            li = ecu.Sensors[i] as LocalIdentifier;
            _device.LocalidentifierData = new byte[] {0x61, (Byte) li.Id, 0xff, 0xff};
            tsm = ecu.ReadSensor(ecu.Sensors[i]);

            Assert.AreEqual(0xffff*0.04f, tsm.Measurement);

            i++;

            Console.WriteLine(string.Format("Testing Sensor: [{0}] {1}", i, ecu.Sensors[i].Name));
            li = ecu.Sensors[i] as LocalIdentifier;
            _device.LocalidentifierData = new byte[] {0x61, (Byte) li.Id, 0xff, 0xff};
            tsm = ecu.ReadSensor(ecu.Sensors[i]);

            Assert.AreEqual(6291.82715f, tsm.Measurement);

            i++;

            Console.WriteLine(string.Format("Testing Sensor: [{0}] {1}", i, ecu.Sensors[i].Name));
            li = ecu.Sensors[i] as LocalIdentifier;
            _device.LocalidentifierData = new byte[] {0x61, (Byte) li.Id, 0xFF, 0xFF};
            tsm = ecu.ReadSensor(ecu.Sensors[i]);

            Assert.AreEqual(-0.100000001f, tsm.Measurement);

            _device.LocalidentifierData = new byte[] {0x61, (Byte) li.Id, 0xFF, 0xFB};
            tsm = ecu.ReadSensor(ecu.Sensors[i]);

            Assert.AreEqual(-102.5f, tsm.Measurement);

            _device.LocalidentifierData = new byte[] {0x61, (Byte) li.Id, 0x00, 0xFF};
            tsm = ecu.ReadSensor(ecu.Sensors[i]);

            Assert.AreEqual(-25.6000004f, tsm.Measurement);

            i++;

            Console.WriteLine(string.Format("Testing Sensor: [{0}] {1}", i, ecu.Sensors[i].Name));
            li = ecu.Sensors[i] as LocalIdentifier;
            _device.LocalidentifierData = new byte[] {0x61, (Byte) li.Id, 0x00, 0xff};
            tsm = ecu.ReadSensor(ecu.Sensors[i]);

            Assert.AreEqual(0xff*0.0281f, tsm.Measurement);

            i++;

            Console.WriteLine(string.Format("Testing Sensor: [{0}] {1}", i, ecu.Sensors[i].Name));
            li = ecu.Sensors[i] as LocalIdentifier;
            _device.LocalidentifierData = new byte[] {0x61, (Byte) li.Id, 0x00, 0xff};
            tsm = ecu.ReadSensor(ecu.Sensors[i]);

            Assert.AreEqual(255, tsm.Measurement);

            i++;

            Console.WriteLine(string.Format("Testing Sensor: [{0}] {1}", i, ecu.Sensors[i].Name));
            li = ecu.Sensors[i] as LocalIdentifier;
            _device.LocalidentifierData = new byte[] {0x61, (Byte) li.Id, 0x00, 0xff};
            tsm = ecu.ReadSensor(ecu.Sensors[i]);

            Assert.AreEqual(255, tsm.Measurement);

            i++;

            Console.WriteLine(string.Format("Testing Sensor: [{0}] {1}", i, ecu.Sensors[i].Name));
            li = ecu.Sensors[i] as LocalIdentifier;
            _device.LocalidentifierData = new byte[] {0x61, (Byte) li.Id, 0x00, 0xff};
            tsm = ecu.ReadSensor(ecu.Sensors[i]);

            Assert.AreEqual(0xff*0.011f, tsm.Measurement);

            i++;

            Console.WriteLine(string.Format("Testing Sensor: [{0}] {1}", i, ecu.Sensors[i].Name));
            li = ecu.Sensors[i] as LocalIdentifier;
            _device.LocalidentifierData = new byte[] {0x61, (Byte) li.Id, 0x00, 0xff};
            tsm = ecu.ReadSensor(ecu.Sensors[i]);

            Assert.AreEqual(2550, tsm.Measurement);

            i++;

            Console.WriteLine(string.Format("Testing Sensor: [{0}] {1}", i, ecu.Sensors[i].Name));
            li = ecu.Sensors[i] as LocalIdentifier;
            _device.LocalidentifierData = new byte[] {0x61, (Byte) li.Id, 0x00, 0xff};
            tsm = ecu.ReadSensor(ecu.Sensors[i]);

            Assert.AreEqual(2550, tsm.Measurement);

            i++;

            Console.WriteLine(string.Format("Testing Sensor: [{0}] {1}", i, ecu.Sensors[i].Name));
            li = ecu.Sensors[i] as LocalIdentifier;
            _device.LocalidentifierData = new byte[] {0x61, (Byte) li.Id, 0x00, 0xff};
            tsm = ecu.ReadSensor(ecu.Sensors[i]);

            Assert.AreEqual(0xff*7.8f, tsm.Measurement);

            i++;

            Console.WriteLine(string.Format("Testing Sensor: [{0}] {1}", i, ecu.Sensors[i].Name));
            li = ecu.Sensors[i] as LocalIdentifier;
            _device.LocalidentifierData = new byte[] {0x61, (Byte) li.Id, 0x0, 0xff};
            tsm = ecu.ReadSensor(ecu.Sensors[i]);

            Assert.AreEqual(255, tsm.Measurement);

            i++;

            Console.WriteLine(string.Format("Testing Sensor: [{0}] {1}", i, ecu.Sensors[i].Name));
            li = ecu.Sensors[i] as LocalIdentifier;
            _device.LocalidentifierData = new byte[] {0x61, (Byte) li.Id, 0x0, 0xff};
            tsm = ecu.ReadSensor(ecu.Sensors[i]);

            Assert.AreEqual(255, tsm.Measurement);

            i++;

            Console.WriteLine(string.Format("Testing Sensor: [{0}] {1}", i, ecu.Sensors[i].Name));
            li = ecu.Sensors[i] as LocalIdentifier;
            _device.LocalidentifierData = new byte[] {0x61, (Byte) li.Id, 0xff};
            tsm = ecu.ReadSensor(ecu.Sensors[i]);
            Assert.AreEqual(true, tsm.IsStatus);
            Assert.AreEqual("No Request", tsm.Status);
            Assert.AreEqual(0xff, tsm.Measurement);

            _device.LocalidentifierData = new byte[] {0x61, (Byte) li.Id, 0x9F};
            tsm = ecu.ReadSensor(ecu.Sensors[i]);
            Assert.AreEqual(true, tsm.IsStatus);
            Assert.AreEqual("Request From Right Button", tsm.Status);
            Assert.AreEqual(0x9f, tsm.Measurement);

            _device.LocalidentifierData = new byte[] {0x61, (Byte) li.Id, 0xAF};
            tsm = ecu.ReadSensor(ecu.Sensors[i]);
            Assert.AreEqual(true, tsm.IsStatus);
            Assert.AreEqual("Request From Left Button", tsm.Status);
            Assert.AreEqual(0xAf, tsm.Measurement);

            _device.LocalidentifierData = new byte[] {0x61, (Byte) li.Id, 0xF9};
            tsm = ecu.ReadSensor(ecu.Sensors[i]);
            Assert.AreEqual(true, tsm.IsStatus);
            Assert.AreEqual("Lever Back", tsm.Status);
            Assert.AreEqual(0xF9, tsm.Measurement);

            _device.LocalidentifierData = new byte[] {0x61, (Byte) li.Id, 0xF3};
            tsm = ecu.ReadSensor(ecu.Sensors[i]);
            Assert.AreEqual(true, tsm.IsStatus);
            Assert.AreEqual("Lever Forward", tsm.Status);
            Assert.AreEqual(0xF3, tsm.Measurement);

            _device.LocalidentifierData = new byte[] {0x61, (Byte) li.Id, 0xFA};
            tsm = ecu.ReadSensor(ecu.Sensors[i]);
            Assert.AreEqual(true, tsm.IsStatus);
            Assert.AreEqual("Request Neutral From Lever", tsm.Status);
            Assert.AreEqual(0xFA, tsm.Measurement);

            _device.LocalidentifierData = new byte[] {0x61, (Byte) li.Id, 0xFC};
            tsm = ecu.ReadSensor(ecu.Sensors[i]);
            Assert.AreEqual(true, tsm.IsStatus);
            Assert.AreEqual("Request Reverse From Lever", tsm.Status);
            Assert.AreEqual(0xFC, tsm.Measurement);

            i++;

            Console.WriteLine(string.Format("Testing Sensor: [{0}] {1}", i, ecu.Sensors[i].Name));
            li = ecu.Sensors[i] as LocalIdentifier;
            _device.LocalidentifierData = new byte[] {0x61, (Byte) li.Id, 0x00, 0};
            tsm = ecu.ReadSensor(ecu.Sensors[i]);
            Assert.AreEqual("Neutral", tsm.Status);
            Assert.AreEqual(0, tsm.Measurement);

            _device.LocalidentifierData = new byte[] {0x61, (Byte) li.Id, 0x00, 6};
            tsm = ecu.ReadSensor(ecu.Sensors[i]);
            Assert.AreEqual("Sixth", tsm.Status);
            Assert.AreEqual(6, tsm.Measurement);

            _device.LocalidentifierData = new byte[] {0x61, (Byte) li.Id, 0x00, 7};
            tsm = ecu.ReadSensor(ecu.Sensors[i]);
            Assert.AreEqual("Reverse", tsm.Status);
            Assert.AreEqual(7, tsm.Measurement);

            i++;

            Console.WriteLine(string.Format("Testing Sensor: [{0}] {1}", i, ecu.Sensors[i].Name));
            li = ecu.Sensors[i] as LocalIdentifier;
            _device.LocalidentifierData = new byte[] {0x61, (Byte) li.Id, 0x00, 0};
            tsm = ecu.ReadSensor(ecu.Sensors[i]);
            Assert.AreEqual("False", tsm.Status);
            Assert.AreEqual(0, tsm.Measurement);

            _device.LocalidentifierData = new byte[] {0x61, (Byte) li.Id, 0x00, 1};
            tsm = ecu.ReadSensor(ecu.Sensors[i]);
            Assert.AreEqual("True", tsm.Status);
            Assert.AreEqual(1, tsm.Measurement);

            i++;

            Console.WriteLine(string.Format("Testing Sensor: [{0}] {1}", i, ecu.Sensors[i].Name));
            li = ecu.Sensors[i] as LocalIdentifier;
            _device.LocalidentifierData = new byte[] {0x61, (Byte) li.Id, 0x00, 0};
            tsm = ecu.ReadSensor(ecu.Sensors[i]);
            Assert.AreEqual("False", tsm.Status);
            Assert.AreEqual(0, tsm.Measurement);

            _device.LocalidentifierData = new byte[] {0x61, (Byte) li.Id, 0x00, 2};
            tsm = ecu.ReadSensor(ecu.Sensors[i]);
            Assert.AreEqual("True", tsm.Status);
            Assert.AreEqual(1, tsm.Measurement);

            i++;

            Console.WriteLine(string.Format("Testing Sensor: [{0}] {1}", i, ecu.Sensors[i].Name));
            li = ecu.Sensors[i] as LocalIdentifier;
            _device.LocalidentifierData = new byte[] {0x61, (Byte) li.Id, 0x00, 0};
            tsm = ecu.ReadSensor(ecu.Sensors[i]);
            Assert.AreEqual("False", tsm.Status);
            Assert.AreEqual(0, tsm.Measurement);

            _device.LocalidentifierData = new byte[] {0x61, (Byte) li.Id, 0x00, 4};
            tsm = ecu.ReadSensor(ecu.Sensors[i]);
            Assert.AreEqual("True", tsm.Status);
            Assert.AreEqual(1, tsm.Measurement);

            i++;


            Console.WriteLine(string.Format("Testing Sensor: [{0}] {1}", i, ecu.Sensors[i].Name));
            li = ecu.Sensors[i] as LocalIdentifier;
            _device.LocalidentifierData = new byte[] {0x61, (Byte) li.Id, 0x00, 0};
            tsm = ecu.ReadSensor(ecu.Sensors[i]);
            Assert.AreEqual("False", tsm.Status);
            Assert.AreEqual(0, tsm.Measurement);

            _device.LocalidentifierData = new byte[] {0x61, (Byte) li.Id, 0x00, 8};
            tsm = ecu.ReadSensor(ecu.Sensors[i]);
            Assert.AreEqual("True", tsm.Status);
            Assert.AreEqual(1, tsm.Measurement);

            i++;


            Console.WriteLine(string.Format("Testing Sensor: [{0}] {1}", i, ecu.Sensors[i].Name));
            li = ecu.Sensors[i] as LocalIdentifier;
            _device.LocalidentifierData = new byte[] {0x61, (Byte) li.Id, 0x00, 0};
            tsm = ecu.ReadSensor(ecu.Sensors[i]);
            Assert.AreEqual("False", tsm.Status);
            Assert.AreEqual(0, tsm.Measurement);

            _device.LocalidentifierData = new byte[] {0x61, (Byte) li.Id, 0x00, 0x10};
            tsm = ecu.ReadSensor(ecu.Sensors[i]);
            Assert.AreEqual("True", tsm.Status);
            Assert.AreEqual(1, tsm.Measurement);

            i++;


            Console.WriteLine(string.Format("Testing Sensor: [{0}] {1}", i, ecu.Sensors[i].Name));
            li = ecu.Sensors[i] as LocalIdentifier;
            _device.LocalidentifierData = new byte[] {0x61, (Byte) li.Id, 0x00, 0};
            tsm = ecu.ReadSensor(ecu.Sensors[i]);
            Assert.AreEqual("Off", tsm.Status);
            Assert.AreEqual(0, tsm.Measurement);

            _device.LocalidentifierData = new byte[] {0x61, (Byte) li.Id, 0x00, 1};
            tsm = ecu.ReadSensor(ecu.Sensors[i]);
            Assert.AreEqual("On", tsm.Status);
            Assert.AreEqual(1, tsm.Measurement);

            i++;

            Console.WriteLine(string.Format("Testing Sensor: [{0}] {1}", i, ecu.Sensors[i].Name));
            li = ecu.Sensors[i] as LocalIdentifier;
            _device.LocalidentifierData = new byte[] {0x61, (Byte) li.Id, 0x00, 0};
            tsm = ecu.ReadSensor(ecu.Sensors[i]);
            Assert.AreEqual("Off", tsm.Status);
            Assert.AreEqual(0, tsm.Measurement);

            _device.LocalidentifierData = new byte[] {0x61, (Byte) li.Id, 0x00, 4};
            tsm = ecu.ReadSensor(ecu.Sensors[i]);
            Assert.AreEqual("On", tsm.Status);
            Assert.AreEqual(1, tsm.Measurement);

            i++;

            Console.WriteLine(string.Format("Testing Sensor: [{0}] {1}", i, ecu.Sensors[i].Name));
            li = ecu.Sensors[i] as LocalIdentifier;
            _device.LocalidentifierData = new byte[] {0x61, (Byte) li.Id, 0x00, 0};
            tsm = ecu.ReadSensor(ecu.Sensors[i]);
            Assert.AreEqual("Off", tsm.Status);
            Assert.AreEqual(0, tsm.Measurement);

            _device.LocalidentifierData = new byte[] {0x61, (Byte) li.Id, 0x00, 8};
            tsm = ecu.ReadSensor(ecu.Sensors[i]);
            Assert.AreEqual("On", tsm.Status);
            Assert.AreEqual(1, tsm.Measurement);

            i++;

            Console.WriteLine(string.Format("Testing Sensor: [{0}] {1}", i, ecu.Sensors[i].Name));
            li = ecu.Sensors[i] as LocalIdentifier;
            _device.LocalidentifierData = new byte[] {0x61, (Byte) li.Id, 0x00, 0};
            tsm = ecu.ReadSensor(ecu.Sensors[i]);
            Assert.AreEqual("Off", tsm.Status);
            Assert.AreEqual(0, tsm.Measurement);

            _device.LocalidentifierData = new byte[] {0x61, (Byte) li.Id, 0x00, 0x20};
            tsm = ecu.ReadSensor(ecu.Sensors[i]);
            Assert.AreEqual("On", tsm.Status);
            Assert.AreEqual(1, tsm.Measurement);

            i++;

            Console.WriteLine(string.Format("Testing Sensor: [{0}] {1}", i, ecu.Sensors[i].Name));
            li = ecu.Sensors[i] as LocalIdentifier;
            _device.LocalidentifierData = new byte[] {0x61, (Byte) li.Id, 0x00, 0};
            tsm = ecu.ReadSensor(ecu.Sensors[i]);
            Assert.AreEqual("Neutral", tsm.Status);
            Assert.AreEqual(0, tsm.Measurement);

            _device.LocalidentifierData = new byte[] {0x61, (Byte) li.Id, 0x00, 6};
            tsm = ecu.ReadSensor(ecu.Sensors[i]);
            Assert.AreEqual("Sixth", tsm.Status);
            Assert.AreEqual(6, tsm.Measurement);

            _device.LocalidentifierData = new byte[] {0x61, (Byte) li.Id, 0x00, 7};
            tsm = ecu.ReadSensor(ecu.Sensors[i]);
            Assert.AreEqual("Reverse", tsm.Status);
            Assert.AreEqual(7, tsm.Measurement);

            i++;

            for (int x = 0; x < 16; x++)
            {
                Console.WriteLine(string.Format("Testing Sensor: [{0}] {1}\r\n", i, ecu.Sensors[i].Name));
                li = ecu.Sensors[i] as LocalIdentifier;
                _device.LocalidentifierData = new byte[] {0x61, (Byte) li.Id, 0x00, 0x64};
                tsm = ecu.ReadSensor(ecu.Sensors[i]);

                Assert.AreEqual(0x64*0.036f, tsm.Measurement);

                i++;
            }

            for (int x = 0; x < 8; x++)
            {
                Console.WriteLine(string.Format("Testing Sensor: [{0}] {1}", i, ecu.Sensors[i].Name));
                li = ecu.Sensors[i] as LocalIdentifier;
                _device.LocalidentifierData = new byte[] {0x61, (Byte) li.Id, 0x00, 0x64};
                tsm = ecu.ReadSensor(ecu.Sensors[i]);

                Assert.AreEqual(0x64*0.032f, tsm.Measurement);

                i++;
            }

            Console.WriteLine(string.Format("Testing Sensor: [{0}] {1}", i, ecu.Sensors[i].Name));
            li = ecu.Sensors[i] as LocalIdentifier;
            _device.LocalidentifierData = new byte[] {0x61, (Byte) li.Id, 0x00, 0x64};
            tsm = ecu.ReadSensor(ecu.Sensors[i]);

            Assert.AreEqual(0x64*0.026f, tsm.Measurement);

            i++;

            Console.WriteLine(string.Format("Testing Sensor: [{0}] {1}", i, ecu.Sensors[i].Name));
            li = ecu.Sensors[i] as LocalIdentifier;
            _device.LocalidentifierData = new byte[] {0x61, (Byte) li.Id, 0x00, 0x64};
            tsm = ecu.ReadSensor(ecu.Sensors[i]);

            Assert.AreEqual(0x64*0.026f, tsm.Measurement);

            i++;

            Console.WriteLine(string.Format("Testing Sensor: [{0}] {1}", i, ecu.Sensors[i].Name));
            li = ecu.Sensors[i] as LocalIdentifier;
            _device.LocalidentifierData = new byte[] {0x61, (Byte) li.Id, 0x00, 0x64};
            tsm = ecu.ReadSensor(ecu.Sensors[i]);

            Assert.AreEqual(780, tsm.Measurement);

            i++;

            Console.WriteLine(string.Format("Testing Sensor: [{0}] {1}", i, ecu.Sensors[i].Name));
            li = ecu.Sensors[i] as LocalIdentifier;
            _device.LocalidentifierData = new byte[] {0x61, (Byte) li.Id, 0x00, 0x64};
            tsm = ecu.ReadSensor(ecu.Sensors[i]);

            Assert.AreEqual(2560f, tsm.Measurement);

            i++;

            Console.WriteLine(string.Format("Testing Sensor: [{0}] {1}", i, ecu.Sensors[i].Name));
            li = ecu.Sensors[i] as LocalIdentifier;
            _device.LocalidentifierData = new byte[] {0x61, (Byte) li.Id, 0x00, 0x64};
            tsm = ecu.ReadSensor(ecu.Sensors[i]);

            Assert.AreEqual(2560.0f, tsm.Measurement);

            i++;

            Console.WriteLine(string.Format("Testing Sensor: [{0}] {1}", i, ecu.Sensors[i].Name));
            li = ecu.Sensors[i] as LocalIdentifier;
            _device.LocalidentifierData = new byte[] {0x61, (Byte) li.Id, 0x00, 0x64};
            tsm = ecu.ReadSensor(ecu.Sensors[i]);

            Assert.AreEqual(20, tsm.Measurement);

            i++;

            for (int x = 0; x < 27; x++)
            {
                Console.WriteLine(string.Format("Testing Sensor: [{0}] {1}", i, ecu.Sensors[i].Name));
                li = ecu.Sensors[i] as LocalIdentifier;
                _device.LocalidentifierData = new byte[] {0x61, (Byte) li.Id, 0x00, 0x64};
                tsm = ecu.ReadSensor(ecu.Sensors[i]);

                Assert.AreEqual(100, tsm.Measurement);

                i++;
            }

            Console.WriteLine(string.Format("Testing Sensor: [{0}] {1}", i, ecu.Sensors[i].Name));
            li = ecu.Sensors[i] as LocalIdentifier;
            _device.LocalidentifierData = new byte[] {0x61, (Byte) li.Id, 0x00, 0};
            tsm = ecu.ReadSensor(ecu.Sensors[i]);
            Assert.AreEqual("AutoCalibration not executed", tsm.Status);
            Assert.AreEqual(0, tsm.Measurement);

            _device.LocalidentifierData = new byte[] {0x61, (Byte) li.Id, 0x00, 0x01};
            tsm = ecu.ReadSensor(ecu.Sensors[i]);
            Assert.AreEqual("Error Selecting F-RM", tsm.Status);
            Assert.AreEqual(1, tsm.Measurement);

            _device.LocalidentifierData = new byte[] {0x61, (Byte) li.Id, 0x00, 0x02};
            tsm = ecu.ReadSensor(ecu.Sensors[i]);
            Assert.AreEqual("Error Selecting 1a-2a", tsm.Status);
            Assert.AreEqual(2, tsm.Measurement);

            _device.LocalidentifierData = new byte[] {0x61, (Byte) li.Id, 0x00, 0x03};
            tsm = ecu.ReadSensor(ecu.Sensors[i]);
            Assert.AreEqual("Error Selecting 3a-4a", tsm.Status);
            Assert.AreEqual(3, tsm.Measurement);

            _device.LocalidentifierData = new byte[] {0x61, (Byte) li.Id, 0x00, 0x04};
            tsm = ecu.ReadSensor(ecu.Sensors[i]);
            Assert.AreEqual("Error Selecting 5a-6a", tsm.Status);
            Assert.AreEqual(4, tsm.Measurement);

            _device.LocalidentifierData = new byte[] {0x61, (Byte) li.Id, 0x00, 0x05};
            tsm = ecu.ReadSensor(ecu.Sensors[i]);
            Assert.AreEqual("Error Inserting Neutral", tsm.Status);
            Assert.AreEqual(5, tsm.Measurement);

            _device.LocalidentifierData = new byte[] {0x61, (Byte) li.Id, 0x00, 0x06};
            tsm = ecu.ReadSensor(ecu.Sensors[i]);
            Assert.AreEqual("Error Inserting 1a", tsm.Status);
            Assert.AreEqual(6, tsm.Measurement);

            _device.LocalidentifierData = new byte[] {0x61, (Byte) li.Id, 0x00, 0x07};
            tsm = ecu.ReadSensor(ecu.Sensors[i]);
            Assert.AreEqual("Error Inserting 2a", tsm.Status);
            Assert.AreEqual(7, tsm.Measurement);

            _device.LocalidentifierData = new byte[] {0x61, (Byte) li.Id, 0x00, 0x08};
            tsm = ecu.ReadSensor(ecu.Sensors[i]);
            Assert.AreEqual("Error Inserting 3a", tsm.Status);
            Assert.AreEqual(8, tsm.Measurement);

            _device.LocalidentifierData = new byte[] {0x61, (Byte) li.Id, 0x00, 0x09};
            tsm = ecu.ReadSensor(ecu.Sensors[i]);
            Assert.AreEqual("Error Inserting 4a", tsm.Status);
            Assert.AreEqual(9, tsm.Measurement);

            _device.LocalidentifierData = new byte[] {0x61, (Byte) li.Id, 0x00, 0x0A};
            tsm = ecu.ReadSensor(ecu.Sensors[i]);
            Assert.AreEqual("Error Inserting 5a", tsm.Status);
            Assert.AreEqual(0xa, tsm.Measurement);

            _device.LocalidentifierData = new byte[] {0x61, (Byte) li.Id, 0x00, 0x0b};
            tsm = ecu.ReadSensor(ecu.Sensors[i]);
            Assert.AreEqual("Error Inserting 6a", tsm.Status);
            Assert.AreEqual(0x0b, tsm.Measurement);

            _device.LocalidentifierData = new byte[] {0x61, (Byte) li.Id, 0x00, 0xc};
            tsm = ecu.ReadSensor(ecu.Sensors[i]);
            Assert.AreEqual("Error Inserting RM", tsm.Status);
            Assert.AreEqual(0x0c, tsm.Measurement);

            _device.LocalidentifierData = new byte[] {0x61, (Byte) li.Id, 0x00, 0x0d};
            tsm = ecu.ReadSensor(ecu.Sensors[i]);
            Assert.AreEqual("Interuption due to Generic Failure Conditions", tsm.Status);
            Assert.AreEqual(0x0d, tsm.Measurement);

            _device.LocalidentifierData = new byte[] {0x61, (Byte) li.Id, 0x00, 0x0e};
            tsm = ecu.ReadSensor(ecu.Sensors[i]);
            Assert.AreEqual("Insufficent clutch speed", tsm.Status);
            Assert.AreEqual(0xe, tsm.Measurement);

            _device.LocalidentifierData = new byte[] {0x61, (Byte) li.Id, 0x00, 0x0f};
            tsm = ecu.ReadSensor(ecu.Sensors[i]);
            Assert.AreEqual("AutoCalibration \"OK\"", tsm.Status);
            Assert.AreEqual(15, tsm.Measurement);

            i++;

            Console.WriteLine(string.Format("Testing Sensor: [{0}] {1}", i, ecu.Sensors[i].Name));
            li = ecu.Sensors[i] as LocalIdentifier;
            _device.LocalidentifierData = new byte[] {0x61, (Byte) li.Id, 0x00, 0x64};
            tsm = ecu.ReadSensor(ecu.Sensors[i]);

            Assert.AreEqual(100, tsm.Measurement);

            i++;

            Console.WriteLine(string.Format("Testing Sensor: [{0}] {1}", i, ecu.Sensors[i].Name));
            li = ecu.Sensors[i] as LocalIdentifier;
            _device.LocalidentifierData = new byte[] {0x61, (Byte) li.Id, 0x00, 0x64};
            tsm = ecu.ReadSensor(ecu.Sensors[i]);

            Assert.AreEqual(6.0f, tsm.Measurement);

            i++;

            Console.WriteLine(string.Format("Testing Sensor: [{0}] {1}", i, ecu.Sensors[i].Name));
            li = ecu.Sensors[i] as LocalIdentifier;
            _device.LocalidentifierData = new byte[] {0x61, (Byte) li.Id, 0x00, 0x64};
            tsm = ecu.ReadSensor(ecu.Sensors[i]);

            Assert.AreEqual(6.0f, tsm.Measurement);

            i++;

            for (int x = 0; x < 16; x++)
            {
                Console.WriteLine(string.Format("Testing Sensor: [{0}] {1}", i, ecu.Sensors[i].Name));
                li = ecu.Sensors[i] as LocalIdentifier;
                _device.LocalidentifierData = new byte[] {0x61, (Byte) li.Id, 0x00, 0x64};
                tsm = ecu.ReadSensor(ecu.Sensors[i]);

                Assert.AreEqual(100, tsm.Measurement);

                i++;
            }
        }

        [TestFixtureTearDown]
        public void Dispose()
        {
            if (_ecu != null)
            {
                _ecu.Close();
                _ecu.Dispose();
            }
        }
    }
}