#region License
// Copyright  2004 - 2010 All Right Reserved, Derek Goslin
// 
// 1. Definitions
// The terms "reproduce," "reproduction," "derivative works," and "distribution" have the same meaning here as under U.S. copyright law.
// A "contribution" is the original software, or any additions or changes to the software.
// A "contributor" is any person that distributes its contribution under this license.
// "Licensed patents" are a contributor's patent claims that read directly on its contribution.
// 
// 2. Grant of Rights
// (A) Copyright Grant- Subject to the terms of this license, including the license
//     conditions and limitations in section 3, each contributor grants you a non-exclusive,
//     worldwide, royalty-free copyright license to reproduce its contribution, prepare derivative 
//     works of its contribution, and distribute its contribution or any derivative works that you create.
// (B) Patent Grant- Subject to the terms of this license, including the license conditions and limitations
//     in section 3, each contributor grants you a non-exclusive, worldwide, royalty-free license under its
//     licensed patents to make, have made, use, sell, offer for sale, import, and/or otherwise dispose of
//     its contribution in the software or derivative works of the contribution in the software.
// 
// 3. Conditions and Limitations
// (A) Reciprocal Grants- For any file you distribute that contains code from the software (in source code or binary format), 
//     you must provide recipients the source code to that file along with a copy of this license, which license will govern that file. 
//     You may license other files that are entirely your own work and do not contain code from the software under any terms you choose.
// (B) No Trademark License- This license does not grant you rights to use any contributors' name, logo, or trademarks.
// (C) If you bring a patent claim against any contributor over patents that you claim are infringed by the software,
//     your patent license from such contributor to the software ends automatically.
// (D) If you distribute any portion of the software, you must retain all copyright, patent, trademark,
//     and attribution notices that are present in the software.
// (E) If you distribute any portion of the software in source code form, you may do so only under this license by including a complete copy 
//     of this license with your distribution. If you distribute any portion of the software in compiled or object code form, you may only do 
//     so under a license that complies with this license.
// (F) The software is licensed "as-is." You bear the risk of using it. The contributors give no express warranties, guarantees or conditions.
//    You may have additional consumer rights under your local laws which this license cannot change. To the extent permitted under your
//    local laws, the contributors exclude the implied warranties of merchantability, fitness for a particular purpose and non-infringement.
#endregion

using System;
using System.Collections.Generic;
using DnEcuDiag.Ecus.Iso;
using DnEcuDiag.Factories;
using NUnit.Framework;

namespace DnEcuDiag.Tests
{
    [TestFixture]
    public class MY99Tests
    {
        private Kwp81Ecu _ecu;
        private IsoTestDevice _device;

        [TestFixtureSetUp]
        public void Init()
        {
            try
            {
                _device = new IsoTestDevice();

                _ecu = EcuFactory.CreateEcu(EcuFactory.EcuList.Find(obj => obj.Name.Equals("MY99"))) as Kwp81Ecu;
                _ecu.DiagnosticInterface = _device;
                _ecu.Connect();
            }
            catch (Exception e)
            {
                Console.WriteLine(e.ToString());
            }
        }

        [Test]
        public void TestConnect()
        {
            Assert.AreNotEqual(string.Empty, _ecu.EcuWarning);
            Assert.AreEqual("55 A7 86 85 97 9E 79", _ecu.IsoCode);
            Assert.AreEqual(string.Empty, _ecu.HardwareNumber);
            Assert.AreEqual("0.1", _ecu.HardwareVersionNumber);
            Assert.AreEqual("MY99", _ecu.Model);
            Assert.AreEqual(string.Empty, _ecu.OmologationNumber);
            Assert.AreEqual("98 331", _ecu.ProductionDate);
            Assert.AreEqual(string.Empty, _ecu.ProgrammingDate);
            Assert.AreEqual("00711", _ecu.SerialNumber);
            Assert.AreEqual("0.1", _ecu.SoftwareVersionNumber);
            Assert.AreEqual("0060658009", _ecu.DrawingNumber);
            Assert.AreEqual(string.Empty, _ecu.SoftwareNumber);
            Assert.AreEqual(0x00, _ecu.EcuAddress);
        }

        [Test]
        public void TestFaults_MY99()
        {
            _device.Faultcodeinfo = new List<byte[]>
                                          {
                                              new byte[]
                                                  {
                                                      0x1B, 0xFC, 0x0C, 0x61, 0x00, 0x66, 0x28, 0x0B, 0x61, 0x00, 0x66,
                                                      0x28,
                                                      0x09
                                                      , 0x61, 0x00, 0x66, 0x28, 0x08, 0x61, 0x00, 0x66, 0x28, 0x01, 0x6F,
                                                      0x00,
                                                      0x6B, 0x30, 0x06
                                                  },
                                              new byte[]
                                                  {
                                                      0x1B, 0xFC, 0x19, 0x29, 0x00, 0x6D, 0xAF, 0x05, 0x61, 0x00, 0x66,
                                                      0x30,
                                                      0x14, 0x61, 0x00, 0x66, 0x30, 0x03, 0x61, 0x00, 0x66, 0x28, 0x04,
                                                      0x69,
                                                      0x00, 0x66, 0x28, 0x69
                                                  }
                                          };
            _ecu.ReadFaultCodes();

            Assert.AreEqual(10, _ecu.ActiveFaultCodes.Count);

            Console.WriteLine(_ecu.ActiveFaultCodes[0].FaultComponent);
            Console.WriteLine(_ecu.ActiveFaultCodes[0].FaultDescription);
            Assert.AreEqual("000C", _ecu.ActiveFaultCodes[0].FaultComponent);
            Assert.AreEqual("Failed Component Code: Passenger Pretensioner Front\r\nError Code Status: Intermittent Fault Present\r\nError Type: Open Circuit\r\nStatic Image Of Lifetime Timer in Snapshot: 130760 Minutes\r\n", _ecu.ActiveFaultCodes[0].FaultDescription);

            Console.WriteLine(_ecu.ActiveFaultCodes[1].FaultComponent);
            Console.WriteLine(_ecu.ActiveFaultCodes[1].FaultDescription);
            Assert.AreEqual("000B", _ecu.ActiveFaultCodes[1].FaultComponent);
            Assert.AreEqual("Failed Component Code: Driver Side Pretensioner Front\r\nError Code Status: Intermittent Fault Present\r\nError Type: Open Circuit\r\nStatic Image Of Lifetime Timer in Snapshot: 130760 Minutes\r\n", _ecu.ActiveFaultCodes[1].FaultDescription);

            Console.WriteLine(_ecu.ActiveFaultCodes[2].FaultComponent);
            Console.WriteLine(_ecu.ActiveFaultCodes[2].FaultDescription);
            Assert.AreEqual("0009", _ecu.ActiveFaultCodes[2].FaultComponent);
            Assert.AreEqual("Failed Component Code: Passenger 1 1st Stage Airbag Front\r\nError Code Status: Intermittent Fault Present\r\nError Type: Open Circuit\r\nStatic Image Of Lifetime Timer in Snapshot: 130760 Minutes\r\n", _ecu.ActiveFaultCodes[2].FaultDescription);

            Console.WriteLine(_ecu.ActiveFaultCodes[3].FaultComponent);
            Console.WriteLine(_ecu.ActiveFaultCodes[3].FaultDescription);
            Assert.AreEqual("0008", _ecu.ActiveFaultCodes[3].FaultComponent);
            Assert.AreEqual("Failed Component Code: Driver 1st Stage Airbag Front\r\nError Code Status: Intermittent Fault Present\r\nError Type: Open Circuit\r\nStatic Image Of Lifetime Timer in Snapshot: 130760 Minutes\r\n", _ecu.ActiveFaultCodes[3].FaultDescription);

            Console.WriteLine(_ecu.ActiveFaultCodes[4].FaultComponent);
            Console.WriteLine(_ecu.ActiveFaultCodes[4].FaultDescription);
            Assert.AreEqual("0001", _ecu.ActiveFaultCodes[4].FaultComponent);
            Assert.AreEqual("Failed Component Code: ECU\r\nError Code Status: Intermittent Fault Present\r\nError Type: Internal Errors\r\nStatic Image Of Lifetime Timer in Snapshot: 137200 Minutes\r\n", _ecu.ActiveFaultCodes[4].FaultDescription);

            Console.WriteLine(_ecu.ActiveFaultCodes[5].FaultComponent);
            Console.WriteLine(_ecu.ActiveFaultCodes[5].FaultDescription);
            Assert.AreEqual("0019", _ecu.ActiveFaultCodes[5].FaultComponent);
            Assert.AreEqual("Failed Component Code: Wiring Identification\r\nError Code Status: Error Present\r\nError Type: Plausibility\r\nStatic Image Of Lifetime Timer in Snapshot: 140395 Minutes\r\n", _ecu.ActiveFaultCodes[5].FaultDescription);

            Console.WriteLine(_ecu.ActiveFaultCodes[6].FaultComponent);
            Console.WriteLine(_ecu.ActiveFaultCodes[6].FaultDescription);
            Assert.AreEqual("0005", _ecu.ActiveFaultCodes[6].FaultComponent);
            Assert.AreEqual("Failed Component Code: Passengers AirBag Inhibition Light\r\nError Code Status: Intermittent Fault Present\r\nError Type: Open Circuit\r\nStatic Image Of Lifetime Timer in Snapshot: 130800 Minutes\r\n", _ecu.ActiveFaultCodes[6].FaultDescription);

            Console.WriteLine(_ecu.ActiveFaultCodes[7].FaultComponent);
            Console.WriteLine(_ecu.ActiveFaultCodes[7].FaultDescription);
            Assert.AreEqual("0014", _ecu.ActiveFaultCodes[7].FaultComponent);
            Assert.AreEqual("Failed Component Code: Deactivation Switch\r\nError Code Status: Intermittent Fault Present\r\nError Type: Open Circuit\r\nStatic Image Of Lifetime Timer in Snapshot: 130800 Minutes\r\n", _ecu.ActiveFaultCodes[7].FaultDescription);

            Console.WriteLine(_ecu.ActiveFaultCodes[8].FaultComponent);
            Console.WriteLine(_ecu.ActiveFaultCodes[8].FaultDescription);
            Assert.AreEqual("0003", _ecu.ActiveFaultCodes[8].FaultComponent);
            Assert.AreEqual("Failed Component Code: Link (or connection) between Central Unit and Fault Light\r\nError Code Status: Intermittent Fault Present\r\nError Type: Open Circuit\r\nStatic Image Of Lifetime Timer in Snapshot: 130760 Minutes\r\n", _ecu.ActiveFaultCodes[8].FaultDescription);

            Console.WriteLine(_ecu.ActiveFaultCodes[9].FaultComponent);
            Console.WriteLine(_ecu.ActiveFaultCodes[9].FaultDescription);
            Assert.AreEqual("0004", _ecu.ActiveFaultCodes[9].FaultComponent);
            Assert.AreEqual("Failed Component Code: Case Attachement\r\nError Code Status: Intermittent Fault Present\r\nError Type: Plausibility\r\nStatic Image Of Lifetime Timer in Snapshot: 130760 Minutes\r\n", _ecu.ActiveFaultCodes[9].FaultDescription);
        }

        [Test]
        public void TestSnapShots_MY99()
        {
            Assert.AreEqual(8, _ecu.SnapshotDefinitions.Count);

            for (int i = 0; i < 4; i++)
            {
                _device.SnapShotData =
                    new byte[]
                        {
                            0x1B, 0xF4, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF,
                            0xFF,
                            0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF
                        };
                Console.WriteLine(_ecu.ReadSnapshot(_ecu.SnapshotDefinitions[i]));
                _device.SnapShotData =
                    new byte[]
                        {
                            0x1B, 0xF4, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
                            0x00,
                            0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00
                        };
                Console.WriteLine(_ecu.ReadSnapshot(_ecu.SnapshotDefinitions[i]));
            }

            _device.SnapShotData =
                new byte[]
                    {
                        0x5, 0xF4, 0xFF, 0xFF, 0xFF, 0xFF
                    };

            Console.WriteLine(_ecu.ReadSnapshot(_ecu.SnapshotDefinitions[4]));

            _device.SnapShotData =
                new byte[]
                    {
                        0x5, 0xF4, 0x0, 0x0, 0x0, 0x0
                    };
            Console.WriteLine(_ecu.ReadSnapshot(_ecu.SnapshotDefinitions[4]));


            _device.SnapShotData =
                new byte[]
                    {
                        0x5, 0xF4, 0x00, 0x00, 0xFF, 0x10, 0x00
                    };

            Console.WriteLine(_ecu.ReadSnapshot(_ecu.SnapshotDefinitions[5]));

            _device.SnapShotData =
                new byte[]
                    {
                        0x5, 0xF4, 0x0, 0x0, 0x0, 0x0, 0x0
                    };
            Console.WriteLine(_ecu.ReadSnapshot(_ecu.SnapshotDefinitions[5]));

            _device.SnapShotData =
                new byte[]
                    {
                        0x4, 0xF4, 0xFF, 0xFF, 0xFF
                    };

            Console.WriteLine(_ecu.ReadSnapshot(_ecu.SnapshotDefinitions[6]));

            _device.SnapShotData =
                new byte[]
                    {
                        0x4, 0xF4, 0x0, 0x0, 0x0
                    };
            Console.WriteLine(_ecu.ReadSnapshot(_ecu.SnapshotDefinitions[6]));

            _device.SnapShotData =
                new byte[]
                    {
                        0x4, 0xF4, 0xFF, 0xFF, 0xFF
                    };

            Console.WriteLine(_ecu.ReadSnapshot(_ecu.SnapshotDefinitions[7]));

            _device.SnapShotData =
                new byte[]
                    {
                        0x4, 0xF4, 0x0, 0x0, 0x0
                    };
            Console.WriteLine(_ecu.ReadSnapshot(_ecu.SnapshotDefinitions[7]));
        }

        [Test]
        public void TestPerformanceMeasurment_MY99()
        {
            Assert.AreEqual(null, _ecu.SpeedSensor);
            Assert.AreEqual(null, _ecu.RpmSensor);
        }

        [TestFixtureTearDown]
        public void Dispose()
        {
            _ecu = null;
        }
    }
}