#region License
// Copyright  2004 - 2010 All Right Reserved, Derek Goslin
// 
// 1. Definitions
// The terms "reproduce," "reproduction," "derivative works," and "distribution" have the same meaning here as under U.S. copyright law.
// A "contribution" is the original software, or any additions or changes to the software.
// A "contributor" is any person that distributes its contribution under this license.
// "Licensed patents" are a contributor's patent claims that read directly on its contribution.
// 
// 2. Grant of Rights
// (A) Copyright Grant- Subject to the terms of this license, including the license
//     conditions and limitations in section 3, each contributor grants you a non-exclusive,
//     worldwide, royalty-free copyright license to reproduce its contribution, prepare derivative 
//     works of its contribution, and distribute its contribution or any derivative works that you create.
// (B) Patent Grant- Subject to the terms of this license, including the license conditions and limitations
//     in section 3, each contributor grants you a non-exclusive, worldwide, royalty-free license under its
//     licensed patents to make, have made, use, sell, offer for sale, import, and/or otherwise dispose of
//     its contribution in the software or derivative works of the contribution in the software.
// 
// 3. Conditions and Limitations
// (A) Reciprocal Grants- For any file you distribute that contains code from the software (in source code or binary format), 
//     you must provide recipients the source code to that file along with a copy of this license, which license will govern that file. 
//     You may license other files that are entirely your own work and do not contain code from the software under any terms you choose.
// (B) No Trademark License- This license does not grant you rights to use any contributors' name, logo, or trademarks.
// (C) If you bring a patent claim against any contributor over patents that you claim are infringed by the software,
//     your patent license from such contributor to the software ends automatically.
// (D) If you distribute any portion of the software, you must retain all copyright, patent, trademark,
//     and attribution notices that are present in the software.
// (E) If you distribute any portion of the software in source code form, you may do so only under this license by including a complete copy 
//     of this license with your distribution. If you distribute any portion of the software in compiled or object code form, you may only do 
//     so under a license that complies with this license.
// (F) The software is licensed "as-is." You bear the risk of using it. The contributors give no express warranties, guarantees or conditions.
//    You may have additional consumer rights under your local laws which this license cannot change. To the extent permitted under your
//    local laws, the contributors exclude the implied warranties of merchantability, fitness for a particular purpose and non-infringement.
#endregion

using System;
using System.Globalization;
using System.IO;
using System.Xml.Serialization;

namespace DnEcuDiag.Configuration
{
    internal class ConfigurationSerializer
    {
        private ConfigurationSerializer()
        { }

        internal static VehiclePerformanceRun DeserializeVehiclePerformanceRun(string fileName)
        {
            var serialiser = new XmlSerializer(typeof(VehiclePerformanceRun));
            VehiclePerformanceRun vehiclePerformanceRun;

            using (var rdr = new StreamReader(fileName))
            {
                vehiclePerformanceRun = serialiser.Deserialize(rdr) as VehiclePerformanceRun;
            }

            return vehiclePerformanceRun;
        }

        internal static void SerializeVehiclePerformanceRun(string fileName, VehiclePerformanceRun vehiclePerformanceRun)
        {
            var serialiser = new XmlSerializer(typeof(VehiclePerformanceRun));

            using (var swr = new StreamWriter(fileName, false))
            {
                serialiser.Serialize(swr, vehiclePerformanceRun);
            }
        }

        internal static VehicleProfile DeserializeVehicleProfile(string fileName)
        {
            var serialiser = new XmlSerializer(typeof(VehicleProfile));
            VehicleProfile profile;

            using (var rdr = new StreamReader(fileName))
            {
                profile = serialiser.Deserialize(rdr) as VehicleProfile;
            }

            return profile;
        }

        internal static void SerializeVehicleProfile(string fileName, VehicleProfile vehicleProfile)
        {
            var serialiser = new XmlSerializer(typeof(VehicleProfile));

            using (var swr = new StreamWriter(fileName, false))
            {
                serialiser.Serialize(swr, vehicleProfile);
            }
        }

        internal static DeviceConfiguration DeserializeDeviceConfigurationFromFile(String fileName)
        {
            var serialiser = new XmlSerializer(typeof(DeviceConfiguration));
            DeviceConfiguration configuration;

            using (var rdr = new StreamReader(fileName))
            {
                configuration = serialiser.Deserialize(rdr) as DeviceConfiguration;
            }

            return configuration;
        }

        internal static void SerializeSerialDeviceConfiguration(string fileName,
                                                                DeviceConfiguration serialDeviceConfiguration)
        {
            var serialiser = new XmlSerializer(typeof(DeviceConfiguration));

            using (var swr = new StreamWriter(fileName, false))
            {
                serialiser.Serialize(swr, serialDeviceConfiguration);
            }
        }

        internal static String SerializeSensorConfigurations(SensorConfigurationContainer sensorConfigurations)
        {
            var serialiser = new XmlSerializer(typeof(SensorConfigurationContainer));
            String serialisedConfiguration;

            using (var swr = new StringWriter(CultureInfo.InvariantCulture))
            {
                serialiser.Serialize(swr, sensorConfigurations);
                serialisedConfiguration = swr.ToString();
            }

            return serialisedConfiguration;
        }

        internal static SensorConfigurationContainer DeserializeSensorConfigurations(String serialisedConfiguration)
        {
            var serialiser = new XmlSerializer(typeof(SensorConfigurationContainer));
            SensorConfigurationContainer sensorConfigurations;
            using (var rdr = new StringReader(serialisedConfiguration))
            {
                sensorConfigurations = (SensorConfigurationContainer)serialiser.Deserialize(rdr);
            }

            return sensorConfigurations;
        }
    }
}
