#region License
// Copyright  2004 - 2010 All Right Reserved, Derek Goslin
// 
// 1. Definitions
// The terms "reproduce," "reproduction," "derivative works," and "distribution" have the same meaning here as under U.S. copyright law.
// A "contribution" is the original software, or any additions or changes to the software.
// A "contributor" is any person that distributes its contribution under this license.
// "Licensed patents" are a contributor's patent claims that read directly on its contribution.
// 
// 2. Grant of Rights
// (A) Copyright Grant- Subject to the terms of this license, including the license
//     conditions and limitations in section 3, each contributor grants you a non-exclusive,
//     worldwide, royalty-free copyright license to reproduce its contribution, prepare derivative 
//     works of its contribution, and distribute its contribution or any derivative works that you create.
// (B) Patent Grant- Subject to the terms of this license, including the license conditions and limitations
//     in section 3, each contributor grants you a non-exclusive, worldwide, royalty-free license under its
//     licensed patents to make, have made, use, sell, offer for sale, import, and/or otherwise dispose of
//     its contribution in the software or derivative works of the contribution in the software.
// 
// 3. Conditions and Limitations
// (A) Reciprocal Grants- For any file you distribute that contains code from the software (in source code or binary format), 
//     you must provide recipients the source code to that file along with a copy of this license, which license will govern that file. 
//     You may license other files that are entirely your own work and do not contain code from the software under any terms you choose.
// (B) No Trademark License- This license does not grant you rights to use any contributors' name, logo, or trademarks.
// (C) If you bring a patent claim against any contributor over patents that you claim are infringed by the software,
//     your patent license from such contributor to the software ends automatically.
// (D) If you distribute any portion of the software, you must retain all copyright, patent, trademark,
//     and attribution notices that are present in the software.
// (E) If you distribute any portion of the software in source code form, you may do so only under this license by including a complete copy 
//     of this license with your distribution. If you distribute any portion of the software in compiled or object code form, you may only do 
//     so under a license that complies with this license.
// (F) The software is licensed "as-is." You bear the risk of using it. The contributors give no express warranties, guarantees or conditions.
//    You may have additional consumer rights under your local laws which this license cannot change. To the extent permitted under your
//    local laws, the contributors exclude the implied warranties of merchantability, fitness for a particular purpose and non-infringement.
#endregion

using System;

namespace DnEcuDiag.Configuration
{
    ///	<summary>
    ///	Summary	description	for	VehiclePerformanceRun.
    ///	</summary>
    public class VehiclePerformanceRun
    {
        #region	Properties

        /// <summary>
        /// Initializes a new instance of the <see cref="VehiclePerformanceRun"/> class.
        /// </summary>
        public VehiclePerformanceRun()
        {
            Clear();
        }

        ///	<summary>
        ///	The	vehicle	that the performance run was measured on.
        ///	</summary>
        public String Vehicle { get; set; }

        /// <summary>
        /// Gets or sets the notes.
        /// </summary>
        /// <value>The notes.</value>
        public string Notes { get; set; }

        /// <summary>
        /// Gets or sets the sixty foot time.
        /// </summary>
        /// <value>The sixty foot time.</value>
        public TimeSpan SixtyFootTime { get; set; }

        /// <summary>
        /// Gets or sets the three thirty time.
        /// </summary>
        /// <value>The three thirty time.</value>
        public TimeSpan ThreeThirtyTime { get; set; }

        /// <summary>
        /// Gets or sets the eighth mile time.
        /// </summary>
        /// <value>The eighth mile time.</value>
        public TimeSpan EighthMileTime { get; set; }

        /// <summary>
        /// Gets or sets the eighth mile speed.
        /// </summary>
        /// <value>The eighth mile speed.</value>
        public double EighthMileSpeed { get; set; }

        /// <summary>
        /// Gets or sets the thousand foot time.
        /// </summary>
        /// <value>The thousand foot time.</value>
        public TimeSpan ThousandFootTime { get; set; }

        /// <summary>
        /// Gets or sets the quarter mile time.
        /// </summary>
        /// <value>The quarter mile time.</value>
        public TimeSpan QuarterMileTime { get; set; }

        /// <summary>
        /// Gets or sets the quarter mile speed.
        /// </summary>
        /// <value>The quarter mile speed.</value>
        public Double QuarterMileSpeed { get; set; }

        /// <summary>
        /// Gets or sets the sixty MPH time.
        /// </summary>
        /// <value>The sixty MPH time.</value>
        public TimeSpan SixtyMphTime { get; set; }

        /// <summary>
        /// Gets or sets the twenty meter time.
        /// </summary>
        /// <value>The twenty meter time.</value>
        public TimeSpan TwentyMeterTime { get; set; }

        /// <summary>
        /// Gets or sets the one hundred meter time.
        /// </summary>
        /// <value>The one hundred meter time.</value>
        public TimeSpan OneHundredMeterTime { get; set; }

        /// <summary>
        /// Gets or sets the two hundred meter time.
        /// </summary>
        /// <value>The two hundred meter time.</value>
        public TimeSpan TwoHundredMeterTime { get; set; }

        /// <summary>
        /// Gets or sets the two hundred meter speed.
        /// </summary>
        /// <value>The two hundred meter speed.</value>
        public double TwoHundredMeterSpeed { get; set; }

        /// <summary>
        /// Gets or sets the three hundred meter time.
        /// </summary>
        /// <value>The three hundred meter time.</value>
        public TimeSpan ThreeHundredMeterTime { get; set; }

        /// <summary>
        /// Gets or sets the four hundred meter speed.
        /// </summary>
        /// <value>The four hundred meter speed.</value>
        public double FourHundredMeterSpeed { get; set; }

        /// <summary>
        /// Gets or sets the four hundred meter time.
        /// </summary>
        /// <value>The four hundred meter time.</value>
        public TimeSpan FourHundredMeterTime { get; set; }

        /// <summary>
        /// Gets or sets the one hundred KPH time.
        /// </summary>
        /// <value>The one hundred KPH time.</value>
        public TimeSpan OneHundredKphTime { get; set; }

        #endregion

        #region Methods
        public void Clear()
        {
            SixtyFootTime = default(TimeSpan);
            ThreeThirtyTime = default(TimeSpan);
            EighthMileTime = default(TimeSpan);
            EighthMileSpeed = 0;
            ThousandFootTime = default(TimeSpan);
            QuarterMileTime = default(TimeSpan);
            QuarterMileSpeed = 0;
            SixtyMphTime = default(TimeSpan);
            TwentyMeterTime = default(TimeSpan);
            OneHundredMeterTime = default(TimeSpan);
            TwoHundredMeterTime = default(TimeSpan);
            TwoHundredMeterSpeed = 0;
            ThreeHundredMeterTime = default(TimeSpan);
            FourHundredMeterSpeed = 0;
            FourHundredMeterTime = default(TimeSpan);
            OneHundredKphTime = default(TimeSpan);
        }
        #endregion

    }
}