#region License
// Copyright  2004 - 2010 All Right Reserved, Derek Goslin
// 
// 1. Definitions
// The terms "reproduce," "reproduction," "derivative works," and "distribution" have the same meaning here as under U.S. copyright law.
// A "contribution" is the original software, or any additions or changes to the software.
// A "contributor" is any person that distributes its contribution under this license.
// "Licensed patents" are a contributor's patent claims that read directly on its contribution.
// 
// 2. Grant of Rights
// (A) Copyright Grant- Subject to the terms of this license, including the license
//     conditions and limitations in section 3, each contributor grants you a non-exclusive,
//     worldwide, royalty-free copyright license to reproduce its contribution, prepare derivative 
//     works of its contribution, and distribute its contribution or any derivative works that you create.
// (B) Patent Grant- Subject to the terms of this license, including the license conditions and limitations
//     in section 3, each contributor grants you a non-exclusive, worldwide, royalty-free license under its
//     licensed patents to make, have made, use, sell, offer for sale, import, and/or otherwise dispose of
//     its contribution in the software or derivative works of the contribution in the software.
// 
// 3. Conditions and Limitations
// (A) Reciprocal Grants- For any file you distribute that contains code from the software (in source code or binary format), 
//     you must provide recipients the source code to that file along with a copy of this license, which license will govern that file. 
//     You may license other files that are entirely your own work and do not contain code from the software under any terms you choose.
// (B) No Trademark License- This license does not grant you rights to use any contributors' name, logo, or trademarks.
// (C) If you bring a patent claim against any contributor over patents that you claim are infringed by the software,
//     your patent license from such contributor to the software ends automatically.
// (D) If you distribute any portion of the software, you must retain all copyright, patent, trademark,
//     and attribution notices that are present in the software.
// (E) If you distribute any portion of the software in source code form, you may do so only under this license by including a complete copy 
//     of this license with your distribution. If you distribute any portion of the software in compiled or object code form, you may only do 
//     so under a license that complies with this license.
// (F) The software is licensed "as-is." You bear the risk of using it. The contributors give no express warranties, guarantees or conditions.
//    You may have additional consumer rights under your local laws which this license cannot change. To the extent permitted under your
//    local laws, the contributors exclude the implied warranties of merchantability, fitness for a particular purpose and non-infringement.
#endregion

using System;
using DnEcuDiag.Ecus.Exceptions;

namespace DnEcuDiag.Conversion
{
    public abstract class Convertor
    {
        /// <summary>
        /// Convert a byte array to an integer
        /// </summary>
        /// <param name="rawData">The byte array to convert.</param>
        /// <returns>An integer.</returns>
        public static UInt32 ConvertByteArray(byte[] rawData)
        {
            uint ret = 0;
            if (rawData.Length == 0) throw new ArgumentOutOfRangeException("rawData", "Lenght==0");

            try
            {
                uint i = rawData[0];
                if (rawData.Length == 1)
                {
                    ret = i;
                }

                if (rawData.Length >= 2)
                {
                    // Shift the first two bytes.
                    ret = i << 8;
                    ret += rawData[1];
                }

                if (rawData.Length >= 3)
                {
                    ret = ret << 8;
                    ret += rawData[2];
                }

                if (rawData.Length == 4)
                {
                    ret = ret << 8;
                    i = rawData[3];
                    ret += i;
                }

                return ret;
            }
            catch (Exception exp)
            {
                throw new ConversionException("Conversion Failed", exp);
            }
        }

        /// <summary>
        /// Convert a byte array to an integer
        /// </summary>
        /// <param name="rawData">The byte array to convert.</param>
        /// <param name="littleEndian">The byte ordering.</param>
        /// <returns>An unsigned integer.</returns>
        public static uint ConvertByteArray(byte[] rawData, bool littleEndian)
        {
            if (rawData == null)
            {
                throw new ArgumentNullException("rawData");
            }

            if (rawData.Length == 0)
            {
                return 0;
            }

            uint ret = 0;

            try
            {
                if ((rawData.Length > 1) & (littleEndian))
                {
                    int k = rawData.Length - 1;

                    for (int j = 0; j < rawData.Length / 2; j++)
                    {
                        byte b = rawData[j];
                        rawData[j] = rawData[k];
                        rawData[k] = b;
                        k -= 1;
                    }
                }

                uint i = rawData[0];
                if (rawData.Length == 1)
                {
                    ret = i;
                }

                if (rawData.Length >= 2)
                {
                    // Shift the first two bytes.
                    ret = i << 8;
                    ret += rawData[1];
                }

                if (rawData.Length >= 3)
                {
                    ret = ret << 8;
                    ret += rawData[2];
                }

                if (rawData.Length == 4)
                {
                    ret = ret << 8;
                    i = rawData[3];
                    ret += i;
                }

                return ret;
            }
            catch (Exception exp)
            {
                throw new ConversionException("Conversion Failed", exp);
            }
        }

        protected float CalculateRpm(byte[] rawData)
        {
            float result;
            if (rawData.Length > 1)
            {
                if ((rawData[0] * 40) > 2500)
                {
                    result = rawData[0] * 40f;
                }
                else
                {
                    result = rawData[1] * 10f;
                }
            }
            else
            {
                result = rawData[0] * 40f;
            }
            return result;
        }
    }
}