﻿#region License
// Copyright © 2004 - 2010 All Right Reserved, Derek Goslin
// 
// 1. Definitions
// The terms "reproduce," "reproduction," "derivative works," and "distribution" have the same meaning here as under U.S. copyright law.
// A "contribution" is the original software, or any additions or changes to the software.
// A "contributor" is any person that distributes its contribution under this license.
// "Licensed patents" are a contributor's patent claims that read directly on its contribution.
// 
// 2. Grant of Rights
// (A) Copyright Grant- Subject to the terms of this license, including the license
//     conditions and limitations in section 3, each contributor grants you a non-exclusive,
//     worldwide, royalty-free copyright license to reproduce its contribution, prepare derivative 
//     works of its contribution, and distribute its contribution or any derivative works that you create.
// (B) Patent Grant- Subject to the terms of this license, including the license conditions and limitations
//     in section 3, each contributor grants you a non-exclusive, worldwide, royalty-free license under its
//     licensed patents to make, have made, use, sell, offer for sale, import, and/or otherwise dispose of
//     its contribution in the software or derivative works of the contribution in the software.
// 
// 3. Conditions and Limitations
// (A) Reciprocal Grants- For any file you distribute that contains code from the software (in source code or binary format), 
//     you must provide recipients the source code to that file along with a copy of this license, which license will govern that file. 
//     You may license other files that are entirely your own work and do not contain code from the software under any terms you choose.
// (B) No Trademark License- This license does not grant you rights to use any contributors' name, logo, or trademarks.
// (C) If you bring a patent claim against any contributor over patents that you claim are infringed by the software,
//     your patent license from such contributor to the software ends automatically.
// (D) If you distribute any portion of the software, you must retain all copyright, patent, trademark,
//     and attribution notices that are present in the software.
// (E) If you distribute any portion of the software in source code form, you may do so only under this license by including a complete copy 
//     of this license with your distribution. If you distribute any portion of the software in compiled or object code form, you may only do 
//     so under a license that complies with this license.
// (F) The software is licensed "as-is." You bear the risk of using it. The contributors give no express warranties, guarantees or conditions.
//    You may have additional consumer rights under your local laws which this license cannot change. To the extent permitted under your
//    local laws, the contributors exclude the implied warranties of merchantability, fitness for a particular purpose and non-infringement.
#endregion

using System;

namespace DnEcuDiag.Conversion
{
    public class Edc15Conversion : Convertor, IConvertor
    {
        /// <summary>
        /// Perform the custom ecu conversion
        /// </summary>
        /// <param name="conversionId"></param>
        /// <param name="rawData"></param>
        /// <returns></returns>
        public float Convert(int conversionId, byte[] rawData)
        {
            float result = 0;
            UInt32 rawValue = ConvertByteArray(rawData);

            switch (conversionId)
            {
                case 400:
                    result = rawValue * 0.1f;
                    break;
                case 401:
                    result = rawValue;
                    break;
                case 402:
                    result = rawValue * 10;
                    break;
                case 403:
                    result = rawValue * 40;
                    break;
                case 404:
                    result = rawValue;
                    break;
                case 405:
                    result = rawValue * 0.1f;
                    break;
                case 406:
                    result = rawValue;
                    break;
                case 409:
                    result = rawValue * 0.01f;
                    break;
                case 410:
                    result = rawValue * 0.02344f;
                    break;
                case 412:
                    result = rawValue * 0.976f;
                    break;
                case 414:
                    result = rawValue * 0.01f;
                    break;
                case 415:
                    result = rawValue * 0.1f;
                    break;
                case 417:
                    result = rawValue * 0.01f;
                    break;
                case 418:
                    result = rawValue * 0.1f;
                    break;
                case 420:
                    result = rawValue * 0.01f;
                    break;
                case 421:
                    result = rawValue;
                    break;
                case 423:
                    result = rawValue * 0.01f;
                    break;
                case 424:
                    result = rawValue;
                    break;
                case 425:
                    result = rawValue * 0.1f - 273.14f;
                    break;
                case 427:
                    result = rawValue * 23.612f / 1000;
                    break;
                case 428:
                    result = rawValue * 0.01f;
                    break;
                case 429:
                    result = rawValue * 4.8876f;
                    break;
                case 430:
                    result = rawValue * 99.74f / 10000;
                    break;
                case 431:
                    result = rawValue * 0.01f;
                    break;
                case 432:
                    result = rawValue * 0.01f;
                    break;
                case 433:
                    result = rawValue * 0.1f - 273.14f;
                    break;
                case 434:
                    result = rawValue * 0.01f;
                    break;
                case 435:
                    result = rawValue;
                    break;
                case 436:
                    result = rawValue * 0.01f;
                    break;
                case 437:
                    result = rawValue;
                    break;
                case 438:
                    result = rawValue * 23.272f;
                    break;
                case 439:
                    result = (rawValue * 0.6549f) - 40.14f;
                    break;
                case 440:
                    result = rawValue * 0.2327f;
                    break;
                case 441:
                    result = rawValue * 6.826f;
                    break;
                case 442:
                    result = rawValue * 1028f;
                    break;
            }

            return result;
        }

    }
}
