#region License
// Copyright  2004 - 2010 All Right Reserved, Derek Goslin
// 
// 1. Definitions
// The terms "reproduce," "reproduction," "derivative works," and "distribution" have the same meaning here as under U.S. copyright law.
// A "contribution" is the original software, or any additions or changes to the software.
// A "contributor" is any person that distributes its contribution under this license.
// "Licensed patents" are a contributor's patent claims that read directly on its contribution.
// 
// 2. Grant of Rights
// (A) Copyright Grant- Subject to the terms of this license, including the license
//     conditions and limitations in section 3, each contributor grants you a non-exclusive,
//     worldwide, royalty-free copyright license to reproduce its contribution, prepare derivative 
//     works of its contribution, and distribute its contribution or any derivative works that you create.
// (B) Patent Grant- Subject to the terms of this license, including the license conditions and limitations
//     in section 3, each contributor grants you a non-exclusive, worldwide, royalty-free license under its
//     licensed patents to make, have made, use, sell, offer for sale, import, and/or otherwise dispose of
//     its contribution in the software or derivative works of the contribution in the software.
// 
// 3. Conditions and Limitations
// (A) Reciprocal Grants- For any file you distribute that contains code from the software (in source code or binary format), 
//     you must provide recipients the source code to that file along with a copy of this license, which license will govern that file. 
//     You may license other files that are entirely your own work and do not contain code from the software under any terms you choose.
// (B) No Trademark License- This license does not grant you rights to use any contributors' name, logo, or trademarks.
// (C) If you bring a patent claim against any contributor over patents that you claim are infringed by the software,
//     your patent license from such contributor to the software ends automatically.
// (D) If you distribute any portion of the software, you must retain all copyright, patent, trademark,
//     and attribution notices that are present in the software.
// (E) If you distribute any portion of the software in source code form, you may do so only under this license by including a complete copy 
//     of this license with your distribution. If you distribute any portion of the software in compiled or object code form, you may only do 
//     so under a license that complies with this license.
// (F) The software is licensed "as-is." You bear the risk of using it. The contributors give no express warranties, guarantees or conditions.
//    You may have additional consumer rights under your local laws which this license cannot change. To the extent permitted under your
//    local laws, the contributors exclude the implied warranties of merchantability, fitness for a particular purpose and non-infringement.
#endregion

using System;

namespace DnEcuDiag.Ecus.Generic
{
    /// <summary>
    /// A match to a bit pattern.
    /// </summary>
    public sealed class MaskMatch
    {
        #region Constructors

        /// <summary>
        /// Initalise an instance of a MaskMatch class.
        /// </summary>
        public MaskMatch() : this(false, 0, String.Empty, String.Empty)
        {
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="MaskMatch"/> class.
        /// </summary>
        /// <param name="matchValue">if set to <c>true</c> [match value].</param>
        /// <param name="maskValue">The mask value.</param>
        /// <param name="maskResultTrue">The mask result true.</param>
        /// <param name="maskResultFalse">The mask result false.</param>
        public MaskMatch(bool matchValue, uint maskValue, string maskResultTrue, string maskResultFalse)
        {
            MatchValue = matchValue;
            MaskValue = maskValue;
            MaskResultTrue = maskResultTrue;
            MaskResultFalse = maskResultFalse;
        }

        #endregion

        #region Fields

        #endregion

        #region Properties

        /// <summary>
        /// Gets or sets a value indicating whether [match value].
        /// </summary>
        /// <value><c>true</c> if [match value]; otherwise, <c>false</c>.</value>
        public bool MatchValue { get; set; }

        /// <summary>
        /// The mask match.
        /// </summary>
        public uint MaskValue { get; set; }

        /// <summary>
        /// Gets or sets the mask result true.
        /// </summary>
        /// <value>The mask result true.</value>
        public string MaskResultTrue { get; set; }

        /// <summary>
        /// Gets or sets the mask result false.
        /// </summary>
        /// <value>The mask result false.</value>
        public string MaskResultFalse { get; set; }

        #endregion Properties
    }
}