﻿#region License
// Copyright © 2004 - 2010 All Right Reserved, Derek Goslin
// 
// 1. Definitions
// The terms "reproduce," "reproduction," "derivative works," and "distribution" have the same meaning here as under U.S. copyright law.
// A "contribution" is the original software, or any additions or changes to the software.
// A "contributor" is any person that distributes its contribution under this license.
// "Licensed patents" are a contributor's patent claims that read directly on its contribution.
// 
// 2. Grant of Rights
// (A) Copyright Grant- Subject to the terms of this license, including the license
//     conditions and limitations in section 3, each contributor grants you a non-exclusive,
//     worldwide, royalty-free copyright license to reproduce its contribution, prepare derivative 
//     works of its contribution, and distribute its contribution or any derivative works that you create.
// (B) Patent Grant- Subject to the terms of this license, including the license conditions and limitations
//     in section 3, each contributor grants you a non-exclusive, worldwide, royalty-free license under its
//     licensed patents to make, have made, use, sell, offer for sale, import, and/or otherwise dispose of
//     its contribution in the software or derivative works of the contribution in the software.
// 
// 3. Conditions and Limitations
// (A) Reciprocal Grants- For any file you distribute that contains code from the software (in source code or binary format), 
//     you must provide recipients the source code to that file along with a copy of this license, which license will govern that file. 
//     You may license other files that are entirely your own work and do not contain code from the software under any terms you choose.
// (B) No Trademark License- This license does not grant you rights to use any contributors' name, logo, or trademarks.
// (C) If you bring a patent claim against any contributor over patents that you claim are infringed by the software,
//     your patent license from such contributor to the software ends automatically.
// (D) If you distribute any portion of the software, you must retain all copyright, patent, trademark,
//     and attribution notices that are present in the software.
// (E) If you distribute any portion of the software in source code form, you may do so only under this license by including a complete copy 
//     of this license with your distribution. If you distribute any portion of the software in compiled or object code form, you may only do 
//     so under a license that complies with this license.
// (F) The software is licensed "as-is." You bear the risk of using it. The contributors give no express warranties, guarantees or conditions.
//    You may have additional consumer rights under your local laws which this license cannot change. To the extent permitted under your
//    local laws, the contributors exclude the implied warranties of merchantability, fitness for a particular purpose and non-infringement.
#endregion

using System;
using System.Collections.Generic;
using System.Globalization;
using System.Text;
using System.Xml.Serialization;
using DnEcuDiag.Devices.Interfaces;
using DnEcuDiag.Ecus.Exceptions;
using DnEcuDiag.Protocols;

namespace DnEcuDiag.Ecus.Iso
{
    public class Kwp71Ecu : Kwp81Ecu
    {
        private Boolean _routineDownloaded;

        public Kwp71Ecu()
        {
            HeaderEndIndex = 3;
            CrcBytesCount = 0;
        }

        public List<Byte> ActuatorRoutine { get; set; }

        [XmlIgnore]
        public override IDevice DiagnosticInterface
        {
            get { return diagnosticInterface; }
            set
            {
                diagnosticInterface = value;
                Protocol = new Kwp71(diagnosticInterface, (Byte) EcuAddress, 0);
            }
        }

        protected override void DecodeIdentificationData(byte[] identificationData)
        {
            if(identificationData.Length == 0x20)
            {
                DecodeLongIdentificationData(identificationData);
            }

            if (identificationData.Length == 0x14)
            {
                DecodeShortIdentificationData(identificationData);
            }
            
            var builder = new StringBuilder();

            foreach (byte b in diagnosticInterface.KeyBytes)
            {
                builder.Append(b.ToString("X", CultureInfo.InvariantCulture));
                builder.Append(" ");
            }

            IsoCode = builder.ToString().Trim();
        }

        private void DecodeShortIdentificationData(byte[] identificationData)
        {
            try
            {
                var builder = new StringBuilder();

                for (int i = 0; i < 12; i++)
                {
                    builder.Append(Convert.ToChar(identificationData[i]));
                }

                HardwareNumber = builder.ToString();

                builder.Remove(0, builder.Length);

                for (int i = 12; i < identificationData.Length; i++)
                {
                    builder.Append(Convert.ToChar(identificationData[i]));
                }

                SoftwareNumber = builder.ToString();
            }
            catch (Exception exp)
            {
                throw new EcuException("Failed executing DecodeIdData.", exp);
            }
        }

        private void DecodeLongIdentificationData(byte[] identificationData)
        {
            try
            {
                var builder = new StringBuilder();

                for (int i = 9; i > 0; i--)
                {
                    builder.Append(Convert.ToChar(identificationData[i]));
                }

                HardwareNumber = builder.ToString();

                builder.Remove(0, builder.Length);

                for (int i = 20; i > 10; i--)
                {
                    builder.Append(Convert.ToChar(identificationData[i]));
                }

                SoftwareNumber = builder.ToString();

                builder.Remove(0, builder.Length);

                for (int i = identificationData.Length - 1; i > 20; i--)
                {
                    builder.Append(Convert.ToChar(identificationData[i]));
                }

                DrawingNumber = builder.ToString();
            }
            catch (Exception exp)
            {
                throw new EcuException("Failed executing DecodeIdData.", exp);
            }
        }

        public override bool ActivateActuator(Generic.Actuator actuator)
        {
            if((!_routineDownloaded) & (ActuatorRoutine.Count != 0))
            {
                Protocol.WriteRamCell(0x400, ActuatorRoutine);
                _routineDownloaded = true;
            }

            return base.ActivateActuator(actuator);
        }
    }
}