#region License
// Copyright  2004 - 2010 All Right Reserved, Derek Goslin
// 
// 1. Definitions
// The terms "reproduce," "reproduction," "derivative works," and "distribution" have the same meaning here as under U.S. copyright law.
// A "contribution" is the original software, or any additions or changes to the software.
// A "contributor" is any person that distributes its contribution under this license.
// "Licensed patents" are a contributor's patent claims that read directly on its contribution.
// 
// 2. Grant of Rights
// (A) Copyright Grant- Subject to the terms of this license, including the license
//     conditions and limitations in section 3, each contributor grants you a non-exclusive,
//     worldwide, royalty-free copyright license to reproduce its contribution, prepare derivative 
//     works of its contribution, and distribute its contribution or any derivative works that you create.
// (B) Patent Grant- Subject to the terms of this license, including the license conditions and limitations
//     in section 3, each contributor grants you a non-exclusive, worldwide, royalty-free license under its
//     licensed patents to make, have made, use, sell, offer for sale, import, and/or otherwise dispose of
//     its contribution in the software or derivative works of the contribution in the software.
// 
// 3. Conditions and Limitations
// (A) Reciprocal Grants- For any file you distribute that contains code from the software (in source code or binary format), 
//     you must provide recipients the source code to that file along with a copy of this license, which license will govern that file. 
//     You may license other files that are entirely your own work and do not contain code from the software under any terms you choose.
// (B) No Trademark License- This license does not grant you rights to use any contributors' name, logo, or trademarks.
// (C) If you bring a patent claim against any contributor over patents that you claim are infringed by the software,
//     your patent license from such contributor to the software ends automatically.
// (D) If you distribute any portion of the software, you must retain all copyright, patent, trademark,
//     and attribution notices that are present in the software.
// (E) If you distribute any portion of the software in source code form, you may do so only under this license by including a complete copy 
//     of this license with your distribution. If you distribute any portion of the software in compiled or object code form, you may only do 
//     so under a license that complies with this license.
// (F) The software is licensed "as-is." You bear the risk of using it. The contributors give no express warranties, guarantees or conditions.
//    You may have additional consumer rights under your local laws which this license cannot change. To the extent permitted under your
//    local laws, the contributors exclude the implied warranties of merchantability, fitness for a particular purpose and non-infringement.
#endregion

using System;
using System.Drawing;
using System.Windows.Forms;
using DnEcuDiag.Ecus.EventArgTypes;
using DnEcuDiag.Ecus.Generic;

namespace DnEcuDiag
{
    public delegate void ResetSensorUiDelegate(Control scope, ButtonBase button, Control max, Control min, Control current, Image image);
    ///	<summary>
    ///	
    ///	</summary>
    public delegate void VoidDelegate();

    ///	<summary>
    ///	
    ///	</summary>
    public class ComboSelectedIndexEventArgs : EventArgs
    {
        private readonly int _index;

        ///	<summary>
        ///	Initializes	a new instance of the <see cref="ComboSelectedIndexEventArgs"/>	class.
        ///	</summary>
        ///	<param name="index">The	index.</param>
        public ComboSelectedIndexEventArgs(int index)
        {
            _index = index;
        }

        ///	<summary>
        ///	Gets the index of the combo	boxs selected item.
        ///	</summary>
        ///	<value>The index.</value>
        public int Index
        {
            get { return _index; }
        }
    }

    ///	<summary>
    ///	
    ///	</summary>
    public class ComboElementEventArgs : EventArgs
    {
        private readonly object _value;

        ///	<summary>
        ///	Initializes	a new instance of the <see cref="ComboElementEventArgs"/> class.
        ///	</summary>
        ///	<param name="value">The	value.</param>
        public ComboElementEventArgs(object value)
        {
            _value = value;
        }

        ///	<summary>
        ///	Gets the value.
        ///	</summary>
        ///	<value>The value.</value>
        public object Value
        {
            get { return _value; }
        }
    }

    ///	<summary>
    ///	
    ///	</summary>
    public class ImageListAddImageEventArgs : EventArgs
    {
        private readonly Bitmap _bitmapImage;

        ///	<summary>
        ///	Initializes	a new instance of the <see cref="ImageListAddImageEventArgs"/> class.
        ///	</summary>
        ///	<param name="bitmapImage">The bitmap image.</param>
        public ImageListAddImageEventArgs(Bitmap bitmapImage)
        {
            _bitmapImage = bitmapImage;
        }

        ///	<summary>
        ///	Gets the bitmap	image.
        ///	</summary>
        ///	<value>The bitmap image.</value>
        public Bitmap BitmapImage
        {
            get { return _bitmapImage; }
        }
    }

    ///	<summary>
    ///	
    ///	</summary>
    public class ListViewAddElementEventArgs : EventArgs
    {
        private readonly int _imageIndex;
        private readonly string _text;

        ///	<summary>
        ///	Initializes	a new instance of the <see cref="ListViewAddElementEventArgs"/>	class.
        ///	</summary>
        ///	<param name="text">The text.</param>
        ///	<param name="imageIndex">Index of the image.</param>
        public ListViewAddElementEventArgs(string text, int imageIndex)
        {
            _text = text;
            _imageIndex = imageIndex;
        }

        ///	<summary>
        ///	Gets the text.
        ///	</summary>
        ///	<value>The text.</value>
        public string Text
        {
            get { return _text; }
        }

        ///	<summary>
        ///	Gets the index of the image.
        ///	</summary>
        ///	<value>The index of	the	image.</value>
        public int ImageIndex
        {
            get { return _imageIndex; }
        }
    }

    ///	<summary>
    ///	
    ///	</summary>
    internal delegate void UpdateSensorUi(SensorUpdatedEventArgs args);

    public delegate Boolean AsyncStartLossMeasure();

    public delegate void AsyncReadFaultCodes();

    public delegate void AsyncCaptureFinalDriveRatio(Int32 selectedGearIndex);

    public delegate Boolean AsyncStartDynoMeasure(Int32 startRpm, Int32 stopRpm, Int32 gearIndex, Double vehicleMass);

    public delegate Boolean AsyncConnectToEcu(EcuDescriptor ecuDescriptor);

    public delegate String AsyncReadSnapshot(SnapshotDefinition snapshotDefinition);

    public delegate Boolean AsyncActivateActuator(Actuator actuator);

    public delegate UInt32 AsyncExecuteRoutine(Routine routine);

    public delegate void AsyncClearFaultCodes();

}