#region License
// Copyright  2004 - 2010 All Right Reserved, Derek Goslin
// 
// 1. Definitions
// The terms "reproduce," "reproduction," "derivative works," and "distribution" have the same meaning here as under U.S. copyright law.
// A "contribution" is the original software, or any additions or changes to the software.
// A "contributor" is any person that distributes its contribution under this license.
// "Licensed patents" are a contributor's patent claims that read directly on its contribution.
// 
// 2. Grant of Rights
// (A) Copyright Grant- Subject to the terms of this license, including the license
//     conditions and limitations in section 3, each contributor grants you a non-exclusive,
//     worldwide, royalty-free copyright license to reproduce its contribution, prepare derivative 
//     works of its contribution, and distribute its contribution or any derivative works that you create.
// (B) Patent Grant- Subject to the terms of this license, including the license conditions and limitations
//     in section 3, each contributor grants you a non-exclusive, worldwide, royalty-free license under its
//     licensed patents to make, have made, use, sell, offer for sale, import, and/or otherwise dispose of
//     its contribution in the software or derivative works of the contribution in the software.
// 
// 3. Conditions and Limitations
// (A) Reciprocal Grants- For any file you distribute that contains code from the software (in source code or binary format), 
//     you must provide recipients the source code to that file along with a copy of this license, which license will govern that file. 
//     You may license other files that are entirely your own work and do not contain code from the software under any terms you choose.
// (B) No Trademark License- This license does not grant you rights to use any contributors' name, logo, or trademarks.
// (C) If you bring a patent claim against any contributor over patents that you claim are infringed by the software,
//     your patent license from such contributor to the software ends automatically.
// (D) If you distribute any portion of the software, you must retain all copyright, patent, trademark,
//     and attribution notices that are present in the software.
// (E) If you distribute any portion of the software in source code form, you may do so only under this license by including a complete copy 
//     of this license with your distribution. If you distribute any portion of the software in compiled or object code form, you may only do 
//     so under a license that complies with this license.
// (F) The software is licensed "as-is." You bear the risk of using it. The contributors give no express warranties, guarantees or conditions.
//    You may have additional consumer rights under your local laws which this license cannot change. To the extent permitted under your
//    local laws, the contributors exclude the implied warranties of merchantability, fitness for a particular purpose and non-infringement.
#endregion

using System;
using System.Diagnostics;
using System.ComponentModel;
using System.Globalization;
using System.IO;
using System.Reflection;
using System.Threading;
using System.Windows.Forms;
using DnEcuDiag.CustomControls;
using DnEcuDiag.Devices;
using DnEcuDiag.Devices.Enums;
using DnEcuDiag.Devices.Exceptions;
using DnEcuDiag.Devices.Responses;
using DnEcuDiag.Ecus.Generic;
using DnEcuDiag.Factories;
using SerialPort;
using DnEcuDiag.Ecus.Enums;
using DnEcuDiag.Devices.Interfaces;

namespace DnEcuDiag.Forms
{
    ///	<summary>
    ///	Summary	description	for	FormConfigSerial.
    ///	</summary>
    public class FormDeviceConfiguration : System.Windows.Forms.Form
    {
        private IContainer components;
        private TabControl tabTimingConfig;
        private TabPage tabDriver;
        private ComboBox cboComPorts;
        private TabPage tabKwp;
        private NumericUpDown kwpResponseInterbyteTimeout;
        private Label lblEcuResponse;
        private NumericUpDown kwpResponseTimeout;
        private Label lblRespTimeOut;
        private TabPage tabISO;
        private Label lblIsoInterByteTime;
        private NumericUpDown isoResponseInterbyteTimeout;
        private NumericUpDown isoResponseTimeout;
        private Label lblIsoEcuResposeTime;
        private TabPage tabTest;
        private Button btnCancel;
        private Button btnOK;
        private ComboBoxExtended cboConnectTo;
        private TextBox txtLog;
        private Label lblConnectTo;
        private Button btnTestConnection;
        private GroupBox gbDeviceDriver;
        private BindingSource serialPortDescriptorBindingSource;
        private BindingSource deviceConfigurationBindingSource;
        private BindingSource kwpTimingConfigBindingSource;
        private BindingSource isoTimingConfigBindingSource;
        private BindingSource ecuDescriptorBindingSource;
        private Label lblComport;
        private CheckBox cbUseAlternateTimings;
        private ToolTip toolTipDeviceConfig;
        private CheckBox cbUseUsb;

        ///	<summary>
        ///	Constructor.
        ///	</summary>
        public FormDeviceConfiguration()
        {
            //
            // Required	for	Windows	Form Designer support
            //
            InitializeComponent();

            serialPortDescriptorBindingSource.DataSource = Model.SerialPorts;
            deviceConfigurationBindingSource.DataSource = Model.Instance.DeviceConfiguration;
            kwpTimingConfigBindingSource.DataSource = Model.Instance.DeviceConfiguration.KwpTiming;
            isoTimingConfigBindingSource.DataSource = Model.Instance.DeviceConfiguration.IsoTiming;
            ecuDescriptorBindingSource.DataSource = Model.EcuConfigurations;

            SetComport(cboComPorts, Model.Instance.DeviceConfiguration.SerialPort);
        }

        ///	<summary>
        ///	Clean up any resources being used.
        ///	</summary>
        protected override void Dispose(bool disposing)
        {
            if (disposing)
            {
                if (components != null)
                {
                    components.Dispose();
                }
            }
            base.Dispose(disposing);
        }

        #region	Windows	Form Designer generated	code

        ///	<summary>
        ///	Required method	for	Designer support - do not modify
        ///	the	contents of	this method	with the code editor.
        ///	</summary>
        private void InitializeComponent()
        {
            this.components = new System.ComponentModel.Container();
            System.ComponentModel.ComponentResourceManager resources = new System.ComponentModel.ComponentResourceManager(typeof(FormDeviceConfiguration));
            this.tabTimingConfig = new System.Windows.Forms.TabControl();
            this.tabDriver = new System.Windows.Forms.TabPage();
            this.gbDeviceDriver = new System.Windows.Forms.GroupBox();
            this.cbUseAlternateTimings = new System.Windows.Forms.CheckBox();
            this.deviceConfigurationBindingSource = new System.Windows.Forms.BindingSource(this.components);
            this.cbUseUsb = new System.Windows.Forms.CheckBox();
            this.lblComport = new System.Windows.Forms.Label();
            this.cboComPorts = new System.Windows.Forms.ComboBox();
            this.serialPortDescriptorBindingSource = new System.Windows.Forms.BindingSource(this.components);
            this.tabKwp = new System.Windows.Forms.TabPage();
            this.kwpResponseInterbyteTimeout = new System.Windows.Forms.NumericUpDown();
            this.kwpTimingConfigBindingSource = new System.Windows.Forms.BindingSource(this.components);
            this.lblEcuResponse = new System.Windows.Forms.Label();
            this.kwpResponseTimeout = new System.Windows.Forms.NumericUpDown();
            this.lblRespTimeOut = new System.Windows.Forms.Label();
            this.tabISO = new System.Windows.Forms.TabPage();
            this.lblIsoInterByteTime = new System.Windows.Forms.Label();
            this.isoResponseInterbyteTimeout = new System.Windows.Forms.NumericUpDown();
            this.isoTimingConfigBindingSource = new System.Windows.Forms.BindingSource(this.components);
            this.isoResponseTimeout = new System.Windows.Forms.NumericUpDown();
            this.lblIsoEcuResposeTime = new System.Windows.Forms.Label();
            this.tabTest = new System.Windows.Forms.TabPage();
            this.btnTestConnection = new System.Windows.Forms.Button();
            this.lblConnectTo = new System.Windows.Forms.Label();
            this.txtLog = new System.Windows.Forms.TextBox();
            this.cboConnectTo = new DnEcuDiag.CustomControls.ComboBoxExtended(this.components);
            this.ecuDescriptorBindingSource = new System.Windows.Forms.BindingSource(this.components);
            this.btnCancel = new System.Windows.Forms.Button();
            this.btnOK = new System.Windows.Forms.Button();
            this.toolTipDeviceConfig = new System.Windows.Forms.ToolTip(this.components);
            this.tabTimingConfig.SuspendLayout();
            this.tabDriver.SuspendLayout();
            this.gbDeviceDriver.SuspendLayout();
            ((System.ComponentModel.ISupportInitialize)(this.deviceConfigurationBindingSource)).BeginInit();
            ((System.ComponentModel.ISupportInitialize)(this.serialPortDescriptorBindingSource)).BeginInit();
            this.tabKwp.SuspendLayout();
            ((System.ComponentModel.ISupportInitialize)(this.kwpResponseInterbyteTimeout)).BeginInit();
            ((System.ComponentModel.ISupportInitialize)(this.kwpTimingConfigBindingSource)).BeginInit();
            ((System.ComponentModel.ISupportInitialize)(this.kwpResponseTimeout)).BeginInit();
            this.tabISO.SuspendLayout();
            ((System.ComponentModel.ISupportInitialize)(this.isoResponseInterbyteTimeout)).BeginInit();
            ((System.ComponentModel.ISupportInitialize)(this.isoTimingConfigBindingSource)).BeginInit();
            ((System.ComponentModel.ISupportInitialize)(this.isoResponseTimeout)).BeginInit();
            this.tabTest.SuspendLayout();
            ((System.ComponentModel.ISupportInitialize)(this.ecuDescriptorBindingSource)).BeginInit();
            this.SuspendLayout();
            // 
            // tabTimingConfig
            // 
            this.tabTimingConfig.AccessibleDescription = null;
            this.tabTimingConfig.AccessibleName = null;
            resources.ApplyResources(this.tabTimingConfig, "tabTimingConfig");
            this.tabTimingConfig.BackgroundImage = null;
            this.tabTimingConfig.Controls.Add(this.tabDriver);
            this.tabTimingConfig.Controls.Add(this.tabKwp);
            this.tabTimingConfig.Controls.Add(this.tabISO);
            this.tabTimingConfig.Controls.Add(this.tabTest);
            this.tabTimingConfig.Font = null;
            this.tabTimingConfig.Name = "tabTimingConfig";
            this.tabTimingConfig.SelectedIndex = 0;
            this.toolTipDeviceConfig.SetToolTip(this.tabTimingConfig, resources.GetString("tabTimingConfig.ToolTip"));
            // 
            // tabDriver
            // 
            this.tabDriver.AccessibleDescription = null;
            this.tabDriver.AccessibleName = null;
            resources.ApplyResources(this.tabDriver, "tabDriver");
            this.tabDriver.BackgroundImage = null;
            this.tabDriver.Controls.Add(this.gbDeviceDriver);
            this.tabDriver.Font = null;
            this.tabDriver.Name = "tabDriver";
            this.toolTipDeviceConfig.SetToolTip(this.tabDriver, resources.GetString("tabDriver.ToolTip"));
            this.tabDriver.UseVisualStyleBackColor = true;
            // 
            // gbDeviceDriver
            // 
            this.gbDeviceDriver.AccessibleDescription = null;
            this.gbDeviceDriver.AccessibleName = null;
            resources.ApplyResources(this.gbDeviceDriver, "gbDeviceDriver");
            this.gbDeviceDriver.BackgroundImage = null;
            this.gbDeviceDriver.Controls.Add(this.cbUseAlternateTimings);
            this.gbDeviceDriver.Controls.Add(this.cbUseUsb);
            this.gbDeviceDriver.Controls.Add(this.lblComport);
            this.gbDeviceDriver.Controls.Add(this.cboComPorts);
            this.gbDeviceDriver.Font = null;
            this.gbDeviceDriver.Name = "gbDeviceDriver";
            this.gbDeviceDriver.TabStop = false;
            this.toolTipDeviceConfig.SetToolTip(this.gbDeviceDriver, resources.GetString("gbDeviceDriver.ToolTip"));
            // 
            // cbUseAlternateTimings
            // 
            this.cbUseAlternateTimings.AccessibleDescription = null;
            this.cbUseAlternateTimings.AccessibleName = null;
            resources.ApplyResources(this.cbUseAlternateTimings, "cbUseAlternateTimings");
            this.cbUseAlternateTimings.BackgroundImage = null;
            this.cbUseAlternateTimings.DataBindings.Add(new System.Windows.Forms.Binding("Checked", this.deviceConfigurationBindingSource, "UseAlternateUsbTimings", true));
            this.cbUseAlternateTimings.Font = null;
            this.cbUseAlternateTimings.Name = "cbUseAlternateTimings";
            this.toolTipDeviceConfig.SetToolTip(this.cbUseAlternateTimings, resources.GetString("cbUseAlternateTimings.ToolTip"));
            this.cbUseAlternateTimings.UseVisualStyleBackColor = true;
            // 
            // deviceConfigurationBindingSource
            // 
            this.deviceConfigurationBindingSource.DataSource = typeof(DnEcuDiag.Configuration.DeviceConfiguration);
            // 
            // cbUseUsb
            // 
            this.cbUseUsb.AccessibleDescription = null;
            this.cbUseUsb.AccessibleName = null;
            resources.ApplyResources(this.cbUseUsb, "cbUseUsb");
            this.cbUseUsb.BackgroundImage = null;
            this.cbUseUsb.DataBindings.Add(new System.Windows.Forms.Binding("Checked", this.deviceConfigurationBindingSource, "UseUsb", true));
            this.cbUseUsb.Font = null;
            this.cbUseUsb.Name = "cbUseUsb";
            this.toolTipDeviceConfig.SetToolTip(this.cbUseUsb, resources.GetString("cbUseUsb.ToolTip"));
            this.cbUseUsb.UseVisualStyleBackColor = true;
            this.cbUseUsb.CheckedChanged += new System.EventHandler(this.cbUseUsb_CheckedChanged);
            // 
            // lblComport
            // 
            this.lblComport.AccessibleDescription = null;
            this.lblComport.AccessibleName = null;
            resources.ApplyResources(this.lblComport, "lblComport");
            this.lblComport.Font = null;
            this.lblComport.Name = "lblComport";
            this.toolTipDeviceConfig.SetToolTip(this.lblComport, resources.GetString("lblComport.ToolTip"));
            // 
            // cboComPorts
            // 
            this.cboComPorts.AccessibleDescription = null;
            this.cboComPorts.AccessibleName = null;
            resources.ApplyResources(this.cboComPorts, "cboComPorts");
            this.cboComPorts.BackgroundImage = null;
            this.cboComPorts.DataBindings.Add(new System.Windows.Forms.Binding("SelectedItem", this.deviceConfigurationBindingSource, "SerialPort", true));
            this.cboComPorts.DataSource = this.serialPortDescriptorBindingSource;
            this.cboComPorts.DisplayMember = "FullName";
            this.cboComPorts.DropDownStyle = System.Windows.Forms.ComboBoxStyle.DropDownList;
            this.cboComPorts.Font = null;
            this.cboComPorts.Name = "cboComPorts";
            this.toolTipDeviceConfig.SetToolTip(this.cboComPorts, resources.GetString("cboComPorts.ToolTip"));
            // 
            // serialPortDescriptorBindingSource
            // 
            this.serialPortDescriptorBindingSource.DataSource = typeof(SerialPort.SerialPortDescriptor);
            // 
            // tabKwp
            // 
            this.tabKwp.AccessibleDescription = null;
            this.tabKwp.AccessibleName = null;
            resources.ApplyResources(this.tabKwp, "tabKwp");
            this.tabKwp.BackgroundImage = null;
            this.tabKwp.Controls.Add(this.kwpResponseInterbyteTimeout);
            this.tabKwp.Controls.Add(this.lblEcuResponse);
            this.tabKwp.Controls.Add(this.kwpResponseTimeout);
            this.tabKwp.Controls.Add(this.lblRespTimeOut);
            this.tabKwp.Font = null;
            this.tabKwp.Name = "tabKwp";
            this.toolTipDeviceConfig.SetToolTip(this.tabKwp, resources.GetString("tabKwp.ToolTip"));
            this.tabKwp.UseVisualStyleBackColor = true;
            // 
            // kwpResponseInterbyteTimeout
            // 
            this.kwpResponseInterbyteTimeout.AccessibleDescription = null;
            this.kwpResponseInterbyteTimeout.AccessibleName = null;
            resources.ApplyResources(this.kwpResponseInterbyteTimeout, "kwpResponseInterbyteTimeout");
            this.kwpResponseInterbyteTimeout.DataBindings.Add(new System.Windows.Forms.Binding("Value", this.kwpTimingConfigBindingSource, "EcuResponseInterbyteTimeout", true));
            this.kwpResponseInterbyteTimeout.Font = null;
            this.kwpResponseInterbyteTimeout.Increment = new decimal(new int[] {
            10,
            0,
            0,
            0});
            this.kwpResponseInterbyteTimeout.Maximum = new decimal(new int[] {
            200,
            0,
            0,
            0});
            this.kwpResponseInterbyteTimeout.Minimum = new decimal(new int[] {
            50,
            0,
            0,
            0});
            this.kwpResponseInterbyteTimeout.Name = "kwpResponseInterbyteTimeout";
            this.toolTipDeviceConfig.SetToolTip(this.kwpResponseInterbyteTimeout, resources.GetString("kwpResponseInterbyteTimeout.ToolTip"));
            this.kwpResponseInterbyteTimeout.Value = new decimal(new int[] {
            100,
            0,
            0,
            0});
            // 
            // kwpTimingConfigBindingSource
            // 
            this.kwpTimingConfigBindingSource.DataSource = typeof(DnEcuDiag.Configuration.TimingConfig);
            // 
            // lblEcuResponse
            // 
            this.lblEcuResponse.AccessibleDescription = null;
            this.lblEcuResponse.AccessibleName = null;
            resources.ApplyResources(this.lblEcuResponse, "lblEcuResponse");
            this.lblEcuResponse.Font = null;
            this.lblEcuResponse.Name = "lblEcuResponse";
            this.toolTipDeviceConfig.SetToolTip(this.lblEcuResponse, resources.GetString("lblEcuResponse.ToolTip"));
            // 
            // kwpResponseTimeout
            // 
            this.kwpResponseTimeout.AccessibleDescription = null;
            this.kwpResponseTimeout.AccessibleName = null;
            resources.ApplyResources(this.kwpResponseTimeout, "kwpResponseTimeout");
            this.kwpResponseTimeout.DataBindings.Add(new System.Windows.Forms.Binding("Value", this.kwpTimingConfigBindingSource, "EcuResponseTimeout", true));
            this.kwpResponseTimeout.Font = null;
            this.kwpResponseTimeout.Increment = new decimal(new int[] {
            500,
            0,
            0,
            0});
            this.kwpResponseTimeout.Maximum = new decimal(new int[] {
            5000,
            0,
            0,
            0});
            this.kwpResponseTimeout.Minimum = new decimal(new int[] {
            500,
            0,
            0,
            0});
            this.kwpResponseTimeout.Name = "kwpResponseTimeout";
            this.toolTipDeviceConfig.SetToolTip(this.kwpResponseTimeout, resources.GetString("kwpResponseTimeout.ToolTip"));
            this.kwpResponseTimeout.Value = new decimal(new int[] {
            1000,
            0,
            0,
            0});
            // 
            // lblRespTimeOut
            // 
            this.lblRespTimeOut.AccessibleDescription = null;
            this.lblRespTimeOut.AccessibleName = null;
            resources.ApplyResources(this.lblRespTimeOut, "lblRespTimeOut");
            this.lblRespTimeOut.Font = null;
            this.lblRespTimeOut.Name = "lblRespTimeOut";
            this.toolTipDeviceConfig.SetToolTip(this.lblRespTimeOut, resources.GetString("lblRespTimeOut.ToolTip"));
            // 
            // tabISO
            // 
            this.tabISO.AccessibleDescription = null;
            this.tabISO.AccessibleName = null;
            resources.ApplyResources(this.tabISO, "tabISO");
            this.tabISO.BackgroundImage = null;
            this.tabISO.Controls.Add(this.lblIsoInterByteTime);
            this.tabISO.Controls.Add(this.isoResponseInterbyteTimeout);
            this.tabISO.Controls.Add(this.isoResponseTimeout);
            this.tabISO.Controls.Add(this.lblIsoEcuResposeTime);
            this.tabISO.Font = null;
            this.tabISO.Name = "tabISO";
            this.toolTipDeviceConfig.SetToolTip(this.tabISO, resources.GetString("tabISO.ToolTip"));
            this.tabISO.UseVisualStyleBackColor = true;
            // 
            // lblIsoInterByteTime
            // 
            this.lblIsoInterByteTime.AccessibleDescription = null;
            this.lblIsoInterByteTime.AccessibleName = null;
            resources.ApplyResources(this.lblIsoInterByteTime, "lblIsoInterByteTime");
            this.lblIsoInterByteTime.Font = null;
            this.lblIsoInterByteTime.Name = "lblIsoInterByteTime";
            this.toolTipDeviceConfig.SetToolTip(this.lblIsoInterByteTime, resources.GetString("lblIsoInterByteTime.ToolTip"));
            // 
            // isoResponseInterbyteTimeout
            // 
            this.isoResponseInterbyteTimeout.AccessibleDescription = null;
            this.isoResponseInterbyteTimeout.AccessibleName = null;
            resources.ApplyResources(this.isoResponseInterbyteTimeout, "isoResponseInterbyteTimeout");
            this.isoResponseInterbyteTimeout.DataBindings.Add(new System.Windows.Forms.Binding("Value", this.isoTimingConfigBindingSource, "EcuResponseInterbyteTimeout", true));
            this.isoResponseInterbyteTimeout.Font = null;
            this.isoResponseInterbyteTimeout.Increment = new decimal(new int[] {
            10,
            0,
            0,
            0});
            this.isoResponseInterbyteTimeout.Maximum = new decimal(new int[] {
            5000,
            0,
            0,
            0});
            this.isoResponseInterbyteTimeout.Minimum = new decimal(new int[] {
            50,
            0,
            0,
            0});
            this.isoResponseInterbyteTimeout.Name = "isoResponseInterbyteTimeout";
            this.toolTipDeviceConfig.SetToolTip(this.isoResponseInterbyteTimeout, resources.GetString("isoResponseInterbyteTimeout.ToolTip"));
            this.isoResponseInterbyteTimeout.Value = new decimal(new int[] {
            100,
            0,
            0,
            0});
            // 
            // isoTimingConfigBindingSource
            // 
            this.isoTimingConfigBindingSource.DataSource = typeof(DnEcuDiag.Configuration.TimingConfig);
            // 
            // isoResponseTimeout
            // 
            this.isoResponseTimeout.AccessibleDescription = null;
            this.isoResponseTimeout.AccessibleName = null;
            resources.ApplyResources(this.isoResponseTimeout, "isoResponseTimeout");
            this.isoResponseTimeout.DataBindings.Add(new System.Windows.Forms.Binding("Value", this.isoTimingConfigBindingSource, "EcuResponseTimeout", true));
            this.isoResponseTimeout.Font = null;
            this.isoResponseTimeout.Increment = new decimal(new int[] {
            500,
            0,
            0,
            0});
            this.isoResponseTimeout.Maximum = new decimal(new int[] {
            2500,
            0,
            0,
            0});
            this.isoResponseTimeout.Minimum = new decimal(new int[] {
            1000,
            0,
            0,
            0});
            this.isoResponseTimeout.Name = "isoResponseTimeout";
            this.toolTipDeviceConfig.SetToolTip(this.isoResponseTimeout, resources.GetString("isoResponseTimeout.ToolTip"));
            this.isoResponseTimeout.Value = new decimal(new int[] {
            1500,
            0,
            0,
            0});
            // 
            // lblIsoEcuResposeTime
            // 
            this.lblIsoEcuResposeTime.AccessibleDescription = null;
            this.lblIsoEcuResposeTime.AccessibleName = null;
            resources.ApplyResources(this.lblIsoEcuResposeTime, "lblIsoEcuResposeTime");
            this.lblIsoEcuResposeTime.Font = null;
            this.lblIsoEcuResposeTime.Name = "lblIsoEcuResposeTime";
            this.toolTipDeviceConfig.SetToolTip(this.lblIsoEcuResposeTime, resources.GetString("lblIsoEcuResposeTime.ToolTip"));
            // 
            // tabTest
            // 
            this.tabTest.AccessibleDescription = null;
            this.tabTest.AccessibleName = null;
            resources.ApplyResources(this.tabTest, "tabTest");
            this.tabTest.BackgroundImage = null;
            this.tabTest.Controls.Add(this.btnTestConnection);
            this.tabTest.Controls.Add(this.lblConnectTo);
            this.tabTest.Controls.Add(this.txtLog);
            this.tabTest.Controls.Add(this.cboConnectTo);
            this.tabTest.Font = null;
            this.tabTest.Name = "tabTest";
            this.toolTipDeviceConfig.SetToolTip(this.tabTest, resources.GetString("tabTest.ToolTip"));
            this.tabTest.UseVisualStyleBackColor = true;
            // 
            // btnTestConnection
            // 
            this.btnTestConnection.AccessibleDescription = null;
            this.btnTestConnection.AccessibleName = null;
            resources.ApplyResources(this.btnTestConnection, "btnTestConnection");
            this.btnTestConnection.BackgroundImage = null;
            this.btnTestConnection.Font = null;
            this.btnTestConnection.Name = "btnTestConnection";
            this.toolTipDeviceConfig.SetToolTip(this.btnTestConnection, resources.GetString("btnTestConnection.ToolTip"));
            this.btnTestConnection.UseVisualStyleBackColor = true;
            this.btnTestConnection.Click += new System.EventHandler(this.btnTestConnection_Click);
            // 
            // lblConnectTo
            // 
            this.lblConnectTo.AccessibleDescription = null;
            this.lblConnectTo.AccessibleName = null;
            resources.ApplyResources(this.lblConnectTo, "lblConnectTo");
            this.lblConnectTo.Font = null;
            this.lblConnectTo.Name = "lblConnectTo";
            this.toolTipDeviceConfig.SetToolTip(this.lblConnectTo, resources.GetString("lblConnectTo.ToolTip"));
            // 
            // txtLog
            // 
            this.txtLog.AccessibleDescription = null;
            this.txtLog.AccessibleName = null;
            resources.ApplyResources(this.txtLog, "txtLog");
            this.txtLog.BackgroundImage = null;
            this.txtLog.Font = null;
            this.txtLog.Name = "txtLog";
            this.txtLog.ReadOnly = true;
            this.txtLog.TabStop = false;
            this.toolTipDeviceConfig.SetToolTip(this.txtLog, resources.GetString("txtLog.ToolTip"));
            // 
            // cboConnectTo
            // 
            this.cboConnectTo.AccessibleDescription = null;
            this.cboConnectTo.AccessibleName = null;
            resources.ApplyResources(this.cboConnectTo, "cboConnectTo");
            this.cboConnectTo.BackgroundImage = null;
            this.cboConnectTo.DataSource = this.ecuDescriptorBindingSource;
            this.cboConnectTo.DisplayMember = "Name";
            this.cboConnectTo.DropDownStyle = System.Windows.Forms.ComboBoxStyle.DropDownList;
            this.cboConnectTo.Font = null;
            this.cboConnectTo.Name = "cboConnectTo";
            this.toolTipDeviceConfig.SetToolTip(this.cboConnectTo, resources.GetString("cboConnectTo.ToolTip"));
            // 
            // ecuDescriptorBindingSource
            // 
            this.ecuDescriptorBindingSource.DataSource = typeof(DnEcuDiag.Ecus.Generic.EcuDescriptor);
            // 
            // btnCancel
            // 
            this.btnCancel.AccessibleDescription = null;
            this.btnCancel.AccessibleName = null;
            resources.ApplyResources(this.btnCancel, "btnCancel");
            this.btnCancel.BackgroundImage = null;
            this.btnCancel.DialogResult = System.Windows.Forms.DialogResult.Cancel;
            this.btnCancel.Font = null;
            this.btnCancel.Name = "btnCancel";
            this.toolTipDeviceConfig.SetToolTip(this.btnCancel, resources.GetString("btnCancel.ToolTip"));
            this.btnCancel.Click += new System.EventHandler(this.btnCancel_Click);
            // 
            // btnOK
            // 
            this.btnOK.AccessibleDescription = null;
            this.btnOK.AccessibleName = null;
            resources.ApplyResources(this.btnOK, "btnOK");
            this.btnOK.BackgroundImage = null;
            this.btnOK.Font = null;
            this.btnOK.Name = "btnOK";
            this.toolTipDeviceConfig.SetToolTip(this.btnOK, resources.GetString("btnOK.ToolTip"));
            this.btnOK.Click += new System.EventHandler(this.btnOK_Click);
            // 
            // FormDeviceConfiguration
            // 
            this.AcceptButton = this.btnOK;
            this.AccessibleDescription = null;
            this.AccessibleName = null;
            resources.ApplyResources(this, "$this");
            this.BackgroundImage = null;
            this.CancelButton = this.btnCancel;
            this.ControlBox = false;
            this.Controls.Add(this.tabTimingConfig);
            this.Controls.Add(this.btnCancel);
            this.Controls.Add(this.btnOK);
            this.Font = null;
            this.FormBorderStyle = System.Windows.Forms.FormBorderStyle.FixedDialog;
            this.MaximizeBox = false;
            this.MinimizeBox = false;
            this.Name = "FormDeviceConfiguration";
            this.toolTipDeviceConfig.SetToolTip(this, resources.GetString("$this.ToolTip"));
            this.tabTimingConfig.ResumeLayout(false);
            this.tabDriver.ResumeLayout(false);
            this.gbDeviceDriver.ResumeLayout(false);
            this.gbDeviceDriver.PerformLayout();
            ((System.ComponentModel.ISupportInitialize)(this.deviceConfigurationBindingSource)).EndInit();
            ((System.ComponentModel.ISupportInitialize)(this.serialPortDescriptorBindingSource)).EndInit();
            this.tabKwp.ResumeLayout(false);
            ((System.ComponentModel.ISupportInitialize)(this.kwpResponseInterbyteTimeout)).EndInit();
            ((System.ComponentModel.ISupportInitialize)(this.kwpTimingConfigBindingSource)).EndInit();
            ((System.ComponentModel.ISupportInitialize)(this.kwpResponseTimeout)).EndInit();
            this.tabISO.ResumeLayout(false);
            ((System.ComponentModel.ISupportInitialize)(this.isoResponseInterbyteTimeout)).EndInit();
            ((System.ComponentModel.ISupportInitialize)(this.isoTimingConfigBindingSource)).EndInit();
            ((System.ComponentModel.ISupportInitialize)(this.isoResponseTimeout)).EndInit();
            this.tabTest.ResumeLayout(false);
            this.tabTest.PerformLayout();
            ((System.ComponentModel.ISupportInitialize)(this.ecuDescriptorBindingSource)).EndInit();
            this.ResumeLayout(false);

        }

        #endregion

        private void btnCancel_Click(object sender, System.EventArgs e)
        {
            Common.ExecuteExceptionHandledDelegate(HandleCancelClick);
        }

        private void btnOK_Click(object sender, System.EventArgs e)
        {
            Common.ExecuteExceptionHandledDelegate(HandleOkClick);
        }

        private void HandleCancelClick()
        {
            Model.Instance.LoadDeviceConfiguration();

            Close();
        }

        private void HandleOkClick()
        {
            Model.Instance.SaveDeviceConfiguration();

            Close();
        }

        private void btnTestConnection_Click(object sender, EventArgs e)
        {
            Common.ExecuteExceptionHandledDelegate(HandleTestConnection);
        }

        private void HandleTestConnection()
        {
            if (cboConnectTo.SelectedIndex == -1)
            {
                Common.ShowMessage("Please select the Connect To control unit.");
                return;
            }

            String fileName = Model.Instance.ApplicationPath + Path.DirectorySeparatorChar + "TestConnect.log";

            using (Stream file = new FileStream(fileName, FileMode.Append, FileAccess.Write))
            {
                using (var writer = new StringWriter(CultureInfo.InvariantCulture))
                {
                    using (var textStringListener = new TextWriterTraceListener(writer))
                    {
                        using (var textListener = new TextWriterTraceListener(file))
                        {
                            Trace.Listeners.Add(textListener);
                            Trace.Listeners.Add(textStringListener);

                            Cursor.Current = Cursors.WaitCursor;
                            Application.DoEvents();

                            try
                            {
                                String deviceName;

                                Trace.WriteLine("".PadLeft(100, '='));
                                if (Model.Instance.DeviceConfiguration.UseUsb)
                                {
                                    deviceName = "FTDI Device";
                                }
                                else
                                {
                                    deviceName = (cboComPorts.SelectedItem).ToString();
                                }

                                var ecuDescriptor = (EcuDescriptor) cboConnectTo.SelectedItem;
                                Trace.WriteLine(String.Format(CultureInfo.InvariantCulture, DnEcuDiag.WinFormStrings.TestingUsingDeviceVersion,
                                                              DateTime.Now, ecuDescriptor.Name, deviceName,
                                                              Assembly.GetExecutingAssembly().GetName().Version));
                                Trace.WriteLine("".PadLeft(100, '='));

                                Trace.WriteLine(Model.Instance.DeviceConfiguration.ToString());

                                IPort port = InterfaceDeviceFactory.CreatePort(Model.Instance.DeviceConfiguration);

                                try
                                {
                                    port.SetBaudRate(
                                        DetermineTestBaudRate(EcuFactory.EcuList[cboConnectTo.SelectedIndex].EcuType));
                                    PerformLineTest(port);
                                }
                                catch (Exception e)
                                {
                                    Trace.WriteLine(String.Format(CultureInfo.InvariantCulture, DnEcuDiag.WinFormStrings.Error, e));
                                }
                                finally
                                {
                                    port.Close();
                                }

                                Thread.Sleep(5000);

                                port = InterfaceDeviceFactory.CreatePort(Model.Instance.DeviceConfiguration);

                                if (EcuFactory.EcuList[cboConnectTo.SelectedIndex].EcuType == EcuType.KwpEcu)
                                {
                                    if (EcuFactory.EcuList[cboConnectTo.SelectedIndex].Name.Contains("M1.5.5"))
                                    {
                                        PerformKWPTest(0x10, port);
                                    }
                                    else if (EcuFactory.EcuList[cboConnectTo.SelectedIndex].Name.Equals("Bosch EDC15", StringComparison.InvariantCultureIgnoreCase))
                                    {
                                        PerformKWPTestSlow(0x0A, port);
                                    }
                                    else if (EcuFactory.EcuList[cboConnectTo.SelectedIndex].Name.Contains("Selespeed"))
                                    {
                                        PerformKWPTestSlow(0x20, port);
                                    }
                                    else if (EcuFactory.EcuList[cboConnectTo.SelectedIndex].Name.Contains("ABS"))
                                    {
                                        PerformKWPTestSlow(0x20, port);
                                    }
                                    else
                                    {
                                        PerformKWPTestSlow(0x10, port);
                                    }
                                }

                                if (EcuFactory.EcuList[cboConnectTo.SelectedIndex].EcuType == EcuType.Kwp71Ecu)
                                {
                                    if (EcuFactory.EcuList[cboConnectTo.SelectedIndex].Name.Contains("ABS"))
                                    {
                                        PerformIsoTest(0x20, new Kwp71SerialDevice(port, true));
                                    }
                                    else if (EcuFactory.EcuList[cboConnectTo.SelectedIndex].Name.Contains("TRW"))
                                    {
                                        PerformIsoTest(0x0, new Kwp71SerialDevice(port, true));
                                    }
                                    else
                                    {
                                        PerformIsoTest(0x10, new Kwp71SerialDevice(port, true));
                                    }
                                }

                                if (EcuFactory.EcuList[cboConnectTo.SelectedIndex].EcuType == EcuType.Kwp81Ecu)
                                {
                                    PerformIsoTest(0, new Kwp81SerialDevice(port, true));
                                }

                                Trace.WriteLine(DnEcuDiag.WinFormStrings.TestComplete);
                            }
                            catch (Exception exp)
                            {
                                Trace.WriteLine(String.Format(CultureInfo.InvariantCulture, DnEcuDiag.WinFormStrings.Error, exp));
                            }
                            finally
                            {
                                Cursor.Current = Cursors.Default;
                            }

                            txtLog.Text = writer.ToString();

                            Trace.Flush();
                            Trace.Listeners.Clear();
                            file.Flush();
                        }
                    }
                }
            }

            Common.ShowMessage(String.Format(CultureInfo.InvariantCulture, DnEcuDiag.WinFormStrings.TestResultWrittenTo, fileName));
        }

        private static Int32 DetermineTestBaudRate(EcuType ecuType)
        {
            Int32 baudRate = 10400;

            switch (ecuType)
            {
                case EcuType.KwpEcu:
                    break;
                case EcuType.Kwp81Ecu:
                    baudRate = 4800;
                    break;
                case EcuType.Kwp71Ecu:
                    baudRate = 4800;
                    break;
            }

            return baudRate;
        }

        private static void PerformIsoTest(Byte ecuId, IDevice device)
        {
            try
            {
                device.EnableTrace = true;
                InterfaceDeviceFactory.SetupDeviceTiming(device, Model.Instance.DeviceConfiguration.IsoTiming);

                Response response = device.SendCommandWithInit(ecuId, 0, new byte[] {0x09}, DeviceInitType.Standard);

                Trace.Write("Key Bytes: ");

                Trace.Write(Response.DumpPacket(device.KeyBytes));

                Trace.WriteLine("");
                Trace.Write("Init Response: ");

                Trace.Write(response.ToString());

                response = device.SendCommand(ecuId, 0, new byte[] {0x07});

                response = device.SendCommand(ecuId, 0, new byte[] { 0x05 });

                Trace.WriteLine("");
                Trace.Write("Clear Response: ");

                Trace.Write(response.ToString());
            }
            catch (SerialDeviceException sde)
            {
                Trace.Write("Error:");
                Trace.WriteLine(sde.ToString());
            }
            catch (DeviceTimeoutException dto)
            {
                Trace.Write("Error:");
                Trace.WriteLine(dto.ToString());
            }
            finally
            {
                if (device != null)
                {
                    device.Close();
                }
            }
        }

        private static void PerformKWPTest(byte ecuId, IPort port)
        {
            Kwp2000SerialDevice device = null;

            try
            {
                device = new Kwp2000SerialDevice(port, true) {EnableTrace = true};
                InterfaceDeviceFactory.SetupDeviceTiming(device, Model.Instance.DeviceConfiguration.KwpTiming);

                Trace.WriteLine(device.SendCommandWithInit(ecuId, 0xF1, new byte[] {0x81}, DeviceInitType.Extended));
                Trace.WriteLine(device.SendCommand(ecuId, 0xF1, new byte[] {0x83, 0}));
                Trace.WriteLine(device.SendCommand(ecuId, 0xF1, new byte[] {0x83, 2}));
                Trace.WriteLine(device.SendCommand(ecuId, 0xF1, new byte[] {0x83, 2, 0, 0xff, 0, 0xff, 0}));
                Trace.WriteLine(device.SendCommand(ecuId, 0xF1, new byte[] {0x10, 0x81}));
                Trace.WriteLine(device.SendCommand(ecuId, 0xF1, new byte[] {0x21, 0x80}));
                Trace.WriteLine(device.SendCommand(ecuId, 0xF1, new byte[] {0x1A, 0x80}));
            }
            catch (SerialDeviceException sde)
            {
                Trace.Write("Error:");
                Trace.WriteLine(sde.ToString());
            }
            catch (DeviceTimeoutException dto)
            {
                Trace.Write("Error:");
                Trace.WriteLine(dto.ToString());
            }
            finally
            {
                if (device != null)
                {
                    device.Close();
                }
            }
        }

        private static void PerformKWPTestSlow(Byte ecuId, IPort port)
        {
            Kwp2000SerialDevice device = null;

            try
            {
                device = new Kwp2000SerialDevice(port, true) {EnableTrace = true};

                Trace.WriteLine(device.SendCommandWithInit(ecuId, 0xF1, new byte[] {0x81}, DeviceInitType.Standard));

                Trace.WriteLine(device.SendCommand(ecuId, 0xF1, new byte[] {0x83, 0}));

                Trace.WriteLine(device.SendCommand(ecuId, 0xF1, new byte[] {0x83, 2}));

                Trace.WriteLine(device.SendCommand(ecuId, 0xF1, new byte[] {0x83, 3, 0, 0xff, 0, 0xff, 0}));

                Trace.WriteLine(device.SendCommand(ecuId, 0xF1, new byte[] {0x10, 0x81}));

                Trace.WriteLine(device.SendCommand(ecuId, 0xF1, new byte[] {0x21, 0x80}));
            }
            catch (SerialDeviceException sde)
            {
                Trace.Write("Error:");
                Trace.WriteLine(sde.ToString());
            }
            catch (DeviceTimeoutException dto)
            {
                Trace.Write("Error:");
                Trace.WriteLine(dto.ToString());
            }
            finally
            {
                if (device != null)
                {
                    device.Close();
                }
            }
        }

        private static void PerformLineTest(IPort port)
        {
            byte[] bOut = new byte[] {0x55, 0xAA, 0x55, 0xAA, 0x55, 0xAA, 0x55, 0xAA};
            byte[] bIn;

            Trace.Write("Sending bytes: ");
            Trace.Write(Response.DumpPacket(bOut));
            Trace.WriteLine(String.Empty);

            foreach (byte b in bOut)
            {
                port.SendImmediate(b);
            }

            Thread.Sleep(1000);

            port.InputLen = 8;
            bIn = port.Input;

            Trace.Write("Received bytes:");
            Trace.Write(Response.DumpPacket(bIn));
            Trace.WriteLine(String.Empty);

            bool missMatch = false;
            int count = 0;
            foreach (byte b in bIn)
            {
                if (count >= bOut.Length)
                {
                    missMatch = true;
                    continue;
                }

                if (b != bOut[count++])
                {
                    missMatch = true;
                }
            }

            if (bOut.Length != bIn.Length)
            {
                Trace.WriteLine(WinFormStrings.ErrorTheSentAndReceiveLengthsMismatchThisCanIndicateAnDiagnosticInterfaceConnectionOrHardwareError);
            }

            if (missMatch)
            {
                Trace.WriteLine(WinFormStrings.ErrorTheSentAndReceivedDataBytesMismatchedThisCanIndicateAnDiagnosticInterfaceConnectionOrHardwareError);
            }
        }

        private static void SetComport(ComboBox cboPorts, SerialPortDescriptor port)
        {
            for (int i = 0; i < cboPorts.Items.Count; i++)
            {
                var portDescriptor = cboPorts.Items[i] as SerialPortDescriptor;

                if (portDescriptor != null)
                {
                    if (portDescriptor.PortName == port.PortName)
                    {
                        cboPorts.SelectedIndex = i;
                        break;
                    }
                }
            }
        }

        private void cbUseUsb_CheckedChanged(object sender, EventArgs e)
        {
            cbUseAlternateTimings.Enabled = cbUseUsb.Checked;
        }
    }
}