#region License
// Copyright  2004 - 2010 All Right Reserved, Derek Goslin
// 
// 1. Definitions
// The terms "reproduce," "reproduction," "derivative works," and "distribution" have the same meaning here as under U.S. copyright law.
// A "contribution" is the original software, or any additions or changes to the software.
// A "contributor" is any person that distributes its contribution under this license.
// "Licensed patents" are a contributor's patent claims that read directly on its contribution.
// 
// 2. Grant of Rights
// (A) Copyright Grant- Subject to the terms of this license, including the license
//     conditions and limitations in section 3, each contributor grants you a non-exclusive,
//     worldwide, royalty-free copyright license to reproduce its contribution, prepare derivative 
//     works of its contribution, and distribute its contribution or any derivative works that you create.
// (B) Patent Grant- Subject to the terms of this license, including the license conditions and limitations
//     in section 3, each contributor grants you a non-exclusive, worldwide, royalty-free license under its
//     licensed patents to make, have made, use, sell, offer for sale, import, and/or otherwise dispose of
//     its contribution in the software or derivative works of the contribution in the software.
// 
// 3. Conditions and Limitations
// (A) Reciprocal Grants- For any file you distribute that contains code from the software (in source code or binary format), 
//     you must provide recipients the source code to that file along with a copy of this license, which license will govern that file. 
//     You may license other files that are entirely your own work and do not contain code from the software under any terms you choose.
// (B) No Trademark License- This license does not grant you rights to use any contributors' name, logo, or trademarks.
// (C) If you bring a patent claim against any contributor over patents that you claim are infringed by the software,
//     your patent license from such contributor to the software ends automatically.
// (D) If you distribute any portion of the software, you must retain all copyright, patent, trademark,
//     and attribution notices that are present in the software.
// (E) If you distribute any portion of the software in source code form, you may do so only under this license by including a complete copy 
//     of this license with your distribution. If you distribute any portion of the software in compiled or object code form, you may only do 
//     so under a license that complies with this license.
// (F) The software is licensed "as-is." You bear the risk of using it. The contributors give no express warranties, guarantees or conditions.
//    You may have additional consumer rights under your local laws which this license cannot change. To the extent permitted under your
//    local laws, the contributors exclude the implied warranties of merchantability, fitness for a particular purpose and non-infringement.
#endregion

using System;
using System.Windows.Forms;
using DnEcuDiag.CustomControls;
using DnEcuDiag.Ecus.Generic;
using DnEcuDiag.Properties;

namespace DnEcuDiag.Forms
{
    /// <summary>
    /// Summary description for FormSensorGroups.
    /// </summary>
    public class FormSensorGroups : System.Windows.Forms.Form
    {
        private System.Windows.Forms.Label lblGroupName;
        private System.Windows.Forms.GroupBox grpSensorGroup;
        private System.Windows.Forms.Label lblDescription;
        private System.Windows.Forms.TextBox txtGroupDescription;
        private System.Windows.Forms.Label lblSensorOne;
        private System.Windows.Forms.Label lblSensorTwo;
        private System.Windows.Forms.Label lblSensorThree;
        private System.Windows.Forms.Label lblSensorFour;
        private ComboBoxExtended cboSensorOne;
        private ComboBoxExtended cboSensorTwo;
        private ComboBoxExtended cboSensorThree;
        private ComboBoxExtended cboSensorFour;
        private System.ComponentModel.IContainer components;
        private System.Windows.Forms.TextBox txtGroupName;
        private System.Windows.Forms.ToolTip toolTip;
        private System.Windows.Forms.Button btnNext;
        private System.Windows.Forms.Button btnLoad;
        private System.Windows.Forms.Button btnDelete;
        private System.Windows.Forms.Button btnAdd;
        private System.Windows.Forms.Button btnPrevious;
        private System.Windows.Forms.Button btnOk;
        private BindingSource sensorsBindingSourceOne;
        private BindingSource sensorsBindingSourceTwo;
        private BindingSource sensorsBindingSourceThree;
        private BindingSource sensorsBindingSourceFour;

        private String ecuName;

        /// <summary>
        /// Initializes a new instance of the <see cref="FormSensorGroups"/> class.
        /// </summary>
        public FormSensorGroups()
        {
            //
            // Required for Windows Form Designer support
            //
            InitializeComponent();
        }

        /// <summary>
        /// Clean up any resources being used.
        /// </summary>
        protected override void Dispose(bool disposing)
        {
            if (disposing)
            {
                if (components != null)
                {
                    components.Dispose();
                }
            }
            base.Dispose(disposing);
        }

        #region Windows Form Designer generated code

        /// <summary>
        /// Required method for Designer support - do not modify
        /// the contents of this method with the code editor.
        /// </summary>
        private void InitializeComponent()
        {
            this.components = new System.ComponentModel.Container();
            System.ComponentModel.ComponentResourceManager resources = new System.ComponentModel.ComponentResourceManager(typeof(FormSensorGroups));
            this.grpSensorGroup = new System.Windows.Forms.GroupBox();
            this.btnPrevious = new System.Windows.Forms.Button();
            this.txtGroupName = new System.Windows.Forms.TextBox();
            this.lblSensorFour = new System.Windows.Forms.Label();
            this.lblSensorThree = new System.Windows.Forms.Label();
            this.lblSensorTwo = new System.Windows.Forms.Label();
            this.lblSensorOne = new System.Windows.Forms.Label();
            this.cboSensorFour = new DnEcuDiag.CustomControls.ComboBoxExtended(this.components);
            this.sensorsBindingSourceFour = new System.Windows.Forms.BindingSource(this.components);
            this.cboSensorThree = new DnEcuDiag.CustomControls.ComboBoxExtended(this.components);
            this.sensorsBindingSourceThree = new System.Windows.Forms.BindingSource(this.components);
            this.cboSensorTwo = new DnEcuDiag.CustomControls.ComboBoxExtended(this.components);
            this.sensorsBindingSourceTwo = new System.Windows.Forms.BindingSource(this.components);
            this.cboSensorOne = new DnEcuDiag.CustomControls.ComboBoxExtended(this.components);
            this.sensorsBindingSourceOne = new System.Windows.Forms.BindingSource(this.components);
            this.txtGroupDescription = new System.Windows.Forms.TextBox();
            this.lblDescription = new System.Windows.Forms.Label();
            this.lblGroupName = new System.Windows.Forms.Label();
            this.btnDelete = new System.Windows.Forms.Button();
            this.btnAdd = new System.Windows.Forms.Button();
            this.btnNext = new System.Windows.Forms.Button();
            this.toolTip = new System.Windows.Forms.ToolTip(this.components);
            this.btnLoad = new System.Windows.Forms.Button();
            this.btnOk = new System.Windows.Forms.Button();
            this.grpSensorGroup.SuspendLayout();
            ((System.ComponentModel.ISupportInitialize)(this.sensorsBindingSourceFour)).BeginInit();
            ((System.ComponentModel.ISupportInitialize)(this.sensorsBindingSourceThree)).BeginInit();
            ((System.ComponentModel.ISupportInitialize)(this.sensorsBindingSourceTwo)).BeginInit();
            ((System.ComponentModel.ISupportInitialize)(this.sensorsBindingSourceOne)).BeginInit();
            this.SuspendLayout();
            // 
            // grpSensorGroup
            // 
            this.grpSensorGroup.AccessibleDescription = null;
            this.grpSensorGroup.AccessibleName = null;
            resources.ApplyResources(this.grpSensorGroup, "grpSensorGroup");
            this.grpSensorGroup.BackgroundImage = null;
            this.grpSensorGroup.Controls.Add(this.btnPrevious);
            this.grpSensorGroup.Controls.Add(this.txtGroupName);
            this.grpSensorGroup.Controls.Add(this.lblSensorFour);
            this.grpSensorGroup.Controls.Add(this.lblSensorThree);
            this.grpSensorGroup.Controls.Add(this.lblSensorTwo);
            this.grpSensorGroup.Controls.Add(this.lblSensorOne);
            this.grpSensorGroup.Controls.Add(this.cboSensorFour);
            this.grpSensorGroup.Controls.Add(this.cboSensorThree);
            this.grpSensorGroup.Controls.Add(this.cboSensorTwo);
            this.grpSensorGroup.Controls.Add(this.cboSensorOne);
            this.grpSensorGroup.Controls.Add(this.txtGroupDescription);
            this.grpSensorGroup.Controls.Add(this.lblDescription);
            this.grpSensorGroup.Controls.Add(this.lblGroupName);
            this.grpSensorGroup.Controls.Add(this.btnDelete);
            this.grpSensorGroup.Controls.Add(this.btnAdd);
            this.grpSensorGroup.Controls.Add(this.btnNext);
            this.grpSensorGroup.Font = null;
            this.grpSensorGroup.Name = "grpSensorGroup";
            this.grpSensorGroup.TabStop = false;
            this.toolTip.SetToolTip(this.grpSensorGroup, resources.GetString("grpSensorGroup.ToolTip"));
            // 
            // btnPrevious
            // 
            this.btnPrevious.AccessibleDescription = null;
            this.btnPrevious.AccessibleName = null;
            resources.ApplyResources(this.btnPrevious, "btnPrevious");
            this.btnPrevious.BackgroundImage = null;
            this.btnPrevious.Font = null;
            this.btnPrevious.Name = "btnPrevious";
            this.toolTip.SetToolTip(this.btnPrevious, resources.GetString("btnPrevious.ToolTip"));
            this.btnPrevious.Click += new System.EventHandler(this.btnPrevious_Click);
            // 
            // txtGroupName
            // 
            this.txtGroupName.AccessibleDescription = null;
            this.txtGroupName.AccessibleName = null;
            resources.ApplyResources(this.txtGroupName, "txtGroupName");
            this.txtGroupName.BackgroundImage = null;
            this.txtGroupName.Font = null;
            this.txtGroupName.Name = "txtGroupName";
            this.toolTip.SetToolTip(this.txtGroupName, resources.GetString("txtGroupName.ToolTip"));
            // 
            // lblSensorFour
            // 
            this.lblSensorFour.AccessibleDescription = null;
            this.lblSensorFour.AccessibleName = null;
            resources.ApplyResources(this.lblSensorFour, "lblSensorFour");
            this.lblSensorFour.Font = null;
            this.lblSensorFour.Name = "lblSensorFour";
            this.toolTip.SetToolTip(this.lblSensorFour, resources.GetString("lblSensorFour.ToolTip"));
            // 
            // lblSensorThree
            // 
            this.lblSensorThree.AccessibleDescription = null;
            this.lblSensorThree.AccessibleName = null;
            resources.ApplyResources(this.lblSensorThree, "lblSensorThree");
            this.lblSensorThree.Font = null;
            this.lblSensorThree.Name = "lblSensorThree";
            this.toolTip.SetToolTip(this.lblSensorThree, resources.GetString("lblSensorThree.ToolTip"));
            // 
            // lblSensorTwo
            // 
            this.lblSensorTwo.AccessibleDescription = null;
            this.lblSensorTwo.AccessibleName = null;
            resources.ApplyResources(this.lblSensorTwo, "lblSensorTwo");
            this.lblSensorTwo.Font = null;
            this.lblSensorTwo.Name = "lblSensorTwo";
            this.toolTip.SetToolTip(this.lblSensorTwo, resources.GetString("lblSensorTwo.ToolTip"));
            // 
            // lblSensorOne
            // 
            this.lblSensorOne.AccessibleDescription = null;
            this.lblSensorOne.AccessibleName = null;
            resources.ApplyResources(this.lblSensorOne, "lblSensorOne");
            this.lblSensorOne.Font = null;
            this.lblSensorOne.Name = "lblSensorOne";
            this.toolTip.SetToolTip(this.lblSensorOne, resources.GetString("lblSensorOne.ToolTip"));
            // 
            // cboSensorFour
            // 
            this.cboSensorFour.AccessibleDescription = null;
            this.cboSensorFour.AccessibleName = null;
            resources.ApplyResources(this.cboSensorFour, "cboSensorFour");
            this.cboSensorFour.BackgroundImage = null;
            this.cboSensorFour.DataSource = this.sensorsBindingSourceFour;
            this.cboSensorFour.DisplayMember = "Name";
            this.cboSensorFour.DropDownStyle = System.Windows.Forms.ComboBoxStyle.DropDownList;
            this.cboSensorFour.Font = null;
            this.cboSensorFour.Name = "cboSensorFour";
            this.toolTip.SetToolTip(this.cboSensorFour, resources.GetString("cboSensorFour.ToolTip"));
            // 
            // sensorsBindingSourceFour
            // 
            this.sensorsBindingSourceFour.DataSource = typeof(DnEcuDiag.Ecus.Generic.Sensor);
            // 
            // cboSensorThree
            // 
            this.cboSensorThree.AccessibleDescription = null;
            this.cboSensorThree.AccessibleName = null;
            resources.ApplyResources(this.cboSensorThree, "cboSensorThree");
            this.cboSensorThree.BackgroundImage = null;
            this.cboSensorThree.DataSource = this.sensorsBindingSourceThree;
            this.cboSensorThree.DisplayMember = "Name";
            this.cboSensorThree.DropDownStyle = System.Windows.Forms.ComboBoxStyle.DropDownList;
            this.cboSensorThree.Font = null;
            this.cboSensorThree.Name = "cboSensorThree";
            this.toolTip.SetToolTip(this.cboSensorThree, resources.GetString("cboSensorThree.ToolTip"));
            // 
            // sensorsBindingSourceThree
            // 
            this.sensorsBindingSourceThree.DataSource = typeof(DnEcuDiag.Ecus.Generic.Sensor);
            // 
            // cboSensorTwo
            // 
            this.cboSensorTwo.AccessibleDescription = null;
            this.cboSensorTwo.AccessibleName = null;
            resources.ApplyResources(this.cboSensorTwo, "cboSensorTwo");
            this.cboSensorTwo.BackgroundImage = null;
            this.cboSensorTwo.DataSource = this.sensorsBindingSourceTwo;
            this.cboSensorTwo.DisplayMember = "Name";
            this.cboSensorTwo.DropDownStyle = System.Windows.Forms.ComboBoxStyle.DropDownList;
            this.cboSensorTwo.Font = null;
            this.cboSensorTwo.Name = "cboSensorTwo";
            this.toolTip.SetToolTip(this.cboSensorTwo, resources.GetString("cboSensorTwo.ToolTip"));
            // 
            // sensorsBindingSourceTwo
            // 
            this.sensorsBindingSourceTwo.DataSource = typeof(DnEcuDiag.Ecus.Generic.Sensor);
            // 
            // cboSensorOne
            // 
            this.cboSensorOne.AccessibleDescription = null;
            this.cboSensorOne.AccessibleName = null;
            resources.ApplyResources(this.cboSensorOne, "cboSensorOne");
            this.cboSensorOne.BackgroundImage = null;
            this.cboSensorOne.DataSource = this.sensorsBindingSourceOne;
            this.cboSensorOne.DisplayMember = "Name";
            this.cboSensorOne.DropDownStyle = System.Windows.Forms.ComboBoxStyle.DropDownList;
            this.cboSensorOne.Font = null;
            this.cboSensorOne.Name = "cboSensorOne";
            this.toolTip.SetToolTip(this.cboSensorOne, resources.GetString("cboSensorOne.ToolTip"));
            // 
            // sensorsBindingSourceOne
            // 
            this.sensorsBindingSourceOne.DataSource = typeof(DnEcuDiag.Ecus.Generic.Sensor);
            // 
            // txtGroupDescription
            // 
            this.txtGroupDescription.AccessibleDescription = null;
            this.txtGroupDescription.AccessibleName = null;
            resources.ApplyResources(this.txtGroupDescription, "txtGroupDescription");
            this.txtGroupDescription.BackgroundImage = null;
            this.txtGroupDescription.Font = null;
            this.txtGroupDescription.Name = "txtGroupDescription";
            this.toolTip.SetToolTip(this.txtGroupDescription, resources.GetString("txtGroupDescription.ToolTip"));
            // 
            // lblDescription
            // 
            this.lblDescription.AccessibleDescription = null;
            this.lblDescription.AccessibleName = null;
            resources.ApplyResources(this.lblDescription, "lblDescription");
            this.lblDescription.Font = null;
            this.lblDescription.Name = "lblDescription";
            this.toolTip.SetToolTip(this.lblDescription, resources.GetString("lblDescription.ToolTip"));
            // 
            // lblGroupName
            // 
            this.lblGroupName.AccessibleDescription = null;
            this.lblGroupName.AccessibleName = null;
            resources.ApplyResources(this.lblGroupName, "lblGroupName");
            this.lblGroupName.Font = null;
            this.lblGroupName.Name = "lblGroupName";
            this.toolTip.SetToolTip(this.lblGroupName, resources.GetString("lblGroupName.ToolTip"));
            // 
            // btnDelete
            // 
            this.btnDelete.AccessibleDescription = null;
            this.btnDelete.AccessibleName = null;
            resources.ApplyResources(this.btnDelete, "btnDelete");
            this.btnDelete.BackgroundImage = null;
            this.btnDelete.Font = null;
            this.btnDelete.Name = "btnDelete";
            this.toolTip.SetToolTip(this.btnDelete, resources.GetString("btnDelete.ToolTip"));
            this.btnDelete.Click += new System.EventHandler(this.btnDelete_Click);
            // 
            // btnAdd
            // 
            this.btnAdd.AccessibleDescription = null;
            this.btnAdd.AccessibleName = null;
            resources.ApplyResources(this.btnAdd, "btnAdd");
            this.btnAdd.BackgroundImage = null;
            this.btnAdd.Font = null;
            this.btnAdd.Name = "btnAdd";
            this.toolTip.SetToolTip(this.btnAdd, resources.GetString("btnAdd.ToolTip"));
            this.btnAdd.Click += new System.EventHandler(this.btnAdd_Click);
            // 
            // btnNext
            // 
            this.btnNext.AccessibleDescription = null;
            this.btnNext.AccessibleName = null;
            resources.ApplyResources(this.btnNext, "btnNext");
            this.btnNext.BackgroundImage = null;
            this.btnNext.Font = null;
            this.btnNext.Name = "btnNext";
            this.toolTip.SetToolTip(this.btnNext, resources.GetString("btnNext.ToolTip"));
            this.btnNext.Click += new System.EventHandler(this.btnNext_Click);
            // 
            // btnLoad
            // 
            this.btnLoad.AccessibleDescription = null;
            this.btnLoad.AccessibleName = null;
            resources.ApplyResources(this.btnLoad, "btnLoad");
            this.btnLoad.BackgroundImage = null;
            this.btnLoad.Font = null;
            this.btnLoad.Name = "btnLoad";
            this.toolTip.SetToolTip(this.btnLoad, resources.GetString("btnLoad.ToolTip"));
            this.btnLoad.Click += new System.EventHandler(this.btnLoad_Click);
            // 
            // btnOk
            // 
            this.btnOk.AccessibleDescription = null;
            this.btnOk.AccessibleName = null;
            resources.ApplyResources(this.btnOk, "btnOk");
            this.btnOk.BackgroundImage = null;
            this.btnOk.DialogResult = System.Windows.Forms.DialogResult.OK;
            this.btnOk.Font = null;
            this.btnOk.Name = "btnOk";
            this.toolTip.SetToolTip(this.btnOk, resources.GetString("btnOk.ToolTip"));
            this.btnOk.Click += new System.EventHandler(this.btnOk_Click);
            // 
            // FormSensorGroups
            // 
            this.AcceptButton = this.btnLoad;
            this.AccessibleDescription = null;
            this.AccessibleName = null;
            resources.ApplyResources(this, "$this");
            this.BackgroundImage = null;
            this.CancelButton = this.btnOk;
            this.ControlBox = false;
            this.Controls.Add(this.btnLoad);
            this.Controls.Add(this.btnOk);
            this.Controls.Add(this.grpSensorGroup);
            this.Font = null;
            this.FormBorderStyle = System.Windows.Forms.FormBorderStyle.FixedSingle;
            this.MaximizeBox = false;
            this.MinimizeBox = false;
            this.Name = "FormSensorGroups";
            this.toolTip.SetToolTip(this, resources.GetString("$this.ToolTip"));
            this.Load += new System.EventHandler(this.FormSensorGroups_Load);
            this.grpSensorGroup.ResumeLayout(false);
            this.grpSensorGroup.PerformLayout();
            ((System.ComponentModel.ISupportInitialize)(this.sensorsBindingSourceFour)).EndInit();
            ((System.ComponentModel.ISupportInitialize)(this.sensorsBindingSourceThree)).EndInit();
            ((System.ComponentModel.ISupportInitialize)(this.sensorsBindingSourceTwo)).EndInit();
            ((System.ComponentModel.ISupportInitialize)(this.sensorsBindingSourceOne)).EndInit();
            this.ResumeLayout(false);

        }

        #endregion

        #region Events

        private void btnOk_Click(object sender, EventArgs e)
        {
            Common.ExecuteExceptionHandledDelegate(delegate { HandleOkButton(); });
        }

        private void btnLoad_Click(object sender, System.EventArgs e)
        {
            Common.ExecuteExceptionHandledDelegate(delegate { HandleLoadButton(); });
        }

        private void btnAdd_Click(object sender, System.EventArgs e)
        {
            Common.ExecuteExceptionHandledDelegate(delegate
                                                       {
                                                           BindingContext[Model.Instance.SensorGroups.Tables[0]].EndCurrentEdit();
                                                           BindingContext[Model.Instance.SensorGroups.Tables[0]].AddNew();

                                                           ConfigureUiElements(true);

                                                           txtGroupName.Text = String.Format(WinFormStrings.NewGroup0,Model.Instance.SensorGroups.Tables[0].Rows.Count);
                                                           txtGroupDescription.Text = String.Format(WinFormStrings.ANewGroup0,Model.Instance.SensorGroups.Tables[0].Rows.Count);
                                                       });
        }

        private void btnPrevious_Click(object sender, System.EventArgs e)
        {
            Common.ExecuteExceptionHandledDelegate(delegate
                                                       {
                                                           if (BindingContext[Model.Instance.SensorGroups.Tables[0]].Position >= 1)
                                                           {
                                                               BindingContext[Model.Instance.SensorGroups.Tables[0]].Position += -1;
                                                           }
                                                       });
        }

        private void btnDelete_Click(object sender, System.EventArgs e)
        {
            Common.ExecuteExceptionHandledDelegate(delegate
                                                       {
                                                           if (this.BindingContext[Model.Instance.SensorGroups.Tables[0]].Position >= 0)
                                                           {
                                                               this.BindingContext[Model.Instance.SensorGroups.Tables[0]].RemoveAt(
                                                                   this.BindingContext[Model.Instance.SensorGroups.Tables[0]].Position);
                                                               Model.Instance.SensorGroups.AcceptChanges();
                                                           }
                                                           ConfigureUiElements(Model.Instance.SensorGroups.Tables[0].Rows.Count == 0 ? false : true);
                                                       });
        }

        private void btnNext_Click(object sender, System.EventArgs e)
        {
            Common.ExecuteExceptionHandledDelegate(delegate
                                                       {
                                                           if (this.BindingContext[Model.Instance.SensorGroups.Tables[0]].Position <
                                                               this.BindingContext[Model.Instance.SensorGroups.Tables[0]].Count)
                                                               this.BindingContext[Model.Instance.SensorGroups.Tables[0]].Position += 1;
                                                       });
        }

        #endregion Events

        #region Private Methods

        private void HandleOkButton()
        {
            try
            {
                Cursor.Current = Cursors.WaitCursor;
                BindingContext[Model.Instance.SensorGroups.Tables[0]].EndCurrentEdit();
                Model.Instance.SaveSensorGroups();
            }
            finally
            {
                Cursor.Current = Cursors.Default;
                Close();
            }
        }

        private void HandleLoadButton()
        {
            try
            {
                BindingContext[Model.Instance.SensorGroups.Tables[0]].EndCurrentEdit();
                Model.Instance.SaveSensorGroups();
                Model.Instance.TargetEcu.SensorMonitorOne = (Sensor)cboSensorOne.SelectedItem;
                Model.Instance.TargetEcu.SensorMonitorTwo = (Sensor)cboSensorTwo.SelectedItem;
                Model.Instance.TargetEcu.SensorMonitorThree = (Sensor)cboSensorThree.SelectedItem;
                Model.Instance.TargetEcu.SensorMonitorFour = (Sensor)cboSensorFour.SelectedItem;
            }
            finally
            {
                Cursor.Current = Cursors.Default;
                Close();
            }
        }

        private void ConfigureUiElements(bool enabled)
        {
            txtGroupDescription.Enabled = txtGroupName.Enabled = btnNext.Enabled = btnPrevious.Enabled = enabled;
            cboSensorFour.Enabled = cboSensorThree.Enabled = cboSensorTwo.Enabled = cboSensorOne.Enabled = enabled;
        }

        #endregion Private Methods

        #region Public Methods

        /// <summary>
        /// Load the list of sensors into the sensors groups UI.
        /// </summary>
        /// <param name="connectedEcu">The currently connected ecu.</param>
        public void PrepareSensorGroups(Ecu connectedEcu)
        {
            try
            {
                Enabled = false;
                Cursor.Current = Cursors.WaitCursor;

                ConfigureUiElements(Model.Instance.SensorGroups.Tables[0].Rows.Count == 0 ? false : true);

                grpSensorGroup.Text = connectedEcu.Model;

                if ((ecuName != connectedEcu.Model) | (ecuName != null && ecuName.Length == 0))
                {
                    ecuName = connectedEcu.Model;

                    txtGroupName.DataBindings.Clear();
                    txtGroupDescription.DataBindings.Clear();
                    cboSensorOne.DataBindings.Clear();
                    cboSensorTwo.DataBindings.Clear();
                    cboSensorThree.DataBindings.Clear();
                    cboSensorFour.DataBindings.Clear();

                    txtGroupName.DataBindings.Add("Text", Model.Instance.SensorGroups.Tables[0], "GroupName");
                    txtGroupDescription.DataBindings.Add("Text", Model.Instance.SensorGroups.Tables[0], "GroupDescription");

                    cboSensorOne.DataBindings.Add("SelectedIndex", Model.Instance.SensorGroups.Tables[0], "SensorIndexOne");
                    cboSensorTwo.DataBindings.Add("SelectedIndex", Model.Instance.SensorGroups.Tables[0], "SensorIndexTwo");
                    cboSensorThree.DataBindings.Add("SelectedIndex", Model.Instance.SensorGroups.Tables[0], "SensorIndexThree");
                    cboSensorFour.DataBindings.Add("SelectedIndex", Model.Instance.SensorGroups.Tables[0], "SensorIndexFour");
                }
            }
            finally
            {
                Enabled = true;
                Cursor.Current = Cursors.Default;
            }
        }

        #endregion Public Methods

        private void FormSensorGroups_Load(object sender, EventArgs e)
        {
            sensorsBindingSourceOne.DataSource = Common.ConfigureSensorBindingSource(Model.Instance.TargetEcu.Sensors);
            sensorsBindingSourceTwo.DataSource = Common.ConfigureSensorBindingSource(Model.Instance.TargetEcu.Sensors);
            sensorsBindingSourceThree.DataSource = Common.ConfigureSensorBindingSource(Model.Instance.TargetEcu.Sensors);
            sensorsBindingSourceFour.DataSource = Common.ConfigureSensorBindingSource(Model.Instance.TargetEcu.Sensors);

            
            if (Settings.Default.SortSensorsAscending)
            {
                sensorsBindingSourceOne.Sort = "Name ASC";
                sensorsBindingSourceTwo.Sort = "Name ASC";
                sensorsBindingSourceThree.Sort = "Name ASC";
                sensorsBindingSourceFour.Sort = "Name ASC";
            }
            
            if (Settings.Default.SortSensorsDescending)
            {
                sensorsBindingSourceOne.Sort = "Name DESC";
                sensorsBindingSourceTwo.Sort = "Name DESC";
                sensorsBindingSourceThree.Sort = "Name DESC";
                sensorsBindingSourceFour.Sort = "Name DESC";
            }

            ConfigureUiElements(Model.Instance.SensorGroups.Tables[0].Rows.Count == 0 ? false : true);

            txtGroupName.DataBindings.Add("Text", Model.Instance.SensorGroups.Tables[0], "GroupName");
            txtGroupDescription.DataBindings.Add("Text", Model.Instance.SensorGroups.Tables[0], "GroupDescription");

            cboSensorOne.DataBindings.Add("SelectedIndex", Model.Instance.SensorGroups.Tables[0], "SensorIndexOne");
            cboSensorTwo.DataBindings.Add("SelectedIndex", Model.Instance.SensorGroups.Tables[0], "SensorIndexTwo");
            cboSensorThree.DataBindings.Add("SelectedIndex", Model.Instance.SensorGroups.Tables[0], "SensorIndexThree");
            cboSensorFour.DataBindings.Add("SelectedIndex", Model.Instance.SensorGroups.Tables[0], "SensorIndexFour");
        }
    }
}