#region License
// Copyright  2004 - 2010 All Right Reserved, Derek Goslin
// 
// 1. Definitions
// The terms "reproduce," "reproduction," "derivative works," and "distribution" have the same meaning here as under U.S. copyright law.
// A "contribution" is the original software, or any additions or changes to the software.
// A "contributor" is any person that distributes its contribution under this license.
// "Licensed patents" are a contributor's patent claims that read directly on its contribution.
// 
// 2. Grant of Rights
// (A) Copyright Grant- Subject to the terms of this license, including the license
//     conditions and limitations in section 3, each contributor grants you a non-exclusive,
//     worldwide, royalty-free copyright license to reproduce its contribution, prepare derivative 
//     works of its contribution, and distribute its contribution or any derivative works that you create.
// (B) Patent Grant- Subject to the terms of this license, including the license conditions and limitations
//     in section 3, each contributor grants you a non-exclusive, worldwide, royalty-free license under its
//     licensed patents to make, have made, use, sell, offer for sale, import, and/or otherwise dispose of
//     its contribution in the software or derivative works of the contribution in the software.
// 
// 3. Conditions and Limitations
// (A) Reciprocal Grants- For any file you distribute that contains code from the software (in source code or binary format), 
//     you must provide recipients the source code to that file along with a copy of this license, which license will govern that file. 
//     You may license other files that are entirely your own work and do not contain code from the software under any terms you choose.
// (B) No Trademark License- This license does not grant you rights to use any contributors' name, logo, or trademarks.
// (C) If you bring a patent claim against any contributor over patents that you claim are infringed by the software,
//     your patent license from such contributor to the software ends automatically.
// (D) If you distribute any portion of the software, you must retain all copyright, patent, trademark,
//     and attribution notices that are present in the software.
// (E) If you distribute any portion of the software in source code form, you may do so only under this license by including a complete copy 
//     of this license with your distribution. If you distribute any portion of the software in compiled or object code form, you may only do 
//     so under a license that complies with this license.
// (F) The software is licensed "as-is." You bear the risk of using it. The contributors give no express warranties, guarantees or conditions.
//    You may have additional consumer rights under your local laws which this license cannot change. To the extent permitted under your
//    local laws, the contributors exclude the implied warranties of merchantability, fitness for a particular purpose and non-infringement.
#endregion

using System;
using System.ComponentModel;
using System.Windows.Forms;

namespace DnEcuDiag.Forms
{
    /// <summary>
    /// This delegate is called when the terminate button is clicked.
    /// </summary>
    public delegate void TerminateHandler();

    /// <summary>
    /// Summary description for TraceExceptionForm.
    /// </summary>
    public class TraceExceptionForm : Form
    {
        private Button btnOk;
        private IContainer components;
        private Label lblStackTrace;
        private Panel panel1;
        private Panel panel2;
        private TextBox txtDetails;

        public TraceExceptionForm()
        {
            //
            // Required for Windows Form Designer support
            //
            InitializeComponent();
        }

        /// <summary>
        /// Clean up any resources being used.
        /// </summary>
        protected override void Dispose(bool disposing)
        {
            if (disposing)
            {
                if (components != null)
                {
                    components.Dispose();
                }
            }
            base.Dispose(disposing);
        }


        /// <summary>
        /// Displays the trace exception form.
        /// </summary>
        /// <param name="exception">The trace exception.</param>
        public static void Display(Exception exception)
        {
            Cursor.Current = Cursors.Default;
            var frm = new TraceExceptionForm();
            frm.Init(exception);
            frm.ShowDialog();
        }

        /// <summary>
        /// Initializes the form.
        /// </summary>
        /// <param name="ex">Root exception.</param>
        private void Init(Exception ex)
        {
            txtDetails.Text = ex.ToString();

            HandleSizeChange();
        }

        private void TraceExceptionForm_Resize(object sender, EventArgs e)
        {
            HandleSizeChange();
        }

        private void HandleSizeChange()
        {
            panel1.Width = Width - 8;
            panel1.Height = panel2.Top - panel1.Top;

            btnOk.Left = (Width/2) - (btnOk.Width/2);
        }

        private void TraceExceptionForm_SizeChanged(object sender, EventArgs e)
        {
            HandleSizeChange();
        }

        #region Windows Form Designer generated code

        /// <summary>
        /// Required method for Designer support - do not modify
        /// the contents of this method with the code editor.
        /// </summary>
        private void InitializeComponent()
        {
            System.ComponentModel.ComponentResourceManager resources = new System.ComponentModel.ComponentResourceManager(typeof(TraceExceptionForm));
            this.lblStackTrace = new System.Windows.Forms.Label();
            this.panel1 = new System.Windows.Forms.Panel();
            this.txtDetails = new System.Windows.Forms.TextBox();
            this.panel2 = new System.Windows.Forms.Panel();
            this.btnOk = new System.Windows.Forms.Button();
            this.panel1.SuspendLayout();
            this.panel2.SuspendLayout();
            this.SuspendLayout();
            // 
            // lblStackTrace
            // 
            this.lblStackTrace.AccessibleDescription = null;
            this.lblStackTrace.AccessibleName = null;
            resources.ApplyResources(this.lblStackTrace, "lblStackTrace");
            this.lblStackTrace.Font = null;
            this.lblStackTrace.Name = "lblStackTrace";
            // 
            // panel1
            // 
            this.panel1.AccessibleDescription = null;
            this.panel1.AccessibleName = null;
            resources.ApplyResources(this.panel1, "panel1");
            this.panel1.BackgroundImage = null;
            this.panel1.Controls.Add(this.txtDetails);
            this.panel1.Font = null;
            this.panel1.Name = "panel1";
            // 
            // txtDetails
            // 
            this.txtDetails.AccessibleDescription = null;
            this.txtDetails.AccessibleName = null;
            resources.ApplyResources(this.txtDetails, "txtDetails");
            this.txtDetails.BackColor = System.Drawing.Color.White;
            this.txtDetails.BackgroundImage = null;
            this.txtDetails.Font = null;
            this.txtDetails.Name = "txtDetails";
            this.txtDetails.ReadOnly = true;
            // 
            // panel2
            // 
            this.panel2.AccessibleDescription = null;
            this.panel2.AccessibleName = null;
            resources.ApplyResources(this.panel2, "panel2");
            this.panel2.BackgroundImage = null;
            this.panel2.Controls.Add(this.btnOk);
            this.panel2.Font = null;
            this.panel2.Name = "panel2";
            // 
            // btnOk
            // 
            this.btnOk.AccessibleDescription = null;
            this.btnOk.AccessibleName = null;
            resources.ApplyResources(this.btnOk, "btnOk");
            this.btnOk.BackgroundImage = null;
            this.btnOk.Font = null;
            this.btnOk.Name = "btnOk";
            this.btnOk.Click += new System.EventHandler(this.btnOk_Click);
            // 
            // TraceExceptionForm
            // 
            this.AccessibleDescription = null;
            this.AccessibleName = null;
            resources.ApplyResources(this, "$this");
            this.BackgroundImage = null;
            this.Controls.Add(this.panel2);
            this.Controls.Add(this.panel1);
            this.Controls.Add(this.lblStackTrace);
            this.Font = null;
            this.FormBorderStyle = System.Windows.Forms.FormBorderStyle.FixedDialog;
            this.MaximizeBox = false;
            this.MinimizeBox = false;
            this.Name = "TraceExceptionForm";
            this.SizeChanged += new System.EventHandler(this.TraceExceptionForm_SizeChanged);
            this.Resize += new System.EventHandler(this.TraceExceptionForm_Resize);
            this.panel1.ResumeLayout(false);
            this.panel1.PerformLayout();
            this.panel2.ResumeLayout(false);
            this.ResumeLayout(false);

        }

        #endregion

        private void btnOk_Click(object sender, EventArgs e)
        {
            Close();
        }
    }
}