#region License
// Copyright  2004 - 2010 All Right Reserved, Derek Goslin
// 
// 1. Definitions
// The terms "reproduce," "reproduction," "derivative works," and "distribution" have the same meaning here as under U.S. copyright law.
// A "contribution" is the original software, or any additions or changes to the software.
// A "contributor" is any person that distributes its contribution under this license.
// "Licensed patents" are a contributor's patent claims that read directly on its contribution.
// 
// 2. Grant of Rights
// (A) Copyright Grant- Subject to the terms of this license, including the license
//     conditions and limitations in section 3, each contributor grants you a non-exclusive,
//     worldwide, royalty-free copyright license to reproduce its contribution, prepare derivative 
//     works of its contribution, and distribute its contribution or any derivative works that you create.
// (B) Patent Grant- Subject to the terms of this license, including the license conditions and limitations
//     in section 3, each contributor grants you a non-exclusive, worldwide, royalty-free license under its
//     licensed patents to make, have made, use, sell, offer for sale, import, and/or otherwise dispose of
//     its contribution in the software or derivative works of the contribution in the software.
// 
// 3. Conditions and Limitations
// (A) Reciprocal Grants- For any file you distribute that contains code from the software (in source code or binary format), 
//     you must provide recipients the source code to that file along with a copy of this license, which license will govern that file. 
//     You may license other files that are entirely your own work and do not contain code from the software under any terms you choose.
// (B) No Trademark License- This license does not grant you rights to use any contributors' name, logo, or trademarks.
// (C) If you bring a patent claim against any contributor over patents that you claim are infringed by the software,
//     your patent license from such contributor to the software ends automatically.
// (D) If you distribute any portion of the software, you must retain all copyright, patent, trademark,
//     and attribution notices that are present in the software.
// (E) If you distribute any portion of the software in source code form, you may do so only under this license by including a complete copy 
//     of this license with your distribution. If you distribute any portion of the software in compiled or object code form, you may only do 
//     so under a license that complies with this license.
// (F) The software is licensed "as-is." You bear the risk of using it. The contributors give no express warranties, guarantees or conditions.
//    You may have additional consumer rights under your local laws which this license cannot change. To the extent permitted under your
//    local laws, the contributors exclude the implied warranties of merchantability, fitness for a particular purpose and non-infringement.
#endregion

using System;
using System.Runtime.Serialization;
using System.Text;
using DnEcuDiag.Ecus.Kwp;

namespace DnEcuDiag.Protocols
{
    [Serializable]
    internal class Kwp2000Exception : Exception
    {
        //
        // For guidelines regarding the creation of new exception types, see
        //    http://msdn.microsoft.com/library/default.asp?url=/library/en-us/cpgenref/html/cpconerrorraisinghandlingguidelines.asp
        // and
        //    http://msdn.microsoft.com/library/default.asp?url=/library/en-us/dncscol/html/csharp07192001.asp
        //

        private readonly ResponseCode _errorCode;

        public Kwp2000Exception()
        {
        }

        public Kwp2000Exception(string message) : base(message)
        {
        }

        public Kwp2000Exception(string message, Exception inner) : base(message, inner)
        {
        }

        ///	<summary>
        ///	Initializes	a new instance of the <see cref="Kwp2000Exception"/> class.
        ///	</summary>
        ///	<param name="message">The error	message.</param>
        ///	<param name="errorCode">The	error code.</param>
        public Kwp2000Exception(string message, ResponseCode errorCode)
            : base(message)
        {
            _errorCode = errorCode;
        }

        ///	<summary>
        ///	Initializes	a new instance of the <see cref="Kwp2000Exception"/> class.
        ///	</summary>
        ///	<param name="message">The error	message.</param>
        ///	<param name="inner">The	inner exception.</param>
        ///	<param name="errorCode">The	error code.</param>
        public Kwp2000Exception(string message, Exception inner, ResponseCode errorCode)
            : base(message, inner)
        {
            _errorCode = errorCode;
        }

        protected Kwp2000Exception(
            SerializationInfo info,
            StreamingContext context)
            : base(info, context)
        {
        }

        /// <summary>
        /// Gets the error code.
        /// </summary>
        /// <value>The error code.</value>
        public ResponseCode ErrorCode
        {
            get { return _errorCode; }
        }

        ///	<summary>
        ///	Convert	the	exception to a string.
        ///	</summary>
        ///	<returns>A string representation of	the	exception.</returns>
        public override string ToString()
        {
            var strbuilder = new StringBuilder();
            strbuilder.Append(base.ToString());

            if (_errorCode != 0)
            {
                strbuilder.Append("ResponseCode[");
                strbuilder.Append(_errorCode.ToString());
                strbuilder.Append("]");
            }

            return (strbuilder.ToString());
        }

        public override string Message
        {
            get
            {
                return base.Message + "ResponseCode[" + _errorCode + "]";
            }
        }
    }
}