#region License
// Copyright  2004 - 2010 All Right Reserved, Derek Goslin
// 
// 1. Definitions
// The terms "reproduce," "reproduction," "derivative works," and "distribution" have the same meaning here as under U.S. copyright law.
// A "contribution" is the original software, or any additions or changes to the software.
// A "contributor" is any person that distributes its contribution under this license.
// "Licensed patents" are a contributor's patent claims that read directly on its contribution.
// 
// 2. Grant of Rights
// (A) Copyright Grant- Subject to the terms of this license, including the license
//     conditions and limitations in section 3, each contributor grants you a non-exclusive,
//     worldwide, royalty-free copyright license to reproduce its contribution, prepare derivative 
//     works of its contribution, and distribute its contribution or any derivative works that you create.
// (B) Patent Grant- Subject to the terms of this license, including the license conditions and limitations
//     in section 3, each contributor grants you a non-exclusive, worldwide, royalty-free license under its
//     licensed patents to make, have made, use, sell, offer for sale, import, and/or otherwise dispose of
//     its contribution in the software or derivative works of the contribution in the software.
// 
// 3. Conditions and Limitations
// (A) Reciprocal Grants- For any file you distribute that contains code from the software (in source code or binary format), 
//     you must provide recipients the source code to that file along with a copy of this license, which license will govern that file. 
//     You may license other files that are entirely your own work and do not contain code from the software under any terms you choose.
// (B) No Trademark License- This license does not grant you rights to use any contributors' name, logo, or trademarks.
// (C) If you bring a patent claim against any contributor over patents that you claim are infringed by the software,
//     your patent license from such contributor to the software ends automatically.
// (D) If you distribute any portion of the software, you must retain all copyright, patent, trademark,
//     and attribution notices that are present in the software.
// (E) If you distribute any portion of the software in source code form, you may do so only under this license by including a complete copy 
//     of this license with your distribution. If you distribute any portion of the software in compiled or object code form, you may only do 
//     so under a license that complies with this license.
// (F) The software is licensed "as-is." You bear the risk of using it. The contributors give no express warranties, guarantees or conditions.
//    You may have additional consumer rights under your local laws which this license cannot change. To the extent permitted under your
//    local laws, the contributors exclude the implied warranties of merchantability, fitness for a particular purpose and non-infringement.
#endregion

using System;

namespace DnEcuDiag.VehiclePerformance
{
    ///	<summary>
    ///	A dyno measurement reading.
    ///	</summary>
    internal struct DynoMeasurementValue 
    {
        private	Double _dPowerMeasurement;
        private	Double _dTorqueMeasurement;
        private	Int32 _irpmMeasurement;
			
        ///	<summary>
        ///	Construct a	dyno measurement.
        ///	</summary>
        ///	<param name="power">The	power expressed	in Kilowatts</param>
        ///	<param name="torque">The torque	expressed in Newton	Meters</param>
        ///	<param name="rpm">The RPM for the measurement.</param>
        public DynoMeasurementValue(double power, double torque, int rpm)
        {
            _dPowerMeasurement	= power;
            _dTorqueMeasurement = torque;
            _irpmMeasurement =	rpm;
        }

        ///	<summary>
        ///	Power in Kilowatts
        ///	</summary>
        public double Power	
        {
            get	{return	_dPowerMeasurement;}
            set	{ _dPowerMeasurement =	value; }
        }
			
        ///	<summary>
        ///	Torque in Newton meters
        ///	</summary>
        public double Torque 
        {
            get	{return	_dTorqueMeasurement;}
            set	{ _dTorqueMeasurement = value;	}
        }
			
        ///	<summary>
        ///	Power in BHP
        ///	</summary>
        public double PowerBhp 
        {
            get	{return	_dPowerMeasurement	* 1.3410;}
        }
			
        ///	<summary>
        ///	Torque in foot pounds
        ///	</summary>
        public double Torqueftlb 
        {
            get	{return	_dTorqueMeasurement * 0.7376;}
        }

        ///	<summary>
        ///	The	unscaled measurement rpm.
        ///	</summary>
        public int Rpm 
        {
            get	{return	_irpmMeasurement; }
            set	{_irpmMeasurement = value;}
        }
    }
}