#region License
// Copyright  2004 - 2010 All Right Reserved, Derek Goslin
// 
// 1. Definitions
// The terms "reproduce," "reproduction," "derivative works," and "distribution" have the same meaning here as under U.S. copyright law.
// A "contribution" is the original software, or any additions or changes to the software.
// A "contributor" is any person that distributes its contribution under this license.
// "Licensed patents" are a contributor's patent claims that read directly on its contribution.
// 
// 2. Grant of Rights
// (A) Copyright Grant- Subject to the terms of this license, including the license
//     conditions and limitations in section 3, each contributor grants you a non-exclusive,
//     worldwide, royalty-free copyright license to reproduce its contribution, prepare derivative 
//     works of its contribution, and distribute its contribution or any derivative works that you create.
// (B) Patent Grant- Subject to the terms of this license, including the license conditions and limitations
//     in section 3, each contributor grants you a non-exclusive, worldwide, royalty-free license under its
//     licensed patents to make, have made, use, sell, offer for sale, import, and/or otherwise dispose of
//     its contribution in the software or derivative works of the contribution in the software.
// 
// 3. Conditions and Limitations
// (A) Reciprocal Grants- For any file you distribute that contains code from the software (in source code or binary format), 
//     you must provide recipients the source code to that file along with a copy of this license, which license will govern that file. 
//     You may license other files that are entirely your own work and do not contain code from the software under any terms you choose.
// (B) No Trademark License- This license does not grant you rights to use any contributors' name, logo, or trademarks.
// (C) If you bring a patent claim against any contributor over patents that you claim are infringed by the software,
//     your patent license from such contributor to the software ends automatically.
// (D) If you distribute any portion of the software, you must retain all copyright, patent, trademark,
//     and attribution notices that are present in the software.
// (E) If you distribute any portion of the software in source code form, you may do so only under this license by including a complete copy 
//     of this license with your distribution. If you distribute any portion of the software in compiled or object code form, you may only do 
//     so under a license that complies with this license.
// (F) The software is licensed "as-is." You bear the risk of using it. The contributors give no express warranties, guarantees or conditions.
//    You may have additional consumer rights under your local laws which this license cannot change. To the extent permitted under your
//    local laws, the contributors exclude the implied warranties of merchantability, fitness for a particular purpose and non-infringement.
#endregion

using System;
using System.Runtime.InteropServices;
using InstrumentationControls.SafeHandles;

namespace InstrumentationControls
{
    public static class NativeMethods
    {
        internal enum RasterOp : uint
        {
            /// <summary>
            /// Copies the source rectangle directly to the destination rectangle.
            /// </summary>
            Srccopy = 0x00CC0020,
            /// <summary>
            /// Combines the colors of the brush currently selected in hdcDest, 
            /// with the colors of the inverted source rectangle by using the Boolean OR operator. 
            /// The result of this operation is combined with the colors of the destination rectangle 
            /// by using the Boolean OR operator.
            /// </summary>
            Srcpaint = 0x00EE0086,
            /// <summary>
            /// 
            /// </summary>
            Srcand = 0x008800C6,
            /// <summary>
            /// 
            /// </summary>
            Srcinvert = 0x00660046,
            /// <summary>
            /// Combines the inverted colors of the destination rectangle with the
            /// colors of the source rectangle by using the Boolean AND operator.
            /// </summary>
            Srcerase = 0x00440328,
            Notsrccopy = 0x00330008,		/* dest = (NOT source)             */
            Notsrcerase = 0x001100A6,		/* dest = (NOT src) AND (NOT dest) */
            Mergecopy = 0x00C000CA,			/* dest = (source AND pattern)     */
            Mergepaint = 0x00BB0226,		/* dest = (NOT source) OR dest     */
            Patcopy = 0x00F00021,			/* dest = pattern                  */
            Patpaint = 0x00FB0A09,			/* dest = DPSnoo                   */
            Patinvert = 0x005A0049,			/* dest = pattern XOR dest         */
            Dstinvert = 0x00550009,			/* dest = (NOT dest)               */
            Blackness = 0x00000042,			/* dest = BLACK                    */
            Whiteness = 0x00FF0062,			/* dest = WHITE                    */
            Nomirrorbitmap = 0x80000000,	/* Do not Mirror the bitmap in this call */
            Captureblt = 0x40000000,		/* Include layered windows */
        }

        [DllImport("Msimg32.dll")]
        public static extern bool TransparentBlt(
            GdiPlusDeviceContextSafeHandle hdcDest, // handle to destination DC
        int nXOriginDest, // x-coord of destination upper-left corner
        int nYOriginDest, // y-coord of destination upper-left corner
        int nWidthDest, // width of destination rectangle
        int hHeightDest, // height of destination rectangle
        GdiDeviceContextSafeHandle hdcSrc, // handle to source DC
        int nXOriginSrc, // x-coord of source upper-left corner
        int nYOriginSrc, // y-coord of source upper-left corner
        int nWidthSrc, // width of source rectangle
        int nHeightSrc, // height of source rectangle
        int crTransparent // color to make transparent
        );

        /// <summary>
        /// 
        /// </summary>
        /// <param name="hdcDest">handle to destination DC</param>
        /// <param name="nXDest">x-coord of destination upper-left corner</param>
        /// <param name="nYDest">y-coord of destination upper-left corner</param>
        /// <param name="nWidth">width of destination rectangle</param>
        /// <param name="nHeight">height of destination rectangle</param>
        /// <param name="hdcSrc">handle to source DC</param>
        /// <param name="nXSrc">x-coordinate of source upper-left corner</param>
        /// <param name="nYSrc">y-coordinate of source upper-left corner</param>
        /// <param name="dwRop">raster operation code</param>
        /// <returns></returns>
        [DllImportAttribute("Gdi32.dll", SetLastError = true)]
        [return: MarshalAs(UnmanagedType.Bool)]
        internal static extern bool BitBlt(
            GdiPlusDeviceContextSafeHandle hdcDest,
            int nXDest,
            int nYDest,
            int nWidth,
            int nHeight,
            GdiDeviceContextSafeHandle hdcSrc,
            int nXSrc,
            int nYSrc,
            RasterOp dwRop
            );

        [DllImportAttribute("Gdi32.dll", SetLastError = true)]
// ReSharper disable InconsistentNaming
        internal static extern GdiDeviceContextSafeHandle CreateCompatibleDC(GdiPlusDeviceContextSafeHandle hdc);
// ReSharper restore InconsistentNaming

        [DllImportAttribute("Gdi32.dll", SetLastError = true)]
        internal static extern IntPtr SelectObject(GdiDeviceContextSafeHandle hdc, GdiObjectSafeHandle hgdiobj);

     }
}
