#region License
// Copyright  2004 - 2010 All Right Reserved, Derek Goslin
// 
// 1. Definitions
// The terms "reproduce," "reproduction," "derivative works," and "distribution" have the same meaning here as under U.S. copyright law.
// A "contribution" is the original software, or any additions or changes to the software.
// A "contributor" is any person that distributes its contribution under this license.
// "Licensed patents" are a contributor's patent claims that read directly on its contribution.
// 
// 2. Grant of Rights
// (A) Copyright Grant- Subject to the terms of this license, including the license
//     conditions and limitations in section 3, each contributor grants you a non-exclusive,
//     worldwide, royalty-free copyright license to reproduce its contribution, prepare derivative 
//     works of its contribution, and distribute its contribution or any derivative works that you create.
// (B) Patent Grant- Subject to the terms of this license, including the license conditions and limitations
//     in section 3, each contributor grants you a non-exclusive, worldwide, royalty-free license under its
//     licensed patents to make, have made, use, sell, offer for sale, import, and/or otherwise dispose of
//     its contribution in the software or derivative works of the contribution in the software.
// 
// 3. Conditions and Limitations
// (A) Reciprocal Grants- For any file you distribute that contains code from the software (in source code or binary format), 
//     you must provide recipients the source code to that file along with a copy of this license, which license will govern that file. 
//     You may license other files that are entirely your own work and do not contain code from the software under any terms you choose.
// (B) No Trademark License- This license does not grant you rights to use any contributors' name, logo, or trademarks.
// (C) If you bring a patent claim against any contributor over patents that you claim are infringed by the software,
//     your patent license from such contributor to the software ends automatically.
// (D) If you distribute any portion of the software, you must retain all copyright, patent, trademark,
//     and attribution notices that are present in the software.
// (E) If you distribute any portion of the software in source code form, you may do so only under this license by including a complete copy 
//     of this license with your distribution. If you distribute any portion of the software in compiled or object code form, you may only do 
//     so under a license that complies with this license.
// (F) The software is licensed "as-is." You bear the risk of using it. The contributors give no express warranties, guarantees or conditions.
//    You may have additional consumer rights under your local laws which this license cannot change. To the extent permitted under your
//    local laws, the contributors exclude the implied warranties of merchantability, fitness for a particular purpose and non-infringement.
#endregion

using System;
using System.Runtime.InteropServices;
using Microsoft.Win32.SafeHandles;
using SerialPort.Api.Enums;
using SerialPort.Api.Structs;

namespace SerialPort.Api
{
    internal class FTD2xxNativeMethods
    {
        private FTD2xxNativeMethods()
        {
        }

        #region Classic Interface Functions

        /// <summary>
        /// 
        /// </summary>
        /// <param name="pvArg1">Meaning depends on dwFlags.</param>
        /// <param name="pvArg2">Meaning depends on dwFlags.</param>
        /// <param name="dwFlags">Determines format of returned information.</param>
        /// <returns>FtOk if successful, otherwise the return value is an FT error code.</returns>
        [DllImport("FTD2XX.dll")]
        public static extern FtStatus FT_ListDevices(IntPtr pvArg1, IntPtr pvArg2, FtListFlags dwFlags);

        /// <summary>
        /// Open the device and return a handle which will be used for subsequent accesses.
        /// </summary>
        /// <param name="iDevice">Must be 0 if only one device is attached. For multiple devices 1,2 etc.</param>
        /// <param name="ftHandle">Pointer to a variable of type IntPtr where the handle will be stored. This handle must be used to access the device.</param>
        /// <returns>FtOk if successful, otherwise the return value is an FT error code.</returns>
        /// <remarks>Although this function can be used to open multiple devices by setting iDevice to 0, 1, 2 etc. there
        /// is no ability to open a specific device. To open named devices, use the function <see cref="FT_OpenEx"/></remarks>
        [DllImport("FTD2XX.dll")]
        public static extern FtStatus FT_Open(UInt32 iDevice, ref IntPtr ftHandle);

        /// <summary>
        /// 
        /// </summary>
        /// <param name="pvArg1">Meaning depends on dwFlags, but it will normally be interpreted as a pointer to a null terminated String.</param>
        /// <param name="dwFlags">FT_OPEN_BY_SERIAL_NUMBER,FT_OPEN_BY_DESCRIPTION or FT_OPEN_BY_LOCATION.</param>
        /// <param name="ftHandle">Pointer to a variable of type IntPtr where the handle will be stored. This handle must be used to access the device.</param>
        /// <returns>FtOk if successful, otherwise the return value is an FT error code.</returns>
        [DllImport("FTD2XX.dll")]
        public static extern FtStatus FT_OpenEx(IntPtr pvArg1, FtOpen dwFlags, ref IntPtr ftHandle);

        /// <summary>
        /// Close an open device.
        /// </summary>
        /// <param name="ftHandle">Handle of the device.</param>
        /// <returns>FtOk if successful, otherwise the return value is an FT error code.</returns>
        [DllImport("FTD2XX.dll")]
        public static extern FtStatus FT_Close(IntPtr ftHandle);

        /// <summary>
        /// Read data from the device.
        /// </summary>
        /// <param name="ftHandle">Handle of the device.</param>
        /// <param name="lpBuffer">Pointer to the buffer that receives the data from the device.</param>
        /// <param name="dwBytesToRead">Number of bytes to be read from the device.</param>
        /// <param name="lpdwBytesReturned">Pointer to a variable of type DWORD which receives the number of bytes read from the device.</param>
        /// <returns>FtOk if successful, FtIoError otherwise.</returns>
        /// <remarks>This function does not return until dwBytesToRead have been read into the buffer.
        /// When a read timeout value has been specified in a previous call to <see cref="FT_SetTimeouts"/> ,
        /// FT_Read returns when the timer expires or dwBytesToRead have been read, whichever occurs
        /// first. If the timeout occurred, FT_Read reads available data into the buffer and returns FtOk.
        /// An application should use the function return value and lpdwBytesReturned when processing the
        /// buffer. If the return value is FtOk, and lpdwBytesReturned is equal to dwBytesToRead then
        /// FT_Read has completed normally. If the return value is FtOk, and lpdwBytesReturned is
        /// less then dwBytesToRead then a timeout has occurred and the read has been partially completed.
        /// Note that if a timeout occurred and no data was read, the return value is still FtOk.
        /// A return value of FtIoError suggests an error in the parameters of the function, or a fatal
        /// error like USB disconnect has occurred.
        /// </remarks>
        [DllImport("FTD2XX.dll")]
        public static extern FtStatus FT_Read(IntPtr ftHandle, Byte[] lpBuffer, UInt32 dwBytesToRead,
                                                      ref UInt32 lpdwBytesReturned);

        /// <summary>
        /// Write data to the device.
        /// </summary>
        /// <param name="ftHandle">Handle of the device.</param>
        /// <param name="lpBuffer">Pointer to the buffer that contains the data to be written to the device.</param>
        /// <param name="dwBytesToWrite">Number of bytes to write to the device.</param>
        /// <param name="lpdwBytesWritten">Pointer to a variable of type DWORD which receives the number of bytes written to the device.</param>
        /// <returns>FtOk if successful, otherwise the return value is an FT error code.</returns>
        [DllImport("FTD2XX.dll")]
        public static extern FtStatus FT_Write(IntPtr ftHandle, Byte[] lpBuffer, UInt32 dwBytesToWrite,
                                                       ref UInt32 lpdwBytesWritten);

        /// <summary>
        /// FtOk if successful, otherwise the return value is an FT error code.
        /// </summary>
        /// <param name="ftHandle">Handle of the device.</param>
        /// <returns>FtOk if successful, otherwise the return value is an FT error code.</returns>
        [DllImport("FTD2XX.dll")]
        public static extern FtStatus FT_ResetDevice(IntPtr ftHandle);

        /// <summary>
        /// This function sets the Baud rate for the device.
        /// </summary>
        /// <param name="ftHandle">Handle of the device.</param>
        /// <param name="dwBaudRate">Baud rate.</param>
        /// <returns>FtOk if successful, otherwise the return value is an FT error code.</returns>
        [DllImport("FTD2XX.dll")]
        public static extern FtStatus FT_SetBaudRate(IntPtr ftHandle, UInt32 dwBaudRate);

        /// <summary>
        /// This function sets the Baud rate for the device. It is used to set non-standard Baud rates.
        /// </summary>
        /// <param name="ftHandle">Handle of the device.</param>
        /// <param name="usDivisor">Divisor.</param>
        /// <returns>FtOk if successful, otherwise the return value is an FT error code.</returns>
        [DllImport("FTD2XX.dll")]
        public static extern FtStatus FT_SetDivisor(IntPtr ftHandle, char usDivisor);

        /// <summary>
        /// This function sets the data characteristics for the device.
        /// </summary>
        /// <param name="ftHandle">Handle of the device.</param>
        /// <param name="uWordLength">Number of bits per word - must be FT_BITS_8 or FT_BITS_7.</param>
        /// <param name="uStopBits">Number of stop bits - must be FT_STOP_BITS_1 or FT_STOP_BITS_2.</param>
        /// <param name="uParity">FT_PARITY_NONE, FT_PARITY_ODD, _FT_PARITY_EVEN,FT_PARITY_MARK, FT_PARITY SPACE.</param>
        /// <returns>FtOk if successful, otherwise the return value is an FT error code.</returns>
        [DllImport("FTD2XX.dll")]
        public static extern FtStatus FT_SetDataCharacteristics(IntPtr ftHandle, byte uWordLength,
                                                                        byte uStopBits, byte uParity);

        /// <summary>
        /// This function sets the flow control for the device.
        /// </summary>
        /// <param name="ftHandle">Handle of the device.</param>
        /// <param name="usFlowControl">Must be one of FT_FLOW_NONE, FT_FLOW_RTS_CTS,FT_FLOW_DTR_DSR or FT_FLOW_XON_XOFF.</param>
        /// <param name="uXon">Character used to signal Xon. Only used if flow control is FT_FLOW_XON_XOFF.</param>
        /// <param name="uXoff">Character used to signal Xoff. Only used if flow control is FT_FLOW_XON_XOFF.</param>
        /// <returns>FtOk if successful, otherwise the return value is an FT error code.</returns>
        [DllImport("FTD2XX.dll")]
        public static extern FtStatus FT_SetFlowControl(IntPtr ftHandle, UInt16 usFlowControl, byte uXon,
                                                                byte uXoff);

        /// <summary>
        /// This function sets the Data Terminal Ready (DTR) control signal.
        /// </summary>
        /// <param name="ftHandle">Handle of the device.</param>
        /// <returns>FtOk if successful, otherwise the return value is an FT error code.</returns>
        [DllImport("FTD2XX.dll")]
        public static extern FtStatus FT_SetDtr(IntPtr ftHandle);

        /// <summary>
        /// This function clears the Data Terminal Ready (DTR) control signal.
        /// </summary>
        /// <param name="ftHandle">Handle of the device.</param>
        /// <returns>FtOk if successful, otherwise the return value is an FT error code.</returns>
        [DllImport("FTD2XX.dll")]
        public static extern FtStatus FT_ClrDtr(IntPtr ftHandle);

        /// <summary>
        /// This function sets the Request To Send (RTS) control signal.
        /// </summary>
        /// <param name="ftHandle">Handle of the device.</param>
        /// <returns>FtOk if successful, otherwise the return value is an FT error code.</returns>
        [DllImport("FTD2XX.dll")]
        public static extern FtStatus FT_SetRts(IntPtr ftHandle);

        /// <summary>
        /// This function clears the Request To Send (RTS) control signal.
        /// </summary>
        /// <param name="ftHandle">Handle of the device.</param>
        /// <returns>FtOk if successful, otherwise the return value is an FT error code.</returns>
        [DllImport("FTD2XX.dll")]
        public static extern FtStatus FT_ClrRts(IntPtr ftHandle);

        /// <summary>
        /// Gets the modem status from the device.
        /// </summary>
        /// <param name="ftHandle">Handle of the device.</param>
        /// <param name="lpdwModemStatus">Pointer to a variable of type DWORD which receives the
        /// modem status from the device. Status lines are bit-mapped as follows:
        /// Cts = 0x10
        /// Dsr = 0x20
        /// Ri = 0x40
        /// Dcd = 0x80</param>
        /// <returns>FtOk if successful, otherwise the return value is an FT error code.</returns>
        [DllImport("FTD2XX.dll")]
        public static extern FtStatus FT_GetModemStatus(IntPtr ftHandle, ref FtModemStatus lpdwModemStatus);

        /// <summary>
        /// This function sets the special characters for the device.
        /// </summary>
        /// <param name="ftHandle">Handle of the device.</param>
        /// <param name="uEventCh">Event character.</param>
        /// <param name="uEventChEn">0 if event character disabled, non-zero otherwise.</param>
        /// <param name="uErrorCh">Error character.</param>
        /// <param name="uErrorChEn">0 if error character disabled, non-zero otherwise.</param>
        /// <returns>FtOk if successful, otherwise the return value is an FT error code.</returns>
        [DllImport("FTD2XX.dll")]
        public static extern FtStatus FT_SetChars(IntPtr ftHandle, byte uEventCh, byte uEventChEn, byte uErrorCh,
                                                          byte uErrorChEn);

        /// <summary>
        /// This function purges receive and transmit buffers in the device.
        /// </summary>
        /// <param name="ftHandle">Handle of the device.</param>
        /// <param name="purgeFlags">Any combination of FtPurgeRx and FtPurgeTx.</param>
        /// <returns>FtOk if successful, otherwise the return value is an FT error code.</returns>
        [DllImport("FTD2XX.dll")]
        public static extern FtStatus FT_Purge(IntPtr ftHandle, FtPurgeFlags purgeFlags);

        /// <summary>
        /// This function sets the read and write timeouts for the device.
        /// </summary>
        /// <param name="ftHandle">Handle of the device.</param>
        /// <param name="dwReadTimeout">Read timeout in milliseconds.</param>
        /// <param name="dwWriteTimeout">Write timeout in milliseconds.</param>
        /// <returns>FtOk if successful, otherwise the return value is an FT error code.</returns>
        [DllImport("FTD2XX.dll")]
        public static extern FtStatus FT_SetTimeouts(IntPtr ftHandle, UInt32 dwReadTimeout,
                                                             UInt32 dwWriteTimeout);

        /// <summary>
        /// Gets the number of characters in the receive queue.
        /// </summary>
        /// <param name="ftHandle">Handle of the device.</param>
        /// <param name="lpdwAmountInRxQueue">Pointer to a variable of type DWORD which receives the number of characters in the receive queue.</param>
        /// <returns>FtOk if successful, otherwise the return value is an FT error code.</returns>
        [DllImport("FTD2XX.dll")]
        public static extern FtStatus FT_GetQueueStatus(IntPtr ftHandle, ref UInt32 lpdwAmountInRxQueue);

        /// <summary>
        /// Sets the BREAK condition for the device.
        /// </summary>
        /// <param name="ftHandle">Handle of the device.</param>
        /// <returns>FtOk if successful, otherwise the return value is an FT error code.</returns>
        [DllImport("FTD2XX.dll")]
        public static extern FtStatus FT_SetBreakOn(IntPtr ftHandle);

        /// <summary>
        /// Resets the BREAK condition for the device.
        /// </summary>
        /// <param name="ftHandle">Handle of the device.</param>
        /// <returns>FtOk if successful, otherwise the return value is an FT error code.</returns>
        [DllImport("FTD2XX.dll")]
        public static extern FtStatus FT_SetBreakOff(IntPtr ftHandle);

        /// <summary>
        /// Gets the device status including number of characters in the receive queue, number of characters 
        /// in the transmit queue, and the current event status.
        /// </summary>
        /// <param name="ftHandle">Handle of the device.</param>
        /// <param name="lpdwAmountInRxQueue">Pointer to a variable of type DWORD which receives the number of characters in the receive queue.</param>
        /// <param name="lpdwAmountInTxQueue">Pointer to a variable of type DWORD which receives the number of characters in the transmit queue.</param>
        /// <param name="lpdwEventStatus">Pointer to a variable of type DWORD which receives the current state of the event status.</param>
        /// <returns>FtOk if successful, otherwise the return value is an FT error code.</returns>
        [DllImport("FTD2XX.dll")]
        public static extern FtStatus FT_GetStatus(IntPtr ftHandle, ref UInt32 lpdwAmountInRxQueue,
                                                           ref UInt32 lpdwAmountInTxQueue, ref FtEventNotificationFlags lpdwEventStatus);

        /// <summary>
        /// Sets conditions for event notification.
        /// </summary>
        /// <param name="ftHandle">Handle of the device.</param>
        /// <param name="dwEventMask">Conditions that cause the event to be set.</param>
        /// <param name="pvArg">Interpreted as the handle of an event.</param>
        /// <returns>FtOk if successful, otherwise the return value is an FT error code.</returns>
        /// <remarks>An application can use this function to setup conditions which allow a thread to block until one of
        /// the conditions is met. Typically, an application will create an event, call this function, then block on
        /// the event. When the conditions are met, the event is set, and the application thread unblocked.
        /// dwEventMask is a bit-map that describes the events the application is interested in. pvArg is
        /// interpreted as the handle of an event which has been created by the application. If one of the
        /// event conditions is met, the event is set.If FtEventRxchar is set in dwEventMask, the event will be set when a character has been
        /// received by the device. If FtEventModemStatus is set in dwEventMask, the event will be
        /// set when a change in the modem signals has been detected by the device.</remarks>
        [DllImport("FTD2XX.dll")]
        public static extern FtStatus FT_SetEventNotification(IntPtr ftHandle, FtEventNotificationFlags dwEventMask, SafeWaitHandle pvArg);

        //public static extern unsafe FtStatus FT_IoCtl(IntPtr ftHandle, UInt32 dwIoControlCode, LPVOID lpInBuf, UInt32 nInBufSize, LPVOID lpOutBuf,
        //                                               UInt32 nOutBufSize, ref UInt32 lpBytesReturned, LPOVERLAPPED lpOverlapped);
        /// <summary>
        /// 
        /// </summary>
        /// <param name="ftHandle"></param>
        /// <param name="dwMask"></param>
        /// <returns></returns>
        [DllImport("FTD2XX.dll")]
        public static extern FtStatus FT_SetWaitMask(IntPtr ftHandle, UInt32 dwMask);

        /// <summary>
        /// 
        /// </summary>
        /// <param name="ftHandle"></param>
        /// <param name="dwMask"></param>
        /// <returns></returns>
        [DllImport("FTD2XX.dll")]
        public static extern FtStatus FT_WaitOnMask(IntPtr ftHandle, UInt32 dwMask);

        /// <summary>
        /// Get device information.
        /// </summary>
        /// <param name="ftHandle"></param>
        /// <param name="pftType"></param>
        /// <param name="lpdwID"></param>
        /// <param name="pcSerialNumber"></param>
        /// <param name="pcDescription"></param>
        /// <param name="pvDummy"></param>
        /// <returns></returns>
        [DllImport("FTD2XX.dll")]
        public static extern FtStatus FT_GetDeviceInfo(IntPtr ftHandle, ref FtDevice pftType, ref UInt32 lpdwID,
                                                               String pcSerialNumber, String pcDescription,
                                                               IntPtr pvDummy);

        /// <summary>
        /// Set the ResetPipeRetryCount.
        /// </summary>
        /// <param name="ftHandle">Handle of the device.</param>
        /// <param name="dwCount">Unsigned long containing required ResetPipeRetryCount.</param>
        /// <returns>FtOk if successful, otherwise the return value is an FT error code.</returns>
        /// <remarks>This function is used to set the ResetPipeRetryCount. ResetPipeRetryCount controls the
        /// maximum number of times that the driver tries to reset a pipe on which an error has occurred.
        /// ResetPipeRequestRetryCount defaults to 50. It may be necessary to increase this value in noisy
        /// environments where a lot of USB errors occur.</remarks>
        [DllImport("FTD2XX.dll")]
        public static extern FtStatus FT_SetResetPipeRetryCount(IntPtr ftHandle, UInt32 dwCount);

        /// <summary>
        /// Stops the driver's IN task.
        /// </summary>
        /// <param name="ftHandle">Handle of the device.</param>
        /// <returns>FtOk if successful, otherwise the return value is an FT error code.</returns>
        /// <remarks>This function is used to put the driver's IN task (read) into a wait state. It can be used in situations
        /// where data is being received continuously, so that the device can be purged without more data
        /// being received. It is used together with FT_RestartInTask which sets the IN task running again.</remarks>
        [DllImport("FTD2XX.dll")]
        public static extern FtStatus FT_StopInTask(IntPtr ftHandle);

        /// <summary>
        /// Restart the driver's IN task.
        /// </summary>
        /// <param name="ftHandle">Handle of the device.</param>
        /// <returns>FtOk if successful, otherwise the return value is an FT error code.</returns>
        [DllImport("FTD2XX.dll")]
        public static extern FtStatus FT_RestartInTask(IntPtr ftHandle);

        /// <summary>
        /// Send a reset command to the port.
        /// </summary>
        /// <param name="ftHandle">Handle of the device.</param>
        /// <returns>FtOk if successful, otherwise the return value is an FT error code.</returns>
        /// <remarks>This function is used to attempt to recover the port after a failure. It is not equivalent to an unplugreplug event.</remarks>
        [DllImport("FTD2XX.dll")]
        public static extern FtStatus FT_ResetPort(IntPtr ftHandle);

        /// <summary>
        /// Send a cycle command to the USB port.
        /// </summary>
        /// <param name="ftHandle">Handle of the device.</param>
        /// <returns>FtOk if successful, otherwise the return value is an FT error code.</returns>
        /// <remarks>The effect of this function is the same as disconnecting then reconnecting the device from USB.
        /// Possible use of this function is in situations where a fatal error has occurred and it is difficult, or not
        /// possible, to recover without unplugging and replugging the USB cable. This function can also be
        /// used after re-programming the EEPROM to force the FTDI device to read the new EEPROM
        /// contents which previously required a physical disconnect-reconnect.
        /// As the current session is not restored when the driver is reloaded, the application must be able to recover after calling this function.</remarks>
        [DllImport("FTD2XX.dll")]
        public static extern FtStatus FT_CyclePort(IntPtr ftHandle);

        /// <summary>
        /// This function builds a device information list and returns the number of D2XX devices connected to the system. The list contains information about both unopen and open devices.
        /// </summary>
        /// <param name="lpdwNumDevs">Pointer to unsigned long to store the number of devices connected.</param>
        /// <returns>FtOk if successful, otherwise the return value is an FT error code.</returns>
        /// <remarks>An application can use this function to get the number of devices attached to the system. It can
        /// then allocate space for the device information list and retrieve the list using
        /// FT_GetDeviceInfoList .If the devices connected to the system change, the device info list will not be updated until
        /// FT_CreateDeviceInfoList is called again.</remarks>
        [DllImport("FTD2XX.dll")]
        public static extern FtStatus FT_CreateDeviceInfoList(ref UInt32 lpdwNumDevs);

        /// <summary>
        /// 
        /// </summary>
        /// <param name="pDest"></param>
        /// <param name="lpdwNumDevs"></param>
        /// <returns></returns>
        [DllImport("FTD2XX.dll")]
        public static extern FtStatus FT_GetDeviceInfo(ref FT_DEVICE_LIST_INFO_NODE pDest, ref UInt32 lpdwNumDevs);

        /// <summary>
        /// This function returns an entry from the device information list.
        /// </summary>
        /// <param name="dwIndex">Index of the entry in the device info list.</param>
        /// <param name="lpdwFlags">Pointer to unsigned long to store the flag value.</param>
        /// <param name="lpdwType">Pointer to unsigned long to store device type.</param>
        /// <param name="lpdwID">Pointer to unsigned long to store device ID.</param>
        /// <param name="lpdwLocId">Pointer to unsigned long to store the device location ID.</param>
        /// <param name="pcSerialNumber">Pointer to buffer to store device serial number as a nullterminated String.</param>
        /// <param name="pcDescription">Pointer to buffer to store device description as a null-terminated String.</param>
        /// <param name="ftHandle">Pointer to a variable of type IntPtr where the handle will be stored.</param>
        /// <returns>FtOk if successful, otherwise the return value is an FT error code.</returns>
        /// <remarks>This function should only be called after calling FT_CreateDeviceInfoList . If the devices
        /// connected to the system change, the device info list will not be updated until
        /// FT_CreateDeviceInfoList is called again.The index value is zero-based.
        /// The flag value is a 4-byte bit map containing miscellaneous data. Bit 0 (least significant bit) of this 
        /// number indicates if the port is open (1) or closed (0). The remaining bits (1 - 31) are reserved at this time.
        /// Location ID information is not returned for devices that are open when FT_CreateDeviceInfoList is called.
        /// To return the whole device</remarks>
        [DllImport("FTD2XX.dll")]
        public static extern FtStatus FT_GetDeviceInfoDetail(UInt32 dwIndex, ref UInt32 lpdwFlags, ref FtDevice lpdwType,
                                                              ref UInt32 lpdwID, ref UInt32 lpdwLocId,
                                                              IntPtr pcSerialNumber, IntPtr pcDescription,
                                                              ref IntPtr ftHandle);
        /// <summary>
        /// This function returns the D2XX driver version number.
        /// </summary>
        /// <param name="ftHandle">Handle of the device.</param>
        /// <param name="lpdwDriverVersion">Pointer to the driver version number.</param>
        /// <returns>FtOk if successful, otherwise the return value is an FT error code.</returns>
        /// <remarks>A version number consists of major, minor and build version numbers contained in a 4-byte field
        /// (unsigned long). Byte0 (least significant) holds the build version, Byte1 holds the minor version,
        /// and Byte2 holds the major version. Byte3 is currently set to zero.
        /// For example, driver version "3.01.02" is represented as 0x00030102. Note that a device has to be
        /// opened before this function can be called.</remarks>
        [DllImport("FTD2XX.dll")]
        public static extern FtStatus FT_GetDriverVersion(IntPtr ftHandle, ref UInt32 lpdwDriverVersion);

        /// <summary>
        /// This function returns D2XX DLL version number.
        /// </summary>  
        /// <param name="lpdwDriverVersion">Pointer to the driver version number.</param>
        /// <returns>FtOk if successful, otherwise the return value is an FT error code.</returns>
        /// <remarks>A version number consists of major, minor and build version numbers contained in a 4-byte field
        /// (unsigned long). Byte0 (least significant) holds the build version, Byte1 holds the minor version,
        /// and Byte2 holds the major version. Byte3 is currently set to zero.
        /// For example, driver version "3.01.02" is represented as 0x00030102. Note that a device has to be
        /// opened before this function can be called.</remarks>
        [DllImport("FTD2XX.dll")]
        public static extern FtStatus FT_GetLibraryVersion(ref UInt32 lpdwDriverVersion);

        /// <summary>
        /// This function allows the maximum time in milliseconds that a USB request can remain outstanding to be set.
        /// </summary>
        /// <param name="ftHandle">Handle of the device.</param>
        /// <param name="dwDeadmanTimeout">Deadman timeout value in milliseconds. Default value is 5000.</param>
        /// <returns>FtOk if successful, otherwise the return value is an FT error code.</returns>
        /// <remarks>The Deadman timeout is referred to in AN232B-10 Advanced Driver Options as the USB timeout.
        /// It is unlikely that this function will be required by most users.</remarks>
        [DllImport("FTD2XX.dll")]
        public static extern FtStatus FT_SetDeadmanTimeout(IntPtr ftHandle, UInt32 dwDeadmanTimeout);

        /// <summary>
        /// This function allows the maximum time in milliseconds that a USB request can remain outstanding to be set.
        /// </summary>
        /// <returns>FtOk if successful, otherwise the return value is an FT error code.</returns>
        /// <remarks>Calling FT_Rescan is equivalent to clicking the "Scan for hardware changes" button in the Device Manager. 
        /// Only USB hardware is checked for new devices.</remarks>
        [DllImport("FTD2XX.dll")]
        public static extern FtStatus FT_Rescan();

        /// <summary>
        /// This function forces a reload of the driver for devices with a specific VID and PID combination.
        /// </summary>
        /// <param name="wVID">Vendor ID of the devices to reload the driver for.</param>
        /// <param name="wPID">Product ID of the devices to reload the driver for.</param>
        /// <returns>FtOk if successful, otherwise the return value is an FT error code.</returns>
        /// <remarks>Calling FT_Reload forces the operating system to unload and reload the driver for the specified
        /// device IDs. If the VID and PID parameters are null, the drivers for USB root hubs will be reloaded,
        /// causing all USB devices connected to reload their drivers. Please note that this function will not
        /// work correctly on 64-bit Windows when called from a 32-bit application.</remarks>
        [DllImport("FTD2XX.dll")]
        public static extern FtStatus FT_Reload(UInt16 wVID, UInt16 wPID);

        #endregion

        #region EEPROM Programming Interface Functions

        #endregion

        #region Extended API Functions

        /// <summary>
        /// Get the current value of the latency timer.
        /// </summary>
        /// <param name="ftHandle">Handle of the device.</param>
        /// <param name="pucTimer">Pointer to unsigned char to store latency timer value.</param>
        /// <returns>FtOk if successful, otherwise the return value is an FT error code.</returns>
        /// <remarks>In the FT8U232AM and FT8U245AM devices, the receive buffer timeout that is used to flush
        /// remaining data from the receive buffer was fixed at 16 ms. In all other FTDI devices, this timeout
        /// is programmable and can be set at 1 ms intervals between 2ms and 255 ms. This allows the
        /// device to be better optimized for protocols requiring faster response times from short data packets.</remarks>
        [DllImport("FTD2XX.dll")]
        public static extern FtStatus FT_GetLatencyTimer(IntPtr ftHandle, ref byte pucTimer);

        /// <summary>
        /// Set the latency timer.
        /// </summary>
        /// <param name="ftHandle">Handle of the device.</param>
        /// <param name="ucTimer">Required value, in milliseconds, of latency timer. Valid range is 2 - 255.</param>
        /// <returns></returns>
        /// <remarks>In the FT8U232AM and FT8U245AM devices, the receive buffer timeout that is used to flush
        /// remaining data from the receive buffer was fixed at 16 ms. In all other FTDI devices, this timeout
        /// is programmable and can be set at 1 ms intervals between 2ms and 255 ms. This allows the
        /// device to be better optimized for protocols requiring faster response times from short data packets.</remarks>
        [DllImport("FTD2XX.dll")]
        public static extern FtStatus FT_SetLatencyTimer(IntPtr ftHandle, byte ucTimer);

        /// <summary>
        /// Gets the instantaneous value of the data bus.
        /// </summary>
        /// <param name="ftHandle">Handle of the device.</param>
        /// <param name="pucMode">Pointer to unsigned char to store bit mode value.</param>
        /// <returns>FtOk if successful, otherwise the return value is an FT error code.</returns>
        /// <remarks>For a description of available bit modes for the FT2232C, see the application note "Bit Mode Functions for the FT2232C".
        /// For a description of Bit Bang Mode for the FT232BM and FT245BM, see the application note "FT232BM/FT245BM Bit Bang Mode".</remarks>
        [DllImport("FTD2XX.dll")]
        public static extern FtStatus FT_GetBitMode(IntPtr ftHandle, ref byte pucMode);

        /// <summary>
        /// Set the value of the bit mode.
        /// </summary>
        /// <param name="ftHandle">Handle of the device.</param>
        /// <param name="ucMask">inputs and outputs. A bit value of 0 sets the corresponding pin to an input, a bit value of 1 
        /// sets the corresponding pin to an output.
        /// In the case of CBUS Bit Bang, the upper nibble of this value controls which pins are inputs and outputs, while the lower
        /// nibble controls which of the outputs are high and low.</param>
        /// <param name="ucEnable">
        /// 0x0 = Reset
        /// 0x1 = Asynchronous Bit Bang
        /// 0x2 = MPSSE (FT2232C devices only)
        /// 0x4 = Synchronous Bit Bang (FT232R, FT245R and FT2232C devices only)
        /// 0x8 = MCU Host Bus Emulation Mode (FT2232C devices only)
        /// 0x10 = Fast Opto-Isolated Serial Mode (FT2232C devices only)
        /// 0x20 = CBUS Bit Bang Mode (FT232R devices only)</param>
        /// <returns>FtOk if successful, otherwise the return value is an FT error code.</returns>
        /// <remarks> For a description of Bit Bang Mode for the FT232BM and FT245BM, see the application note "FT232BM/FT245BM Bit Bang Mode".
        /// For a description of available bit modes for the FT2232C, see the application note "Bit Mode Functions for the FT2232C".
        /// For a description of the Bit Bang modes available for the FT232R and FT245R devices, see the application note "Bit Bang Modes for the FT232R and FT245R".
        /// </remarks>
        [DllImport("FTD2XX.dll")]
        public static extern FtStatus FT_SetBitMode(IntPtr ftHandle, byte ucMask, byte ucEnable);

        /// <summary>
        /// Set the USB request transfer size.
        /// </summary>
        /// <param name="ftHandle">Handle of the device.</param>
        /// <param name="dwInTransferSize">Transfer size for USB IN request.</param>
        /// <param name="dwOutTransferSize">Transfer size for USB OUT request.</param>
        /// <returns>FtOk if successful, otherwise the return value is an FT error code.</returns>
        /// <remarks>Previously, USB request transfer sizes have been set at 4096 bytes and have not been configurable.
        ///  This function can be used to change the transfer sizes to better suit the application requirements.
        /// Transfer sizes must be set to a multiple of 64 bytes between 64 bytes and 64k bytes.
        /// When FT_SetUSBParameters is called, the change comes into effect immediately and any data that was held in the driver at the time of the change is lost.
        /// Note that, at present, only dwInTransferSize is supported.</remarks>
        [DllImport("FTD2XX.dll")]
        public static extern FtStatus FT_SetUSBParameters(IntPtr ftHandle, UInt32 dwInTransferSize,
                                                                  UInt32 dwOutTransferSize);

        #endregion
    }
}