#region License
// Copyright  2004 - 2010 All Right Reserved, Derek Goslin
// 
// 1. Definitions
// The terms "reproduce," "reproduction," "derivative works," and "distribution" have the same meaning here as under U.S. copyright law.
// A "contribution" is the original software, or any additions or changes to the software.
// A "contributor" is any person that distributes its contribution under this license.
// "Licensed patents" are a contributor's patent claims that read directly on its contribution.
// 
// 2. Grant of Rights
// (A) Copyright Grant- Subject to the terms of this license, including the license
//     conditions and limitations in section 3, each contributor grants you a non-exclusive,
//     worldwide, royalty-free copyright license to reproduce its contribution, prepare derivative 
//     works of its contribution, and distribute its contribution or any derivative works that you create.
// (B) Patent Grant- Subject to the terms of this license, including the license conditions and limitations
//     in section 3, each contributor grants you a non-exclusive, worldwide, royalty-free license under its
//     licensed patents to make, have made, use, sell, offer for sale, import, and/or otherwise dispose of
//     its contribution in the software or derivative works of the contribution in the software.
// 
// 3. Conditions and Limitations
// (A) Reciprocal Grants- For any file you distribute that contains code from the software (in source code or binary format), 
//     you must provide recipients the source code to that file along with a copy of this license, which license will govern that file. 
//     You may license other files that are entirely your own work and do not contain code from the software under any terms you choose.
// (B) No Trademark License- This license does not grant you rights to use any contributors' name, logo, or trademarks.
// (C) If you bring a patent claim against any contributor over patents that you claim are infringed by the software,
//     your patent license from such contributor to the software ends automatically.
// (D) If you distribute any portion of the software, you must retain all copyright, patent, trademark,
//     and attribution notices that are present in the software.
// (E) If you distribute any portion of the software in source code form, you may do so only under this license by including a complete copy 
//     of this license with your distribution. If you distribute any portion of the software in compiled or object code form, you may only do 
//     so under a license that complies with this license.
// (F) The software is licensed "as-is." You bear the risk of using it. The contributors give no express warranties, guarantees or conditions.
//    You may have additional consumer rights under your local laws which this license cannot change. To the extent permitted under your
//    local laws, the contributors exclude the implied warranties of merchantability, fitness for a particular purpose and non-infringement.
#endregion

using System;
using System.Collections;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Reflection;
using System.Threading;
using System.Timers;
using System.Windows.Forms;
using DnEcuDiag.CustomControls;
using DnEcuDiag.Ecus.Enums;
using DnEcuDiag.Ecus.EventArgTypes;
using DnEcuDiag.Ecus.Exceptions;
using DnEcuDiag.Ecus.Generic;
using DnEcuDiag.VehiclePerformance;
using InstrumentationControls;
using ZedGraph;
using Timer = System.Timers.Timer;
using System.Globalization;
using DnEcuDiag.Factories;
using DnEcuDiag.Properties;
using System.IO;
using DnEcuDiag.Exceptions;

namespace DnEcuDiag.Forms
{
    /// <summary>
    /// Summary description for Form1.
    /// </summary>
    public class MainForm : Form
    {
        private string vehiclePerformanceFileTypeFilter = DnEcuDiag.WinFormStrings.VehiclePerformanceRunVprVpr;
        private string vehicleProfileFileTypeFilter = DnEcuDiag.WinFormStrings.VehicleProfilesVprofVprof;
        private Button btnAutoActuator;
        private Button btnCaptureFinalDriveRatio;
        private Button btnClearDTCs;
        private Button btnConnect;
        private Button btnDisconnect;
        private Button btnExecuteRoutine;
        private Button btnLoadPerfRun;
        private Button btnLoadVehicleProfile;
        private Button btnMeasureLoss;
        private Button btnReadDTCs;
        private Button btnReadSnapShot;
        private Button btnRoutineExecuteCancel;
        private Button btnSavePerfRun;
        private Button btnSaveVehicleProfile;
        private Button btnSensorFourCapture;
        private Button btnSensorOneCapture;
        private Button btnSensorThreeCapture;
        private Button btnSensorTwoCapture;
        private Button btnStartDynoRun;
        private Button btnStartPerformance;
        private Button btnStopDynoRun;
        private Button btnStopLoss;
        private Button btnStopPerformance;
        private ComboBoxExtended cboEcuList;
        private ComboBox cboGear;
        private ComboBoxExtended cboRoutines;
        private ComboBoxExtended cboSensorFour;
        private ComboBoxExtended cboSensorOne;
        private ComboBoxExtended cboSensorThree;
        private ComboBoxExtended cboSensorTwo;
        private ComboBoxExtended cboSnapShots;
        private CheckBox chkImperial;
        private CheckBox chkIncLoss;
        private IContainer components;
        private DigitalClock dCCPerf;
        private GroupBox gbConnect;
        private GroupBox gbDynoControl;
        private GroupBox gbDynoResults;
        private GroupBox gbDynoSetup;
        private GroupBox gbECU;
        private GroupBox gbFaultCodeList;
        private GroupBox gbFaultCodes;
        private GroupBox gbLoss;
        private GroupBox gbPerformance;
        private GroupBox gbPerformanceRunData;
        private GroupBox gbPerformanceRunStore;
        private GroupBox gbSensorFour;
        private GroupBox gbSensorOne;
        private GroupBox gbSensorThree;
        private GroupBox gbSensorTwo;
        private GroupBox gbSnapShotDescription;
        private GroupBox gbSnapshots;
        private GroupBox gbVehicleProfile;
        private GroupBox grpFaultCodeDescription;
        private ImageList imgLstButtons;
        private ImageList imgLstActuators;
        private ImageList imgLstStatus;
        private ImageList imgLstToolbar;
        private Label label10;
        private Label label11;
        private Label label12;
        private Label label13;
        private Label label14;
        private Label label15;
        private Label label16;
        private Label label17;
        private Label label18;
        private Label label19;
        private Label label2;
        private Label label20;
        private Label label4;
        private Label label5;
        private Label label6;
        private Label label7;
        private Label label8;
        private Label label9;
        private Label lblCurrOne;
        private Label lblCurrThree;
        private Label lblCurrTwo;
        private Label lblDate;
        private Label lblDragLoss;
        private Label lblDrawingNumber;
        private Label lblEcuHWNumber;
        private Label lblECUModel;
        private Label lblFinalDrive;
        private Label lblGear;
        private Label lblHWVerNo;
        private Label lblISOCode;
        private Label lblOmoCode;
        private Label lblProgDate;
        private Label lblRoutine;
        private Label lblRpmStop;
        private Label lblSensorFourCurrent;
        private Label lblSensorFourMax;
        private Label lblSensorFourMaxValue;
        private Label lblSensorFourMin;
        private Label lblSensorFourMinValue;
        private Label lblSensorOneCurrent;
        private Label lblSensorOneMax;
        private Label lblSensorOneMaxValue;
        private Label lblSensorOneMin;
        private Label lblSensorOneMinValue;
        private Label lblSensorThreeCurrent;
        private Label lblSensorThreeMax;
        private Label lblSensorThreeMaxValue;
        private Label lblSensorThreeMin;
        private Label lblSensorThreeMinValue;
        private Label lblSensorTwoCurrent;
        private Label lblSensorTwoMax;
        private Label lblSensorTwoMaxValue;
        private Label lblSensorTwoMin;
        private Label lblSensorTwoMinValue;
        private Label lblSixty;
        private Label lblStartRpm;
        private Label lblSWNo;
        private Label lblSWVerNo;
        private Label lblVehicle;
        private Label lblVehicleProfile;
        private Label lblWeghtMeasure;
        private Label lblWeight;
        private ListBox lstBoxFaultCodes;
        private ListView lvActuators;
        private MenuItem menuItem6;
        private MenuItem menuItemHelp;
        private MenuItem menuItemInterface;
        private MenuItem mnuItemAbout;
        private MenuItem mnuItemApplication;
        private MenuItem mnuItemConfigure;
        private MenuItem mnuItemConnect;
        private MenuItem mnuItemDisconnect;
        private MenuItem mnuItemEcu;
        private MenuItem mnuItemEnableLogging;
        private MenuItem mnuItemHelp;
        private MenuItem mnuItemPlotAll;
        private MenuItem mnuItemSensorGroup;
        private MainMenu mnuMain;
        private MenuItem mnuSensors;
        private NumericUpDown nudRpmStart;
        private NumericUpDown nudRpmStop;
        private NumericUpDown nudVehicleWeight;
        private OpenFileDialog openFileDialog;
        private SingleChannelScope oScopeFour;
        private SingleChannelScope oScopeOne;
        private SingleChannelScope oScopeThree;
        private SingleChannelScope oScopeTwo;
        private ProgressBar pbActuator;
        private ProgressBar pbCalibration;
        private ProgressBar pbDyno;
        private SaveFileDialog saveFileDialog;
        private StatusBarPanel statusBarPanelApp;
        private StatusBarPanel statusBarPanelConnect;
        private StatusBarPanel statusBarPanelLog;
        private StatusBarPanel statusBarPanelTab;
        private StatusBar stsbarMain;
        private TabPage tabActuatorControl;
        private TabPage tabActuators;
        private TabPage tabConnect;
        private TabPage tabDyno;
        private TabPage tabFaultCodes;
        private TabPage tabIdentification;
        private TabControlExtended tabMain;
        private TabPage tabPerformance;
        private TabPage tabRoutines;
        private TabPage tabSensors;
        private TabControlExtended tcActuators;
        private Timer tmrProgress;
        private ToolTip toolTips;
        private TextBox txtActuatorTestStat;
        private TextBox txtDLoss;
        private TextBox txtDrawingNo;
        private TextBox txtDynoVehicle;
        private TextBox txtEcuHwNo;
        private TextBox txtEcuHWVer;
        private TextBox txtECUModel;
        private MaskedTextBox txtEightMileSpeed;
        private MaskedTextBox txtEightMileTime;
        private TextBox txtFaultCodeDescription;
        private TextBox txtFinalDriveRatio;
        private TextBox txtFLoss;
        private MaskedTextBox txtFourHundredMtrSpeed;
        private MaskedTextBox txtFourHundredMtrTime;
        private TextBox txtISOCode;
        private MaskedTextBox txtNoughtOneHundredkphTime;
        private MaskedTextBox txtNoughtSixtyMphTime;
        private TextBox txtOmolCode;
        private MaskedTextBox txtOneHundredMtrTime;
        private TextBox txtPerformanceNotes;
        private TextBox txtPerformanceVehicle;
        private TextBox txtProgDate;
        private MaskedTextBox txtQuarterMileSpeed;
        private MaskedTextBox txtQuarterMileTime;
        private TextBox txtRoutineExecution;
        private TextBox txtRoutineInstructions;
        private MaskedTextBox txtSixtyFtTime;
        private TextBox txtSnapShotText;
        private TextBox txtSwNo;
        private TextBox txtSwVerNo;
        private MaskedTextBox txtThousandFtTime;
        private MaskedTextBox txtThreeHundredMtrTime;
        private MaskedTextBox txtThreeThirtyFtTime;
        private MaskedTextBox txtTwentyMtrTime;
        private MaskedTextBox txtTwoHundredMtrSpeed;
        private MaskedTextBox txtTwoHundredMtrTime;
        private BindingSource ecuDescriptorsBindingSource;
        private BindingSource connectedEcuBindingSource;
        private BindingSource sensorOneBindingSource;
        private BindingSource routinesBindingSource;
        private BindingSource snapshotsBindingSource;
        private BindingSource faultCodesBindingSource;
        private BindingSource vehiclePerformanceRunBindingSource;
        private BindingSource sensorTwoBindingSource;
        private BindingSource sensorThreeBindingSource;
        private BindingSource sensorFourBindingSource;
        private BindingSource vehicleProfileBindingSource;
        private MenuItem menuItemSensorSort;
        private MenuItem menuItemAscending;
        private MenuItem menuItemDescending;
        private MenuItem menuItemNone;
        private BindingSource modelBindingSource;
        private ZedGraphControl zedGraphDyno;

        /// <summary>
        /// Main form constructor.
        /// </summary>
        public MainForm()
        {
            //
            // Required for Windows Form Designer support
            //
            InitializeComponent();

            ecuDescriptorsBindingSource.DataSource = EcuFactory.EcuList;
            vehiclePerformanceRunBindingSource.DataSource = Model.Instance.VehiclePerformance.PerformanceRun;
            vehicleProfileBindingSource.DataSource = Model.Instance.DynoVehicleProfile;
            modelBindingSource.DataSource = Model.Instance;
            
            splashScreenHandler = MainForm_Activated;
            Activated += splashScreenHandler;

            Thread.CurrentThread.Name = "ui";

            ConfigureDynoGraph();

        }

        /// <summary>
        /// Clean up any resources being used.
        /// </summary>
        protected override void Dispose(bool disposing)
        {
            if (disposing)
            {
                if (components != null)
                {
                    components.Dispose();
                }
            }
            base.Dispose(disposing);
        }

        #region Windows Form Designer generated code

        /// <summary>
        /// Required method for Designer support - do not modify
        /// the contents of this method with the code editor.
        /// </summary>
        private void InitializeComponent()
        {
            this.components = new System.ComponentModel.Container();
            System.ComponentModel.ComponentResourceManager resources = new System.ComponentModel.ComponentResourceManager(typeof(MainForm));
            this.mnuMain = new System.Windows.Forms.MainMenu(this.components);
            this.mnuItemEcu = new System.Windows.Forms.MenuItem();
            this.mnuItemConnect = new System.Windows.Forms.MenuItem();
            this.mnuItemDisconnect = new System.Windows.Forms.MenuItem();
            this.mnuItemConfigure = new System.Windows.Forms.MenuItem();
            this.mnuItemApplication = new System.Windows.Forms.MenuItem();
            this.menuItemInterface = new System.Windows.Forms.MenuItem();
            this.mnuSensors = new System.Windows.Forms.MenuItem();
            this.mnuItemPlotAll = new System.Windows.Forms.MenuItem();
            this.mnuItemEnableLogging = new System.Windows.Forms.MenuItem();
            this.menuItem6 = new System.Windows.Forms.MenuItem();
            this.mnuItemSensorGroup = new System.Windows.Forms.MenuItem();
            this.menuItemSensorSort = new System.Windows.Forms.MenuItem();
            this.menuItemAscending = new System.Windows.Forms.MenuItem();
            this.menuItemDescending = new System.Windows.Forms.MenuItem();
            this.menuItemNone = new System.Windows.Forms.MenuItem();
            this.menuItemHelp = new System.Windows.Forms.MenuItem();
            this.mnuItemAbout = new System.Windows.Forms.MenuItem();
            this.mnuItemHelp = new System.Windows.Forms.MenuItem();
            this.imgLstToolbar = new System.Windows.Forms.ImageList(this.components);
            this.imgLstButtons = new System.Windows.Forms.ImageList(this.components);
            this.imgLstActuators = new System.Windows.Forms.ImageList(this.components);
            this.tmrProgress = new System.Timers.Timer();
            this.toolTips = new System.Windows.Forms.ToolTip(this.components);
            this.btnDisconnect = new System.Windows.Forms.Button();
            this.btnConnect = new System.Windows.Forms.Button();
            this.cboEcuList = new DnEcuDiag.CustomControls.ComboBoxExtended(this.components);
            this.modelBindingSource = new System.Windows.Forms.BindingSource(this.components);
            this.ecuDescriptorsBindingSource = new System.Windows.Forms.BindingSource(this.components);
            this.tabIdentification = new System.Windows.Forms.TabPage();
            this.gbECU = new System.Windows.Forms.GroupBox();
            this.lblDate = new System.Windows.Forms.Label();
            this.txtProgDate = new System.Windows.Forms.TextBox();
            this.connectedEcuBindingSource = new System.Windows.Forms.BindingSource(this.components);
            this.txtISOCode = new System.Windows.Forms.TextBox();
            this.txtECUModel = new System.Windows.Forms.TextBox();
            this.lblECUModel = new System.Windows.Forms.Label();
            this.txtOmolCode = new System.Windows.Forms.TextBox();
            this.txtSwVerNo = new System.Windows.Forms.TextBox();
            this.txtSwNo = new System.Windows.Forms.TextBox();
            this.txtEcuHWVer = new System.Windows.Forms.TextBox();
            this.txtEcuHwNo = new System.Windows.Forms.TextBox();
            this.txtDrawingNo = new System.Windows.Forms.TextBox();
            this.lblProgDate = new System.Windows.Forms.Label();
            this.lblISOCode = new System.Windows.Forms.Label();
            this.lblOmoCode = new System.Windows.Forms.Label();
            this.lblSWVerNo = new System.Windows.Forms.Label();
            this.lblSWNo = new System.Windows.Forms.Label();
            this.lblHWVerNo = new System.Windows.Forms.Label();
            this.lblEcuHWNumber = new System.Windows.Forms.Label();
            this.lblDrawingNumber = new System.Windows.Forms.Label();
            this.tabSensors = new System.Windows.Forms.TabPage();
            this.gbSensorFour = new System.Windows.Forms.GroupBox();
            this.lblSensorFourCurrent = new System.Windows.Forms.Label();
            this.label8 = new System.Windows.Forms.Label();
            this.lblSensorFourMinValue = new System.Windows.Forms.Label();
            this.lblSensorFourMaxValue = new System.Windows.Forms.Label();
            this.lblSensorFourMin = new System.Windows.Forms.Label();
            this.lblSensorFourMax = new System.Windows.Forms.Label();
            this.btnSensorFourCapture = new System.Windows.Forms.Button();
            this.cboSensorFour = new DnEcuDiag.CustomControls.ComboBoxExtended(this.components);
            this.sensorFourBindingSource = new System.Windows.Forms.BindingSource(this.components);
            this.gbSensorThree = new System.Windows.Forms.GroupBox();
            this.lblSensorThreeCurrent = new System.Windows.Forms.Label();
            this.lblCurrThree = new System.Windows.Forms.Label();
            this.lblSensorThreeMinValue = new System.Windows.Forms.Label();
            this.lblSensorThreeMaxValue = new System.Windows.Forms.Label();
            this.lblSensorThreeMin = new System.Windows.Forms.Label();
            this.lblSensorThreeMax = new System.Windows.Forms.Label();
            this.btnSensorThreeCapture = new System.Windows.Forms.Button();
            this.cboSensorThree = new DnEcuDiag.CustomControls.ComboBoxExtended(this.components);
            this.sensorThreeBindingSource = new System.Windows.Forms.BindingSource(this.components);
            this.gbSensorTwo = new System.Windows.Forms.GroupBox();
            this.lblSensorTwoCurrent = new System.Windows.Forms.Label();
            this.lblCurrTwo = new System.Windows.Forms.Label();
            this.lblSensorTwoMinValue = new System.Windows.Forms.Label();
            this.lblSensorTwoMaxValue = new System.Windows.Forms.Label();
            this.lblSensorTwoMin = new System.Windows.Forms.Label();
            this.lblSensorTwoMax = new System.Windows.Forms.Label();
            this.btnSensorTwoCapture = new System.Windows.Forms.Button();
            this.cboSensorTwo = new DnEcuDiag.CustomControls.ComboBoxExtended(this.components);
            this.sensorTwoBindingSource = new System.Windows.Forms.BindingSource(this.components);
            this.gbSensorOne = new System.Windows.Forms.GroupBox();
            this.lblSensorOneCurrent = new System.Windows.Forms.Label();
            this.lblCurrOne = new System.Windows.Forms.Label();
            this.lblSensorOneMinValue = new System.Windows.Forms.Label();
            this.lblSensorOneMaxValue = new System.Windows.Forms.Label();
            this.lblSensorOneMin = new System.Windows.Forms.Label();
            this.lblSensorOneMax = new System.Windows.Forms.Label();
            this.btnSensorOneCapture = new System.Windows.Forms.Button();
            this.cboSensorOne = new DnEcuDiag.CustomControls.ComboBoxExtended(this.components);
            this.sensorOneBindingSource = new System.Windows.Forms.BindingSource(this.components);
            this.oScopeFour = new InstrumentationControls.SingleChannelScope();
            this.oScopeThree = new InstrumentationControls.SingleChannelScope();
            this.oScopeTwo = new InstrumentationControls.SingleChannelScope();
            this.oScopeOne = new InstrumentationControls.SingleChannelScope();
            this.tabActuators = new System.Windows.Forms.TabPage();
            this.tcActuators = new DnEcuDiag.CustomControls.TabControlExtended();
            this.tabActuatorControl = new System.Windows.Forms.TabPage();
            this.lvActuators = new System.Windows.Forms.ListView();
            this.txtActuatorTestStat = new System.Windows.Forms.TextBox();
            this.btnAutoActuator = new System.Windows.Forms.Button();
            this.pbActuator = new System.Windows.Forms.ProgressBar();
            this.tabRoutines = new System.Windows.Forms.TabPage();
            this.btnRoutineExecuteCancel = new System.Windows.Forms.Button();
            this.btnExecuteRoutine = new System.Windows.Forms.Button();
            this.txtRoutineInstructions = new System.Windows.Forms.TextBox();
            this.cboRoutines = new DnEcuDiag.CustomControls.ComboBoxExtended(this.components);
            this.routinesBindingSource = new System.Windows.Forms.BindingSource(this.components);
            this.lblRoutine = new System.Windows.Forms.Label();
            this.pbCalibration = new System.Windows.Forms.ProgressBar();
            this.txtRoutineExecution = new System.Windows.Forms.TextBox();
            this.tabFaultCodes = new System.Windows.Forms.TabPage();
            this.gbSnapshots = new System.Windows.Forms.GroupBox();
            this.cboSnapShots = new DnEcuDiag.CustomControls.ComboBoxExtended(this.components);
            this.snapshotsBindingSource = new System.Windows.Forms.BindingSource(this.components);
            this.gbSnapShotDescription = new System.Windows.Forms.GroupBox();
            this.txtSnapShotText = new System.Windows.Forms.TextBox();
            this.btnReadSnapShot = new System.Windows.Forms.Button();
            this.gbFaultCodes = new System.Windows.Forms.GroupBox();
            this.grpFaultCodeDescription = new System.Windows.Forms.GroupBox();
            this.txtFaultCodeDescription = new System.Windows.Forms.TextBox();
            this.faultCodesBindingSource = new System.Windows.Forms.BindingSource(this.components);
            this.gbFaultCodeList = new System.Windows.Forms.GroupBox();
            this.lstBoxFaultCodes = new System.Windows.Forms.ListBox();
            this.btnClearDTCs = new System.Windows.Forms.Button();
            this.btnReadDTCs = new System.Windows.Forms.Button();
            this.tabPerformance = new System.Windows.Forms.TabPage();
            this.gbPerformance = new System.Windows.Forms.GroupBox();
            this.gbPerformanceRunStore = new System.Windows.Forms.GroupBox();
            this.lblVehicle = new System.Windows.Forms.Label();
            this.btnLoadPerfRun = new System.Windows.Forms.Button();
            this.btnSavePerfRun = new System.Windows.Forms.Button();
            this.btnStopPerformance = new System.Windows.Forms.Button();
            this.btnStartPerformance = new System.Windows.Forms.Button();
            this.txtPerformanceNotes = new System.Windows.Forms.TextBox();
            this.vehiclePerformanceRunBindingSource = new System.Windows.Forms.BindingSource(this.components);
            this.label2 = new System.Windows.Forms.Label();
            this.txtPerformanceVehicle = new System.Windows.Forms.TextBox();
            this.gbPerformanceRunData = new System.Windows.Forms.GroupBox();
            this.dCCPerf = new InstrumentationControls.DigitalClock();
            this.txtFourHundredMtrSpeed = new System.Windows.Forms.MaskedTextBox();
            this.txtFourHundredMtrTime = new System.Windows.Forms.MaskedTextBox();
            this.txtThreeHundredMtrTime = new System.Windows.Forms.MaskedTextBox();
            this.txtTwoHundredMtrSpeed = new System.Windows.Forms.MaskedTextBox();
            this.txtTwoHundredMtrTime = new System.Windows.Forms.MaskedTextBox();
            this.txtOneHundredMtrTime = new System.Windows.Forms.MaskedTextBox();
            this.txtNoughtOneHundredkphTime = new System.Windows.Forms.MaskedTextBox();
            this.txtTwentyMtrTime = new System.Windows.Forms.MaskedTextBox();
            this.label12 = new System.Windows.Forms.Label();
            this.label13 = new System.Windows.Forms.Label();
            this.label14 = new System.Windows.Forms.Label();
            this.label15 = new System.Windows.Forms.Label();
            this.label16 = new System.Windows.Forms.Label();
            this.label17 = new System.Windows.Forms.Label();
            this.label18 = new System.Windows.Forms.Label();
            this.label19 = new System.Windows.Forms.Label();
            this.txtQuarterMileSpeed = new System.Windows.Forms.MaskedTextBox();
            this.txtQuarterMileTime = new System.Windows.Forms.MaskedTextBox();
            this.txtThousandFtTime = new System.Windows.Forms.MaskedTextBox();
            this.txtEightMileSpeed = new System.Windows.Forms.MaskedTextBox();
            this.txtEightMileTime = new System.Windows.Forms.MaskedTextBox();
            this.txtThreeThirtyFtTime = new System.Windows.Forms.MaskedTextBox();
            this.txtNoughtSixtyMphTime = new System.Windows.Forms.MaskedTextBox();
            this.txtSixtyFtTime = new System.Windows.Forms.MaskedTextBox();
            this.label10 = new System.Windows.Forms.Label();
            this.label11 = new System.Windows.Forms.Label();
            this.label4 = new System.Windows.Forms.Label();
            this.label9 = new System.Windows.Forms.Label();
            this.label7 = new System.Windows.Forms.Label();
            this.label6 = new System.Windows.Forms.Label();
            this.label5 = new System.Windows.Forms.Label();
            this.lblSixty = new System.Windows.Forms.Label();
            this.tabDyno = new System.Windows.Forms.TabPage();
            this.gbDynoSetup = new System.Windows.Forms.GroupBox();
            this.nudRpmStart = new System.Windows.Forms.NumericUpDown();
            this.nudRpmStop = new System.Windows.Forms.NumericUpDown();
            this.lblRpmStop = new System.Windows.Forms.Label();
            this.lblStartRpm = new System.Windows.Forms.Label();
            this.gbDynoResults = new System.Windows.Forms.GroupBox();
            this.pbDyno = new System.Windows.Forms.ProgressBar();
            this.zedGraphDyno = new ZedGraph.ZedGraphControl();
            this.gbVehicleProfile = new System.Windows.Forms.GroupBox();
            this.lblWeghtMeasure = new System.Windows.Forms.Label();
            this.nudVehicleWeight = new System.Windows.Forms.NumericUpDown();
            this.vehicleProfileBindingSource = new System.Windows.Forms.BindingSource(this.components);
            this.btnCaptureFinalDriveRatio = new System.Windows.Forms.Button();
            this.txtFinalDriveRatio = new System.Windows.Forms.TextBox();
            this.lblFinalDrive = new System.Windows.Forms.Label();
            this.lblWeight = new System.Windows.Forms.Label();
            this.cboGear = new System.Windows.Forms.ComboBox();
            this.lblGear = new System.Windows.Forms.Label();
            this.lblVehicleProfile = new System.Windows.Forms.Label();
            this.txtDynoVehicle = new System.Windows.Forms.TextBox();
            this.btnSaveVehicleProfile = new System.Windows.Forms.Button();
            this.btnLoadVehicleProfile = new System.Windows.Forms.Button();
            this.gbDynoControl = new System.Windows.Forms.GroupBox();
            this.chkIncLoss = new System.Windows.Forms.CheckBox();
            this.chkImperial = new System.Windows.Forms.CheckBox();
            this.gbLoss = new System.Windows.Forms.GroupBox();
            this.btnStopLoss = new System.Windows.Forms.Button();
            this.txtFLoss = new System.Windows.Forms.TextBox();
            this.txtDLoss = new System.Windows.Forms.TextBox();
            this.label20 = new System.Windows.Forms.Label();
            this.btnMeasureLoss = new System.Windows.Forms.Button();
            this.lblDragLoss = new System.Windows.Forms.Label();
            this.btnStopDynoRun = new System.Windows.Forms.Button();
            this.btnStartDynoRun = new System.Windows.Forms.Button();
            this.stsbarMain = new System.Windows.Forms.StatusBar();
            this.statusBarPanelConnect = new System.Windows.Forms.StatusBarPanel();
            this.statusBarPanelLog = new System.Windows.Forms.StatusBarPanel();
            this.statusBarPanelApp = new System.Windows.Forms.StatusBarPanel();
            this.statusBarPanelTab = new System.Windows.Forms.StatusBarPanel();
            this.tabMain = new DnEcuDiag.CustomControls.TabControlExtended();
            this.tabConnect = new System.Windows.Forms.TabPage();
            this.gbConnect = new System.Windows.Forms.GroupBox();
            this.openFileDialog = new System.Windows.Forms.OpenFileDialog();
            this.saveFileDialog = new System.Windows.Forms.SaveFileDialog();
            this.imgLstStatus = new System.Windows.Forms.ImageList(this.components);
            ((System.ComponentModel.ISupportInitialize)(this.tmrProgress)).BeginInit();
            ((System.ComponentModel.ISupportInitialize)(this.modelBindingSource)).BeginInit();
            ((System.ComponentModel.ISupportInitialize)(this.ecuDescriptorsBindingSource)).BeginInit();
            this.tabIdentification.SuspendLayout();
            this.gbECU.SuspendLayout();
            ((System.ComponentModel.ISupportInitialize)(this.connectedEcuBindingSource)).BeginInit();
            this.tabSensors.SuspendLayout();
            this.gbSensorFour.SuspendLayout();
            ((System.ComponentModel.ISupportInitialize)(this.sensorFourBindingSource)).BeginInit();
            this.gbSensorThree.SuspendLayout();
            ((System.ComponentModel.ISupportInitialize)(this.sensorThreeBindingSource)).BeginInit();
            this.gbSensorTwo.SuspendLayout();
            ((System.ComponentModel.ISupportInitialize)(this.sensorTwoBindingSource)).BeginInit();
            this.gbSensorOne.SuspendLayout();
            ((System.ComponentModel.ISupportInitialize)(this.sensorOneBindingSource)).BeginInit();
            this.tabActuators.SuspendLayout();
            this.tcActuators.SuspendLayout();
            this.tabActuatorControl.SuspendLayout();
            this.tabRoutines.SuspendLayout();
            ((System.ComponentModel.ISupportInitialize)(this.routinesBindingSource)).BeginInit();
            this.tabFaultCodes.SuspendLayout();
            this.gbSnapshots.SuspendLayout();
            ((System.ComponentModel.ISupportInitialize)(this.snapshotsBindingSource)).BeginInit();
            this.gbSnapShotDescription.SuspendLayout();
            this.gbFaultCodes.SuspendLayout();
            this.grpFaultCodeDescription.SuspendLayout();
            ((System.ComponentModel.ISupportInitialize)(this.faultCodesBindingSource)).BeginInit();
            this.gbFaultCodeList.SuspendLayout();
            this.tabPerformance.SuspendLayout();
            this.gbPerformance.SuspendLayout();
            this.gbPerformanceRunStore.SuspendLayout();
            ((System.ComponentModel.ISupportInitialize)(this.vehiclePerformanceRunBindingSource)).BeginInit();
            this.gbPerformanceRunData.SuspendLayout();
            this.tabDyno.SuspendLayout();
            this.gbDynoSetup.SuspendLayout();
            ((System.ComponentModel.ISupportInitialize)(this.nudRpmStart)).BeginInit();
            ((System.ComponentModel.ISupportInitialize)(this.nudRpmStop)).BeginInit();
            this.gbDynoResults.SuspendLayout();
            this.gbVehicleProfile.SuspendLayout();
            ((System.ComponentModel.ISupportInitialize)(this.nudVehicleWeight)).BeginInit();
            ((System.ComponentModel.ISupportInitialize)(this.vehicleProfileBindingSource)).BeginInit();
            this.gbDynoControl.SuspendLayout();
            this.gbLoss.SuspendLayout();
            ((System.ComponentModel.ISupportInitialize)(this.statusBarPanelConnect)).BeginInit();
            ((System.ComponentModel.ISupportInitialize)(this.statusBarPanelLog)).BeginInit();
            ((System.ComponentModel.ISupportInitialize)(this.statusBarPanelApp)).BeginInit();
            ((System.ComponentModel.ISupportInitialize)(this.statusBarPanelTab)).BeginInit();
            this.tabMain.SuspendLayout();
            this.tabConnect.SuspendLayout();
            this.gbConnect.SuspendLayout();
            this.SuspendLayout();
            // 
            // mnuMain
            // 
            this.mnuMain.MenuItems.AddRange(new System.Windows.Forms.MenuItem[] {
            this.mnuItemEcu,
            this.mnuItemConfigure,
            this.mnuSensors,
            this.menuItemHelp});
            resources.ApplyResources(this.mnuMain, "mnuMain");
            // 
            // mnuItemEcu
            // 
            resources.ApplyResources(this.mnuItemEcu, "mnuItemEcu");
            this.mnuItemEcu.Index = 0;
            this.mnuItemEcu.MenuItems.AddRange(new System.Windows.Forms.MenuItem[] {
            this.mnuItemConnect,
            this.mnuItemDisconnect});
            // 
            // mnuItemConnect
            // 
            resources.ApplyResources(this.mnuItemConnect, "mnuItemConnect");
            this.mnuItemConnect.Index = 0;
            this.mnuItemConnect.Click += new System.EventHandler(this.mnuItemConnect_Click);
            // 
            // mnuItemDisconnect
            // 
            resources.ApplyResources(this.mnuItemDisconnect, "mnuItemDisconnect");
            this.mnuItemDisconnect.Index = 1;
            this.mnuItemDisconnect.Click += new System.EventHandler(this.mnuItemDisconnect_Click);
            // 
            // mnuItemConfigure
            // 
            resources.ApplyResources(this.mnuItemConfigure, "mnuItemConfigure");
            this.mnuItemConfigure.Index = 1;
            this.mnuItemConfigure.MenuItems.AddRange(new System.Windows.Forms.MenuItem[] {
            this.mnuItemApplication,
            this.menuItemInterface});
            // 
            // mnuItemApplication
            // 
            resources.ApplyResources(this.mnuItemApplication, "mnuItemApplication");
            this.mnuItemApplication.Index = 0;
            this.mnuItemApplication.Click += new System.EventHandler(this.mnuItemApplication_Click);
            // 
            // menuItemInterface
            // 
            resources.ApplyResources(this.menuItemInterface, "menuItemInterface");
            this.menuItemInterface.Index = 1;
            this.menuItemInterface.Click += new System.EventHandler(this.menuItemConfigure_Click);
            // 
            // mnuSensors
            // 
            resources.ApplyResources(this.mnuSensors, "mnuSensors");
            this.mnuSensors.Index = 2;
            this.mnuSensors.MenuItems.AddRange(new System.Windows.Forms.MenuItem[] {
            this.mnuItemPlotAll,
            this.mnuItemEnableLogging,
            this.menuItem6,
            this.mnuItemSensorGroup,
            this.menuItemSensorSort});
            // 
            // mnuItemPlotAll
            // 
            resources.ApplyResources(this.mnuItemPlotAll, "mnuItemPlotAll");
            this.mnuItemPlotAll.Index = 0;
            this.mnuItemPlotAll.Click += new System.EventHandler(this.mnuItemPlotAll_Click);
            // 
            // mnuItemEnableLogging
            // 
            resources.ApplyResources(this.mnuItemEnableLogging, "mnuItemEnableLogging");
            this.mnuItemEnableLogging.Index = 1;
            this.mnuItemEnableLogging.Click += new System.EventHandler(this.mnuItemEnableLogging_Click);
            // 
            // menuItem6
            // 
            resources.ApplyResources(this.menuItem6, "menuItem6");
            this.menuItem6.Index = 2;
            // 
            // mnuItemSensorGroup
            // 
            resources.ApplyResources(this.mnuItemSensorGroup, "mnuItemSensorGroup");
            this.mnuItemSensorGroup.Index = 3;
            this.mnuItemSensorGroup.Click += new System.EventHandler(this.mnuItemSensorGroup_Click);
            // 
            // menuItemSensorSort
            // 
            resources.ApplyResources(this.menuItemSensorSort, "menuItemSensorSort");
            this.menuItemSensorSort.Index = 4;
            this.menuItemSensorSort.MenuItems.AddRange(new System.Windows.Forms.MenuItem[] {
            this.menuItemAscending,
            this.menuItemDescending,
            this.menuItemNone});
            // 
            // menuItemAscending
            // 
            this.menuItemAscending.Checked = global::DnEcuDiag.Properties.Settings.Default.SortSensorsAscending;
            resources.ApplyResources(this.menuItemAscending, "menuItemAscending");
            this.menuItemAscending.Index = 0;
            this.menuItemAscending.Click += new System.EventHandler(this.menuItemAscending_Click);
            // 
            // menuItemDescending
            // 
            this.menuItemDescending.Checked = global::DnEcuDiag.Properties.Settings.Default.SortSensorsDescending;
            resources.ApplyResources(this.menuItemDescending, "menuItemDescending");
            this.menuItemDescending.Index = 1;
            this.menuItemDescending.Click += new System.EventHandler(this.menuItemDescending_Click);
            // 
            // menuItemNone
            // 
            this.menuItemNone.Checked = global::DnEcuDiag.Properties.Settings.Default.SortSensorsNone;
            resources.ApplyResources(this.menuItemNone, "menuItemNone");
            this.menuItemNone.Index = 2;
            this.menuItemNone.Click += new System.EventHandler(this.menuItemNone_Click);
            // 
            // menuItemHelp
            // 
            resources.ApplyResources(this.menuItemHelp, "menuItemHelp");
            this.menuItemHelp.Index = 3;
            this.menuItemHelp.MenuItems.AddRange(new System.Windows.Forms.MenuItem[] {
            this.mnuItemAbout,
            this.mnuItemHelp});
            // 
            // mnuItemAbout
            // 
            resources.ApplyResources(this.mnuItemAbout, "mnuItemAbout");
            this.mnuItemAbout.Index = 0;
            this.mnuItemAbout.Click += new System.EventHandler(this.mnuItemAbout_Click);
            // 
            // mnuItemHelp
            // 
            resources.ApplyResources(this.mnuItemHelp, "mnuItemHelp");
            this.mnuItemHelp.Index = 1;
            this.mnuItemHelp.Click += new System.EventHandler(this.mnuItemHelp_Click);
            // 
            // imgLstToolbar
            // 
            this.imgLstToolbar.ImageStream = ((System.Windows.Forms.ImageListStreamer)(resources.GetObject("imgLstToolbar.ImageStream")));
            this.imgLstToolbar.TransparentColor = System.Drawing.Color.Transparent;
            this.imgLstToolbar.Images.SetKeyName(0, "");
            this.imgLstToolbar.Images.SetKeyName(1, "");
            this.imgLstToolbar.Images.SetKeyName(2, "");
            this.imgLstToolbar.Images.SetKeyName(3, "");
            this.imgLstToolbar.Images.SetKeyName(4, "");
            this.imgLstToolbar.Images.SetKeyName(5, "");
            this.imgLstToolbar.Images.SetKeyName(6, "");
            this.imgLstToolbar.Images.SetKeyName(7, "");
            this.imgLstToolbar.Images.SetKeyName(8, "");
            // 
            // imgLstButtons
            // 
            this.imgLstButtons.ImageStream = ((System.Windows.Forms.ImageListStreamer)(resources.GetObject("imgLstButtons.ImageStream")));
            this.imgLstButtons.TransparentColor = System.Drawing.Color.White;
            this.imgLstButtons.Images.SetKeyName(0, "");
            this.imgLstButtons.Images.SetKeyName(1, "");
            this.imgLstButtons.Images.SetKeyName(2, "");
            // 
            // imgLstActuators
            // 
            this.imgLstActuators.ColorDepth = System.Windows.Forms.ColorDepth.Depth8Bit;
            resources.ApplyResources(this.imgLstActuators, "imgLstActuators");
            this.imgLstActuators.TransparentColor = System.Drawing.Color.Transparent;
            // 
            // tmrProgress
            // 
            this.tmrProgress.Interval = 1000;
            this.tmrProgress.SynchronizingObject = this;
            this.tmrProgress.Elapsed += new System.Timers.ElapsedEventHandler(this.tmrProgress_Elapsed);
            // 
            // btnDisconnect
            // 
            this.btnDisconnect.AccessibleDescription = null;
            this.btnDisconnect.AccessibleName = null;
            resources.ApplyResources(this.btnDisconnect, "btnDisconnect");
            this.btnDisconnect.BackgroundImage = null;
            this.btnDisconnect.Font = null;
            this.btnDisconnect.ImageList = this.imgLstToolbar;
            this.btnDisconnect.Name = "btnDisconnect";
            this.toolTips.SetToolTip(this.btnDisconnect, resources.GetString("btnDisconnect.ToolTip"));
            this.btnDisconnect.Click += new System.EventHandler(this.btnDisconnect_Click);
            // 
            // btnConnect
            // 
            this.btnConnect.AccessibleDescription = null;
            this.btnConnect.AccessibleName = null;
            resources.ApplyResources(this.btnConnect, "btnConnect");
            this.btnConnect.BackgroundImage = null;
            this.btnConnect.Font = null;
            this.btnConnect.ImageList = this.imgLstToolbar;
            this.btnConnect.Name = "btnConnect";
            this.toolTips.SetToolTip(this.btnConnect, resources.GetString("btnConnect.ToolTip"));
            this.btnConnect.Click += new System.EventHandler(this.btnConnect_Click);
            // 
            // cboEcuList
            // 
            this.cboEcuList.AccessibleDescription = null;
            this.cboEcuList.AccessibleName = null;
            resources.ApplyResources(this.cboEcuList, "cboEcuList");
            this.cboEcuList.BackgroundImage = null;
            this.cboEcuList.DataBindings.Add(new System.Windows.Forms.Binding("SelectedItem", this.modelBindingSource, "LastConnectedEcu", true));
            this.cboEcuList.DataSource = this.ecuDescriptorsBindingSource;
            this.cboEcuList.DisplayMember = "Name";
            this.cboEcuList.DropDownStyle = System.Windows.Forms.ComboBoxStyle.DropDownList;
            this.cboEcuList.Font = null;
            this.cboEcuList.Name = "cboEcuList";
            this.toolTips.SetToolTip(this.cboEcuList, resources.GetString("cboEcuList.ToolTip"));
            this.cboEcuList.SelectedValueChanged += new System.EventHandler(this.cboEcuList_SelectedValueChanged);
            // 
            // modelBindingSource
            // 
            this.modelBindingSource.DataSource = typeof(DnEcuDiag.Model);
            // 
            // ecuDescriptorsBindingSource
            // 
            this.ecuDescriptorsBindingSource.DataSource = typeof(DnEcuDiag.Ecus.Generic.EcuDescriptor);
            // 
            // tabIdentification
            // 
            this.tabIdentification.AccessibleDescription = null;
            this.tabIdentification.AccessibleName = null;
            resources.ApplyResources(this.tabIdentification, "tabIdentification");
            this.tabIdentification.BackgroundImage = null;
            this.tabIdentification.Controls.Add(this.gbECU);
            this.tabIdentification.Font = null;
            this.tabIdentification.Name = "tabIdentification";
            this.toolTips.SetToolTip(this.tabIdentification, resources.GetString("tabIdentification.ToolTip"));
            this.tabIdentification.UseVisualStyleBackColor = true;
            // 
            // gbECU
            // 
            this.gbECU.AccessibleDescription = null;
            this.gbECU.AccessibleName = null;
            resources.ApplyResources(this.gbECU, "gbECU");
            this.gbECU.BackgroundImage = null;
            this.gbECU.Controls.Add(this.lblDate);
            this.gbECU.Controls.Add(this.txtProgDate);
            this.gbECU.Controls.Add(this.txtISOCode);
            this.gbECU.Controls.Add(this.txtECUModel);
            this.gbECU.Controls.Add(this.lblECUModel);
            this.gbECU.Controls.Add(this.txtOmolCode);
            this.gbECU.Controls.Add(this.txtSwVerNo);
            this.gbECU.Controls.Add(this.txtSwNo);
            this.gbECU.Controls.Add(this.txtEcuHWVer);
            this.gbECU.Controls.Add(this.txtEcuHwNo);
            this.gbECU.Controls.Add(this.txtDrawingNo);
            this.gbECU.Controls.Add(this.lblProgDate);
            this.gbECU.Controls.Add(this.lblISOCode);
            this.gbECU.Controls.Add(this.lblOmoCode);
            this.gbECU.Controls.Add(this.lblSWVerNo);
            this.gbECU.Controls.Add(this.lblSWNo);
            this.gbECU.Controls.Add(this.lblHWVerNo);
            this.gbECU.Controls.Add(this.lblEcuHWNumber);
            this.gbECU.Controls.Add(this.lblDrawingNumber);
            this.gbECU.Font = null;
            this.gbECU.Name = "gbECU";
            this.gbECU.TabStop = false;
            this.toolTips.SetToolTip(this.gbECU, resources.GetString("gbECU.ToolTip"));
            // 
            // lblDate
            // 
            this.lblDate.AccessibleDescription = null;
            this.lblDate.AccessibleName = null;
            resources.ApplyResources(this.lblDate, "lblDate");
            this.lblDate.Font = null;
            this.lblDate.Name = "lblDate";
            this.toolTips.SetToolTip(this.lblDate, resources.GetString("lblDate.ToolTip"));
            // 
            // txtProgDate
            // 
            this.txtProgDate.AccessibleDescription = null;
            this.txtProgDate.AccessibleName = null;
            resources.ApplyResources(this.txtProgDate, "txtProgDate");
            this.txtProgDate.BackColor = System.Drawing.SystemColors.Window;
            this.txtProgDate.BackgroundImage = null;
            this.txtProgDate.DataBindings.Add(new System.Windows.Forms.Binding("Text", this.connectedEcuBindingSource, "ProductionDate", true));
            this.txtProgDate.Font = null;
            this.txtProgDate.Name = "txtProgDate";
            this.txtProgDate.ReadOnly = true;
            this.txtProgDate.TabStop = false;
            this.toolTips.SetToolTip(this.txtProgDate, resources.GetString("txtProgDate.ToolTip"));
            // 
            // connectedEcuBindingSource
            // 
            this.connectedEcuBindingSource.DataSource = typeof(DnEcuDiag.Ecus.Generic.Ecu);
            // 
            // txtISOCode
            // 
            this.txtISOCode.AccessibleDescription = null;
            this.txtISOCode.AccessibleName = null;
            resources.ApplyResources(this.txtISOCode, "txtISOCode");
            this.txtISOCode.BackColor = System.Drawing.SystemColors.Window;
            this.txtISOCode.BackgroundImage = null;
            this.txtISOCode.DataBindings.Add(new System.Windows.Forms.Binding("Text", this.connectedEcuBindingSource, "IsoCode", true));
            this.txtISOCode.Font = null;
            this.txtISOCode.Name = "txtISOCode";
            this.txtISOCode.ReadOnly = true;
            this.txtISOCode.TabStop = false;
            this.toolTips.SetToolTip(this.txtISOCode, resources.GetString("txtISOCode.ToolTip"));
            // 
            // txtECUModel
            // 
            this.txtECUModel.AccessibleDescription = null;
            this.txtECUModel.AccessibleName = null;
            resources.ApplyResources(this.txtECUModel, "txtECUModel");
            this.txtECUModel.BackColor = System.Drawing.SystemColors.Window;
            this.txtECUModel.BackgroundImage = null;
            this.txtECUModel.DataBindings.Add(new System.Windows.Forms.Binding("Text", this.connectedEcuBindingSource, "Model", true));
            this.txtECUModel.Font = null;
            this.txtECUModel.Name = "txtECUModel";
            this.txtECUModel.ReadOnly = true;
            this.txtECUModel.TabStop = false;
            this.toolTips.SetToolTip(this.txtECUModel, resources.GetString("txtECUModel.ToolTip"));
            // 
            // lblECUModel
            // 
            this.lblECUModel.AccessibleDescription = null;
            this.lblECUModel.AccessibleName = null;
            resources.ApplyResources(this.lblECUModel, "lblECUModel");
            this.lblECUModel.Font = null;
            this.lblECUModel.Name = "lblECUModel";
            this.toolTips.SetToolTip(this.lblECUModel, resources.GetString("lblECUModel.ToolTip"));
            // 
            // txtOmolCode
            // 
            this.txtOmolCode.AccessibleDescription = null;
            this.txtOmolCode.AccessibleName = null;
            resources.ApplyResources(this.txtOmolCode, "txtOmolCode");
            this.txtOmolCode.BackColor = System.Drawing.SystemColors.Window;
            this.txtOmolCode.BackgroundImage = null;
            this.txtOmolCode.DataBindings.Add(new System.Windows.Forms.Binding("Text", this.connectedEcuBindingSource, "OmologationNumber", true));
            this.txtOmolCode.Font = null;
            this.txtOmolCode.Name = "txtOmolCode";
            this.txtOmolCode.ReadOnly = true;
            this.txtOmolCode.TabStop = false;
            this.toolTips.SetToolTip(this.txtOmolCode, resources.GetString("txtOmolCode.ToolTip"));
            // 
            // txtSwVerNo
            // 
            this.txtSwVerNo.AccessibleDescription = null;
            this.txtSwVerNo.AccessibleName = null;
            resources.ApplyResources(this.txtSwVerNo, "txtSwVerNo");
            this.txtSwVerNo.BackColor = System.Drawing.SystemColors.Window;
            this.txtSwVerNo.BackgroundImage = null;
            this.txtSwVerNo.DataBindings.Add(new System.Windows.Forms.Binding("Text", this.connectedEcuBindingSource, "SoftwareVersionNumber", true));
            this.txtSwVerNo.Font = null;
            this.txtSwVerNo.Name = "txtSwVerNo";
            this.txtSwVerNo.ReadOnly = true;
            this.txtSwVerNo.TabStop = false;
            this.toolTips.SetToolTip(this.txtSwVerNo, resources.GetString("txtSwVerNo.ToolTip"));
            // 
            // txtSwNo
            // 
            this.txtSwNo.AccessibleDescription = null;
            this.txtSwNo.AccessibleName = null;
            resources.ApplyResources(this.txtSwNo, "txtSwNo");
            this.txtSwNo.BackColor = System.Drawing.SystemColors.Window;
            this.txtSwNo.BackgroundImage = null;
            this.txtSwNo.DataBindings.Add(new System.Windows.Forms.Binding("Text", this.connectedEcuBindingSource, "SoftwareNumber", true));
            this.txtSwNo.Font = null;
            this.txtSwNo.Name = "txtSwNo";
            this.txtSwNo.ReadOnly = true;
            this.txtSwNo.TabStop = false;
            this.toolTips.SetToolTip(this.txtSwNo, resources.GetString("txtSwNo.ToolTip"));
            // 
            // txtEcuHWVer
            // 
            this.txtEcuHWVer.AccessibleDescription = null;
            this.txtEcuHWVer.AccessibleName = null;
            resources.ApplyResources(this.txtEcuHWVer, "txtEcuHWVer");
            this.txtEcuHWVer.BackColor = System.Drawing.SystemColors.Window;
            this.txtEcuHWVer.BackgroundImage = null;
            this.txtEcuHWVer.DataBindings.Add(new System.Windows.Forms.Binding("Text", this.connectedEcuBindingSource, "HardwareVersionNumber", true));
            this.txtEcuHWVer.Font = null;
            this.txtEcuHWVer.Name = "txtEcuHWVer";
            this.txtEcuHWVer.ReadOnly = true;
            this.txtEcuHWVer.TabStop = false;
            this.toolTips.SetToolTip(this.txtEcuHWVer, resources.GetString("txtEcuHWVer.ToolTip"));
            // 
            // txtEcuHwNo
            // 
            this.txtEcuHwNo.AccessibleDescription = null;
            this.txtEcuHwNo.AccessibleName = null;
            resources.ApplyResources(this.txtEcuHwNo, "txtEcuHwNo");
            this.txtEcuHwNo.BackColor = System.Drawing.SystemColors.Window;
            this.txtEcuHwNo.BackgroundImage = null;
            this.txtEcuHwNo.DataBindings.Add(new System.Windows.Forms.Binding("Text", this.connectedEcuBindingSource, "HardwareNumber", true));
            this.txtEcuHwNo.Font = null;
            this.txtEcuHwNo.Name = "txtEcuHwNo";
            this.txtEcuHwNo.ReadOnly = true;
            this.txtEcuHwNo.TabStop = false;
            this.toolTips.SetToolTip(this.txtEcuHwNo, resources.GetString("txtEcuHwNo.ToolTip"));
            // 
            // txtDrawingNo
            // 
            this.txtDrawingNo.AccessibleDescription = null;
            this.txtDrawingNo.AccessibleName = null;
            resources.ApplyResources(this.txtDrawingNo, "txtDrawingNo");
            this.txtDrawingNo.BackColor = System.Drawing.SystemColors.Window;
            this.txtDrawingNo.BackgroundImage = null;
            this.txtDrawingNo.DataBindings.Add(new System.Windows.Forms.Binding("Text", this.connectedEcuBindingSource, "DrawingNumber", true));
            this.txtDrawingNo.Font = null;
            this.txtDrawingNo.Name = "txtDrawingNo";
            this.txtDrawingNo.ReadOnly = true;
            this.txtDrawingNo.TabStop = false;
            this.toolTips.SetToolTip(this.txtDrawingNo, resources.GetString("txtDrawingNo.ToolTip"));
            // 
            // lblProgDate
            // 
            this.lblProgDate.AccessibleDescription = null;
            this.lblProgDate.AccessibleName = null;
            resources.ApplyResources(this.lblProgDate, "lblProgDate");
            this.lblProgDate.Font = null;
            this.lblProgDate.Name = "lblProgDate";
            this.toolTips.SetToolTip(this.lblProgDate, resources.GetString("lblProgDate.ToolTip"));
            // 
            // lblISOCode
            // 
            this.lblISOCode.AccessibleDescription = null;
            this.lblISOCode.AccessibleName = null;
            resources.ApplyResources(this.lblISOCode, "lblISOCode");
            this.lblISOCode.Font = null;
            this.lblISOCode.Name = "lblISOCode";
            this.toolTips.SetToolTip(this.lblISOCode, resources.GetString("lblISOCode.ToolTip"));
            // 
            // lblOmoCode
            // 
            this.lblOmoCode.AccessibleDescription = null;
            this.lblOmoCode.AccessibleName = null;
            resources.ApplyResources(this.lblOmoCode, "lblOmoCode");
            this.lblOmoCode.Font = null;
            this.lblOmoCode.Name = "lblOmoCode";
            this.toolTips.SetToolTip(this.lblOmoCode, resources.GetString("lblOmoCode.ToolTip"));
            // 
            // lblSWVerNo
            // 
            this.lblSWVerNo.AccessibleDescription = null;
            this.lblSWVerNo.AccessibleName = null;
            resources.ApplyResources(this.lblSWVerNo, "lblSWVerNo");
            this.lblSWVerNo.Font = null;
            this.lblSWVerNo.Name = "lblSWVerNo";
            this.toolTips.SetToolTip(this.lblSWVerNo, resources.GetString("lblSWVerNo.ToolTip"));
            // 
            // lblSWNo
            // 
            this.lblSWNo.AccessibleDescription = null;
            this.lblSWNo.AccessibleName = null;
            resources.ApplyResources(this.lblSWNo, "lblSWNo");
            this.lblSWNo.Font = null;
            this.lblSWNo.Name = "lblSWNo";
            this.toolTips.SetToolTip(this.lblSWNo, resources.GetString("lblSWNo.ToolTip"));
            // 
            // lblHWVerNo
            // 
            this.lblHWVerNo.AccessibleDescription = null;
            this.lblHWVerNo.AccessibleName = null;
            resources.ApplyResources(this.lblHWVerNo, "lblHWVerNo");
            this.lblHWVerNo.Font = null;
            this.lblHWVerNo.Name = "lblHWVerNo";
            this.toolTips.SetToolTip(this.lblHWVerNo, resources.GetString("lblHWVerNo.ToolTip"));
            // 
            // lblEcuHWNumber
            // 
            this.lblEcuHWNumber.AccessibleDescription = null;
            this.lblEcuHWNumber.AccessibleName = null;
            resources.ApplyResources(this.lblEcuHWNumber, "lblEcuHWNumber");
            this.lblEcuHWNumber.Font = null;
            this.lblEcuHWNumber.Name = "lblEcuHWNumber";
            this.toolTips.SetToolTip(this.lblEcuHWNumber, resources.GetString("lblEcuHWNumber.ToolTip"));
            // 
            // lblDrawingNumber
            // 
            this.lblDrawingNumber.AccessibleDescription = null;
            this.lblDrawingNumber.AccessibleName = null;
            resources.ApplyResources(this.lblDrawingNumber, "lblDrawingNumber");
            this.lblDrawingNumber.Font = null;
            this.lblDrawingNumber.Name = "lblDrawingNumber";
            this.toolTips.SetToolTip(this.lblDrawingNumber, resources.GetString("lblDrawingNumber.ToolTip"));
            // 
            // tabSensors
            // 
            this.tabSensors.AccessibleDescription = null;
            this.tabSensors.AccessibleName = null;
            resources.ApplyResources(this.tabSensors, "tabSensors");
            this.tabSensors.BackgroundImage = null;
            this.tabSensors.Controls.Add(this.gbSensorFour);
            this.tabSensors.Controls.Add(this.gbSensorThree);
            this.tabSensors.Controls.Add(this.gbSensorTwo);
            this.tabSensors.Controls.Add(this.gbSensorOne);
            this.tabSensors.Controls.Add(this.oScopeFour);
            this.tabSensors.Controls.Add(this.oScopeThree);
            this.tabSensors.Controls.Add(this.oScopeTwo);
            this.tabSensors.Controls.Add(this.oScopeOne);
            this.tabSensors.Font = null;
            this.tabSensors.Name = "tabSensors";
            this.toolTips.SetToolTip(this.tabSensors, resources.GetString("tabSensors.ToolTip"));
            this.tabSensors.UseVisualStyleBackColor = true;
            // 
            // gbSensorFour
            // 
            this.gbSensorFour.AccessibleDescription = null;
            this.gbSensorFour.AccessibleName = null;
            resources.ApplyResources(this.gbSensorFour, "gbSensorFour");
            this.gbSensorFour.BackgroundImage = null;
            this.gbSensorFour.Controls.Add(this.lblSensorFourCurrent);
            this.gbSensorFour.Controls.Add(this.label8);
            this.gbSensorFour.Controls.Add(this.lblSensorFourMinValue);
            this.gbSensorFour.Controls.Add(this.lblSensorFourMaxValue);
            this.gbSensorFour.Controls.Add(this.lblSensorFourMin);
            this.gbSensorFour.Controls.Add(this.lblSensorFourMax);
            this.gbSensorFour.Controls.Add(this.btnSensorFourCapture);
            this.gbSensorFour.Controls.Add(this.cboSensorFour);
            this.gbSensorFour.Font = null;
            this.gbSensorFour.Name = "gbSensorFour";
            this.gbSensorFour.TabStop = false;
            this.toolTips.SetToolTip(this.gbSensorFour, resources.GetString("gbSensorFour.ToolTip"));
            // 
            // lblSensorFourCurrent
            // 
            this.lblSensorFourCurrent.AccessibleDescription = null;
            this.lblSensorFourCurrent.AccessibleName = null;
            resources.ApplyResources(this.lblSensorFourCurrent, "lblSensorFourCurrent");
            this.lblSensorFourCurrent.Name = "lblSensorFourCurrent";
            this.toolTips.SetToolTip(this.lblSensorFourCurrent, resources.GetString("lblSensorFourCurrent.ToolTip"));
            // 
            // label8
            // 
            this.label8.AccessibleDescription = null;
            this.label8.AccessibleName = null;
            resources.ApplyResources(this.label8, "label8");
            this.label8.Name = "label8";
            this.toolTips.SetToolTip(this.label8, resources.GetString("label8.ToolTip"));
            // 
            // lblSensorFourMinValue
            // 
            this.lblSensorFourMinValue.AccessibleDescription = null;
            this.lblSensorFourMinValue.AccessibleName = null;
            resources.ApplyResources(this.lblSensorFourMinValue, "lblSensorFourMinValue");
            this.lblSensorFourMinValue.Name = "lblSensorFourMinValue";
            this.toolTips.SetToolTip(this.lblSensorFourMinValue, resources.GetString("lblSensorFourMinValue.ToolTip"));
            // 
            // lblSensorFourMaxValue
            // 
            this.lblSensorFourMaxValue.AccessibleDescription = null;
            this.lblSensorFourMaxValue.AccessibleName = null;
            resources.ApplyResources(this.lblSensorFourMaxValue, "lblSensorFourMaxValue");
            this.lblSensorFourMaxValue.Name = "lblSensorFourMaxValue";
            this.toolTips.SetToolTip(this.lblSensorFourMaxValue, resources.GetString("lblSensorFourMaxValue.ToolTip"));
            // 
            // lblSensorFourMin
            // 
            this.lblSensorFourMin.AccessibleDescription = null;
            this.lblSensorFourMin.AccessibleName = null;
            resources.ApplyResources(this.lblSensorFourMin, "lblSensorFourMin");
            this.lblSensorFourMin.Name = "lblSensorFourMin";
            this.toolTips.SetToolTip(this.lblSensorFourMin, resources.GetString("lblSensorFourMin.ToolTip"));
            // 
            // lblSensorFourMax
            // 
            this.lblSensorFourMax.AccessibleDescription = null;
            this.lblSensorFourMax.AccessibleName = null;
            resources.ApplyResources(this.lblSensorFourMax, "lblSensorFourMax");
            this.lblSensorFourMax.Name = "lblSensorFourMax";
            this.toolTips.SetToolTip(this.lblSensorFourMax, resources.GetString("lblSensorFourMax.ToolTip"));
            // 
            // btnSensorFourCapture
            // 
            this.btnSensorFourCapture.AccessibleDescription = null;
            this.btnSensorFourCapture.AccessibleName = null;
            resources.ApplyResources(this.btnSensorFourCapture, "btnSensorFourCapture");
            this.btnSensorFourCapture.BackgroundImage = null;
            this.btnSensorFourCapture.Font = null;
            this.btnSensorFourCapture.ImageList = this.imgLstButtons;
            this.btnSensorFourCapture.Name = "btnSensorFourCapture";
            this.toolTips.SetToolTip(this.btnSensorFourCapture, resources.GetString("btnSensorFourCapture.ToolTip"));
            this.btnSensorFourCapture.Click += new System.EventHandler(this.btnSensorFourCapture_Click);
            // 
            // cboSensorFour
            // 
            this.cboSensorFour.AccessibleDescription = null;
            this.cboSensorFour.AccessibleName = null;
            resources.ApplyResources(this.cboSensorFour, "cboSensorFour");
            this.cboSensorFour.BackgroundImage = null;
            this.cboSensorFour.DataBindings.Add(new System.Windows.Forms.Binding("SelectedItem", this.connectedEcuBindingSource, "SensorMonitorFour", true));
            this.cboSensorFour.DataBindings.Add(new System.Windows.Forms.Binding("SelectedValue", this.connectedEcuBindingSource, "SensorMonitorFour", true));
            this.cboSensorFour.DataSource = this.sensorFourBindingSource;
            this.cboSensorFour.DisplayMember = "Name";
            this.cboSensorFour.DropDownStyle = System.Windows.Forms.ComboBoxStyle.DropDownList;
            this.cboSensorFour.Font = null;
            this.cboSensorFour.Name = "cboSensorFour";
            this.toolTips.SetToolTip(this.cboSensorFour, resources.GetString("cboSensorFour.ToolTip"));
            this.cboSensorFour.SelectedIndexChanged += new System.EventHandler(this.cboSensorFour_SelectedIndexChanged);
            // 
            // sensorFourBindingSource
            // 
            this.sensorFourBindingSource.DataSource = typeof(DnEcuDiag.Ecus.Generic.Sensor);
            // 
            // gbSensorThree
            // 
            this.gbSensorThree.AccessibleDescription = null;
            this.gbSensorThree.AccessibleName = null;
            resources.ApplyResources(this.gbSensorThree, "gbSensorThree");
            this.gbSensorThree.BackgroundImage = null;
            this.gbSensorThree.Controls.Add(this.lblSensorThreeCurrent);
            this.gbSensorThree.Controls.Add(this.lblCurrThree);
            this.gbSensorThree.Controls.Add(this.lblSensorThreeMinValue);
            this.gbSensorThree.Controls.Add(this.lblSensorThreeMaxValue);
            this.gbSensorThree.Controls.Add(this.lblSensorThreeMin);
            this.gbSensorThree.Controls.Add(this.lblSensorThreeMax);
            this.gbSensorThree.Controls.Add(this.btnSensorThreeCapture);
            this.gbSensorThree.Controls.Add(this.cboSensorThree);
            this.gbSensorThree.Font = null;
            this.gbSensorThree.Name = "gbSensorThree";
            this.gbSensorThree.TabStop = false;
            this.toolTips.SetToolTip(this.gbSensorThree, resources.GetString("gbSensorThree.ToolTip"));
            // 
            // lblSensorThreeCurrent
            // 
            this.lblSensorThreeCurrent.AccessibleDescription = null;
            this.lblSensorThreeCurrent.AccessibleName = null;
            resources.ApplyResources(this.lblSensorThreeCurrent, "lblSensorThreeCurrent");
            this.lblSensorThreeCurrent.Name = "lblSensorThreeCurrent";
            this.toolTips.SetToolTip(this.lblSensorThreeCurrent, resources.GetString("lblSensorThreeCurrent.ToolTip"));
            // 
            // lblCurrThree
            // 
            this.lblCurrThree.AccessibleDescription = null;
            this.lblCurrThree.AccessibleName = null;
            resources.ApplyResources(this.lblCurrThree, "lblCurrThree");
            this.lblCurrThree.Name = "lblCurrThree";
            this.toolTips.SetToolTip(this.lblCurrThree, resources.GetString("lblCurrThree.ToolTip"));
            // 
            // lblSensorThreeMinValue
            // 
            this.lblSensorThreeMinValue.AccessibleDescription = null;
            this.lblSensorThreeMinValue.AccessibleName = null;
            resources.ApplyResources(this.lblSensorThreeMinValue, "lblSensorThreeMinValue");
            this.lblSensorThreeMinValue.Name = "lblSensorThreeMinValue";
            this.toolTips.SetToolTip(this.lblSensorThreeMinValue, resources.GetString("lblSensorThreeMinValue.ToolTip"));
            // 
            // lblSensorThreeMaxValue
            // 
            this.lblSensorThreeMaxValue.AccessibleDescription = null;
            this.lblSensorThreeMaxValue.AccessibleName = null;
            resources.ApplyResources(this.lblSensorThreeMaxValue, "lblSensorThreeMaxValue");
            this.lblSensorThreeMaxValue.Name = "lblSensorThreeMaxValue";
            this.toolTips.SetToolTip(this.lblSensorThreeMaxValue, resources.GetString("lblSensorThreeMaxValue.ToolTip"));
            // 
            // lblSensorThreeMin
            // 
            this.lblSensorThreeMin.AccessibleDescription = null;
            this.lblSensorThreeMin.AccessibleName = null;
            resources.ApplyResources(this.lblSensorThreeMin, "lblSensorThreeMin");
            this.lblSensorThreeMin.Name = "lblSensorThreeMin";
            this.toolTips.SetToolTip(this.lblSensorThreeMin, resources.GetString("lblSensorThreeMin.ToolTip"));
            // 
            // lblSensorThreeMax
            // 
            this.lblSensorThreeMax.AccessibleDescription = null;
            this.lblSensorThreeMax.AccessibleName = null;
            resources.ApplyResources(this.lblSensorThreeMax, "lblSensorThreeMax");
            this.lblSensorThreeMax.Name = "lblSensorThreeMax";
            this.toolTips.SetToolTip(this.lblSensorThreeMax, resources.GetString("lblSensorThreeMax.ToolTip"));
            // 
            // btnSensorThreeCapture
            // 
            this.btnSensorThreeCapture.AccessibleDescription = null;
            this.btnSensorThreeCapture.AccessibleName = null;
            resources.ApplyResources(this.btnSensorThreeCapture, "btnSensorThreeCapture");
            this.btnSensorThreeCapture.BackgroundImage = null;
            this.btnSensorThreeCapture.Font = null;
            this.btnSensorThreeCapture.ImageList = this.imgLstButtons;
            this.btnSensorThreeCapture.Name = "btnSensorThreeCapture";
            this.toolTips.SetToolTip(this.btnSensorThreeCapture, resources.GetString("btnSensorThreeCapture.ToolTip"));
            this.btnSensorThreeCapture.Click += new System.EventHandler(this.btnSensorThreeCapture_Click);
            // 
            // cboSensorThree
            // 
            this.cboSensorThree.AccessibleDescription = null;
            this.cboSensorThree.AccessibleName = null;
            resources.ApplyResources(this.cboSensorThree, "cboSensorThree");
            this.cboSensorThree.BackgroundImage = null;
            this.cboSensorThree.DataBindings.Add(new System.Windows.Forms.Binding("SelectedItem", this.connectedEcuBindingSource, "SensorMonitorThree", true));
            this.cboSensorThree.DataBindings.Add(new System.Windows.Forms.Binding("SelectedValue", this.connectedEcuBindingSource, "SensorMonitorThree", true));
            this.cboSensorThree.DataSource = this.sensorThreeBindingSource;
            this.cboSensorThree.DisplayMember = "Name";
            this.cboSensorThree.DropDownStyle = System.Windows.Forms.ComboBoxStyle.DropDownList;
            this.cboSensorThree.Font = null;
            this.cboSensorThree.Name = "cboSensorThree";
            this.toolTips.SetToolTip(this.cboSensorThree, resources.GetString("cboSensorThree.ToolTip"));
            this.cboSensorThree.SelectedIndexChanged += new System.EventHandler(this.cboSensorThree_SelectedIndexChanged);
            // 
            // sensorThreeBindingSource
            // 
            this.sensorThreeBindingSource.DataSource = typeof(DnEcuDiag.Ecus.Generic.Sensor);
            // 
            // gbSensorTwo
            // 
            this.gbSensorTwo.AccessibleDescription = null;
            this.gbSensorTwo.AccessibleName = null;
            resources.ApplyResources(this.gbSensorTwo, "gbSensorTwo");
            this.gbSensorTwo.BackgroundImage = null;
            this.gbSensorTwo.Controls.Add(this.lblSensorTwoCurrent);
            this.gbSensorTwo.Controls.Add(this.lblCurrTwo);
            this.gbSensorTwo.Controls.Add(this.lblSensorTwoMinValue);
            this.gbSensorTwo.Controls.Add(this.lblSensorTwoMaxValue);
            this.gbSensorTwo.Controls.Add(this.lblSensorTwoMin);
            this.gbSensorTwo.Controls.Add(this.lblSensorTwoMax);
            this.gbSensorTwo.Controls.Add(this.btnSensorTwoCapture);
            this.gbSensorTwo.Controls.Add(this.cboSensorTwo);
            this.gbSensorTwo.Name = "gbSensorTwo";
            this.gbSensorTwo.TabStop = false;
            this.toolTips.SetToolTip(this.gbSensorTwo, resources.GetString("gbSensorTwo.ToolTip"));
            // 
            // lblSensorTwoCurrent
            // 
            this.lblSensorTwoCurrent.AccessibleDescription = null;
            this.lblSensorTwoCurrent.AccessibleName = null;
            resources.ApplyResources(this.lblSensorTwoCurrent, "lblSensorTwoCurrent");
            this.lblSensorTwoCurrent.Name = "lblSensorTwoCurrent";
            this.toolTips.SetToolTip(this.lblSensorTwoCurrent, resources.GetString("lblSensorTwoCurrent.ToolTip"));
            // 
            // lblCurrTwo
            // 
            this.lblCurrTwo.AccessibleDescription = null;
            this.lblCurrTwo.AccessibleName = null;
            resources.ApplyResources(this.lblCurrTwo, "lblCurrTwo");
            this.lblCurrTwo.Name = "lblCurrTwo";
            this.toolTips.SetToolTip(this.lblCurrTwo, resources.GetString("lblCurrTwo.ToolTip"));
            // 
            // lblSensorTwoMinValue
            // 
            this.lblSensorTwoMinValue.AccessibleDescription = null;
            this.lblSensorTwoMinValue.AccessibleName = null;
            resources.ApplyResources(this.lblSensorTwoMinValue, "lblSensorTwoMinValue");
            this.lblSensorTwoMinValue.Name = "lblSensorTwoMinValue";
            this.toolTips.SetToolTip(this.lblSensorTwoMinValue, resources.GetString("lblSensorTwoMinValue.ToolTip"));
            // 
            // lblSensorTwoMaxValue
            // 
            this.lblSensorTwoMaxValue.AccessibleDescription = null;
            this.lblSensorTwoMaxValue.AccessibleName = null;
            resources.ApplyResources(this.lblSensorTwoMaxValue, "lblSensorTwoMaxValue");
            this.lblSensorTwoMaxValue.Name = "lblSensorTwoMaxValue";
            this.toolTips.SetToolTip(this.lblSensorTwoMaxValue, resources.GetString("lblSensorTwoMaxValue.ToolTip"));
            // 
            // lblSensorTwoMin
            // 
            this.lblSensorTwoMin.AccessibleDescription = null;
            this.lblSensorTwoMin.AccessibleName = null;
            resources.ApplyResources(this.lblSensorTwoMin, "lblSensorTwoMin");
            this.lblSensorTwoMin.Name = "lblSensorTwoMin";
            this.toolTips.SetToolTip(this.lblSensorTwoMin, resources.GetString("lblSensorTwoMin.ToolTip"));
            // 
            // lblSensorTwoMax
            // 
            this.lblSensorTwoMax.AccessibleDescription = null;
            this.lblSensorTwoMax.AccessibleName = null;
            resources.ApplyResources(this.lblSensorTwoMax, "lblSensorTwoMax");
            this.lblSensorTwoMax.Name = "lblSensorTwoMax";
            this.toolTips.SetToolTip(this.lblSensorTwoMax, resources.GetString("lblSensorTwoMax.ToolTip"));
            // 
            // btnSensorTwoCapture
            // 
            this.btnSensorTwoCapture.AccessibleDescription = null;
            this.btnSensorTwoCapture.AccessibleName = null;
            resources.ApplyResources(this.btnSensorTwoCapture, "btnSensorTwoCapture");
            this.btnSensorTwoCapture.BackgroundImage = null;
            this.btnSensorTwoCapture.Font = null;
            this.btnSensorTwoCapture.ImageList = this.imgLstButtons;
            this.btnSensorTwoCapture.Name = "btnSensorTwoCapture";
            this.toolTips.SetToolTip(this.btnSensorTwoCapture, resources.GetString("btnSensorTwoCapture.ToolTip"));
            this.btnSensorTwoCapture.Click += new System.EventHandler(this.btnSensorTwoCapture_Click);
            // 
            // cboSensorTwo
            // 
            this.cboSensorTwo.AccessibleDescription = null;
            this.cboSensorTwo.AccessibleName = null;
            resources.ApplyResources(this.cboSensorTwo, "cboSensorTwo");
            this.cboSensorTwo.BackgroundImage = null;
            this.cboSensorTwo.DataBindings.Add(new System.Windows.Forms.Binding("SelectedItem", this.connectedEcuBindingSource, "SensorMonitorTwo", true));
            this.cboSensorTwo.DataBindings.Add(new System.Windows.Forms.Binding("SelectedValue", this.connectedEcuBindingSource, "SensorMonitorTwo", true));
            this.cboSensorTwo.DataSource = this.sensorTwoBindingSource;
            this.cboSensorTwo.DisplayMember = "Name";
            this.cboSensorTwo.DropDownStyle = System.Windows.Forms.ComboBoxStyle.DropDownList;
            this.cboSensorTwo.Name = "cboSensorTwo";
            this.toolTips.SetToolTip(this.cboSensorTwo, resources.GetString("cboSensorTwo.ToolTip"));
            this.cboSensorTwo.SelectedIndexChanged += new System.EventHandler(this.cboSensorTwo_SelectedIndexChanged);
            // 
            // sensorTwoBindingSource
            // 
            this.sensorTwoBindingSource.DataSource = typeof(DnEcuDiag.Ecus.Generic.Sensor);
            // 
            // gbSensorOne
            // 
            this.gbSensorOne.AccessibleDescription = null;
            this.gbSensorOne.AccessibleName = null;
            resources.ApplyResources(this.gbSensorOne, "gbSensorOne");
            this.gbSensorOne.BackgroundImage = null;
            this.gbSensorOne.Controls.Add(this.lblSensorOneCurrent);
            this.gbSensorOne.Controls.Add(this.lblCurrOne);
            this.gbSensorOne.Controls.Add(this.lblSensorOneMinValue);
            this.gbSensorOne.Controls.Add(this.lblSensorOneMaxValue);
            this.gbSensorOne.Controls.Add(this.lblSensorOneMin);
            this.gbSensorOne.Controls.Add(this.lblSensorOneMax);
            this.gbSensorOne.Controls.Add(this.btnSensorOneCapture);
            this.gbSensorOne.Controls.Add(this.cboSensorOne);
            this.gbSensorOne.Font = null;
            this.gbSensorOne.Name = "gbSensorOne";
            this.gbSensorOne.TabStop = false;
            this.toolTips.SetToolTip(this.gbSensorOne, resources.GetString("gbSensorOne.ToolTip"));
            // 
            // lblSensorOneCurrent
            // 
            this.lblSensorOneCurrent.AccessibleDescription = null;
            this.lblSensorOneCurrent.AccessibleName = null;
            resources.ApplyResources(this.lblSensorOneCurrent, "lblSensorOneCurrent");
            this.lblSensorOneCurrent.Name = "lblSensorOneCurrent";
            this.toolTips.SetToolTip(this.lblSensorOneCurrent, resources.GetString("lblSensorOneCurrent.ToolTip"));
            // 
            // lblCurrOne
            // 
            this.lblCurrOne.AccessibleDescription = null;
            this.lblCurrOne.AccessibleName = null;
            resources.ApplyResources(this.lblCurrOne, "lblCurrOne");
            this.lblCurrOne.Name = "lblCurrOne";
            this.toolTips.SetToolTip(this.lblCurrOne, resources.GetString("lblCurrOne.ToolTip"));
            // 
            // lblSensorOneMinValue
            // 
            this.lblSensorOneMinValue.AccessibleDescription = null;
            this.lblSensorOneMinValue.AccessibleName = null;
            resources.ApplyResources(this.lblSensorOneMinValue, "lblSensorOneMinValue");
            this.lblSensorOneMinValue.Name = "lblSensorOneMinValue";
            this.toolTips.SetToolTip(this.lblSensorOneMinValue, resources.GetString("lblSensorOneMinValue.ToolTip"));
            // 
            // lblSensorOneMaxValue
            // 
            this.lblSensorOneMaxValue.AccessibleDescription = null;
            this.lblSensorOneMaxValue.AccessibleName = null;
            resources.ApplyResources(this.lblSensorOneMaxValue, "lblSensorOneMaxValue");
            this.lblSensorOneMaxValue.Name = "lblSensorOneMaxValue";
            this.toolTips.SetToolTip(this.lblSensorOneMaxValue, resources.GetString("lblSensorOneMaxValue.ToolTip"));
            // 
            // lblSensorOneMin
            // 
            this.lblSensorOneMin.AccessibleDescription = null;
            this.lblSensorOneMin.AccessibleName = null;
            resources.ApplyResources(this.lblSensorOneMin, "lblSensorOneMin");
            this.lblSensorOneMin.Name = "lblSensorOneMin";
            this.toolTips.SetToolTip(this.lblSensorOneMin, resources.GetString("lblSensorOneMin.ToolTip"));
            // 
            // lblSensorOneMax
            // 
            this.lblSensorOneMax.AccessibleDescription = null;
            this.lblSensorOneMax.AccessibleName = null;
            resources.ApplyResources(this.lblSensorOneMax, "lblSensorOneMax");
            this.lblSensorOneMax.Name = "lblSensorOneMax";
            this.toolTips.SetToolTip(this.lblSensorOneMax, resources.GetString("lblSensorOneMax.ToolTip"));
            // 
            // btnSensorOneCapture
            // 
            this.btnSensorOneCapture.AccessibleDescription = null;
            this.btnSensorOneCapture.AccessibleName = null;
            resources.ApplyResources(this.btnSensorOneCapture, "btnSensorOneCapture");
            this.btnSensorOneCapture.BackgroundImage = null;
            this.btnSensorOneCapture.Font = null;
            this.btnSensorOneCapture.ImageList = this.imgLstButtons;
            this.btnSensorOneCapture.Name = "btnSensorOneCapture";
            this.toolTips.SetToolTip(this.btnSensorOneCapture, resources.GetString("btnSensorOneCapture.ToolTip"));
            this.btnSensorOneCapture.Click += new System.EventHandler(this.btnSensorOneCapture_Click);
            // 
            // cboSensorOne
            // 
            this.cboSensorOne.AccessibleDescription = null;
            this.cboSensorOne.AccessibleName = null;
            resources.ApplyResources(this.cboSensorOne, "cboSensorOne");
            this.cboSensorOne.BackgroundImage = null;
            this.cboSensorOne.DataBindings.Add(new System.Windows.Forms.Binding("SelectedItem", this.connectedEcuBindingSource, "SensorMonitorOne", true));
            this.cboSensorOne.DataBindings.Add(new System.Windows.Forms.Binding("SelectedValue", this.connectedEcuBindingSource, "SensorMonitorOne", true));
            this.cboSensorOne.DataSource = this.sensorOneBindingSource;
            this.cboSensorOne.DisplayMember = "Name";
            this.cboSensorOne.DropDownStyle = System.Windows.Forms.ComboBoxStyle.DropDownList;
            this.cboSensorOne.Font = null;
            this.cboSensorOne.Name = "cboSensorOne";
            this.toolTips.SetToolTip(this.cboSensorOne, resources.GetString("cboSensorOne.ToolTip"));
            this.cboSensorOne.SelectedIndexChanged += new System.EventHandler(this.cboSensorOne_SelectedIndexChanged);
            // 
            // sensorOneBindingSource
            // 
            this.sensorOneBindingSource.DataSource = typeof(DnEcuDiag.Ecus.Generic.Sensor);
            // 
            // oScopeFour
            // 
            this.oScopeFour.AccessibleDescription = null;
            this.oScopeFour.AccessibleName = null;
            resources.ApplyResources(this.oScopeFour, "oScopeFour");
            this.oScopeFour.BackColor = global::DnEcuDiag.Properties.Settings.Default.BackColor;
            this.oScopeFour.BackgroundImage = null;
            this.oScopeFour.DataBindings.Add(new System.Windows.Forms.Binding("ForeColor", global::DnEcuDiag.Properties.Settings.Default, "PlotColor", true, System.Windows.Forms.DataSourceUpdateMode.OnPropertyChanged));
            this.oScopeFour.DataBindings.Add(new System.Windows.Forms.Binding("GridColor", global::DnEcuDiag.Properties.Settings.Default, "GridColor", true, System.Windows.Forms.DataSourceUpdateMode.OnPropertyChanged));
            this.oScopeFour.DataBindings.Add(new System.Windows.Forms.Binding("BackColor", global::DnEcuDiag.Properties.Settings.Default, "BackColor", true, System.Windows.Forms.DataSourceUpdateMode.OnPropertyChanged));
            this.oScopeFour.DataBindings.Add(new System.Windows.Forms.Binding("LegendColor", global::DnEcuDiag.Properties.Settings.Default, "LegendColor", true, System.Windows.Forms.DataSourceUpdateMode.OnPropertyChanged));
            this.oScopeFour.ForeColor = global::DnEcuDiag.Properties.Settings.Default.PlotColor;
            this.oScopeFour.GridColor = global::DnEcuDiag.Properties.Settings.Default.GridColor;
            this.oScopeFour.LegendColor = global::DnEcuDiag.Properties.Settings.Default.LegendColor;
            this.oScopeFour.Name = "oScopeFour";
            this.oScopeFour.TabStop = false;
            this.toolTips.SetToolTip(this.oScopeFour, resources.GetString("oScopeFour.ToolTip"));
            // 
            // oScopeThree
            // 
            this.oScopeThree.AccessibleDescription = null;
            this.oScopeThree.AccessibleName = null;
            resources.ApplyResources(this.oScopeThree, "oScopeThree");
            this.oScopeThree.BackColor = global::DnEcuDiag.Properties.Settings.Default.BackColor;
            this.oScopeThree.BackgroundImage = null;
            this.oScopeThree.DataBindings.Add(new System.Windows.Forms.Binding("BackColor", global::DnEcuDiag.Properties.Settings.Default, "BackColor", true, System.Windows.Forms.DataSourceUpdateMode.OnPropertyChanged));
            this.oScopeThree.DataBindings.Add(new System.Windows.Forms.Binding("ForeColor", global::DnEcuDiag.Properties.Settings.Default, "PlotColor", true, System.Windows.Forms.DataSourceUpdateMode.OnPropertyChanged));
            this.oScopeThree.DataBindings.Add(new System.Windows.Forms.Binding("GridColor", global::DnEcuDiag.Properties.Settings.Default, "GridColor", true, System.Windows.Forms.DataSourceUpdateMode.OnPropertyChanged));
            this.oScopeThree.DataBindings.Add(new System.Windows.Forms.Binding("LegendColor", global::DnEcuDiag.Properties.Settings.Default, "LegendColor", true, System.Windows.Forms.DataSourceUpdateMode.OnPropertyChanged));
            this.oScopeThree.ForeColor = global::DnEcuDiag.Properties.Settings.Default.PlotColor;
            this.oScopeThree.GridColor = global::DnEcuDiag.Properties.Settings.Default.GridColor;
            this.oScopeThree.LegendColor = global::DnEcuDiag.Properties.Settings.Default.LegendColor;
            this.oScopeThree.Name = "oScopeThree";
            this.oScopeThree.TabStop = false;
            this.toolTips.SetToolTip(this.oScopeThree, resources.GetString("oScopeThree.ToolTip"));
            // 
            // oScopeTwo
            // 
            this.oScopeTwo.AccessibleDescription = null;
            this.oScopeTwo.AccessibleName = null;
            resources.ApplyResources(this.oScopeTwo, "oScopeTwo");
            this.oScopeTwo.BackColor = global::DnEcuDiag.Properties.Settings.Default.BackColor;
            this.oScopeTwo.BackgroundImage = null;
            this.oScopeTwo.DataBindings.Add(new System.Windows.Forms.Binding("BackColor", global::DnEcuDiag.Properties.Settings.Default, "BackColor", true, System.Windows.Forms.DataSourceUpdateMode.OnPropertyChanged));
            this.oScopeTwo.DataBindings.Add(new System.Windows.Forms.Binding("ForeColor", global::DnEcuDiag.Properties.Settings.Default, "PlotColor", true, System.Windows.Forms.DataSourceUpdateMode.OnPropertyChanged));
            this.oScopeTwo.DataBindings.Add(new System.Windows.Forms.Binding("GridColor", global::DnEcuDiag.Properties.Settings.Default, "GridColor", true, System.Windows.Forms.DataSourceUpdateMode.OnPropertyChanged));
            this.oScopeTwo.DataBindings.Add(new System.Windows.Forms.Binding("LegendColor", global::DnEcuDiag.Properties.Settings.Default, "LegendColor", true, System.Windows.Forms.DataSourceUpdateMode.OnPropertyChanged));
            this.oScopeTwo.ForeColor = global::DnEcuDiag.Properties.Settings.Default.PlotColor;
            this.oScopeTwo.GridColor = global::DnEcuDiag.Properties.Settings.Default.GridColor;
            this.oScopeTwo.LegendColor = global::DnEcuDiag.Properties.Settings.Default.LegendColor;
            this.oScopeTwo.Name = "oScopeTwo";
            this.oScopeTwo.TabStop = false;
            this.toolTips.SetToolTip(this.oScopeTwo, resources.GetString("oScopeTwo.ToolTip"));
            // 
            // oScopeOne
            // 
            this.oScopeOne.AccessibleDescription = null;
            this.oScopeOne.AccessibleName = null;
            resources.ApplyResources(this.oScopeOne, "oScopeOne");
            this.oScopeOne.BackColor = global::DnEcuDiag.Properties.Settings.Default.BackColor;
            this.oScopeOne.BackgroundImage = null;
            this.oScopeOne.DataBindings.Add(new System.Windows.Forms.Binding("BackColor", global::DnEcuDiag.Properties.Settings.Default, "BackColor", true, System.Windows.Forms.DataSourceUpdateMode.OnPropertyChanged));
            this.oScopeOne.DataBindings.Add(new System.Windows.Forms.Binding("LegendColor", global::DnEcuDiag.Properties.Settings.Default, "LegendColor", true, System.Windows.Forms.DataSourceUpdateMode.OnPropertyChanged));
            this.oScopeOne.DataBindings.Add(new System.Windows.Forms.Binding("GridColor", global::DnEcuDiag.Properties.Settings.Default, "GridColor", true, System.Windows.Forms.DataSourceUpdateMode.OnPropertyChanged));
            this.oScopeOne.DataBindings.Add(new System.Windows.Forms.Binding("ForeColor", global::DnEcuDiag.Properties.Settings.Default, "PlotColor", true, System.Windows.Forms.DataSourceUpdateMode.OnPropertyChanged));
            this.oScopeOne.ForeColor = global::DnEcuDiag.Properties.Settings.Default.PlotColor;
            this.oScopeOne.GridColor = global::DnEcuDiag.Properties.Settings.Default.GridColor;
            this.oScopeOne.LegendColor = global::DnEcuDiag.Properties.Settings.Default.LegendColor;
            this.oScopeOne.Name = "oScopeOne";
            this.oScopeOne.TabStop = false;
            this.toolTips.SetToolTip(this.oScopeOne, resources.GetString("oScopeOne.ToolTip"));
            // 
            // tabActuators
            // 
            this.tabActuators.AccessibleDescription = null;
            this.tabActuators.AccessibleName = null;
            resources.ApplyResources(this.tabActuators, "tabActuators");
            this.tabActuators.BackgroundImage = null;
            this.tabActuators.Controls.Add(this.tcActuators);
            this.tabActuators.Font = null;
            this.tabActuators.Name = "tabActuators";
            this.toolTips.SetToolTip(this.tabActuators, resources.GetString("tabActuators.ToolTip"));
            this.tabActuators.UseVisualStyleBackColor = true;
            // 
            // tcActuators
            // 
            this.tcActuators.AccessibleDescription = null;
            this.tcActuators.AccessibleName = null;
            resources.ApplyResources(this.tcActuators, "tcActuators");
            this.tcActuators.BackgroundImage = null;
            this.tcActuators.Controls.Add(this.tabActuatorControl);
            this.tcActuators.Controls.Add(this.tabRoutines);
            this.tcActuators.Font = null;
            this.tcActuators.Name = "tcActuators";
            this.tcActuators.SelectedIndex = 0;
            this.toolTips.SetToolTip(this.tcActuators, resources.GetString("tcActuators.ToolTip"));
            this.tcActuators.SelectedIndexChanging += new System.EventHandler<DnEcuDiag.CustomControls.TabPageChangeEventArgs>(this.tcActuators_SelectedIndexChanging);
            // 
            // tabActuatorControl
            // 
            this.tabActuatorControl.AccessibleDescription = null;
            this.tabActuatorControl.AccessibleName = null;
            resources.ApplyResources(this.tabActuatorControl, "tabActuatorControl");
            this.tabActuatorControl.BackgroundImage = null;
            this.tabActuatorControl.Controls.Add(this.lvActuators);
            this.tabActuatorControl.Controls.Add(this.txtActuatorTestStat);
            this.tabActuatorControl.Controls.Add(this.btnAutoActuator);
            this.tabActuatorControl.Controls.Add(this.pbActuator);
            this.tabActuatorControl.Font = null;
            this.tabActuatorControl.Name = "tabActuatorControl";
            this.toolTips.SetToolTip(this.tabActuatorControl, resources.GetString("tabActuatorControl.ToolTip"));
            this.tabActuatorControl.UseVisualStyleBackColor = true;
            // 
            // lvActuators
            // 
            this.lvActuators.AccessibleDescription = null;
            this.lvActuators.AccessibleName = null;
            resources.ApplyResources(this.lvActuators, "lvActuators");
            this.lvActuators.AutoArrange = false;
            this.lvActuators.BackgroundImage = null;
            this.lvActuators.Font = null;
            this.lvActuators.FullRowSelect = true;
            this.lvActuators.HeaderStyle = System.Windows.Forms.ColumnHeaderStyle.None;
            this.lvActuators.HideSelection = false;
            this.lvActuators.MultiSelect = false;
            this.lvActuators.Name = "lvActuators";
            this.lvActuators.SmallImageList = this.imgLstActuators;
            this.toolTips.SetToolTip(this.lvActuators, resources.GetString("lvActuators.ToolTip"));
            this.lvActuators.UseCompatibleStateImageBehavior = false;
            this.lvActuators.View = System.Windows.Forms.View.List;
            // 
            // txtActuatorTestStat
            // 
            this.txtActuatorTestStat.AccessibleDescription = null;
            this.txtActuatorTestStat.AccessibleName = null;
            resources.ApplyResources(this.txtActuatorTestStat, "txtActuatorTestStat");
            this.txtActuatorTestStat.BackColor = System.Drawing.SystemColors.Window;
            this.txtActuatorTestStat.BackgroundImage = null;
            this.txtActuatorTestStat.Font = null;
            this.txtActuatorTestStat.Name = "txtActuatorTestStat";
            this.txtActuatorTestStat.ReadOnly = true;
            this.toolTips.SetToolTip(this.txtActuatorTestStat, resources.GetString("txtActuatorTestStat.ToolTip"));
            // 
            // btnAutoActuator
            // 
            this.btnAutoActuator.AccessibleDescription = null;
            this.btnAutoActuator.AccessibleName = null;
            resources.ApplyResources(this.btnAutoActuator, "btnAutoActuator");
            this.btnAutoActuator.BackgroundImage = null;
            this.btnAutoActuator.Font = null;
            this.btnAutoActuator.Name = "btnAutoActuator";
            this.toolTips.SetToolTip(this.btnAutoActuator, resources.GetString("btnAutoActuator.ToolTip"));
            this.btnAutoActuator.Click += new System.EventHandler(this.btnAutoActuator_Click);
            // 
            // pbActuator
            // 
            this.pbActuator.AccessibleDescription = null;
            this.pbActuator.AccessibleName = null;
            resources.ApplyResources(this.pbActuator, "pbActuator");
            this.pbActuator.BackgroundImage = null;
            this.pbActuator.Font = null;
            this.pbActuator.Name = "pbActuator";
            this.pbActuator.Step = 1;
            this.toolTips.SetToolTip(this.pbActuator, resources.GetString("pbActuator.ToolTip"));
            // 
            // tabRoutines
            // 
            this.tabRoutines.AccessibleDescription = null;
            this.tabRoutines.AccessibleName = null;
            resources.ApplyResources(this.tabRoutines, "tabRoutines");
            this.tabRoutines.BackgroundImage = null;
            this.tabRoutines.Controls.Add(this.btnRoutineExecuteCancel);
            this.tabRoutines.Controls.Add(this.btnExecuteRoutine);
            this.tabRoutines.Controls.Add(this.txtRoutineInstructions);
            this.tabRoutines.Controls.Add(this.cboRoutines);
            this.tabRoutines.Controls.Add(this.lblRoutine);
            this.tabRoutines.Controls.Add(this.pbCalibration);
            this.tabRoutines.Controls.Add(this.txtRoutineExecution);
            this.tabRoutines.Font = null;
            this.tabRoutines.Name = "tabRoutines";
            this.toolTips.SetToolTip(this.tabRoutines, resources.GetString("tabRoutines.ToolTip"));
            this.tabRoutines.UseVisualStyleBackColor = true;
            // 
            // btnRoutineExecuteCancel
            // 
            this.btnRoutineExecuteCancel.AccessibleDescription = null;
            this.btnRoutineExecuteCancel.AccessibleName = null;
            resources.ApplyResources(this.btnRoutineExecuteCancel, "btnRoutineExecuteCancel");
            this.btnRoutineExecuteCancel.BackgroundImage = null;
            this.btnRoutineExecuteCancel.Font = null;
            this.btnRoutineExecuteCancel.Name = "btnRoutineExecuteCancel";
            this.toolTips.SetToolTip(this.btnRoutineExecuteCancel, resources.GetString("btnRoutineExecuteCancel.ToolTip"));
            this.btnRoutineExecuteCancel.Click += new System.EventHandler(this.btnCancelRoutineExecution_Click);
            // 
            // btnExecuteRoutine
            // 
            this.btnExecuteRoutine.AccessibleDescription = null;
            this.btnExecuteRoutine.AccessibleName = null;
            resources.ApplyResources(this.btnExecuteRoutine, "btnExecuteRoutine");
            this.btnExecuteRoutine.BackgroundImage = null;
            this.btnExecuteRoutine.Font = null;
            this.btnExecuteRoutine.Name = "btnExecuteRoutine";
            this.toolTips.SetToolTip(this.btnExecuteRoutine, resources.GetString("btnExecuteRoutine.ToolTip"));
            this.btnExecuteRoutine.Click += new System.EventHandler(this.btnExecuteRoutine_Click);
            // 
            // txtRoutineInstructions
            // 
            this.txtRoutineInstructions.AcceptsReturn = true;
            this.txtRoutineInstructions.AccessibleDescription = null;
            this.txtRoutineInstructions.AccessibleName = null;
            resources.ApplyResources(this.txtRoutineInstructions, "txtRoutineInstructions");
            this.txtRoutineInstructions.BackColor = System.Drawing.SystemColors.Window;
            this.txtRoutineInstructions.BackgroundImage = null;
            this.txtRoutineInstructions.Font = null;
            this.txtRoutineInstructions.Name = "txtRoutineInstructions";
            this.txtRoutineInstructions.ReadOnly = true;
            this.toolTips.SetToolTip(this.txtRoutineInstructions, resources.GetString("txtRoutineInstructions.ToolTip"));
            // 
            // cboRoutines
            // 
            this.cboRoutines.AccessibleDescription = null;
            this.cboRoutines.AccessibleName = null;
            resources.ApplyResources(this.cboRoutines, "cboRoutines");
            this.cboRoutines.BackgroundImage = null;
            this.cboRoutines.DataSource = this.routinesBindingSource;
            this.cboRoutines.DisplayMember = "Name";
            this.cboRoutines.DropDownStyle = System.Windows.Forms.ComboBoxStyle.DropDownList;
            this.cboRoutines.Font = null;
            this.cboRoutines.Name = "cboRoutines";
            this.toolTips.SetToolTip(this.cboRoutines, resources.GetString("cboRoutines.ToolTip"));
            this.cboRoutines.SelectedIndexChanged += new System.EventHandler(this.cboRoutines_SelectedIndexChanged);
            // 
            // routinesBindingSource
            // 
            this.routinesBindingSource.DataSource = typeof(DnEcuDiag.Ecus.Generic.Routine);
            // 
            // lblRoutine
            // 
            this.lblRoutine.AccessibleDescription = null;
            this.lblRoutine.AccessibleName = null;
            resources.ApplyResources(this.lblRoutine, "lblRoutine");
            this.lblRoutine.Font = null;
            this.lblRoutine.Name = "lblRoutine";
            this.toolTips.SetToolTip(this.lblRoutine, resources.GetString("lblRoutine.ToolTip"));
            // 
            // pbCalibration
            // 
            this.pbCalibration.AccessibleDescription = null;
            this.pbCalibration.AccessibleName = null;
            resources.ApplyResources(this.pbCalibration, "pbCalibration");
            this.pbCalibration.BackgroundImage = null;
            this.pbCalibration.Font = null;
            this.pbCalibration.Name = "pbCalibration";
            this.pbCalibration.Step = 1;
            this.toolTips.SetToolTip(this.pbCalibration, resources.GetString("pbCalibration.ToolTip"));
            // 
            // txtRoutineExecution
            // 
            this.txtRoutineExecution.AccessibleDescription = null;
            this.txtRoutineExecution.AccessibleName = null;
            resources.ApplyResources(this.txtRoutineExecution, "txtRoutineExecution");
            this.txtRoutineExecution.BackColor = System.Drawing.SystemColors.Window;
            this.txtRoutineExecution.BackgroundImage = null;
            this.txtRoutineExecution.Font = null;
            this.txtRoutineExecution.Name = "txtRoutineExecution";
            this.txtRoutineExecution.ReadOnly = true;
            this.toolTips.SetToolTip(this.txtRoutineExecution, resources.GetString("txtRoutineExecution.ToolTip"));
            // 
            // tabFaultCodes
            // 
            this.tabFaultCodes.AccessibleDescription = null;
            this.tabFaultCodes.AccessibleName = null;
            resources.ApplyResources(this.tabFaultCodes, "tabFaultCodes");
            this.tabFaultCodes.BackgroundImage = null;
            this.tabFaultCodes.Controls.Add(this.gbSnapshots);
            this.tabFaultCodes.Controls.Add(this.gbFaultCodes);
            this.tabFaultCodes.Font = null;
            this.tabFaultCodes.Name = "tabFaultCodes";
            this.toolTips.SetToolTip(this.tabFaultCodes, resources.GetString("tabFaultCodes.ToolTip"));
            this.tabFaultCodes.UseVisualStyleBackColor = true;
            // 
            // gbSnapshots
            // 
            this.gbSnapshots.AccessibleDescription = null;
            this.gbSnapshots.AccessibleName = null;
            resources.ApplyResources(this.gbSnapshots, "gbSnapshots");
            this.gbSnapshots.BackgroundImage = null;
            this.gbSnapshots.Controls.Add(this.cboSnapShots);
            this.gbSnapshots.Controls.Add(this.gbSnapShotDescription);
            this.gbSnapshots.Controls.Add(this.btnReadSnapShot);
            this.gbSnapshots.Font = null;
            this.gbSnapshots.Name = "gbSnapshots";
            this.gbSnapshots.TabStop = false;
            this.toolTips.SetToolTip(this.gbSnapshots, resources.GetString("gbSnapshots.ToolTip"));
            // 
            // cboSnapShots
            // 
            this.cboSnapShots.AccessibleDescription = null;
            this.cboSnapShots.AccessibleName = null;
            resources.ApplyResources(this.cboSnapShots, "cboSnapShots");
            this.cboSnapShots.BackgroundImage = null;
            this.cboSnapShots.DataSource = this.snapshotsBindingSource;
            this.cboSnapShots.DisplayMember = "Name";
            this.cboSnapShots.DropDownStyle = System.Windows.Forms.ComboBoxStyle.DropDownList;
            this.cboSnapShots.Name = "cboSnapShots";
            this.toolTips.SetToolTip(this.cboSnapShots, resources.GetString("cboSnapShots.ToolTip"));
            this.cboSnapShots.SelectedIndexChanged += new System.EventHandler(this.cboSnapShots_SelectedIndexChanged);
            // 
            // snapshotsBindingSource
            // 
            this.snapshotsBindingSource.DataSource = typeof(DnEcuDiag.Ecus.Generic.SnapshotDefinition);
            // 
            // gbSnapShotDescription
            // 
            this.gbSnapShotDescription.AccessibleDescription = null;
            this.gbSnapShotDescription.AccessibleName = null;
            resources.ApplyResources(this.gbSnapShotDescription, "gbSnapShotDescription");
            this.gbSnapShotDescription.BackgroundImage = null;
            this.gbSnapShotDescription.Controls.Add(this.txtSnapShotText);
            this.gbSnapShotDescription.Font = null;
            this.gbSnapShotDescription.Name = "gbSnapShotDescription";
            this.gbSnapShotDescription.TabStop = false;
            this.toolTips.SetToolTip(this.gbSnapShotDescription, resources.GetString("gbSnapShotDescription.ToolTip"));
            // 
            // txtSnapShotText
            // 
            this.txtSnapShotText.AccessibleDescription = null;
            this.txtSnapShotText.AccessibleName = null;
            resources.ApplyResources(this.txtSnapShotText, "txtSnapShotText");
            this.txtSnapShotText.BackColor = System.Drawing.SystemColors.Window;
            this.txtSnapShotText.BackgroundImage = null;
            this.txtSnapShotText.Name = "txtSnapShotText";
            this.txtSnapShotText.ReadOnly = true;
            this.toolTips.SetToolTip(this.txtSnapShotText, resources.GetString("txtSnapShotText.ToolTip"));
            // 
            // btnReadSnapShot
            // 
            this.btnReadSnapShot.AccessibleDescription = null;
            this.btnReadSnapShot.AccessibleName = null;
            resources.ApplyResources(this.btnReadSnapShot, "btnReadSnapShot");
            this.btnReadSnapShot.BackgroundImage = null;
            this.btnReadSnapShot.Font = null;
            this.btnReadSnapShot.Name = "btnReadSnapShot";
            this.toolTips.SetToolTip(this.btnReadSnapShot, resources.GetString("btnReadSnapShot.ToolTip"));
            this.btnReadSnapShot.Click += new System.EventHandler(this.btnReadSnapShot_Click);
            // 
            // gbFaultCodes
            // 
            this.gbFaultCodes.AccessibleDescription = null;
            this.gbFaultCodes.AccessibleName = null;
            resources.ApplyResources(this.gbFaultCodes, "gbFaultCodes");
            this.gbFaultCodes.BackgroundImage = null;
            this.gbFaultCodes.Controls.Add(this.grpFaultCodeDescription);
            this.gbFaultCodes.Controls.Add(this.gbFaultCodeList);
            this.gbFaultCodes.Controls.Add(this.btnClearDTCs);
            this.gbFaultCodes.Controls.Add(this.btnReadDTCs);
            this.gbFaultCodes.Font = null;
            this.gbFaultCodes.Name = "gbFaultCodes";
            this.gbFaultCodes.TabStop = false;
            this.toolTips.SetToolTip(this.gbFaultCodes, resources.GetString("gbFaultCodes.ToolTip"));
            // 
            // grpFaultCodeDescription
            // 
            this.grpFaultCodeDescription.AccessibleDescription = null;
            this.grpFaultCodeDescription.AccessibleName = null;
            resources.ApplyResources(this.grpFaultCodeDescription, "grpFaultCodeDescription");
            this.grpFaultCodeDescription.BackgroundImage = null;
            this.grpFaultCodeDescription.Controls.Add(this.txtFaultCodeDescription);
            this.grpFaultCodeDescription.Font = null;
            this.grpFaultCodeDescription.Name = "grpFaultCodeDescription";
            this.grpFaultCodeDescription.TabStop = false;
            this.toolTips.SetToolTip(this.grpFaultCodeDescription, resources.GetString("grpFaultCodeDescription.ToolTip"));
            // 
            // txtFaultCodeDescription
            // 
            this.txtFaultCodeDescription.AccessibleDescription = null;
            this.txtFaultCodeDescription.AccessibleName = null;
            resources.ApplyResources(this.txtFaultCodeDescription, "txtFaultCodeDescription");
            this.txtFaultCodeDescription.BackColor = System.Drawing.SystemColors.Window;
            this.txtFaultCodeDescription.BackgroundImage = null;
            this.txtFaultCodeDescription.DataBindings.Add(new System.Windows.Forms.Binding("Text", this.faultCodesBindingSource, "FaultDescription", true));
            this.txtFaultCodeDescription.Name = "txtFaultCodeDescription";
            this.txtFaultCodeDescription.ReadOnly = true;
            this.toolTips.SetToolTip(this.txtFaultCodeDescription, resources.GetString("txtFaultCodeDescription.ToolTip"));
            // 
            // faultCodesBindingSource
            // 
            this.faultCodesBindingSource.DataSource = typeof(DnEcuDiag.Ecus.Generic.FaultCode);
            // 
            // gbFaultCodeList
            // 
            this.gbFaultCodeList.AccessibleDescription = null;
            this.gbFaultCodeList.AccessibleName = null;
            resources.ApplyResources(this.gbFaultCodeList, "gbFaultCodeList");
            this.gbFaultCodeList.BackgroundImage = null;
            this.gbFaultCodeList.Controls.Add(this.lstBoxFaultCodes);
            this.gbFaultCodeList.Font = null;
            this.gbFaultCodeList.Name = "gbFaultCodeList";
            this.gbFaultCodeList.TabStop = false;
            this.toolTips.SetToolTip(this.gbFaultCodeList, resources.GetString("gbFaultCodeList.ToolTip"));
            // 
            // lstBoxFaultCodes
            // 
            this.lstBoxFaultCodes.AccessibleDescription = null;
            this.lstBoxFaultCodes.AccessibleName = null;
            resources.ApplyResources(this.lstBoxFaultCodes, "lstBoxFaultCodes");
            this.lstBoxFaultCodes.BackgroundImage = null;
            this.lstBoxFaultCodes.DataSource = this.faultCodesBindingSource;
            this.lstBoxFaultCodes.DisplayMember = "FaultComponent";
            this.lstBoxFaultCodes.Name = "lstBoxFaultCodes";
            this.toolTips.SetToolTip(this.lstBoxFaultCodes, resources.GetString("lstBoxFaultCodes.ToolTip"));
            // 
            // btnClearDTCs
            // 
            this.btnClearDTCs.AccessibleDescription = null;
            this.btnClearDTCs.AccessibleName = null;
            resources.ApplyResources(this.btnClearDTCs, "btnClearDTCs");
            this.btnClearDTCs.BackgroundImage = null;
            this.btnClearDTCs.Font = null;
            this.btnClearDTCs.Name = "btnClearDTCs";
            this.toolTips.SetToolTip(this.btnClearDTCs, resources.GetString("btnClearDTCs.ToolTip"));
            this.btnClearDTCs.Click += new System.EventHandler(this.btnClearDTCs_Click);
            // 
            // btnReadDTCs
            // 
            this.btnReadDTCs.AccessibleDescription = null;
            this.btnReadDTCs.AccessibleName = null;
            resources.ApplyResources(this.btnReadDTCs, "btnReadDTCs");
            this.btnReadDTCs.BackgroundImage = null;
            this.btnReadDTCs.Font = null;
            this.btnReadDTCs.Name = "btnReadDTCs";
            this.toolTips.SetToolTip(this.btnReadDTCs, resources.GetString("btnReadDTCs.ToolTip"));
            this.btnReadDTCs.Click += new System.EventHandler(this.btnReadDTCs_Click);
            // 
            // tabPerformance
            // 
            this.tabPerformance.AccessibleDescription = null;
            this.tabPerformance.AccessibleName = null;
            resources.ApplyResources(this.tabPerformance, "tabPerformance");
            this.tabPerformance.BackgroundImage = null;
            this.tabPerformance.Controls.Add(this.gbPerformance);
            this.tabPerformance.Font = null;
            this.tabPerformance.Name = "tabPerformance";
            this.toolTips.SetToolTip(this.tabPerformance, resources.GetString("tabPerformance.ToolTip"));
            this.tabPerformance.UseVisualStyleBackColor = true;
            // 
            // gbPerformance
            // 
            this.gbPerformance.AccessibleDescription = null;
            this.gbPerformance.AccessibleName = null;
            resources.ApplyResources(this.gbPerformance, "gbPerformance");
            this.gbPerformance.BackgroundImage = null;
            this.gbPerformance.Controls.Add(this.gbPerformanceRunStore);
            this.gbPerformance.Controls.Add(this.gbPerformanceRunData);
            this.gbPerformance.Font = null;
            this.gbPerformance.Name = "gbPerformance";
            this.gbPerformance.TabStop = false;
            this.toolTips.SetToolTip(this.gbPerformance, resources.GetString("gbPerformance.ToolTip"));
            // 
            // gbPerformanceRunStore
            // 
            this.gbPerformanceRunStore.AccessibleDescription = null;
            this.gbPerformanceRunStore.AccessibleName = null;
            resources.ApplyResources(this.gbPerformanceRunStore, "gbPerformanceRunStore");
            this.gbPerformanceRunStore.BackgroundImage = null;
            this.gbPerformanceRunStore.Controls.Add(this.lblVehicle);
            this.gbPerformanceRunStore.Controls.Add(this.btnLoadPerfRun);
            this.gbPerformanceRunStore.Controls.Add(this.btnSavePerfRun);
            this.gbPerformanceRunStore.Controls.Add(this.btnStopPerformance);
            this.gbPerformanceRunStore.Controls.Add(this.btnStartPerformance);
            this.gbPerformanceRunStore.Controls.Add(this.txtPerformanceNotes);
            this.gbPerformanceRunStore.Controls.Add(this.label2);
            this.gbPerformanceRunStore.Controls.Add(this.txtPerformanceVehicle);
            this.gbPerformanceRunStore.Font = null;
            this.gbPerformanceRunStore.Name = "gbPerformanceRunStore";
            this.gbPerformanceRunStore.TabStop = false;
            this.toolTips.SetToolTip(this.gbPerformanceRunStore, resources.GetString("gbPerformanceRunStore.ToolTip"));
            // 
            // lblVehicle
            // 
            this.lblVehicle.AccessibleDescription = null;
            this.lblVehicle.AccessibleName = null;
            resources.ApplyResources(this.lblVehicle, "lblVehicle");
            this.lblVehicle.Font = null;
            this.lblVehicle.Name = "lblVehicle";
            this.toolTips.SetToolTip(this.lblVehicle, resources.GetString("lblVehicle.ToolTip"));
            // 
            // btnLoadPerfRun
            // 
            this.btnLoadPerfRun.AccessibleDescription = null;
            this.btnLoadPerfRun.AccessibleName = null;
            resources.ApplyResources(this.btnLoadPerfRun, "btnLoadPerfRun");
            this.btnLoadPerfRun.BackgroundImage = null;
            this.btnLoadPerfRun.Font = null;
            this.btnLoadPerfRun.Name = "btnLoadPerfRun";
            this.toolTips.SetToolTip(this.btnLoadPerfRun, resources.GetString("btnLoadPerfRun.ToolTip"));
            this.btnLoadPerfRun.Click += new System.EventHandler(this.btnLoadPerfRun_Click);
            // 
            // btnSavePerfRun
            // 
            this.btnSavePerfRun.AccessibleDescription = null;
            this.btnSavePerfRun.AccessibleName = null;
            resources.ApplyResources(this.btnSavePerfRun, "btnSavePerfRun");
            this.btnSavePerfRun.BackgroundImage = null;
            this.btnSavePerfRun.Font = null;
            this.btnSavePerfRun.Name = "btnSavePerfRun";
            this.toolTips.SetToolTip(this.btnSavePerfRun, resources.GetString("btnSavePerfRun.ToolTip"));
            this.btnSavePerfRun.Click += new System.EventHandler(this.btnSavePerfRun_Click);
            // 
            // btnStopPerformance
            // 
            this.btnStopPerformance.AccessibleDescription = null;
            this.btnStopPerformance.AccessibleName = null;
            resources.ApplyResources(this.btnStopPerformance, "btnStopPerformance");
            this.btnStopPerformance.BackgroundImage = null;
            this.btnStopPerformance.Font = null;
            this.btnStopPerformance.Name = "btnStopPerformance";
            this.toolTips.SetToolTip(this.btnStopPerformance, resources.GetString("btnStopPerformance.ToolTip"));
            this.btnStopPerformance.Click += new System.EventHandler(this.btnStopPerformance_Click);
            // 
            // btnStartPerformance
            // 
            this.btnStartPerformance.AccessibleDescription = null;
            this.btnStartPerformance.AccessibleName = null;
            resources.ApplyResources(this.btnStartPerformance, "btnStartPerformance");
            this.btnStartPerformance.BackgroundImage = null;
            this.btnStartPerformance.Font = null;
            this.btnStartPerformance.Name = "btnStartPerformance";
            this.toolTips.SetToolTip(this.btnStartPerformance, resources.GetString("btnStartPerformance.ToolTip"));
            this.btnStartPerformance.Click += new System.EventHandler(this.btnStartPerformance_Click);
            // 
            // txtPerformanceNotes
            // 
            this.txtPerformanceNotes.AccessibleDescription = null;
            this.txtPerformanceNotes.AccessibleName = null;
            resources.ApplyResources(this.txtPerformanceNotes, "txtPerformanceNotes");
            this.txtPerformanceNotes.BackgroundImage = null;
            this.txtPerformanceNotes.DataBindings.Add(new System.Windows.Forms.Binding("Text", this.vehiclePerformanceRunBindingSource, "Notes", true));
            this.txtPerformanceNotes.Font = null;
            this.txtPerformanceNotes.Name = "txtPerformanceNotes";
            this.toolTips.SetToolTip(this.txtPerformanceNotes, resources.GetString("txtPerformanceNotes.ToolTip"));
            // 
            // vehiclePerformanceRunBindingSource
            // 
            this.vehiclePerformanceRunBindingSource.DataSource = typeof(DnEcuDiag.Configuration.VehiclePerformanceRun);
            // 
            // label2
            // 
            this.label2.AccessibleDescription = null;
            this.label2.AccessibleName = null;
            resources.ApplyResources(this.label2, "label2");
            this.label2.Font = null;
            this.label2.Name = "label2";
            this.toolTips.SetToolTip(this.label2, resources.GetString("label2.ToolTip"));
            // 
            // txtPerformanceVehicle
            // 
            this.txtPerformanceVehicle.AccessibleDescription = null;
            this.txtPerformanceVehicle.AccessibleName = null;
            resources.ApplyResources(this.txtPerformanceVehicle, "txtPerformanceVehicle");
            this.txtPerformanceVehicle.BackgroundImage = null;
            this.txtPerformanceVehicle.DataBindings.Add(new System.Windows.Forms.Binding("Text", this.vehiclePerformanceRunBindingSource, "Vehicle", true));
            this.txtPerformanceVehicle.Font = null;
            this.txtPerformanceVehicle.Name = "txtPerformanceVehicle";
            this.toolTips.SetToolTip(this.txtPerformanceVehicle, resources.GetString("txtPerformanceVehicle.ToolTip"));
            // 
            // gbPerformanceRunData
            // 
            this.gbPerformanceRunData.AccessibleDescription = null;
            this.gbPerformanceRunData.AccessibleName = null;
            resources.ApplyResources(this.gbPerformanceRunData, "gbPerformanceRunData");
            this.gbPerformanceRunData.BackgroundImage = null;
            this.gbPerformanceRunData.Controls.Add(this.dCCPerf);
            this.gbPerformanceRunData.Controls.Add(this.txtFourHundredMtrSpeed);
            this.gbPerformanceRunData.Controls.Add(this.txtFourHundredMtrTime);
            this.gbPerformanceRunData.Controls.Add(this.txtThreeHundredMtrTime);
            this.gbPerformanceRunData.Controls.Add(this.txtTwoHundredMtrSpeed);
            this.gbPerformanceRunData.Controls.Add(this.txtTwoHundredMtrTime);
            this.gbPerformanceRunData.Controls.Add(this.txtOneHundredMtrTime);
            this.gbPerformanceRunData.Controls.Add(this.txtNoughtOneHundredkphTime);
            this.gbPerformanceRunData.Controls.Add(this.txtTwentyMtrTime);
            this.gbPerformanceRunData.Controls.Add(this.label12);
            this.gbPerformanceRunData.Controls.Add(this.label13);
            this.gbPerformanceRunData.Controls.Add(this.label14);
            this.gbPerformanceRunData.Controls.Add(this.label15);
            this.gbPerformanceRunData.Controls.Add(this.label16);
            this.gbPerformanceRunData.Controls.Add(this.label17);
            this.gbPerformanceRunData.Controls.Add(this.label18);
            this.gbPerformanceRunData.Controls.Add(this.label19);
            this.gbPerformanceRunData.Controls.Add(this.txtQuarterMileSpeed);
            this.gbPerformanceRunData.Controls.Add(this.txtQuarterMileTime);
            this.gbPerformanceRunData.Controls.Add(this.txtThousandFtTime);
            this.gbPerformanceRunData.Controls.Add(this.txtEightMileSpeed);
            this.gbPerformanceRunData.Controls.Add(this.txtEightMileTime);
            this.gbPerformanceRunData.Controls.Add(this.txtThreeThirtyFtTime);
            this.gbPerformanceRunData.Controls.Add(this.txtNoughtSixtyMphTime);
            this.gbPerformanceRunData.Controls.Add(this.txtSixtyFtTime);
            this.gbPerformanceRunData.Controls.Add(this.label10);
            this.gbPerformanceRunData.Controls.Add(this.label11);
            this.gbPerformanceRunData.Controls.Add(this.label4);
            this.gbPerformanceRunData.Controls.Add(this.label9);
            this.gbPerformanceRunData.Controls.Add(this.label7);
            this.gbPerformanceRunData.Controls.Add(this.label6);
            this.gbPerformanceRunData.Controls.Add(this.label5);
            this.gbPerformanceRunData.Controls.Add(this.lblSixty);
            this.gbPerformanceRunData.Font = null;
            this.gbPerformanceRunData.Name = "gbPerformanceRunData";
            this.gbPerformanceRunData.TabStop = false;
            this.toolTips.SetToolTip(this.gbPerformanceRunData, resources.GetString("gbPerformanceRunData.ToolTip"));
            // 
            // dCCPerf
            // 
            this.dCCPerf.AccessibleDescription = null;
            this.dCCPerf.AccessibleName = null;
            resources.ApplyResources(this.dCCPerf, "dCCPerf");
            this.dCCPerf.BackColor = System.Drawing.Color.Black;
            this.dCCPerf.BackgroundImage = null;
            this.dCCPerf.ClockDisplayFormat = InstrumentationControls.ClockFormat.TwelveHourFormat;
            this.dCCPerf.CountdownTime = 10000;
            this.dCCPerf.Font = null;
            this.dCCPerf.Name = "dCCPerf";
            this.dCCPerf.SetClockType = InstrumentationControls.ClockType.Freeze;
            this.dCCPerf.SetDigitalColor = InstrumentationControls.DigitalColor.RedColor;
            this.toolTips.SetToolTip(this.dCCPerf, resources.GetString("dCCPerf.ToolTip"));
            // 
            // txtFourHundredMtrSpeed
            // 
            this.txtFourHundredMtrSpeed.AccessibleDescription = null;
            this.txtFourHundredMtrSpeed.AccessibleName = null;
            resources.ApplyResources(this.txtFourHundredMtrSpeed, "txtFourHundredMtrSpeed");
            this.txtFourHundredMtrSpeed.BackgroundImage = null;
            this.txtFourHundredMtrSpeed.DataBindings.Add(new System.Windows.Forms.Binding("Text", this.vehiclePerformanceRunBindingSource, "FourHundredMeterSpeed", true));
            this.txtFourHundredMtrSpeed.Font = null;
            this.txtFourHundredMtrSpeed.Name = "txtFourHundredMtrSpeed";
            this.txtFourHundredMtrSpeed.ReadOnly = true;
            this.toolTips.SetToolTip(this.txtFourHundredMtrSpeed, resources.GetString("txtFourHundredMtrSpeed.ToolTip"));
            // 
            // txtFourHundredMtrTime
            // 
            this.txtFourHundredMtrTime.AccessibleDescription = null;
            this.txtFourHundredMtrTime.AccessibleName = null;
            resources.ApplyResources(this.txtFourHundredMtrTime, "txtFourHundredMtrTime");
            this.txtFourHundredMtrTime.BackgroundImage = null;
            this.txtFourHundredMtrTime.DataBindings.Add(new System.Windows.Forms.Binding("Text", this.vehiclePerformanceRunBindingSource, "FourHundredMeterTime", true));
            this.txtFourHundredMtrTime.Font = null;
            this.txtFourHundredMtrTime.Name = "txtFourHundredMtrTime";
            this.txtFourHundredMtrTime.ReadOnly = true;
            this.toolTips.SetToolTip(this.txtFourHundredMtrTime, resources.GetString("txtFourHundredMtrTime.ToolTip"));
            // 
            // txtThreeHundredMtrTime
            // 
            this.txtThreeHundredMtrTime.AccessibleDescription = null;
            this.txtThreeHundredMtrTime.AccessibleName = null;
            resources.ApplyResources(this.txtThreeHundredMtrTime, "txtThreeHundredMtrTime");
            this.txtThreeHundredMtrTime.BackgroundImage = null;
            this.txtThreeHundredMtrTime.DataBindings.Add(new System.Windows.Forms.Binding("Text", this.vehiclePerformanceRunBindingSource, "ThreeHundredMeterTime", true));
            this.txtThreeHundredMtrTime.Font = null;
            this.txtThreeHundredMtrTime.Name = "txtThreeHundredMtrTime";
            this.txtThreeHundredMtrTime.ReadOnly = true;
            this.toolTips.SetToolTip(this.txtThreeHundredMtrTime, resources.GetString("txtThreeHundredMtrTime.ToolTip"));
            // 
            // txtTwoHundredMtrSpeed
            // 
            this.txtTwoHundredMtrSpeed.AccessibleDescription = null;
            this.txtTwoHundredMtrSpeed.AccessibleName = null;
            resources.ApplyResources(this.txtTwoHundredMtrSpeed, "txtTwoHundredMtrSpeed");
            this.txtTwoHundredMtrSpeed.BackgroundImage = null;
            this.txtTwoHundredMtrSpeed.DataBindings.Add(new System.Windows.Forms.Binding("Text", this.vehiclePerformanceRunBindingSource, "TwoHundredMeterSpeed", true));
            this.txtTwoHundredMtrSpeed.Font = null;
            this.txtTwoHundredMtrSpeed.Name = "txtTwoHundredMtrSpeed";
            this.txtTwoHundredMtrSpeed.ReadOnly = true;
            this.toolTips.SetToolTip(this.txtTwoHundredMtrSpeed, resources.GetString("txtTwoHundredMtrSpeed.ToolTip"));
            // 
            // txtTwoHundredMtrTime
            // 
            this.txtTwoHundredMtrTime.AccessibleDescription = null;
            this.txtTwoHundredMtrTime.AccessibleName = null;
            resources.ApplyResources(this.txtTwoHundredMtrTime, "txtTwoHundredMtrTime");
            this.txtTwoHundredMtrTime.BackgroundImage = null;
            this.txtTwoHundredMtrTime.DataBindings.Add(new System.Windows.Forms.Binding("Text", this.vehiclePerformanceRunBindingSource, "TwentyMeterTime", true));
            this.txtTwoHundredMtrTime.Font = null;
            this.txtTwoHundredMtrTime.Name = "txtTwoHundredMtrTime";
            this.txtTwoHundredMtrTime.ReadOnly = true;
            this.toolTips.SetToolTip(this.txtTwoHundredMtrTime, resources.GetString("txtTwoHundredMtrTime.ToolTip"));
            // 
            // txtOneHundredMtrTime
            // 
            this.txtOneHundredMtrTime.AccessibleDescription = null;
            this.txtOneHundredMtrTime.AccessibleName = null;
            resources.ApplyResources(this.txtOneHundredMtrTime, "txtOneHundredMtrTime");
            this.txtOneHundredMtrTime.BackgroundImage = null;
            this.txtOneHundredMtrTime.DataBindings.Add(new System.Windows.Forms.Binding("Text", this.vehiclePerformanceRunBindingSource, "OneHundredKphTime", true));
            this.txtOneHundredMtrTime.Font = null;
            this.txtOneHundredMtrTime.Name = "txtOneHundredMtrTime";
            this.txtOneHundredMtrTime.ReadOnly = true;
            this.toolTips.SetToolTip(this.txtOneHundredMtrTime, resources.GetString("txtOneHundredMtrTime.ToolTip"));
            // 
            // txtNoughtOneHundredkphTime
            // 
            this.txtNoughtOneHundredkphTime.AccessibleDescription = null;
            this.txtNoughtOneHundredkphTime.AccessibleName = null;
            resources.ApplyResources(this.txtNoughtOneHundredkphTime, "txtNoughtOneHundredkphTime");
            this.txtNoughtOneHundredkphTime.BackgroundImage = null;
            this.txtNoughtOneHundredkphTime.DataBindings.Add(new System.Windows.Forms.Binding("Text", this.vehiclePerformanceRunBindingSource, "OneHundredKphTime", true));
            this.txtNoughtOneHundredkphTime.Font = null;
            this.txtNoughtOneHundredkphTime.Name = "txtNoughtOneHundredkphTime";
            this.txtNoughtOneHundredkphTime.ReadOnly = true;
            this.toolTips.SetToolTip(this.txtNoughtOneHundredkphTime, resources.GetString("txtNoughtOneHundredkphTime.ToolTip"));
            // 
            // txtTwentyMtrTime
            // 
            this.txtTwentyMtrTime.AccessibleDescription = null;
            this.txtTwentyMtrTime.AccessibleName = null;
            resources.ApplyResources(this.txtTwentyMtrTime, "txtTwentyMtrTime");
            this.txtTwentyMtrTime.BackgroundImage = null;
            this.txtTwentyMtrTime.DataBindings.Add(new System.Windows.Forms.Binding("Text", this.vehiclePerformanceRunBindingSource, "TwentyMeterTime", true));
            this.txtTwentyMtrTime.Font = null;
            this.txtTwentyMtrTime.Name = "txtTwentyMtrTime";
            this.txtTwentyMtrTime.ReadOnly = true;
            this.toolTips.SetToolTip(this.txtTwentyMtrTime, resources.GetString("txtTwentyMtrTime.ToolTip"));
            // 
            // label12
            // 
            this.label12.AccessibleDescription = null;
            this.label12.AccessibleName = null;
            resources.ApplyResources(this.label12, "label12");
            this.label12.Font = null;
            this.label12.Name = "label12";
            this.toolTips.SetToolTip(this.label12, resources.GetString("label12.ToolTip"));
            // 
            // label13
            // 
            this.label13.AccessibleDescription = null;
            this.label13.AccessibleName = null;
            resources.ApplyResources(this.label13, "label13");
            this.label13.Font = null;
            this.label13.Name = "label13";
            this.toolTips.SetToolTip(this.label13, resources.GetString("label13.ToolTip"));
            // 
            // label14
            // 
            this.label14.AccessibleDescription = null;
            this.label14.AccessibleName = null;
            resources.ApplyResources(this.label14, "label14");
            this.label14.Font = null;
            this.label14.Name = "label14";
            this.toolTips.SetToolTip(this.label14, resources.GetString("label14.ToolTip"));
            // 
            // label15
            // 
            this.label15.AccessibleDescription = null;
            this.label15.AccessibleName = null;
            resources.ApplyResources(this.label15, "label15");
            this.label15.Font = null;
            this.label15.Name = "label15";
            this.toolTips.SetToolTip(this.label15, resources.GetString("label15.ToolTip"));
            // 
            // label16
            // 
            this.label16.AccessibleDescription = null;
            this.label16.AccessibleName = null;
            resources.ApplyResources(this.label16, "label16");
            this.label16.Font = null;
            this.label16.Name = "label16";
            this.toolTips.SetToolTip(this.label16, resources.GetString("label16.ToolTip"));
            // 
            // label17
            // 
            this.label17.AccessibleDescription = null;
            this.label17.AccessibleName = null;
            resources.ApplyResources(this.label17, "label17");
            this.label17.Font = null;
            this.label17.Name = "label17";
            this.toolTips.SetToolTip(this.label17, resources.GetString("label17.ToolTip"));
            // 
            // label18
            // 
            this.label18.AccessibleDescription = null;
            this.label18.AccessibleName = null;
            resources.ApplyResources(this.label18, "label18");
            this.label18.Font = null;
            this.label18.Name = "label18";
            this.toolTips.SetToolTip(this.label18, resources.GetString("label18.ToolTip"));
            // 
            // label19
            // 
            this.label19.AccessibleDescription = null;
            this.label19.AccessibleName = null;
            resources.ApplyResources(this.label19, "label19");
            this.label19.Font = null;
            this.label19.Name = "label19";
            this.toolTips.SetToolTip(this.label19, resources.GetString("label19.ToolTip"));
            // 
            // txtQuarterMileSpeed
            // 
            this.txtQuarterMileSpeed.AccessibleDescription = null;
            this.txtQuarterMileSpeed.AccessibleName = null;
            resources.ApplyResources(this.txtQuarterMileSpeed, "txtQuarterMileSpeed");
            this.txtQuarterMileSpeed.BackgroundImage = null;
            this.txtQuarterMileSpeed.DataBindings.Add(new System.Windows.Forms.Binding("Text", this.vehiclePerformanceRunBindingSource, "QuarterMileSpeed", true));
            this.txtQuarterMileSpeed.Font = null;
            this.txtQuarterMileSpeed.Name = "txtQuarterMileSpeed";
            this.txtQuarterMileSpeed.ReadOnly = true;
            this.toolTips.SetToolTip(this.txtQuarterMileSpeed, resources.GetString("txtQuarterMileSpeed.ToolTip"));
            // 
            // txtQuarterMileTime
            // 
            this.txtQuarterMileTime.AccessibleDescription = null;
            this.txtQuarterMileTime.AccessibleName = null;
            resources.ApplyResources(this.txtQuarterMileTime, "txtQuarterMileTime");
            this.txtQuarterMileTime.BackgroundImage = null;
            this.txtQuarterMileTime.DataBindings.Add(new System.Windows.Forms.Binding("Text", this.vehiclePerformanceRunBindingSource, "QuarterMileTime", true));
            this.txtQuarterMileTime.Font = null;
            this.txtQuarterMileTime.Name = "txtQuarterMileTime";
            this.txtQuarterMileTime.ReadOnly = true;
            this.toolTips.SetToolTip(this.txtQuarterMileTime, resources.GetString("txtQuarterMileTime.ToolTip"));
            // 
            // txtThousandFtTime
            // 
            this.txtThousandFtTime.AccessibleDescription = null;
            this.txtThousandFtTime.AccessibleName = null;
            resources.ApplyResources(this.txtThousandFtTime, "txtThousandFtTime");
            this.txtThousandFtTime.BackgroundImage = null;
            this.txtThousandFtTime.DataBindings.Add(new System.Windows.Forms.Binding("Text", this.vehiclePerformanceRunBindingSource, "ThousandFootTime", true));
            this.txtThousandFtTime.Font = null;
            this.txtThousandFtTime.Name = "txtThousandFtTime";
            this.txtThousandFtTime.ReadOnly = true;
            this.toolTips.SetToolTip(this.txtThousandFtTime, resources.GetString("txtThousandFtTime.ToolTip"));
            // 
            // txtEightMileSpeed
            // 
            this.txtEightMileSpeed.AccessibleDescription = null;
            this.txtEightMileSpeed.AccessibleName = null;
            resources.ApplyResources(this.txtEightMileSpeed, "txtEightMileSpeed");
            this.txtEightMileSpeed.BackgroundImage = null;
            this.txtEightMileSpeed.DataBindings.Add(new System.Windows.Forms.Binding("Text", this.vehiclePerformanceRunBindingSource, "EighthMileSpeed", true));
            this.txtEightMileSpeed.Font = null;
            this.txtEightMileSpeed.Name = "txtEightMileSpeed";
            this.txtEightMileSpeed.ReadOnly = true;
            this.toolTips.SetToolTip(this.txtEightMileSpeed, resources.GetString("txtEightMileSpeed.ToolTip"));
            // 
            // txtEightMileTime
            // 
            this.txtEightMileTime.AccessibleDescription = null;
            this.txtEightMileTime.AccessibleName = null;
            resources.ApplyResources(this.txtEightMileTime, "txtEightMileTime");
            this.txtEightMileTime.BackgroundImage = null;
            this.txtEightMileTime.DataBindings.Add(new System.Windows.Forms.Binding("Text", this.vehiclePerformanceRunBindingSource, "EighthMileTime", true));
            this.txtEightMileTime.Font = null;
            this.txtEightMileTime.Name = "txtEightMileTime";
            this.txtEightMileTime.ReadOnly = true;
            this.toolTips.SetToolTip(this.txtEightMileTime, resources.GetString("txtEightMileTime.ToolTip"));
            // 
            // txtThreeThirtyFtTime
            // 
            this.txtThreeThirtyFtTime.AccessibleDescription = null;
            this.txtThreeThirtyFtTime.AccessibleName = null;
            resources.ApplyResources(this.txtThreeThirtyFtTime, "txtThreeThirtyFtTime");
            this.txtThreeThirtyFtTime.BackgroundImage = null;
            this.txtThreeThirtyFtTime.DataBindings.Add(new System.Windows.Forms.Binding("Text", this.vehiclePerformanceRunBindingSource, "ThreeThirtyTime", true));
            this.txtThreeThirtyFtTime.Font = null;
            this.txtThreeThirtyFtTime.Name = "txtThreeThirtyFtTime";
            this.txtThreeThirtyFtTime.ReadOnly = true;
            this.toolTips.SetToolTip(this.txtThreeThirtyFtTime, resources.GetString("txtThreeThirtyFtTime.ToolTip"));
            // 
            // txtNoughtSixtyMphTime
            // 
            this.txtNoughtSixtyMphTime.AccessibleDescription = null;
            this.txtNoughtSixtyMphTime.AccessibleName = null;
            resources.ApplyResources(this.txtNoughtSixtyMphTime, "txtNoughtSixtyMphTime");
            this.txtNoughtSixtyMphTime.BackgroundImage = null;
            this.txtNoughtSixtyMphTime.DataBindings.Add(new System.Windows.Forms.Binding("Text", this.vehiclePerformanceRunBindingSource, "SixtyMphTime", true));
            this.txtNoughtSixtyMphTime.Font = null;
            this.txtNoughtSixtyMphTime.Name = "txtNoughtSixtyMphTime";
            this.txtNoughtSixtyMphTime.ReadOnly = true;
            this.toolTips.SetToolTip(this.txtNoughtSixtyMphTime, resources.GetString("txtNoughtSixtyMphTime.ToolTip"));
            // 
            // txtSixtyFtTime
            // 
            this.txtSixtyFtTime.AccessibleDescription = null;
            this.txtSixtyFtTime.AccessibleName = null;
            resources.ApplyResources(this.txtSixtyFtTime, "txtSixtyFtTime");
            this.txtSixtyFtTime.BackgroundImage = null;
            this.txtSixtyFtTime.DataBindings.Add(new System.Windows.Forms.Binding("Text", this.vehiclePerformanceRunBindingSource, "SixtyFootTime", true));
            this.txtSixtyFtTime.Font = null;
            this.txtSixtyFtTime.Name = "txtSixtyFtTime";
            this.txtSixtyFtTime.ReadOnly = true;
            this.toolTips.SetToolTip(this.txtSixtyFtTime, resources.GetString("txtSixtyFtTime.ToolTip"));
            // 
            // label10
            // 
            this.label10.AccessibleDescription = null;
            this.label10.AccessibleName = null;
            resources.ApplyResources(this.label10, "label10");
            this.label10.Font = null;
            this.label10.Name = "label10";
            this.toolTips.SetToolTip(this.label10, resources.GetString("label10.ToolTip"));
            // 
            // label11
            // 
            this.label11.AccessibleDescription = null;
            this.label11.AccessibleName = null;
            resources.ApplyResources(this.label11, "label11");
            this.label11.Font = null;
            this.label11.Name = "label11";
            this.toolTips.SetToolTip(this.label11, resources.GetString("label11.ToolTip"));
            // 
            // label4
            // 
            this.label4.AccessibleDescription = null;
            this.label4.AccessibleName = null;
            resources.ApplyResources(this.label4, "label4");
            this.label4.Font = null;
            this.label4.Name = "label4";
            this.toolTips.SetToolTip(this.label4, resources.GetString("label4.ToolTip"));
            // 
            // label9
            // 
            this.label9.AccessibleDescription = null;
            this.label9.AccessibleName = null;
            resources.ApplyResources(this.label9, "label9");
            this.label9.Font = null;
            this.label9.Name = "label9";
            this.toolTips.SetToolTip(this.label9, resources.GetString("label9.ToolTip"));
            // 
            // label7
            // 
            this.label7.AccessibleDescription = null;
            this.label7.AccessibleName = null;
            resources.ApplyResources(this.label7, "label7");
            this.label7.Font = null;
            this.label7.Name = "label7";
            this.toolTips.SetToolTip(this.label7, resources.GetString("label7.ToolTip"));
            // 
            // label6
            // 
            this.label6.AccessibleDescription = null;
            this.label6.AccessibleName = null;
            resources.ApplyResources(this.label6, "label6");
            this.label6.Font = null;
            this.label6.Name = "label6";
            this.toolTips.SetToolTip(this.label6, resources.GetString("label6.ToolTip"));
            // 
            // label5
            // 
            this.label5.AccessibleDescription = null;
            this.label5.AccessibleName = null;
            resources.ApplyResources(this.label5, "label5");
            this.label5.Font = null;
            this.label5.Name = "label5";
            this.toolTips.SetToolTip(this.label5, resources.GetString("label5.ToolTip"));
            // 
            // lblSixty
            // 
            this.lblSixty.AccessibleDescription = null;
            this.lblSixty.AccessibleName = null;
            resources.ApplyResources(this.lblSixty, "lblSixty");
            this.lblSixty.Font = null;
            this.lblSixty.Name = "lblSixty";
            this.toolTips.SetToolTip(this.lblSixty, resources.GetString("lblSixty.ToolTip"));
            // 
            // tabDyno
            // 
            this.tabDyno.AccessibleDescription = null;
            this.tabDyno.AccessibleName = null;
            resources.ApplyResources(this.tabDyno, "tabDyno");
            this.tabDyno.BackgroundImage = null;
            this.tabDyno.Controls.Add(this.gbDynoSetup);
            this.tabDyno.Controls.Add(this.gbDynoResults);
            this.tabDyno.Controls.Add(this.gbVehicleProfile);
            this.tabDyno.Controls.Add(this.gbDynoControl);
            this.tabDyno.Font = null;
            this.tabDyno.Name = "tabDyno";
            this.toolTips.SetToolTip(this.tabDyno, resources.GetString("tabDyno.ToolTip"));
            this.tabDyno.UseVisualStyleBackColor = true;
            // 
            // gbDynoSetup
            // 
            this.gbDynoSetup.AccessibleDescription = null;
            this.gbDynoSetup.AccessibleName = null;
            resources.ApplyResources(this.gbDynoSetup, "gbDynoSetup");
            this.gbDynoSetup.BackgroundImage = null;
            this.gbDynoSetup.Controls.Add(this.nudRpmStart);
            this.gbDynoSetup.Controls.Add(this.nudRpmStop);
            this.gbDynoSetup.Controls.Add(this.lblRpmStop);
            this.gbDynoSetup.Controls.Add(this.lblStartRpm);
            this.gbDynoSetup.Font = null;
            this.gbDynoSetup.Name = "gbDynoSetup";
            this.gbDynoSetup.TabStop = false;
            this.toolTips.SetToolTip(this.gbDynoSetup, resources.GetString("gbDynoSetup.ToolTip"));
            // 
            // nudRpmStart
            // 
            this.nudRpmStart.AccessibleDescription = null;
            this.nudRpmStart.AccessibleName = null;
            resources.ApplyResources(this.nudRpmStart, "nudRpmStart");
            this.nudRpmStart.Font = null;
            this.nudRpmStart.Increment = new decimal(new int[] {
            50,
            0,
            0,
            0});
            this.nudRpmStart.Maximum = new decimal(new int[] {
            3500,
            0,
            0,
            0});
            this.nudRpmStart.Minimum = new decimal(new int[] {
            1000,
            0,
            0,
            0});
            this.nudRpmStart.Name = "nudRpmStart";
            this.toolTips.SetToolTip(this.nudRpmStart, resources.GetString("nudRpmStart.ToolTip"));
            this.nudRpmStart.Value = new decimal(new int[] {
            1000,
            0,
            0,
            0});
            // 
            // nudRpmStop
            // 
            this.nudRpmStop.AccessibleDescription = null;
            this.nudRpmStop.AccessibleName = null;
            resources.ApplyResources(this.nudRpmStop, "nudRpmStop");
            this.nudRpmStop.Font = null;
            this.nudRpmStop.Increment = new decimal(new int[] {
            50,
            0,
            0,
            0});
            this.nudRpmStop.Maximum = new decimal(new int[] {
            7300,
            0,
            0,
            0});
            this.nudRpmStop.Minimum = new decimal(new int[] {
            3500,
            0,
            0,
            0});
            this.nudRpmStop.Name = "nudRpmStop";
            this.toolTips.SetToolTip(this.nudRpmStop, resources.GetString("nudRpmStop.ToolTip"));
            this.nudRpmStop.Value = new decimal(new int[] {
            3500,
            0,
            0,
            0});
            // 
            // lblRpmStop
            // 
            this.lblRpmStop.AccessibleDescription = null;
            this.lblRpmStop.AccessibleName = null;
            resources.ApplyResources(this.lblRpmStop, "lblRpmStop");
            this.lblRpmStop.Font = null;
            this.lblRpmStop.Name = "lblRpmStop";
            this.toolTips.SetToolTip(this.lblRpmStop, resources.GetString("lblRpmStop.ToolTip"));
            // 
            // lblStartRpm
            // 
            this.lblStartRpm.AccessibleDescription = null;
            this.lblStartRpm.AccessibleName = null;
            resources.ApplyResources(this.lblStartRpm, "lblStartRpm");
            this.lblStartRpm.Font = null;
            this.lblStartRpm.Name = "lblStartRpm";
            this.toolTips.SetToolTip(this.lblStartRpm, resources.GetString("lblStartRpm.ToolTip"));
            // 
            // gbDynoResults
            // 
            this.gbDynoResults.AccessibleDescription = null;
            this.gbDynoResults.AccessibleName = null;
            resources.ApplyResources(this.gbDynoResults, "gbDynoResults");
            this.gbDynoResults.BackgroundImage = null;
            this.gbDynoResults.Controls.Add(this.pbDyno);
            this.gbDynoResults.Controls.Add(this.zedGraphDyno);
            this.gbDynoResults.Font = null;
            this.gbDynoResults.Name = "gbDynoResults";
            this.gbDynoResults.TabStop = false;
            this.toolTips.SetToolTip(this.gbDynoResults, resources.GetString("gbDynoResults.ToolTip"));
            // 
            // pbDyno
            // 
            this.pbDyno.AccessibleDescription = null;
            this.pbDyno.AccessibleName = null;
            resources.ApplyResources(this.pbDyno, "pbDyno");
            this.pbDyno.BackgroundImage = null;
            this.pbDyno.Font = null;
            this.pbDyno.Name = "pbDyno";
            this.toolTips.SetToolTip(this.pbDyno, resources.GetString("pbDyno.ToolTip"));
            // 
            // zedGraphDyno
            // 
            this.zedGraphDyno.AccessibleDescription = null;
            this.zedGraphDyno.AccessibleName = null;
            resources.ApplyResources(this.zedGraphDyno, "zedGraphDyno");
            this.zedGraphDyno.BackgroundImage = null;
            this.zedGraphDyno.Font = null;
            this.zedGraphDyno.IsEnablePan = true;
            this.zedGraphDyno.IsEnableZoom = true;
            this.zedGraphDyno.IsShowContextMenu = true;
            this.zedGraphDyno.IsShowPointValues = false;
            this.zedGraphDyno.Name = "zedGraphDyno";
            this.zedGraphDyno.PointDateFormat = "g";
            this.zedGraphDyno.PointValueFormat = "G";
            this.toolTips.SetToolTip(this.zedGraphDyno, resources.GetString("zedGraphDyno.ToolTip"));
            // 
            // gbVehicleProfile
            // 
            this.gbVehicleProfile.AccessibleDescription = null;
            this.gbVehicleProfile.AccessibleName = null;
            resources.ApplyResources(this.gbVehicleProfile, "gbVehicleProfile");
            this.gbVehicleProfile.BackgroundImage = null;
            this.gbVehicleProfile.Controls.Add(this.lblWeghtMeasure);
            this.gbVehicleProfile.Controls.Add(this.nudVehicleWeight);
            this.gbVehicleProfile.Controls.Add(this.btnCaptureFinalDriveRatio);
            this.gbVehicleProfile.Controls.Add(this.txtFinalDriveRatio);
            this.gbVehicleProfile.Controls.Add(this.lblFinalDrive);
            this.gbVehicleProfile.Controls.Add(this.lblWeight);
            this.gbVehicleProfile.Controls.Add(this.cboGear);
            this.gbVehicleProfile.Controls.Add(this.lblGear);
            this.gbVehicleProfile.Controls.Add(this.lblVehicleProfile);
            this.gbVehicleProfile.Controls.Add(this.txtDynoVehicle);
            this.gbVehicleProfile.Controls.Add(this.btnSaveVehicleProfile);
            this.gbVehicleProfile.Controls.Add(this.btnLoadVehicleProfile);
            this.gbVehicleProfile.Font = null;
            this.gbVehicleProfile.Name = "gbVehicleProfile";
            this.gbVehicleProfile.TabStop = false;
            this.toolTips.SetToolTip(this.gbVehicleProfile, resources.GetString("gbVehicleProfile.ToolTip"));
            // 
            // lblWeghtMeasure
            // 
            this.lblWeghtMeasure.AccessibleDescription = null;
            this.lblWeghtMeasure.AccessibleName = null;
            resources.ApplyResources(this.lblWeghtMeasure, "lblWeghtMeasure");
            this.lblWeghtMeasure.Font = null;
            this.lblWeghtMeasure.Name = "lblWeghtMeasure";
            this.toolTips.SetToolTip(this.lblWeghtMeasure, resources.GetString("lblWeghtMeasure.ToolTip"));
            // 
            // nudVehicleWeight
            // 
            this.nudVehicleWeight.AccessibleDescription = null;
            this.nudVehicleWeight.AccessibleName = null;
            resources.ApplyResources(this.nudVehicleWeight, "nudVehicleWeight");
            this.nudVehicleWeight.DataBindings.Add(new System.Windows.Forms.Binding("Value", this.vehicleProfileBindingSource, "VehicleMass", true));
            this.nudVehicleWeight.Font = null;
            this.nudVehicleWeight.Maximum = new decimal(new int[] {
            2500,
            0,
            0,
            0});
            this.nudVehicleWeight.Minimum = new decimal(new int[] {
            1000,
            0,
            0,
            0});
            this.nudVehicleWeight.Name = "nudVehicleWeight";
            this.toolTips.SetToolTip(this.nudVehicleWeight, resources.GetString("nudVehicleWeight.ToolTip"));
            this.nudVehicleWeight.Value = new decimal(new int[] {
            1000,
            0,
            0,
            0});
            // 
            // vehicleProfileBindingSource
            // 
            this.vehicleProfileBindingSource.DataSource = typeof(DnEcuDiag.Configuration.VehicleProfile);
            // 
            // btnCaptureFinalDriveRatio
            // 
            this.btnCaptureFinalDriveRatio.AccessibleDescription = null;
            this.btnCaptureFinalDriveRatio.AccessibleName = null;
            resources.ApplyResources(this.btnCaptureFinalDriveRatio, "btnCaptureFinalDriveRatio");
            this.btnCaptureFinalDriveRatio.BackgroundImage = null;
            this.btnCaptureFinalDriveRatio.Font = null;
            this.btnCaptureFinalDriveRatio.Name = "btnCaptureFinalDriveRatio";
            this.toolTips.SetToolTip(this.btnCaptureFinalDriveRatio, resources.GetString("btnCaptureFinalDriveRatio.ToolTip"));
            this.btnCaptureFinalDriveRatio.Click += new System.EventHandler(this.btnCaptureFinalDriveRatio_Click);
            // 
            // txtFinalDriveRatio
            // 
            this.txtFinalDriveRatio.AccessibleDescription = null;
            this.txtFinalDriveRatio.AccessibleName = null;
            resources.ApplyResources(this.txtFinalDriveRatio, "txtFinalDriveRatio");
            this.txtFinalDriveRatio.BackgroundImage = null;
            this.txtFinalDriveRatio.Font = null;
            this.txtFinalDriveRatio.Name = "txtFinalDriveRatio";
            this.txtFinalDriveRatio.ReadOnly = true;
            this.toolTips.SetToolTip(this.txtFinalDriveRatio, resources.GetString("txtFinalDriveRatio.ToolTip"));
            // 
            // lblFinalDrive
            // 
            this.lblFinalDrive.AccessibleDescription = null;
            this.lblFinalDrive.AccessibleName = null;
            resources.ApplyResources(this.lblFinalDrive, "lblFinalDrive");
            this.lblFinalDrive.Font = null;
            this.lblFinalDrive.Name = "lblFinalDrive";
            this.toolTips.SetToolTip(this.lblFinalDrive, resources.GetString("lblFinalDrive.ToolTip"));
            // 
            // lblWeight
            // 
            this.lblWeight.AccessibleDescription = null;
            this.lblWeight.AccessibleName = null;
            resources.ApplyResources(this.lblWeight, "lblWeight");
            this.lblWeight.Font = null;
            this.lblWeight.Name = "lblWeight";
            this.toolTips.SetToolTip(this.lblWeight, resources.GetString("lblWeight.ToolTip"));
            // 
            // cboGear
            // 
            this.cboGear.AccessibleDescription = null;
            this.cboGear.AccessibleName = null;
            resources.ApplyResources(this.cboGear, "cboGear");
            this.cboGear.BackgroundImage = null;
            this.cboGear.DropDownStyle = System.Windows.Forms.ComboBoxStyle.DropDownList;
            this.cboGear.Font = null;
            this.cboGear.Items.AddRange(new object[] {
            resources.GetString("cboGear.Items"),
            resources.GetString("cboGear.Items1"),
            resources.GetString("cboGear.Items2"),
            resources.GetString("cboGear.Items3"),
            resources.GetString("cboGear.Items4"),
            resources.GetString("cboGear.Items5")});
            this.cboGear.Name = "cboGear";
            this.toolTips.SetToolTip(this.cboGear, resources.GetString("cboGear.ToolTip"));
            this.cboGear.SelectedIndexChanged += new System.EventHandler(this.cboGear_SelectedIndexChanged);
            // 
            // lblGear
            // 
            this.lblGear.AccessibleDescription = null;
            this.lblGear.AccessibleName = null;
            resources.ApplyResources(this.lblGear, "lblGear");
            this.lblGear.Font = null;
            this.lblGear.Name = "lblGear";
            this.toolTips.SetToolTip(this.lblGear, resources.GetString("lblGear.ToolTip"));
            // 
            // lblVehicleProfile
            // 
            this.lblVehicleProfile.AccessibleDescription = null;
            this.lblVehicleProfile.AccessibleName = null;
            resources.ApplyResources(this.lblVehicleProfile, "lblVehicleProfile");
            this.lblVehicleProfile.Font = null;
            this.lblVehicleProfile.Name = "lblVehicleProfile";
            this.toolTips.SetToolTip(this.lblVehicleProfile, resources.GetString("lblVehicleProfile.ToolTip"));
            // 
            // txtDynoVehicle
            // 
            this.txtDynoVehicle.AccessibleDescription = null;
            this.txtDynoVehicle.AccessibleName = null;
            resources.ApplyResources(this.txtDynoVehicle, "txtDynoVehicle");
            this.txtDynoVehicle.BackgroundImage = null;
            this.txtDynoVehicle.DataBindings.Add(new System.Windows.Forms.Binding("Text", this.vehicleProfileBindingSource, "VehicleName", true));
            this.txtDynoVehicle.Font = null;
            this.txtDynoVehicle.Name = "txtDynoVehicle";
            this.toolTips.SetToolTip(this.txtDynoVehicle, resources.GetString("txtDynoVehicle.ToolTip"));
            this.txtDynoVehicle.TextChanged += new System.EventHandler(this.txtDynoVehicle_TextChanged);
            // 
            // btnSaveVehicleProfile
            // 
            this.btnSaveVehicleProfile.AccessibleDescription = null;
            this.btnSaveVehicleProfile.AccessibleName = null;
            resources.ApplyResources(this.btnSaveVehicleProfile, "btnSaveVehicleProfile");
            this.btnSaveVehicleProfile.BackgroundImage = null;
            this.btnSaveVehicleProfile.Font = null;
            this.btnSaveVehicleProfile.Name = "btnSaveVehicleProfile";
            this.toolTips.SetToolTip(this.btnSaveVehicleProfile, resources.GetString("btnSaveVehicleProfile.ToolTip"));
            this.btnSaveVehicleProfile.Click += new System.EventHandler(this.btnSaveVehicleProfile_Click);
            // 
            // btnLoadVehicleProfile
            // 
            this.btnLoadVehicleProfile.AccessibleDescription = null;
            this.btnLoadVehicleProfile.AccessibleName = null;
            resources.ApplyResources(this.btnLoadVehicleProfile, "btnLoadVehicleProfile");
            this.btnLoadVehicleProfile.BackgroundImage = null;
            this.btnLoadVehicleProfile.Font = null;
            this.btnLoadVehicleProfile.Name = "btnLoadVehicleProfile";
            this.toolTips.SetToolTip(this.btnLoadVehicleProfile, resources.GetString("btnLoadVehicleProfile.ToolTip"));
            this.btnLoadVehicleProfile.Click += new System.EventHandler(this.btnLoadVehicleProfile_Click);
            // 
            // gbDynoControl
            // 
            this.gbDynoControl.AccessibleDescription = null;
            this.gbDynoControl.AccessibleName = null;
            resources.ApplyResources(this.gbDynoControl, "gbDynoControl");
            this.gbDynoControl.BackgroundImage = null;
            this.gbDynoControl.Controls.Add(this.chkIncLoss);
            this.gbDynoControl.Controls.Add(this.chkImperial);
            this.gbDynoControl.Controls.Add(this.gbLoss);
            this.gbDynoControl.Controls.Add(this.btnStopDynoRun);
            this.gbDynoControl.Controls.Add(this.btnStartDynoRun);
            this.gbDynoControl.Font = null;
            this.gbDynoControl.Name = "gbDynoControl";
            this.gbDynoControl.TabStop = false;
            this.toolTips.SetToolTip(this.gbDynoControl, resources.GetString("gbDynoControl.ToolTip"));
            // 
            // chkIncLoss
            // 
            this.chkIncLoss.AccessibleDescription = null;
            this.chkIncLoss.AccessibleName = null;
            resources.ApplyResources(this.chkIncLoss, "chkIncLoss");
            this.chkIncLoss.BackgroundImage = null;
            this.chkIncLoss.Font = null;
            this.chkIncLoss.Name = "chkIncLoss";
            this.toolTips.SetToolTip(this.chkIncLoss, resources.GetString("chkIncLoss.ToolTip"));
            this.chkIncLoss.CheckedChanged += new System.EventHandler(this.chkIncLoss_CheckedChanged);
            // 
            // chkImperial
            // 
            this.chkImperial.AccessibleDescription = null;
            this.chkImperial.AccessibleName = null;
            resources.ApplyResources(this.chkImperial, "chkImperial");
            this.chkImperial.BackgroundImage = null;
            this.chkImperial.Font = null;
            this.chkImperial.Name = "chkImperial";
            this.toolTips.SetToolTip(this.chkImperial, resources.GetString("chkImperial.ToolTip"));
            this.chkImperial.CheckedChanged += new System.EventHandler(this.chkImperial_CheckedChanged);
            // 
            // gbLoss
            // 
            this.gbLoss.AccessibleDescription = null;
            this.gbLoss.AccessibleName = null;
            resources.ApplyResources(this.gbLoss, "gbLoss");
            this.gbLoss.BackgroundImage = null;
            this.gbLoss.Controls.Add(this.btnStopLoss);
            this.gbLoss.Controls.Add(this.txtFLoss);
            this.gbLoss.Controls.Add(this.txtDLoss);
            this.gbLoss.Controls.Add(this.label20);
            this.gbLoss.Controls.Add(this.btnMeasureLoss);
            this.gbLoss.Controls.Add(this.lblDragLoss);
            this.gbLoss.Font = null;
            this.gbLoss.Name = "gbLoss";
            this.gbLoss.TabStop = false;
            this.toolTips.SetToolTip(this.gbLoss, resources.GetString("gbLoss.ToolTip"));
            // 
            // btnStopLoss
            // 
            this.btnStopLoss.AccessibleDescription = null;
            this.btnStopLoss.AccessibleName = null;
            resources.ApplyResources(this.btnStopLoss, "btnStopLoss");
            this.btnStopLoss.BackgroundImage = null;
            this.btnStopLoss.Font = null;
            this.btnStopLoss.Name = "btnStopLoss";
            this.toolTips.SetToolTip(this.btnStopLoss, resources.GetString("btnStopLoss.ToolTip"));
            this.btnStopLoss.Click += new System.EventHandler(this.btnStopLoss_Click);
            // 
            // txtFLoss
            // 
            this.txtFLoss.AccessibleDescription = null;
            this.txtFLoss.AccessibleName = null;
            resources.ApplyResources(this.txtFLoss, "txtFLoss");
            this.txtFLoss.BackgroundImage = null;
            this.txtFLoss.DataBindings.Add(new System.Windows.Forms.Binding("Text", this.vehicleProfileBindingSource, "FrictionLoss", true));
            this.txtFLoss.Font = null;
            this.txtFLoss.Name = "txtFLoss";
            this.txtFLoss.ReadOnly = true;
            this.toolTips.SetToolTip(this.txtFLoss, resources.GetString("txtFLoss.ToolTip"));
            // 
            // txtDLoss
            // 
            this.txtDLoss.AccessibleDescription = null;
            this.txtDLoss.AccessibleName = null;
            resources.ApplyResources(this.txtDLoss, "txtDLoss");
            this.txtDLoss.BackgroundImage = null;
            this.txtDLoss.DataBindings.Add(new System.Windows.Forms.Binding("Text", this.vehicleProfileBindingSource, "DynamicLoss", true));
            this.txtDLoss.Font = null;
            this.txtDLoss.Name = "txtDLoss";
            this.txtDLoss.ReadOnly = true;
            this.toolTips.SetToolTip(this.txtDLoss, resources.GetString("txtDLoss.ToolTip"));
            // 
            // label20
            // 
            this.label20.AccessibleDescription = null;
            this.label20.AccessibleName = null;
            resources.ApplyResources(this.label20, "label20");
            this.label20.Font = null;
            this.label20.Name = "label20";
            this.toolTips.SetToolTip(this.label20, resources.GetString("label20.ToolTip"));
            // 
            // btnMeasureLoss
            // 
            this.btnMeasureLoss.AccessibleDescription = null;
            this.btnMeasureLoss.AccessibleName = null;
            resources.ApplyResources(this.btnMeasureLoss, "btnMeasureLoss");
            this.btnMeasureLoss.BackgroundImage = null;
            this.btnMeasureLoss.Font = null;
            this.btnMeasureLoss.Name = "btnMeasureLoss";
            this.toolTips.SetToolTip(this.btnMeasureLoss, resources.GetString("btnMeasureLoss.ToolTip"));
            this.btnMeasureLoss.Click += new System.EventHandler(this.btnMeasureLoss_Click);
            // 
            // lblDragLoss
            // 
            this.lblDragLoss.AccessibleDescription = null;
            this.lblDragLoss.AccessibleName = null;
            resources.ApplyResources(this.lblDragLoss, "lblDragLoss");
            this.lblDragLoss.Font = null;
            this.lblDragLoss.Name = "lblDragLoss";
            this.toolTips.SetToolTip(this.lblDragLoss, resources.GetString("lblDragLoss.ToolTip"));
            // 
            // btnStopDynoRun
            // 
            this.btnStopDynoRun.AccessibleDescription = null;
            this.btnStopDynoRun.AccessibleName = null;
            resources.ApplyResources(this.btnStopDynoRun, "btnStopDynoRun");
            this.btnStopDynoRun.BackgroundImage = null;
            this.btnStopDynoRun.Font = null;
            this.btnStopDynoRun.Name = "btnStopDynoRun";
            this.toolTips.SetToolTip(this.btnStopDynoRun, resources.GetString("btnStopDynoRun.ToolTip"));
            this.btnStopDynoRun.Click += new System.EventHandler(this.btnStopDynoRun_Click);
            // 
            // btnStartDynoRun
            // 
            this.btnStartDynoRun.AccessibleDescription = null;
            this.btnStartDynoRun.AccessibleName = null;
            resources.ApplyResources(this.btnStartDynoRun, "btnStartDynoRun");
            this.btnStartDynoRun.BackgroundImage = null;
            this.btnStartDynoRun.Font = null;
            this.btnStartDynoRun.Name = "btnStartDynoRun";
            this.toolTips.SetToolTip(this.btnStartDynoRun, resources.GetString("btnStartDynoRun.ToolTip"));
            this.btnStartDynoRun.Click += new System.EventHandler(this.btnStartDynoRun_Click);
            // 
            // stsbarMain
            // 
            this.stsbarMain.AccessibleDescription = null;
            this.stsbarMain.AccessibleName = null;
            resources.ApplyResources(this.stsbarMain, "stsbarMain");
            this.stsbarMain.BackgroundImage = null;
            this.stsbarMain.Font = null;
            this.stsbarMain.Name = "stsbarMain";
            this.stsbarMain.Panels.AddRange(new System.Windows.Forms.StatusBarPanel[] {
            this.statusBarPanelConnect,
            this.statusBarPanelLog,
            this.statusBarPanelApp,
            this.statusBarPanelTab});
            this.stsbarMain.ShowPanels = true;
            this.stsbarMain.SizingGrip = false;
            this.toolTips.SetToolTip(this.stsbarMain, resources.GetString("stsbarMain.ToolTip"));
            // 
            // statusBarPanelConnect
            // 
            resources.ApplyResources(this.statusBarPanelConnect, "statusBarPanelConnect");
            // 
            // statusBarPanelLog
            // 
            resources.ApplyResources(this.statusBarPanelLog, "statusBarPanelLog");
            // 
            // statusBarPanelApp
            // 
            resources.ApplyResources(this.statusBarPanelApp, "statusBarPanelApp");
            this.statusBarPanelApp.Icon = null;
            // 
            // statusBarPanelTab
            // 
            resources.ApplyResources(this.statusBarPanelTab, "statusBarPanelTab");
            this.statusBarPanelTab.AutoSize = System.Windows.Forms.StatusBarPanelAutoSize.Spring;
            this.statusBarPanelTab.Icon = null;
            // 
            // tabMain
            // 
            this.tabMain.AccessibleDescription = null;
            this.tabMain.AccessibleName = null;
            resources.ApplyResources(this.tabMain, "tabMain");
            this.tabMain.BackgroundImage = null;
            this.tabMain.Controls.Add(this.tabConnect);
            this.tabMain.Controls.Add(this.tabIdentification);
            this.tabMain.Controls.Add(this.tabSensors);
            this.tabMain.Controls.Add(this.tabActuators);
            this.tabMain.Controls.Add(this.tabFaultCodes);
            this.tabMain.Controls.Add(this.tabPerformance);
            this.tabMain.Controls.Add(this.tabDyno);
            this.tabMain.Font = null;
            this.tabMain.ImageList = this.imgLstToolbar;
            this.tabMain.Name = "tabMain";
            this.tabMain.SelectedIndex = 0;
            this.toolTips.SetToolTip(this.tabMain, resources.GetString("tabMain.ToolTip"));
            this.tabMain.SelectedIndexChanging += new System.EventHandler<DnEcuDiag.CustomControls.TabPageChangeEventArgs>(this.tabMain_SelectedIndexChanging);
            this.tabMain.SelectedIndexChanged += new System.EventHandler(this.tabMain_SelectedIndexChanged);
            // 
            // tabConnect
            // 
            this.tabConnect.AccessibleDescription = null;
            this.tabConnect.AccessibleName = null;
            resources.ApplyResources(this.tabConnect, "tabConnect");
            this.tabConnect.BackgroundImage = null;
            this.tabConnect.Controls.Add(this.gbConnect);
            this.tabConnect.Font = null;
            this.tabConnect.ForeColor = System.Drawing.SystemColors.ControlText;
            this.tabConnect.Name = "tabConnect";
            this.toolTips.SetToolTip(this.tabConnect, resources.GetString("tabConnect.ToolTip"));
            this.tabConnect.UseVisualStyleBackColor = true;
            // 
            // gbConnect
            // 
            this.gbConnect.AccessibleDescription = null;
            this.gbConnect.AccessibleName = null;
            resources.ApplyResources(this.gbConnect, "gbConnect");
            this.gbConnect.BackgroundImage = null;
            this.gbConnect.Controls.Add(this.btnDisconnect);
            this.gbConnect.Controls.Add(this.btnConnect);
            this.gbConnect.Controls.Add(this.cboEcuList);
            this.gbConnect.Font = null;
            this.gbConnect.Name = "gbConnect";
            this.gbConnect.TabStop = false;
            this.toolTips.SetToolTip(this.gbConnect, resources.GetString("gbConnect.ToolTip"));
            // 
            // openFileDialog
            // 
            resources.ApplyResources(this.openFileDialog, "openFileDialog");
            // 
            // saveFileDialog
            // 
            resources.ApplyResources(this.saveFileDialog, "saveFileDialog");
            // 
            // imgLstStatus
            // 
            this.imgLstStatus.ImageStream = ((System.Windows.Forms.ImageListStreamer)(resources.GetObject("imgLstStatus.ImageStream")));
            this.imgLstStatus.TransparentColor = System.Drawing.Color.Transparent;
            this.imgLstStatus.Images.SetKeyName(0, "");
            this.imgLstStatus.Images.SetKeyName(1, "");
            this.imgLstStatus.Images.SetKeyName(2, "");
            this.imgLstStatus.Images.SetKeyName(3, "");
            // 
            // MainForm
            // 
            this.AccessibleDescription = null;
            this.AccessibleName = null;
            resources.ApplyResources(this, "$this");
            this.BackgroundImage = null;
            this.Controls.Add(this.tabMain);
            this.Controls.Add(this.stsbarMain);
            this.Font = null;
            this.MaximizeBox = false;
            this.Menu = this.mnuMain;
            this.Name = "MainForm";
            this.toolTips.SetToolTip(this, resources.GetString("$this.ToolTip"));
            this.Closing += new System.ComponentModel.CancelEventHandler(this.MainForm_Closing);
            ((System.ComponentModel.ISupportInitialize)(this.tmrProgress)).EndInit();
            ((System.ComponentModel.ISupportInitialize)(this.modelBindingSource)).EndInit();
            ((System.ComponentModel.ISupportInitialize)(this.ecuDescriptorsBindingSource)).EndInit();
            this.tabIdentification.ResumeLayout(false);
            this.gbECU.ResumeLayout(false);
            this.gbECU.PerformLayout();
            ((System.ComponentModel.ISupportInitialize)(this.connectedEcuBindingSource)).EndInit();
            this.tabSensors.ResumeLayout(false);
            this.gbSensorFour.ResumeLayout(false);
            ((System.ComponentModel.ISupportInitialize)(this.sensorFourBindingSource)).EndInit();
            this.gbSensorThree.ResumeLayout(false);
            ((System.ComponentModel.ISupportInitialize)(this.sensorThreeBindingSource)).EndInit();
            this.gbSensorTwo.ResumeLayout(false);
            ((System.ComponentModel.ISupportInitialize)(this.sensorTwoBindingSource)).EndInit();
            this.gbSensorOne.ResumeLayout(false);
            ((System.ComponentModel.ISupportInitialize)(this.sensorOneBindingSource)).EndInit();
            this.tabActuators.ResumeLayout(false);
            this.tcActuators.ResumeLayout(false);
            this.tabActuatorControl.ResumeLayout(false);
            this.tabActuatorControl.PerformLayout();
            this.tabRoutines.ResumeLayout(false);
            this.tabRoutines.PerformLayout();
            ((System.ComponentModel.ISupportInitialize)(this.routinesBindingSource)).EndInit();
            this.tabFaultCodes.ResumeLayout(false);
            this.gbSnapshots.ResumeLayout(false);
            ((System.ComponentModel.ISupportInitialize)(this.snapshotsBindingSource)).EndInit();
            this.gbSnapShotDescription.ResumeLayout(false);
            this.gbSnapShotDescription.PerformLayout();
            this.gbFaultCodes.ResumeLayout(false);
            this.grpFaultCodeDescription.ResumeLayout(false);
            this.grpFaultCodeDescription.PerformLayout();
            ((System.ComponentModel.ISupportInitialize)(this.faultCodesBindingSource)).EndInit();
            this.gbFaultCodeList.ResumeLayout(false);
            this.tabPerformance.ResumeLayout(false);
            this.gbPerformance.ResumeLayout(false);
            this.gbPerformanceRunStore.ResumeLayout(false);
            this.gbPerformanceRunStore.PerformLayout();
            ((System.ComponentModel.ISupportInitialize)(this.vehiclePerformanceRunBindingSource)).EndInit();
            this.gbPerformanceRunData.ResumeLayout(false);
            this.gbPerformanceRunData.PerformLayout();
            this.tabDyno.ResumeLayout(false);
            this.gbDynoSetup.ResumeLayout(false);
            ((System.ComponentModel.ISupportInitialize)(this.nudRpmStart)).EndInit();
            ((System.ComponentModel.ISupportInitialize)(this.nudRpmStop)).EndInit();
            this.gbDynoResults.ResumeLayout(false);
            this.gbVehicleProfile.ResumeLayout(false);
            this.gbVehicleProfile.PerformLayout();
            ((System.ComponentModel.ISupportInitialize)(this.nudVehicleWeight)).EndInit();
            ((System.ComponentModel.ISupportInitialize)(this.vehicleProfileBindingSource)).EndInit();
            this.gbDynoControl.ResumeLayout(false);
            this.gbLoss.ResumeLayout(false);
            this.gbLoss.PerformLayout();
            ((System.ComponentModel.ISupportInitialize)(this.statusBarPanelConnect)).EndInit();
            ((System.ComponentModel.ISupportInitialize)(this.statusBarPanelLog)).EndInit();
            ((System.ComponentModel.ISupportInitialize)(this.statusBarPanelApp)).EndInit();
            ((System.ComponentModel.ISupportInitialize)(this.statusBarPanelTab)).EndInit();
            this.tabMain.ResumeLayout(false);
            this.tabConnect.ResumeLayout(false);
            this.gbConnect.ResumeLayout(false);
            this.ResumeLayout(false);

        }

        #endregion

        private void MainForm_Activated(object sender, EventArgs e)
        {
            if (splashIsDone)
            {
                HideSplash();
            }
            else
            {
                ShowSplash();

                var t = new System.Windows.Forms.Timer { Interval = 1500 };
                t.Tick += Tick;
                t.Start();
            }
        }

        private void ShowSplash()
        {
            splash = new AboutForm { StartPosition = FormStartPosition.Manual };
            splash.Left = Left + (Width / 2) - (splash.Width / 2);
            splash.Top = Top + (Height / 2) - (splash.Height / 2);
            splash.Show();

            splashIsDone = true;
        }

        private void Tick(object sender, EventArgs e)
        {
            var t = sender as System.Windows.Forms.Timer;
            if (t != null)
            {
                t.Stop();
                t.Enabled = false;
            }

            HideSplash();
        }

        private void HideSplash()
        {
            // Need to lock for threading.
            // Cannot lock splash because that can be null.
            lock (splashScreenHandler)
            {
                if (splash != null)
                {
                    splash.Close();
                    splash = null;
                }
            }
        }

        #region Events

        private void mnuItemHelp_Click(object sender, EventArgs e)
        {
            Common.ExecuteExceptionHandledDelegate(HandleHelp);
        }

        private void btnConnect_Click(object sender, EventArgs e)
        {
            Common.ExecuteExceptionHandledDelegate(HandleConnectButton);
        }

        private void btnDisconnect_Click(object sender, EventArgs e)
        {
            Common.ExecuteExceptionHandledDelegate(HandleDisconnectButton);    
        }

        private void mnuItemConnect_Click(object sender, EventArgs e)
        {
            Common.ExecuteExceptionHandledDelegate(HandleConnectButton);
        }

        private void mnuItemDisconnect_Click(object sender, EventArgs e)
        {
            Common.ExecuteExceptionHandledDelegate(HandleDisconnectButton);
        }

        private void menuItemConfigure_Click(object sender, EventArgs e)
        {
            Common.ExecuteExceptionHandledDelegate(HandleInterfaceConfigure);
        }

        private void mnuItemAbout_Click(object sender, EventArgs e)
        {
            Common.ExecuteExceptionHandledDelegate(HandleAbout);
        }

        private void MainForm_Closing(object sender, CancelEventArgs e)
        {
            Common.ExecuteExceptionHandledDelegate(HandleShutdown);
        }

        private void btnAutoActuator_Click(object sender, EventArgs e)
        {
            Common.ExecuteExceptionHandledDelegate(HandleAutomaticActuatorTest);
        }

        private void tmrProgress_Elapsed(object sender, ElapsedEventArgs e)
        {
            Common.ExecuteExceptionHandledDelegate(delegate
            {
                if (pbIndicator.Value == pbIndicator.Maximum)
                {
                    pbIndicator.Value = pbIndicator.Minimum;
                }
                else
                {
                    pbIndicator.Increment(1);
                }
            });
        }

        private void btnSensorOneCapture_Click(object sender, EventArgs e)
        {
            Common.ExecuteExceptionHandledDelegate(
            delegate
            {
                Model.Instance.TargetEcu.SensorMonitorOneEnable = HandleSensorCaptureSelection(Model.Instance.TargetEcu.SensorMonitorOne, (Button)sender, oScopeOne,
                                                                                               SensorChannel.ChannelOne, lblSensorOneMinValue, lblSensorOneMaxValue,
                                                                                               lblSensorOneCurrent, Model.Instance.TargetEcu.SensorMonitorOneEnable);
            }
            )
            ;
        }

        private void btnSensorTwoCapture_Click(object sender, EventArgs e)
        {
            Common.ExecuteExceptionHandledDelegate(
                delegate
                {
                    Model.Instance.TargetEcu.SensorMonitorTwoEnable = HandleSensorCaptureSelection(Model.Instance.TargetEcu.SensorMonitorTwo, (Button)sender, oScopeTwo, SensorChannel.ChannelTwo, lblSensorTwoMinValue,
                                                 lblSensorTwoMaxValue,
                                                 lblSensorTwoCurrent, Model.Instance.TargetEcu.SensorMonitorTwoEnable);
                });
        }

        private void btnSensorThreeCapture_Click(object sender, EventArgs e)
        {
            Common.ExecuteExceptionHandledDelegate(
                delegate
                {
                    Model.Instance.TargetEcu.SensorMonitorThreeEnable = HandleSensorCaptureSelection(Model.Instance.TargetEcu.SensorMonitorThree, (Button)sender, oScopeThree, SensorChannel.ChannelThree, lblSensorThreeMinValue,
                                                 lblSensorThreeMaxValue,
                                                 lblSensorThreeCurrent, Model.Instance.TargetEcu.SensorMonitorThreeEnable);
                });
        }

        private void btnSensorFourCapture_Click(object sender, EventArgs e)
        {
            Common.ExecuteExceptionHandledDelegate(
                delegate
                {
                    Model.Instance.TargetEcu.SensorMonitorFourEnable = HandleSensorCaptureSelection(Model.Instance.TargetEcu.SensorMonitorFour, (Button)sender, oScopeFour, SensorChannel.ChannelFour, lblSensorFourMinValue,
                                                 lblSensorFourMaxValue,
                                                 lblSensorFourCurrent, Model.Instance.TargetEcu.SensorMonitorFourEnable);
                });
        }

        private void cboSensorOne_SelectedIndexChanged(object sender, EventArgs e)
        {
            Common.ExecuteExceptionHandledDelegate(() => HandleSensorSelection(0));
        }

        private void cboSensorTwo_SelectedIndexChanged(object sender, EventArgs e)
        {
            Common.ExecuteExceptionHandledDelegate(() => HandleSensorSelection(1));
        }

        private void cboSensorThree_SelectedIndexChanged(object sender, EventArgs e)
        {
            Common.ExecuteExceptionHandledDelegate(() => HandleSensorSelection(2));
        }

        private void cboSensorFour_SelectedIndexChanged(object sender, EventArgs e)
        {
            Common.ExecuteExceptionHandledDelegate(() => HandleSensorSelection(3));
        }

        private void btnReadDTCs_Click(object sender, EventArgs e)
        {
            Common.ExecuteExceptionHandledDelegate(HandleReadFaultCodes);
        }

        private void btnClearDTCs_Click(object sender, EventArgs e)
        {
            Common.ExecuteExceptionHandledDelegate(HandleClearFaultCodes);
        }

        private void tabMain_SelectedIndexChanged(object sender, EventArgs e)
        {
            Common.ExecuteExceptionHandledDelegate(() => HandleMainTabSelection(tabMain.SelectedIndex));
        }

        private void mnuItemApplication_Click(object sender, EventArgs e)
        {
            Common.ExecuteExceptionHandledDelegate(HandleApplicationConfigure);
        }

        private void btnStartPerformance_Click(object sender, EventArgs e)
        {
            Common.ExecuteExceptionHandledDelegate(HandleStartPerformanceRun);
        }

        private void btnStopPerformance_Click(object sender, EventArgs e)
        {
            Common.ExecuteExceptionHandledDelegate(Model.Instance.StopPerformanceMeasurement);
        }

        private void btnSavePerfRun_Click(object sender, EventArgs e)
        {
            Common.ExecuteExceptionHandledDelegate(HandleSavePerformanceRun);
        }

        private void btnLoadPerfRun_Click(object sender, EventArgs e)
        {
            Common.ExecuteExceptionHandledDelegate(HandleLoadPerformanceRun);
        }

        private void btnSaveVehicleProfile_Click(object sender, EventArgs e)
        {
            Common.ExecuteExceptionHandledDelegate(HandleSaveVehicleProfile);
        }

        private void btnLoadVehicleProfile_Click(object sender, EventArgs e)
        {
            Common.ExecuteExceptionHandledDelegate(HandleLoadVehicleProfile);
        }

        private void btnCaptureFinalDriveRatio_Click(object sender, EventArgs e)
        {
            Common.ExecuteExceptionHandledDelegate(HandleCaptureFinalDriveRatio);
        }

        private void cboGear_SelectedIndexChanged(object sender, EventArgs e)
        {
            Common.ExecuteExceptionHandledDelegate(HandleSelectedGearIndexChanged);
        }

        private void txtDynoVehicle_TextChanged(object sender, EventArgs e)
        {
            Common.ExecuteExceptionHandledDelegate(HandleDynoVehicleTextChanged);
        }

        private void btnStopDynoRun_Click(object sender, EventArgs e)
        {
            Common.ExecuteExceptionHandledDelegate(Model.Instance.StopPerformanceMeasurement);
        }

        private void btnStartDynoRun_Click(object sender, EventArgs e)
        {
            Common.ExecuteExceptionHandledDelegate(HandleStartDynoRun);
        }

        private void btnStopLoss_Click(object sender, EventArgs e)
        {
            Common.ExecuteExceptionHandledDelegate(Model.Instance.StopPerformanceMeasurement);
        }

        private void btnMeasureLoss_Click(object sender, EventArgs e)
        {
            Common.ExecuteExceptionHandledDelegate(HandleMeasureLoss);
        }

        private void btnReadSnapShot_Click(object sender, EventArgs e)
        {
            Common.ExecuteExceptionHandledDelegate(HandleReadShapShot);
        }

        private void chkIncLoss_CheckedChanged(object sender, EventArgs e)
        {
            Common.ExecuteExceptionHandledDelegate(HandleIncludeLossCheck);
        }

        private void chkImperial_CheckedChanged(object sender, EventArgs e)
        {
            Common.ExecuteExceptionHandledDelegate(HandleDisplayImperialMeasures);
        }

        private void mnuItemPlotAll_Click(object sender, EventArgs e)
        {
            Common.ExecuteExceptionHandledDelegate(delegate
            {
                mnuItemPlotAll.Checked = !mnuItemPlotAll.Checked;
                HandlePlotSensors(mnuItemPlotAll.Checked);
            }
                );
        }

        private void mnuItemEnableLogging_Click(object sender, EventArgs e)
        {
            Common.ExecuteExceptionHandledDelegate(delegate
            {
                mnuItemEnableLogging.Checked = !mnuItemEnableLogging.Checked;
                mnuItemSensorGroup.Enabled = mnuItemPlotAll.Enabled = !mnuItemEnableLogging.Checked;

                if (mnuItemEnableLogging.Checked)
                {
                    statusBarPanelLog.Icon = Icon.FromHandle(((Bitmap)imgLstStatus.Images[2]).GetHicon());
                    statusBarPanelLog.ToolTipText = "Logging Sensors";
                }
                else
                {
                    statusBarPanelLog.Icon = Icon.FromHandle(((Bitmap)imgLstStatus.Images[3]).GetHicon());
                    statusBarPanelLog.ToolTipText = WinFormStrings.NotLoggingSensors;
                }

                btnSensorOneCapture.Enabled = !mnuItemEnableLogging.Checked;
                btnSensorTwoCapture.Enabled = !mnuItemEnableLogging.Checked;
                btnSensorThreeCapture.Enabled = !mnuItemEnableLogging.Checked;
                btnSensorFourCapture.Enabled = !mnuItemEnableLogging.Checked;

                if (mnuItemEnableLogging.Checked)
                {
                    var sensors = new List<Sensor>();

                    if (Model.Instance.TargetEcu.SensorMonitorOneEnable)
                    {
                        sensors.Add(Model.Instance.TargetEcu.SensorMonitorOne);
                    }

                    if (Model.Instance.TargetEcu.SensorMonitorTwoEnable)
                    {
                        sensors.Add(Model.Instance.TargetEcu.SensorMonitorTwo);
                    }

                    if (Model.Instance.TargetEcu.SensorMonitorThreeEnable)
                    {
                        sensors.Add(Model.Instance.TargetEcu.SensorMonitorThree);
                    }

                    if (Model.Instance.TargetEcu.SensorMonitorFourEnable)
                    {
                        sensors.Add(Model.Instance.TargetEcu.SensorMonitorFour);
                    }

                    Model.Instance.Logger.StartLogging(sensors);
                }
                else
                {
                    Model.Instance.Logger.StopLogging();
                }
            }
                );
        }

        private void mnuItemSensorGroup_Click(object sender, EventArgs e)
        {
            Common.ExecuteExceptionHandledDelegate(delegate
            {
                var formSensorGroups = new FormSensorGroups();
                BindingContext[connectedEcuBindingSource].SuspendBinding();

                formSensorGroups.ShowDialog();

                BindingContext[connectedEcuBindingSource].ResumeBinding();
            });
        }

        private void btnExecuteRoutine_Click(object sender, EventArgs e)
        {
            Common.ExecuteExceptionHandledDelegate(HandleExecuteRoutine);
        }

        private void cboSnapShots_SelectedIndexChanged(object sender, EventArgs e)
        {
            txtSnapShotText.Text = String.Empty;
        }

        private void cboRoutines_SelectedIndexChanged(object sender, EventArgs e)
        {
            Common.ExecuteExceptionHandledDelegate(() => HandleRoutineSelectionChange(cboRoutines.SelectedIndex));
        }

        private void btnCancelRoutineExecution_Click(object sender, EventArgs e)
        {
            Common.ExecuteExceptionHandledDelegate(HandleExecuteRoutineCancel);
        }

        private void tabMain_SelectedIndexChanging(object sender, TabPageChangeEventArgs e)
        {
            if (disableTabbing)
            {
                e.Cancel = true;
            }
        }

        private void tcActuators_SelectedIndexChanging(object sender, TabPageChangeEventArgs e)
        {
            if (disableTabbing)
            {
                e.Cancel = true;
            }
        }

        #endregion Events

        #region Fields

        private AboutForm splash;
        private readonly EventHandler splashScreenHandler;
        private readonly Color PowerPlotColor = Color.Red;
        private readonly ArrayList tabs = new ArrayList();
        private readonly Color TorquePlotColor = Color.Blue;
        private Boolean splashIsDone;
        private Boolean disconnecting;
        private Boolean disableTabbing;
        private ProgressBar pbIndicator;

        #endregion

        #region MVC Logic

        private void HandleExecuteRoutineCancel()
        {
            Model.Instance.TargetEcu.ExecuteRoutineCancel();
        }

        private void HandleReadShapShot()
        {
            try
            {
#if DEMO
				Common.ShowMessageExclamation("This feature is not availabe in the Demo version");
				return;
#else
                SetUIBusy();
                btnReadSnapShot.Enabled = false;

                statusBarPanelTab.Text = DnEcuDiag.WinFormStrings.ReadingTheSnapShot;

                AsyncReadSnapshot asyncReadSnapshot = Model.Instance.TargetEcu.ReadSnapshot;
                IAsyncResult result = asyncReadSnapshot.BeginInvoke(Model.Instance.TargetEcu.SnapshotDefinitions[cboSnapShots.SelectedIndex], null, null);

                while (!result.IsCompleted)
                {
                    Application.DoEvents();

                    Thread.Sleep(200);

                    statusBarPanelTab.Text += ".";
                }

                txtSnapShotText.Text = asyncReadSnapshot.EndInvoke(result);

                statusBarPanelTab.Text = String.Empty;

#endif
            }
            catch (SnapshotNotSupportedException)
            {
                txtSnapShotText.Text += String.Format(CultureInfo.InvariantCulture, DnEcuDiag.WinFormStrings.ErrorThisEcuDoesNotSupportThisSnapshot);
            }
            finally
            {
                SetUIActive();
                btnReadSnapShot.Enabled = true;
            }
        }

        private void HandlePlotSensors(Boolean enable)
        {
            if (Model.Instance.TargetEcu.SensorMonitorOneEnable != enable)
            {
                Model.Instance.TargetEcu.SensorMonitorOneEnable = HandleSensorCaptureSelection(Model.Instance.TargetEcu.SensorMonitorOne, btnSensorOneCapture, oScopeOne, SensorChannel.ChannelOne, lblSensorOneMinValue,
                                             lblSensorOneMaxValue,
                                             lblSensorOneCurrent, Model.Instance.TargetEcu.SensorMonitorOneEnable);
            }

            if (Model.Instance.TargetEcu.SensorMonitorTwoEnable != enable)
            {
                Model.Instance.TargetEcu.SensorMonitorTwoEnable = HandleSensorCaptureSelection(Model.Instance.TargetEcu.SensorMonitorTwo, btnSensorTwoCapture, oScopeTwo, SensorChannel.ChannelTwo, lblSensorTwoMinValue,
                                              lblSensorTwoMaxValue,
                                              lblSensorTwoCurrent, Model.Instance.TargetEcu.SensorMonitorTwoEnable);
            }

            if (Model.Instance.TargetEcu.SensorMonitorThreeEnable != enable)
            {
                Model.Instance.TargetEcu.SensorMonitorThreeEnable = HandleSensorCaptureSelection(Model.Instance.TargetEcu.SensorMonitorThree, btnSensorThreeCapture, oScopeThree, SensorChannel.ChannelThree, lblSensorThreeMinValue,
                                             lblSensorThreeMaxValue,
                                             lblSensorThreeCurrent, Model.Instance.TargetEcu.SensorMonitorThreeEnable);
            }

            if (Model.Instance.TargetEcu.SensorMonitorFourEnable != enable)
            {
                Model.Instance.TargetEcu.SensorMonitorFourEnable = HandleSensorCaptureSelection(Model.Instance.TargetEcu.SensorMonitorFour, btnSensorFourCapture, oScopeFour, SensorChannel.ChannelFour, lblSensorFourMinValue,
                                             lblSensorFourMaxValue,
                                             lblSensorFourCurrent, Model.Instance.TargetEcu.SensorMonitorFourEnable);
            }
        }


        private void HandleMeasureLoss()
        {
#if DEMO
				Common.ShowMessageExclamation("This feature is not availabe in the demo version");
				return;
#else
            try
            {
                pbDyno.Maximum = 0;
                pbDyno.Value = 0;
                pbDyno.Step = 1;

                gbDynoSetup.Enabled = false;
                btnStartDynoRun.Enabled = false;
                gbVehicleProfile.Enabled = false;
                btnStopLoss.Enabled = true;
                btnMeasureLoss.Enabled = false;

                statusBarPanelTab.Text = DnEcuDiag.WinFormStrings.LossMeasureStarted;

                AsyncStartLossMeasure asyncStartLossMeasure = Model.Instance.StartLossMeasure;
                IAsyncResult res = asyncStartLossMeasure.BeginInvoke(null, null);

                while (!res.IsCompleted)
                {
                    Application.DoEvents();

                    Thread.Sleep(200);

                    if ((pbDyno.Maximum == 0) && (Model.Instance.LastLossMeasureSpeed != 0))
                    {
                        pbDyno.Maximum = Convert.ToInt32(Model.Instance.LastLossMeasureSpeed);
                        pbDyno.Value = Convert.ToInt32(Model.Instance.LastLossMeasureSpeed);
                    }

                    if (pbDyno.Maximum != 0)
                    {
                        pbDyno.Value = Convert.ToInt32(Model.Instance.LastLossMeasureSpeed);
                    }
                }

                if (asyncStartLossMeasure.EndInvoke(res) == false)
                {
                    statusBarPanelTab.Text = "";
                    Common.ShowMessage(DnEcuDiag.WinFormStrings.LossMeasurementNotFinishedSuccessfully);
                }
                else
                {
                    statusBarPanelTab.Text = DnEcuDiag.WinFormStrings.LossMeasurementFinished;
                    Common.ShowMessage(DnEcuDiag.WinFormStrings.LossMeasurementFinished);
                }
            }
            finally
            {
                gbDynoSetup.Enabled = true;
                gbVehicleProfile.Enabled = true;
                btnStartDynoRun.Enabled = true;
                btnStopLoss.Enabled = false;
                btnMeasureLoss.Enabled = true;
            }
#endif
        }

        private void HandleDisplayImperialMeasures()
        {
            DisplayDynoResults(Model.Instance.VehiclePerformance.DynoMeasurements, chkImperial.Checked);
        }

        private void HandleIncludeLossCheck()
        {
            Model.Instance.VehiclePerformance.DynoCalculateResults(chkIncLoss.Checked,
                                                                   (int)Settings.Default.DynoFilterCount);
            DisplayDynoResults(Model.Instance.VehiclePerformance.DynoMeasurements, chkImperial.Checked);
        }

        private void HandleStartDynoRun()
        {
#if DEMO
            Common.ShowMessageExclamation("This feature is not availabe in the demo version");
				return;
#else
            try
            {
                btnStopDynoRun.Enabled = true;

                pbDyno.Maximum = (int)nudRpmStop.Value;
                pbDyno.Value = pbDyno.Minimum = (int)nudRpmStart.Value;
                pbDyno.Step = 1;

                gbDynoSetup.Enabled = false;
                gbLoss.Enabled = false;
                gbVehicleProfile.Enabled = false;

                statusBarPanelTab.Text = DnEcuDiag.WinFormStrings.DynoRunStarted;

                AsyncStartDynoMeasure asyncStartDynoMeasure = Model.Instance.StartDynoMeasure;
                var result = asyncStartDynoMeasure.BeginInvoke((Int32)nudRpmStart.Value, (int)nudRpmStop.Value, cboGear.SelectedIndex,
                                                               (int)nudVehicleWeight.Value, null, null);

                while (!result.IsCompleted)
                {
                    Application.DoEvents();

                    Thread.Sleep(200);

                    if (Model.Instance.LastDynoRpm >= (double)nudRpmStart.Value)
                    {
                        pbDyno.Value = (int)Model.Instance.LastDynoRpm;
                    }
                }

                if (asyncStartDynoMeasure.EndInvoke(result) == false)
                {
                    statusBarPanelTab.Text = "";
                    Common.ShowMessage(DnEcuDiag.WinFormStrings.DynoRunNotFinishedSuccessfully);
                }
                else
                {
                    Model.Instance.VehiclePerformance.DynoCalculateResults(chkIncLoss.Checked, (int)Settings.Default.DynoFilterCount);
                    DisplayDynoResults(Model.Instance.VehiclePerformance.DynoMeasurements, chkImperial.Checked);
                    statusBarPanelTab.Text = DnEcuDiag.WinFormStrings.DynoRunFinished;
                    Common.ShowMessage(DnEcuDiag.WinFormStrings.DynoRunFinished);
                }
            }
            finally
            {
                gbDynoSetup.Enabled = true;
                gbLoss.Enabled = true;
                gbVehicleProfile.Enabled = true;

                btnStopDynoRun.Enabled = false;
            }
#endif
        }

        private void HandleDynoVehicleTextChanged()
        {
            Model.Instance.DynoVehicleProfile.VehicleName = txtDynoVehicle.Text;
            zedGraphDyno.GraphPane.Title = DnEcuDiag.WinFormStrings.DynoMeasurementsFor + txtDynoVehicle.Text;
            zedGraphDyno.Refresh();
        }


        private void HandleSelectedGearIndexChanged()
        {
            if (cboGear.SelectedIndex != -1)
            {
                txtFinalDriveRatio.Text =
                    Model.Instance.DynoVehicleProfile.FinalDriveRatios[cboGear.SelectedIndex].ToString("0.00", CultureInfo.InvariantCulture);
            }
        }

        private void HandleCaptureFinalDriveRatio()
        {
#if DEMO
				Common.ShowMessageExclamation("This feature is not availabe in the demo version");
				return;
#else

            try
            {
                SetUIBusy();

                statusBarPanelTab.Text = DnEcuDiag.WinFormStrings.CapturingDriveRatio;
                pbDyno.Maximum = 0;
                pbDyno.Maximum = 25;
                pbDyno.Value = 0;
                pbDyno.Step = 1;

                AsyncCaptureFinalDriveRatio asyncCaptureFinalDriveRatio = Model.Instance.CaptureFinalDriveRatio;
                IAsyncResult result = asyncCaptureFinalDriveRatio.BeginInvoke(cboGear.SelectedIndex, null, null);

                while (!result.IsCompleted)
                {
                    Application.DoEvents();

                    Thread.Sleep(200);

                    pbDyno.Value = Model.Instance.FinalDriveProgress;
                }

                asyncCaptureFinalDriveRatio.EndInvoke(result);

                statusBarPanelTab.Text = DnEcuDiag.WinFormStrings.CapturingDriveRatioFinished;

                if (cboGear.SelectedIndex >= 0)
                {
                    txtFinalDriveRatio.Text =
                        Model.Instance.DynoVehicleProfile.FinalDriveRatios[cboGear.SelectedIndex].ToString("0.00", CultureInfo.InvariantCulture);
                }
            }
            catch (PerformanceRunNotSupportedException e)
            {
                Common.ShowMessageExclamation(e.Message);
            }
            finally
            {
                SetUIActive();
            }
#endif
        }

        private void HandleLoadVehicleProfile()
        {
#if DEMO
				Common.ShowMessageExclamation("This feature is not availabe in the demo version");
				return;
#else
            try
            {
                SetUIBusy();

                openFileDialog.InitialDirectory = Model.Instance.VehicleProfileFolder;
                openFileDialog.Filter = vehicleProfileFileTypeFilter;

                if (openFileDialog.ShowDialog() == DialogResult.OK)
                {
                    LoadVehicleProfile(openFileDialog.FileName);

                    SetupDynoUi();

                    Common.ShowMessage("Vehicle Profile loaded.");
                }
            }
            finally
            {
                SetUIActive();
            }
#endif
        }

        private void SetupDynoUi()
        {
            if (String.IsNullOrEmpty(Model.Instance.DynoVehicleProfile.VehicleName))
            {
                gbDynoSetup.Enabled = gbDynoControl.Enabled = false;
            }
            else
            {
                gbDynoSetup.Enabled = gbDynoControl.Enabled = true;
            }
        }

        private void LoadVehicleProfile(String vehicleProfileFilename)
        {
            Model.Instance.HandleLoadVehicleProfile(vehicleProfileFilename);
            vehicleProfileBindingSource.DataSource = Model.Instance.DynoVehicleProfile;
        }

        private void HandleSaveVehicleProfile()
        {
#if DEMO
				Common.ShowMessageExclamation("This feature is not availabe in the demo version");
				return;
#else
            if (txtDynoVehicle.Text.Length == 0)
            {
                Common.ShowMessageExclamation("You must enter a name for the Vehicle before you can save the profile.");
                return;
            }

            saveFileDialog.InitialDirectory = Model.Instance.VehicleProfileFolder;
            saveFileDialog.Filter = vehicleProfileFileTypeFilter;
            saveFileDialog.FileName = txtDynoVehicle.Text;

            if (saveFileDialog.ShowDialog() == DialogResult.OK)
            {
                Model.Instance.SaveVehicleProfile(saveFileDialog.FileName);
                Common.ShowMessage("Vehicle Profile saved.");
            }
#endif
        }

        private void HandleStartPerformanceRun()
        {
#if DEMO
				Common.ShowMessageExclamation("This feature is not availabe in the demo version");
				return;
#else
            bool started = false;

            try
            {
                SetUIBusy();
                btnStartPerformance.Enabled = false;
                btnLoadPerfRun.Enabled = false;
                btnSavePerfRun.Enabled = false;
                btnStopPerformance.Enabled = true;

                statusBarPanelTab.Text = DnEcuDiag.WinFormStrings.PerformanceRunMeasurementStarted;

                dCCPerf.Reset();

                VoidDelegate del = Model.Instance.StartPerformanceRun;
                IAsyncResult res = del.BeginInvoke(null, null);

                while (!res.IsCompleted)
                {
                    Application.DoEvents();

                    // Start timer.
                    if ((Model.Instance.LastSpeedMeasurement > 0) & !started)
                    {
                        dCCPerf.SetClockType = ClockType.Stopwatch;
                        started = true;
                    }

                    Thread.Sleep(100);

                    vehiclePerformanceRunBindingSource.ResetBindings(false);
                }

                del.EndInvoke(res);

                dCCPerf.SetClockType = ClockType.Freeze;

                statusBarPanelTab.Text = DnEcuDiag.WinFormStrings.PerformanceRunMeasurementFinished;
            }
            finally
            {
                SetUIActive();
                btnStartPerformance.Enabled = true;
                btnLoadPerfRun.Enabled = true;
                btnSavePerfRun.Enabled = true;
                btnStopPerformance.Enabled = false;
                dCCPerf.SetClockType = ClockType.Freeze;
            }
#endif
        }


        private void HandleSavePerformanceRun()
        {
#if DEMO
				Common.ShowMessageExclamation("This feature is not availabe in the demo version");
				return;
#else
            try
            {
                SetUIBusy();

                if (txtPerformanceVehicle.Text.Length == 0)
                {
                    Common.ShowMessageExclamation(DnEcuDiag.WinFormStrings.YouMustEnterANameInTheVehicleTextBoxBeforeYouCanSaveAPerformanceRun);
                }
                else if (!Model.Instance.VehiclePerformance.Complete)
                {
                    Common.ShowMessageExclamation(DnEcuDiag.WinFormStrings.APerformanceRunHasNotBeenPerformedSoThereIsNothingToSave);
                }
                else
                {
                    saveFileDialog.InitialDirectory = Model.Instance.PerformanceRunFolder;
                    saveFileDialog.Filter = vehiclePerformanceFileTypeFilter;
                    saveFileDialog.FileName = txtPerformanceVehicle.Text;

                    if (saveFileDialog.ShowDialog() == DialogResult.OK)
                    {
                        Model.Instance.SaveVehiclePerformanceRun(saveFileDialog.FileName);
                    }
                }
            }
            finally
            {
                SetUIActive();
            }
#endif
        }

        private void HandleLoadPerformanceRun()
        {
#if DEMO
				Common.ShowMessageExclamation("This feature is not availabe in the demo version");
				return;
#else
            try
            {
                SetUIBusy();
                openFileDialog.InitialDirectory = Model.Instance.PerformanceRunFolder;
                openFileDialog.Filter = vehiclePerformanceFileTypeFilter;

                if (openFileDialog.ShowDialog() == DialogResult.OK)
                {
                    Model.Instance.LoadPerformanceRun(openFileDialog.FileName);
                    vehiclePerformanceRunBindingSource.DataSource = Model.Instance.VehiclePerformance.PerformanceRun;
                }
            }
            finally
            {
                SetUIActive();
            }
#endif
        }

        private void HandleApplicationConfigure()
        {
            Form applicationConfiguration = new FormApplicationConfiguration();
            applicationConfiguration.ShowDialog();
            oScopeOne.BackColor = oScopeTwo.BackColor = oScopeThree.BackColor = oScopeFour.BackColor = Settings.Default.BackColor;
            oScopeOne.ForeColor = oScopeTwo.ForeColor = oScopeThree.ForeColor = oScopeFour.ForeColor = Settings.Default.PlotColor;
            oScopeOne.LegendColor = oScopeTwo.LegendColor = oScopeThree.LegendColor = oScopeFour.LegendColor = Settings.Default.LegendColor;
            oScopeOne.GridColor = oScopeTwo.GridColor = oScopeThree.GridColor = oScopeFour.GridColor = Settings.Default.GridColor;
        }

        private void HandleClearFaultCodes()
        {
            try
            {
#if DEMO
				Common.ShowMessageExclamation("This feature is not availabe in the Demo version");
				return;
#else
                btnClearDTCs.Enabled = false;

                if (Common.ShowYesNo(DnEcuDiag.WinFormStrings.ClearTheFaultCodes) == DialogResult.No)
                {
                    return;
                }

                statusBarPanelTab.Text = DnEcuDiag.WinFormStrings.ClearingTheECUsFaultCodes;


                AsyncClearFaultCodes invokeClearFaults = Model.Instance.TargetEcu.ClearFaultCodes;
                IAsyncResult result = invokeClearFaults.BeginInvoke(null, null);

                while (!result.IsCompleted)
                {
                    Application.DoEvents();

                    Thread.Sleep(200);

                    statusBarPanelTab.Text += ".";
                }

                invokeClearFaults.EndInvoke(result);

                statusBarPanelTab.Text = String.Empty;

                MessageBox.Show(DnEcuDiag.WinFormStrings.ClearedFaultCodesSucessfully);

#endif
            }
            finally
            {
                btnClearDTCs.Enabled = true;
            }
        }

        private void HandleReadFaultCodes()
        {
            try
            {
#if DEMO
				Common.ShowMessageExclamation("This feature is not availabe in the Demo version");
				return;
#else
                SetUIBusy();

                btnReadDTCs.Enabled = false;

                statusBarPanelTab.Text = DnEcuDiag.WinFormStrings.ReadingTheECUsFaultCodes;

                AsyncReadFaultCodes asyncReadFaultCodes = Model.Instance.TargetEcu.ReadFaultCodes;
                IAsyncResult result = asyncReadFaultCodes.BeginInvoke(null, null);

                while (!result.IsCompleted)
                {
                    Application.DoEvents();

                    Thread.Sleep(200);

                    statusBarPanelTab.Text += ".";
                }

                asyncReadFaultCodes.EndInvoke(result);

                Common.ShowMessage(DnEcuDiag.WinFormStrings.Read0FaultCodesFromTheECU, Model.Instance.TargetEcu.ActiveFaultCodes.Count);

                statusBarPanelTab.Text = String.Empty;

                faultCodesBindingSource.ResetBindings(false);
#endif
            }
            finally
            {
                SetUIActive();
                btnReadDTCs.Enabled = true;
            }
        }

        private void HandleMainTabSelection(int index)
        {
            if (index == (int)FormTab.Actuators)
            {
                ResetActuatorUI();
            }

            if (index != (int)FormTab.Sensors)
            {
                mnuSensors.Enabled = false;
                if (Model.Instance.TargetEcu.Connected)
                {
                    Model.Instance.TargetEcu.StopSensorMonitoring();
                }
            }

            if (index == (int)FormTab.Sensors)
            {
                if (Model.Instance.TargetEcu.Connected)
                {
                    Model.Instance.TargetEcu.StartSensorMonitoring();
                    mnuSensors.Enabled = Model.Instance.TargetEcu.Sensors.Count == 0 ? false : true;
                }
            }

            // Reset the clock timer.
            if (index == (int)FormTab.Performance)
            {
                TimerReset();
            }

            if (index == (int)FormTab.Dyno)
            {
                if (!String.IsNullOrEmpty(Settings.Default.LastDynoVehicleProfile))
                {
                    if (File.Exists(Settings.Default.LastDynoVehicleProfile))
                    {
                        LoadVehicleProfile(Settings.Default.LastDynoVehicleProfile);
                    }
                    else
                    {
                        Settings.Default.LastDynoVehicleProfile = String.Empty;
                    }
                }

                SetupDynoUi();
            }

            statusBarPanelTab.Text = String.Empty;
        }

        private void HandleShutdown()
        {
            // prevent shutdown during disconnect.
            if (disconnecting)
            {
                return;
            }

            Model.Instance.ShutDown();
        }

        private void HandleSensorSelection(int index)
        {
            ComboBox comboBox = null;

            switch (index)
            {
                case 0:
                    Model.Instance.TargetEcu.SensorMonitorOneEnable = false;
                    ResetSensorUi(oScopeOne, btnSensorOneCapture, lblSensorOneMaxValue, lblSensorOneMinValue, lblSensorOneCurrent, imgLstButtons.Images[1]);
                    toolTips.SetToolTip(cboSensorOne, cboSensorOne.Text);
                    comboBox = cboSensorOne;
                    break;
                case 1:
                    Model.Instance.TargetEcu.SensorMonitorTwoEnable = false;
                    ResetSensorUi(oScopeTwo, btnSensorTwoCapture, lblSensorTwoMaxValue, lblSensorTwoMinValue, lblSensorTwoCurrent, imgLstButtons.Images[1]);
                    toolTips.SetToolTip(cboSensorTwo, cboSensorTwo.Text);
                    comboBox = cboSensorTwo;
                    break;
                case 2:
                    Model.Instance.TargetEcu.SensorMonitorThreeEnable = false;
                    ResetSensorUi(oScopeThree, btnSensorThreeCapture, lblSensorThreeMaxValue, lblSensorThreeMinValue, lblSensorThreeCurrent, imgLstButtons.Images[1]);
                    toolTips.SetToolTip(cboSensorThree, cboSensorThree.Text);
                    comboBox = cboSensorThree;
                    break;
                case 3:
                    Model.Instance.TargetEcu.SensorMonitorFourEnable = false;
                    ResetSensorUi(oScopeFour, btnSensorFourCapture, lblSensorFourMaxValue, lblSensorFourMinValue, lblSensorFourCurrent, imgLstButtons.Images[1]);
                    toolTips.SetToolTip(cboSensorFour, cboSensorFour.Text);
                    comboBox = cboSensorFour;
                    break;
            }

            if (comboBox != null)
            {
                if (comboBox.DataBindings[0] != null)
                {
                    if (comboBox.DataBindings[0].BindingManagerBase != null)
                    {
                        comboBox.DataBindings[0].BindingManagerBase.EndCurrentEdit();
                    }
                }
            }   
        }

        private Boolean HandleSensorCaptureSelection(Sensor selectedSensor, ButtonBase selectedButton, SingleChannelScope selectedScope, SensorChannel sensorChannel, Control min,
                                                     Control max, Control current, Boolean capture)
        {
            String tooltip;

            capture = !capture;
            selectedScope.Enabled = selectedScope.Visible = capture;
            SetScopeProperties(selectedScope, selectedSensor);

            Model.Instance.SaveSelectedSensorSetting(sensorChannel, selectedSensor.Id);

            min.Text = max.Text = current.Text = String.Empty;

            if (capture)
            {
                var ip = new ImageProcess
                {
                    ImageToProcess = imgLstButtons.Images[2]
                };

                selectedButton.Image = ImageProcessor.Overlay(imgLstButtons.Images[1], new[] { ip });
                tooltip = "Stop capture of ";
            }
            else
            {
                mnuItemPlotAll.Checked = false;
                selectedButton.Image = imgLstButtons.Images[1];
                tooltip = "Start capture of ";
            }

            toolTips.SetToolTip(selectedScope, "Plot of " + selectedSensor.Name);
            toolTips.SetToolTip(selectedButton, tooltip + selectedSensor.Name);

            mnuItemEnableLogging.Enabled |= capture;

            return capture;
        }


        private static void SetScopeProperties(SingleChannelScope scope, Sensor sensor)
        {
            scope.Channel.SetLimits(sensor.UpperLimit, sensor.LowerLimit);
            scope.Channel.SetUnits(sensor.Name, sensor.Units);
        }

        private void HandleHelp()
        {
            Help.ShowHelp(this, Model.Instance.HelpFile);
        }

        private void HandleAbout()
        {
            ShowSplash();
        }

        private void HandleAutomaticActuatorTest()
        {
            try
            {
#if DEMO
				Common.ShowMessageExclamation("This feature is not availabe in the demo version");
				return;
#else
                var x = 0;

                SetUIBusy();
                btnAutoActuator.Enabled = false;

                if (lvActuators.SelectedItems.Count == 0)
                {
                    Common.ShowMessageExclamation("Please select an Actuator to test from the list of Actuators.");
                    SetUIActive();
                    return;
                }

                Int32 selIndex = lvActuators.SelectedItems[0].Index;

                if (selIndex < 0)
                {
                    Common.ShowMessageExclamation("Please select an Actuator to test from the list of Actuators.");
                    SetUIActive();
                    return;
                }
                string warn = Model.Instance.TargetEcu.Actuators[selIndex].Warning;

                if (warn.Length != 0)
                {
                    if (Common.ShowYesNo(warn) == DialogResult.No)
                    {
                        return;
                    }
                }

                pbIndicator = pbActuator;

                AsyncActivateActuator asyncActivateActuator = Model.Instance.TargetEcu.ActivateActuator;
                IAsyncResult result = asyncActivateActuator.BeginInvoke(Model.Instance.TargetEcu.Actuators[selIndex], null, null);

                pbActuator.Value = 0;
                pbActuator.Maximum = Model.Instance.TargetEcu.Actuators[selIndex].Duration;
                txtActuatorTestStat.Clear();
                txtActuatorTestStat.Text +=
                    String.Format(CultureInfo.InvariantCulture, DnEcuDiag.WinFormStrings.StartingAutomaticTestOn, Model.Instance.TargetEcu.Actuators[selIndex].Name);
                tmrProgress.Start();
                txtActuatorTestStat.Text += DnEcuDiag.WinFormStrings.TestStarted;
                txtActuatorTestStat.Text += Model.Instance.TargetEcu.Actuators[selIndex].Message;

                while (!result.IsCompleted)
                {
                    Application.DoEvents();
                    Thread.Sleep(100);
                    if (x++ == 10)
                    {
                        txtActuatorTestStat.Text += ".";
                        x = 0;
                    }
                }

                try
                {
                    asyncActivateActuator.EndInvoke(result);
                    txtActuatorTestStat.Text += DnEcuDiag.WinFormStrings.TestComplete;
                }
                catch (EngineRunningException)
                {
                    txtActuatorTestStat.Text += DnEcuDiag.WinFormStrings.TestFailedReasonEngineCannotBeRunning;
                }
                catch (ActuatorNotSupportedException actuatorException)
                {
                    txtActuatorTestStat.Text += String.Format(DnEcuDiag.WinFormStrings.TestFailedReason,actuatorException.Message);
                }
#endif
            }
            finally
            {
                btnAutoActuator.Enabled = true;
                tmrProgress.Stop();
                SetUIActive();
            }
        }

        private static void HandleInterfaceConfigure()
        {
            var formConfigDevice = new FormDeviceConfiguration();
            formConfigDevice.ShowDialog();
        }

        private void HandleDisconnectButton()
        {
            try
            {
                disconnecting = true;
                Model.Instance.DisconnectFromEcu();
                disconnecting = false;  
            }
            finally
            {
                HandleDisconnect();
            }
        }

        private void HandleDisconnect()
        {
            ConfigureUserInterface();
            ResetUserInterface();
            DetatchEventHandlers(Model.Instance.TargetEcu);
            connectedEcuBindingSource.ResetBindings(false);
            faultCodesBindingSource.ResetBindings(false);  
        }

        private void HandleConnectButton()
        {
            int count = 0;

            if (!Model.Instance.DeviceConfigured)
            {
                Common.ShowMessage(DnEcuDiag.WinFormStrings.TheSerialDeviceNeedsToBeConfigured);
                HandleInterfaceConfigure();
                return;
            }

            String detectedMonitor = String.Empty;

            if (!String.IsNullOrEmpty(detectedMonitor))
            {
                Common.ShowMessageError(DnEcuDiag.WinFormStrings.DetecedARunningPortMonitor, detectedMonitor);
            }
            else
            {
                try
                {
                    ConfigureConnectionUserInterface();
                    
                    AsyncConnectToEcu connectionDelegate = Model.Instance.ConnectToEcu;
                    IAsyncResult asyncConnect = connectionDelegate.BeginInvoke((EcuDescriptor)cboEcuList.SelectedItem, null, null);

                    statusBarPanelApp.Text = WinFormStrings.WaitingForResponseFromEcu;

                    while (!asyncConnect.IsCompleted)
                    {
                        Application.DoEvents();
                        Cursor.Current = Cursors.WaitCursor;

                        Thread.Sleep(100);

                        count++;

                        if (count % 4 == 0)
                        {
                            statusBarPanelApp.Text += ".";
                        }
                    }

                    if (connectionDelegate.EndInvoke(asyncConnect) == false)
                    {
                        statusBarPanelApp.Text = String.Empty;
                    }
                    else
                    {
                        AttachEventHandlers(Model.Instance.TargetEcu);

                        tabMain.SelectedTab = tabIdentification;

                        connectedEcuBindingSource.ResetBindings(false);

                        statusBarPanelConnect.Icon = Icon.FromHandle(((Bitmap)imgLstStatus.Images[0]).GetHicon());
                        statusBarPanelConnect.ToolTipText =
                            statusBarPanelApp.Text = string.Format(CultureInfo.InvariantCulture, WinFormStrings.Connected, Model.Instance.TargetEcu.Model);    
                    }
                }
                catch (EcuException e)
                {
                    if (!Settings.Default.DisablePopups)
                    {
                        if (Common.ShowYesNoError(WinFormStrings.ErrorUnableToConnectToTheECU) == DialogResult.Yes)
                        {
                            TraceExceptionForm.Display(e);
                        }
                    }
                }
                finally
                {
                    ConfigureUserInterface();
                    
                    Cursor.Current = Cursors.Default;
                }
            }
        }

        private void SetupDatabinding()
        {
            connectedEcuBindingSource.DataSource = Model.Instance.TargetEcu;

            sensorOneBindingSource.DataSource = Common.ConfigureSensorBindingSource(Model.Instance.TargetEcu.Sensors);
            sensorTwoBindingSource.DataSource = Common.ConfigureSensorBindingSource(Model.Instance.TargetEcu.Sensors);
            sensorThreeBindingSource.DataSource = Common.ConfigureSensorBindingSource(Model.Instance.TargetEcu.Sensors);
            sensorFourBindingSource.DataSource = Common.ConfigureSensorBindingSource(Model.Instance.TargetEcu.Sensors);

            routinesBindingSource.DataSource = Model.Instance.TargetEcu.Routines;
            snapshotsBindingSource.DataSource = Model.Instance.TargetEcu.SnapshotDefinitions;
            faultCodesBindingSource.DataSource = Model.Instance.TargetEcu.ActiveFaultCodes;
        }

        private void AttachEventHandlers(Ecu ecu)
        {
            ecu.SensorOneUpdated += OnSensorOneValueChanged;
            ecu.SensorTwoUpdated += OnSensorTwoValueChanged;
            ecu.SensorThreeUpdated += OnSensorThreeValueChanged;
            ecu.SensorFourUpdated += OnSensorFourValueChanged;
            ecu.EcuDisconnected += OnEcuDisconnected;
            ecu.SensorOneReadError += OnSensorOneError;
            ecu.SensorTwoReadError += OnSensorTwoError;
            ecu.SensorThreeReadError += OnSensorThreeError;
            ecu.SensorFourReadError += OnSensorFourError;
        }

        private void DetatchEventHandlers(Ecu ecu)
        {
            ecu.SensorOneUpdated -= OnSensorOneValueChanged;
            ecu.SensorTwoUpdated -= OnSensorTwoValueChanged;
            ecu.SensorThreeUpdated -= OnSensorThreeValueChanged;
            ecu.SensorFourUpdated -= OnSensorFourValueChanged;
            ecu.EcuDisconnected -= OnEcuDisconnected;
            ecu.SensorOneReadError -= OnSensorOneError;
            ecu.SensorTwoReadError -= OnSensorTwoError;
            ecu.SensorThreeReadError -= OnSensorThreeError;
            ecu.SensorFourReadError -= OnSensorFourError;
        }

        private void HandleExecuteRoutine()
        {
            try
            {
#if DEMO
				Common.ShowMessageExclamation("This feature is not availabe in the demo version");
				return;
#else
                SetUIBusy();

                btnExecuteRoutine.Enabled = false;
                btnRoutineExecuteCancel.Enabled = true;

                if (cboRoutines.SelectedIndex == -1)
                {
                    Common.ShowMessageExclamation(DnEcuDiag.WinFormStrings.PleaseSelectARoutineFromTheDropDown);
                    return;
                }

                int selectedIndex = cboRoutines.SelectedIndex;

                pbIndicator = pbCalibration;

                pbCalibration.Value = 0;
                pbCalibration.Maximum = 100;
                txtRoutineExecution.Clear();
                txtRoutineExecution.Text += DnEcuDiag.WinFormStrings.StartingRoutine;
                txtRoutineExecution.Text += Model.Instance.TargetEcu.Routines[selectedIndex].Name;
                tmrProgress.Start();
                txtRoutineExecution.Text += DnEcuDiag.WinFormStrings.RoutineStarted;

                AsyncExecuteRoutine asyncExecuteRoutine = Model.Instance.TargetEcu.ExecuteRoutine;
                IAsyncResult asyncResult = asyncExecuteRoutine.BeginInvoke(Model.Instance.TargetEcu.Routines[selectedIndex], null, null);
                int x = 0;

                while (!asyncResult.IsCompleted)
                {
                    Application.DoEvents();
                    Thread.Sleep(100);

                    if (x++ == 10)
                    {
                        txtRoutineExecution.Text += ".";
                        x = 0;
                    }
                }

                try
                {
                    uint result = asyncExecuteRoutine.EndInvoke(asyncResult);

                    txtRoutineExecution.Text += String.Format(CultureInfo.InvariantCulture, DnEcuDiag.WinFormStrings.RoutineResult, result.ToString("x", CultureInfo.InvariantCulture));

                    if (result == Model.Instance.TargetEcu.Routines[selectedIndex].Result)
                    {
                        txtRoutineExecution.Text += Model.Instance.TargetEcu.Routines[selectedIndex].RoutineType ==
                                                    RoutineType.Activation
                                                        ? DnEcuDiag.WinFormStrings.StatusActive
                                                        : DnEcuDiag.WinFormStrings.RoutineSucceeded;
                    }
                    else
                    {
                        txtRoutineExecution.Text += Model.Instance.TargetEcu.Routines[selectedIndex].RoutineType ==
                                                    RoutineType.Activation
                                                        ? DnEcuDiag.WinFormStrings.StatusInActive
                                                        : DnEcuDiag.WinFormStrings.RoutineFailed;
                    }


                    txtRoutineExecution.Text += DnEcuDiag.WinFormStrings.RoutineComplete;
                }
                catch (RoutineNotSupportedException)
                {
                    txtRoutineExecution.Text += DnEcuDiag.WinFormStrings.ErrorThisEcuDoesNotSupportThisRoutine;
                }
                catch (RoutineExecutionCanceledException)
                {
                    txtRoutineExecution.Text += DnEcuDiag.WinFormStrings.ExecutionCancelled;
                }
#endif
            }
            finally
            {
                SetUIActive();
                tmrProgress.Stop();
                btnExecuteRoutine.Enabled = true;
                btnRoutineExecuteCancel.Enabled = false;
            }
        }

        private void HandleRoutineSelectionChange(int index)
        {
            txtRoutineInstructions.Text =
                Model.Instance.TargetEcu.Routines[index].OperationDescription.Replace("&", Environment.NewLine);
        }

        private void HandleSelectEcu(EcuDescriptor ecuDescriptor)
        {
            if (ecuDescriptor != null)
            {
                Model.Instance.SelectTargetEcu(ecuDescriptor);

                LoadEcuActuators(Model.Instance.TargetEcu.Actuators);

                SetupDatabinding();

                ApplySensorSort();

                ConfigureUserInterface();
            }
        }

        #endregion

        #region UserInterface Configuration

        private void ConfigureUserInterface()
        {
            ConfigureConfigurationMenu();
            ConfigureConnectionUserInterface();
            ConfigureSensorsUserInterface();
            ConfigureActuatorsTab();
            ConfigureFaultCodeTab();
            ConfigurePerformanceTab();
            ConfigureDynoTab();
        }

        private void ConfigureConfigurationMenu()
        {
            menuItemInterface.Enabled = !Model.Instance.TargetEcu.Connected;
        }

        private void ConfigureConnectionUserInterface()
        {
            btnConnect.Enabled = !Model.Instance.TargetEcu.Connected;
            btnDisconnect.Enabled = Model.Instance.TargetEcu.Connected;
            cboEcuList.Enabled = !Model.Instance.TargetEcu.Connected;
            mnuItemConnect.Enabled = !Model.Instance.TargetEcu.Connected;
            mnuItemDisconnect.Enabled = Model.Instance.TargetEcu.Connected;
        }

        private void ConfigureDynoTab()
        {
            btnStartDynoRun.Enabled = Model.Instance.TargetEcu.Connected & Model.Instance.TargetEcu.RpmSensor != null;
            btnStopDynoRun.Enabled = Model.Instance.TargetEcu.Connected & Model.Instance.TargetEcu.RpmSensor != null;
            btnCaptureFinalDriveRatio.Enabled = Model.Instance.TargetEcu.Connected & Model.Instance.TargetEcu.RpmSensor != null;
        }

        private void ConfigurePerformanceTab()
        {
            btnStartPerformance.Enabled = Model.Instance.TargetEcu.Connected & Model.Instance.TargetEcu.SpeedSensor != null;
            btnStopPerformance.Enabled = !btnStartPerformance.Enabled & Model.Instance.TargetEcu.Connected;
        }

        private void ConfigureFaultCodeTab()
        {
            btnReadDTCs.Enabled = Model.Instance.TargetEcu.Connected;
            btnClearDTCs.Enabled = Model.Instance.TargetEcu.Connected;
            txtFaultCodeDescription.Text = String.Empty;

            btnReadSnapShot.Enabled = Model.Instance.TargetEcu.Connected & Model.Instance.TargetEcu.SnapshotDefinitions.Count != 0;
            cboSnapShots.Enabled = Model.Instance.TargetEcu.SnapshotDefinitions.Count != 0;
            txtSnapShotText.Text = String.Empty;
        }

        private void ConfigureActuatorsTab()
        {
            btnAutoActuator.Enabled = Model.Instance.TargetEcu.Connected & Model.Instance.TargetEcu.Actuators.Count != 0;
            txtActuatorTestStat.Text = String.Empty;
            
            if (Model.Instance.TargetEcu.Actuators.Count != 0)
            {
                tcActuators.SelectedIndex = 0;
            }

            cboRoutines.Enabled = Model.Instance.TargetEcu.Routines.Count != 0;
            btnRoutineExecuteCancel.Enabled = Model.Instance.TargetEcu.Connected & Model.Instance.TargetEcu.Routines.Count != 0;
            btnExecuteRoutine.Enabled = Model.Instance.TargetEcu.Connected & Model.Instance.TargetEcu.Routines.Count != 0;
            txtRoutineInstructions.Text = String.Empty;
            txtRoutineExecution.Text = String.Empty;
        }

        private void ConfigureSensorsUserInterface()
        {
            mnuItemPlotAll.Enabled = Model.Instance.TargetEcu.Sensors.Count != 0;
            mnuItemEnableLogging.Enabled = Model.Instance.TargetEcu.Sensors.Count != 0;
            mnuItemSensorGroup.Enabled = Model.Instance.TargetEcu.Sensors.Count != 0;
            menuItemSensorSort.Enabled = Model.Instance.TargetEcu.Sensors.Count != 0;

            btnSensorOneCapture.Enabled = Model.Instance.TargetEcu.Connected & Model.Instance.TargetEcu.Sensors.Count != 0;
            btnSensorTwoCapture.Enabled = Model.Instance.TargetEcu.Connected & Model.Instance.TargetEcu.Sensors.Count != 0;
            btnSensorThreeCapture.Enabled = Model.Instance.TargetEcu.Connected & Model.Instance.TargetEcu.Sensors.Count != 0;
            btnSensorFourCapture.Enabled = Model.Instance.TargetEcu.Connected & Model.Instance.TargetEcu.Sensors.Count != 0;

            cboSensorOne.Enabled = Model.Instance.TargetEcu.Sensors.Count != 0;
            cboSensorTwo.Enabled = Model.Instance.TargetEcu.Sensors.Count != 0;
            cboSensorThree.Enabled = Model.Instance.TargetEcu.Sensors.Count != 0;
            cboSensorFour.Enabled = Model.Instance.TargetEcu.Sensors.Count != 0;


        }

        private void ConfigureDynoGraph()
        {
            zedGraphDyno.GraphPane.Title = DnEcuDiag.WinFormStrings.DynoMeasurementsFor;
            zedGraphDyno.GraphPane.XAxis.Title = "Rpm";
            zedGraphDyno.GraphPane.YAxis.Title = DnEcuDiag.WinFormStrings.Power;
            zedGraphDyno.GraphPane.YAxis.Color = PowerPlotColor;
            zedGraphDyno.GraphPane.Y2Axis.Title = DnEcuDiag.WinFormStrings.Torque;
            zedGraphDyno.GraphPane.Y2Axis.Color = TorquePlotColor;
            zedGraphDyno.GraphPane.Y2Axis.IsVisible = true;

            // Make the Y axis scale red
            zedGraphDyno.GraphPane.YAxis.ScaleFontSpec.FontColor = PowerPlotColor;
            zedGraphDyno.GraphPane.YAxis.TitleFontSpec.FontColor = PowerPlotColor;
            // turn off the opposite tics so the Y tics don't show up on the Y2 axis
            zedGraphDyno.GraphPane.YAxis.IsOppositeTic = false;
            zedGraphDyno.GraphPane.YAxis.IsMinorOppositeTic = false;
            // Don't display the Y zero line
            zedGraphDyno.GraphPane.YAxis.IsZeroLine = false;
            // Align the Y axis labels so they are flush to the axis
            zedGraphDyno.GraphPane.YAxis.ScaleAlign = AlignP.Inside;

            // Enable the Y2 axis display
            zedGraphDyno.GraphPane.Y2Axis.IsVisible = true;
            // Make the Y2 axis scale blue
            zedGraphDyno.GraphPane.Y2Axis.ScaleFontSpec.FontColor = TorquePlotColor;
            zedGraphDyno.GraphPane.Y2Axis.TitleFontSpec.FontColor = TorquePlotColor;
            // turn off the opposite tics so the Y2 tics don't show up on the Y axis
            zedGraphDyno.GraphPane.Y2Axis.IsOppositeTic = false;
            zedGraphDyno.GraphPane.Y2Axis.IsMinorOppositeTic = false;
            // Display the Y2 axis grid lines
            zedGraphDyno.GraphPane.Y2Axis.IsShowGrid = true;
            // Align the Y2 axis labels so they are flush to the axis
            zedGraphDyno.GraphPane.Y2Axis.ScaleAlign = AlignP.Inside;

            // Fill the axis background with a gradient
            zedGraphDyno.GraphPane.AxisFill = new Fill(Color.White, Color.LightGray, 45.0f);
        }

        #endregion 
        #region Boundary Artifacts

        #region Sensors

        /// <summary>
        /// Gets or sets the index of the selected sensor one.
        /// </summary>
        /// <value>The index of the selected sensor one.</value>
        public int SelectedSensorOneIndex
        {
            get { return cboSensorOne.SelectedIndex; }
            set { cboSensorOne.SelectedIndex = value; }
        }

        /// <summary>
        /// Gets or sets the index of the selected sensor two.
        /// </summary>
        /// <value>The index of the selected sensor two.</value>
        public int SelectedSensorTwoIndex
        {
            get { return cboSensorTwo.SelectedIndex; }
            set { cboSensorTwo.SelectedIndex = value; }
        }

        /// <summary>
        /// Gets or sets the index of the selected sensor three.
        /// </summary>
        /// <value>The index of the selected sensor three.</value>
        public int SelectedSensorThreeIndex
        {
            get { return cboSensorThree.SelectedIndex; }
            set { cboSensorThree.SelectedIndex = value; }
        }

        /// <summary>
        /// Gets or sets the index of the selected sensor four.
        /// </summary>
        /// <value>The index of the selected sensor four.</value>
        public int SelectedSensorFourIndex
        {
            get { return cboSensorFour.SelectedIndex; }
            set { cboSensorFour.SelectedIndex = value; }
        }

        #endregion

        #region Properties

        #endregion

        #region Methods

        /// <summary>
        /// Reset the actuator tab.
        /// </summary>
        public void ResetActuatorUI()
        {
            txtRoutineExecution.Text = String.Empty;
            txtActuatorTestStat.Text = String.Empty;
            pbActuator.Value = pbActuator.Minimum;
            pbCalibration.Value = pbCalibration.Minimum;
        }

        /// <summary>
        /// Reset the performance timer.
        /// </summary>
        public void TimerReset()
        {
            dCCPerf.Reset();
        }

        private void DisplayDynoResults(DynoMeasurementValue[] dynoMeasurements, bool showImperial)
        {
            string torqueLabel = DnEcuDiag.WinFormStrings.Torque;
            string powerLabel = DnEcuDiag.WinFormStrings.Power;

            var maxPower = new DynoMeasurementValue(0, 0, 0);
            var maxTorque = new DynoMeasurementValue(0, 0, 0);

            var power = new PointPairList();
            var torque = new PointPairList();

            // Exclude the first and last measurement as its never filtered.
            for (int i = 1; i < dynoMeasurements.Length - 1; i++)
            {
                if (showImperial)
                {
                    power.Add(dynoMeasurements[i].Rpm, Math.Round(dynoMeasurements[i].PowerBhp, 2));
                    torque.Add(dynoMeasurements[i].Rpm, Math.Round(dynoMeasurements[i].Torqueftlb, 2));
                }
                else
                {
                    power.Add(dynoMeasurements[i].Rpm, Math.Round(dynoMeasurements[i].Power, 2));
                    torque.Add(dynoMeasurements[i].Rpm, Math.Round(dynoMeasurements[i].Torque, 2));
                }

                // Find the maximum power.
                if (dynoMeasurements[i].Power > maxPower.Power)
                {
                    maxPower.Power = dynoMeasurements[i].Power;
                    maxPower.Rpm = dynoMeasurements[i].Rpm;
                }

                // Find the maximum torque.
                if (dynoMeasurements[i].Torque > maxTorque.Torque)
                {
                    maxTorque.Torque = dynoMeasurements[i].Torque;
                    maxTorque.Rpm = dynoMeasurements[i].Rpm;
                }
            }

            // Add the curves.
            GraphPane myPane = zedGraphDyno.GraphPane;
            myPane.CurveList.Clear();
            myPane.GraphItemList.Clear();
            LineItem myCurve = myPane.AddCurve(powerLabel, power, PowerPlotColor, SymbolType.Diamond);
            myCurve.Symbol.Fill = new Fill(Color.White);

            myCurve = myPane.AddCurve(torqueLabel, torque, TorquePlotColor, SymbolType.Circle);
            myCurve.Symbol.Fill = new Fill(Color.White);
            myCurve.IsY2Axis = true;

            if (showImperial)
            {
                torqueLabel += " (ft*lbs)";
                powerLabel += " (Bhp)";
            }
            else
            {
                torqueLabel += " (Nm)";
                powerLabel += " (Kw)";
            }

            myPane.YAxis.Title = powerLabel;
            myPane.Y2Axis.Title = torqueLabel;

            // Add the max torque and power.
            TextItem text;
            if (showImperial)
            {
                text =
                    new TextItem(
                        String.Format(CultureInfo.InvariantCulture, DnEcuDiag.WinFormStrings.MaxPowerRpmMaxTorque,
                                      maxPower.PowerBhp.ToString("0.00", CultureInfo.InvariantCulture), maxPower.Rpm,
                                      maxTorque.Torqueftlb.ToString("0.00", CultureInfo.InvariantCulture), maxTorque.Rpm), 0.79F, -0.062F);
            }
            else
            {
                text =
                    new TextItem(
                        String.Format(CultureInfo.InvariantCulture, DnEcuDiag.WinFormStrings.MaxPowerRpmMaxTorque,
                                      maxPower.Power.ToString("0.00", CultureInfo.InvariantCulture), maxPower.Rpm,
                                      maxTorque.Torque.ToString("0.00", CultureInfo.InvariantCulture), maxTorque.Rpm), 0.79F, -0.062F);
            }
            text.FontSpec.Size = 11.0f;
            text.Location.CoordinateFrame = CoordType.AxisFraction;
            myPane.GraphItemList.Add(text);

            // Show the x axis grid
            myPane.XAxis.IsShowGrid = true;
            myPane.AxisChange(zedGraphDyno.CreateGraphics());
            zedGraphDyno.Refresh();
        }

        private void ResetSensorMenu()
        {
            mnuItemSensorGroup.Enabled = true;
            mnuItemSensorGroup.Checked = false;

            mnuItemEnableLogging.Checked = mnuItemEnableLogging.Enabled = false;
            mnuItemPlotAll.Checked = false;
            mnuItemPlotAll.Enabled = true;
        }

        private void ResetUserInterface()
        {
            tabMain.SelectedTab = tabConnect;
            ResetSensorUi(oScopeOne, btnSensorOneCapture, lblSensorOneMaxValue, lblSensorOneMinValue, lblSensorOneCurrent, imgLstButtons.Images[1]);
            ResetSensorUi(oScopeTwo, btnSensorTwoCapture, lblSensorTwoMaxValue, lblSensorTwoMinValue, lblSensorTwoCurrent, imgLstButtons.Images[1]);
            ResetSensorUi(oScopeThree, btnSensorThreeCapture, lblSensorThreeMaxValue, lblSensorThreeMinValue, lblSensorThreeCurrent, imgLstButtons.Images[1]);
            ResetSensorUi(oScopeFour, btnSensorFourCapture, lblSensorFourMaxValue, lblSensorFourMinValue, lblSensorFourCurrent, imgLstButtons.Images[1]);
            
            statusBarPanelApp.Text = string.Empty;

            statusBarPanelLog.Icon = Icon.FromHandle(((Bitmap)imgLstStatus.Images[3]).GetHicon());
            statusBarPanelLog.ToolTipText = DnEcuDiag.WinFormStrings.NotLoggingSensors;

            statusBarPanelConnect.Icon = Icon.FromHandle(((Bitmap)imgLstStatus.Images[1]).GetHicon());
            statusBarPanelConnect.ToolTipText = DnEcuDiag.WinFormStrings.ConnectionDisconnected;

            ResetSensorMenu();
        }

        /// <summary>
        /// Load the actuators to the main form list view.
        /// </summary>
        private void LoadEcuActuators(ICollection<Actuator> actuators)
        {
            int index = 0;

            imgLstActuators.Images.Clear();
            lvActuators.Items.Clear();

            foreach (Actuator a in actuators)
            {
                imgLstActuators.Images.Add(LoadResourcedBitmap("DnEcuDiag.Images." + a.ImageName));
            }
            foreach (Actuator a in actuators)
            {
                lvActuators.Items.Add(a.Name, index++);
            }
        }

        private static void ResetSensorUi(Control scope, ButtonBase button, Control max, Control min, Control current, Image image)
        {
            scope.Enabled = scope.Visible = false;
            button.Image = image;
            max.Text = min.Text = "0";
            current.Text = String.Empty;
        }

        internal void OnSensorOneError(object sender, SensorReadErrorEventArgs e)
        {
            BeginInvoke(new ResetSensorUiDelegate(ResetSensorUi), oScopeOne, btnSensorOneCapture, lblSensorOneMaxValue, lblSensorOneMinValue, lblSensorOneCurrent, imgLstButtons.Images[1]);
            Common.ExecuteExceptionHandledDelegate(() => HandleSensorError(e, DnEcuDiag.WinFormStrings.One, Model.Instance.TargetEcu.SensorMonitorOne.Name, ResetSensorMenu));
        }

        internal void OnSensorOneValueChanged(object sender, SensorUpdatedEventArgs args)
        {
            if (InvokeRequired)
            {
                BeginInvoke(new UpdateSensorUi(UpdateSensorOneUI), new object[] { args });
            }
            else
            {
                UpdateSensorOneUI(args);
            }
        }

        private void UpdateSensorOneUI(SensorUpdatedEventArgs args)
        {
            if (!Model.Instance.TargetEcu.SensorMonitorOneEnable) return;

            oScopeOne.Channel.AppendPoint(args.Current.Measurement);

            if (!args.Current.IsStatus)
            {
                lblSensorOneMaxValue.Text = String.Format(CultureInfo.InvariantCulture, "{0:0.00}", args.Maximum.Measurement);
                lblSensorOneMinValue.Text = String.Format(CultureInfo.InvariantCulture, "{0:0.00}", args.Minimum.Measurement);
                lblSensorOneCurrent.Text = String.Format(CultureInfo.InvariantCulture, "{0:0.00}", args.Current.Measurement);
            }
            else
            {
                lblSensorOneMaxValue.Text = String.Empty;
                lblSensorOneMinValue.Text = String.Empty;
                lblSensorOneCurrent.Text = args.Current.Status;
            }
        }

        internal void OnSensorTwoError(object sender, SensorReadErrorEventArgs e)
        {
            BeginInvoke(new ResetSensorUiDelegate(ResetSensorUi), oScopeTwo, btnSensorTwoCapture, lblSensorTwoMaxValue, lblSensorTwoMinValue, lblSensorTwoCurrent, imgLstButtons.Images[1]);
            Common.ExecuteExceptionHandledDelegate(() => HandleSensorError(e, DnEcuDiag.WinFormStrings.Two, Model.Instance.TargetEcu.SensorMonitorTwo.Name, ResetSensorMenu));
        }

        internal void OnSensorTwoValueChanged(object sender, SensorUpdatedEventArgs args)
        {
            // Check if the calling thread of this method is not on the UI thread.
            if (InvokeRequired)
            {
                // Invoke the delegate async.
                BeginInvoke(new UpdateSensorUi(UpdateSensorTwoUI), new object[] { args });
            }
            else
            {
                // Already on the UI thread then just invoke the UpdateMethod.
                UpdateSensorTwoUI(args);
            }
        }

        private void UpdateSensorTwoUI(SensorUpdatedEventArgs args)
        {
            if (!Model.Instance.TargetEcu.SensorMonitorTwoEnable) return;

            oScopeTwo.Channel.AppendPoint(args.Current.Measurement);

            if (!args.Current.IsStatus)
            {
                lblSensorTwoMaxValue.Text = String.Format(CultureInfo.InvariantCulture, "{0:0.00}", args.Maximum.Measurement);
                lblSensorTwoMinValue.Text = String.Format(CultureInfo.InvariantCulture, "{0:0.00}", args.Minimum.Measurement);
                lblSensorTwoCurrent.Text = String.Format(CultureInfo.InvariantCulture, "{0:0.00}", args.Current.Measurement);
            }
            else
            {
                lblSensorTwoMaxValue.Text = String.Empty;
                lblSensorTwoMinValue.Text = String.Empty;
                lblSensorTwoCurrent.Text = args.Current.Status;
            }
        }

        internal void OnSensorThreeError(object sender, SensorReadErrorEventArgs e)
        {
            BeginInvoke(new ResetSensorUiDelegate(ResetSensorUi), oScopeThree, btnSensorThreeCapture, lblSensorThreeMaxValue, lblSensorThreeMinValue, lblSensorThreeCurrent, imgLstButtons.Images[1]);
            Common.ExecuteExceptionHandledDelegate(() => HandleSensorError(e, DnEcuDiag.WinFormStrings.Three, Model.Instance.TargetEcu.SensorMonitorThree.Name, ResetSensorMenu));
        }

        internal void OnSensorThreeValueChanged(object sender, SensorUpdatedEventArgs args)
        {
            // Check if the calling thread of this method is not on the UI thread.
            if (InvokeRequired)
            {
                // Invoke the delegate async.
                BeginInvoke(new UpdateSensorUi(UpdateSensorThreeUI), new object[] { args });
            }
            else
            {
                // Already on the UI thread then just invoke the UpdateMethod.
                UpdateSensorThreeUI(args);
            }
        }

        private void UpdateSensorThreeUI(SensorUpdatedEventArgs args)
        {
            if (!Model.Instance.TargetEcu.SensorMonitorThreeEnable) return;

            oScopeThree.Channel.AppendPoint(args.Current.Measurement);

            if (!args.Current.IsStatus)
            {
                lblSensorThreeMaxValue.Text = String.Format(CultureInfo.InvariantCulture, "{0:0.00}", args.Maximum.Measurement);
                lblSensorThreeMinValue.Text = String.Format(CultureInfo.InvariantCulture, "{0:0.00}", args.Minimum.Measurement);
                lblSensorThreeCurrent.Text = String.Format(CultureInfo.InvariantCulture, "{0:0.00}", args.Current.Measurement);
            }
            else
            {
                lblSensorThreeMaxValue.Text = String.Empty;
                lblSensorThreeMinValue.Text = String.Empty;
                lblSensorThreeCurrent.Text = args.Current.Status;
            }
        }

        internal void OnSensorFourError(object sender, SensorReadErrorEventArgs e)
        {
            BeginInvoke(new ResetSensorUiDelegate(ResetSensorUi), oScopeFour, btnSensorFourCapture, lblSensorFourMaxValue, lblSensorFourMinValue, lblSensorFourCurrent, imgLstButtons.Images[1]);
            Common.ExecuteExceptionHandledDelegate(() => HandleSensorError(e, DnEcuDiag.WinFormStrings.Four, Model.Instance.TargetEcu.SensorMonitorFour.Name, ResetSensorMenu));
        }

        internal void OnSensorFourValueChanged(object sender, SensorUpdatedEventArgs args)
        {
            // Check if the calling thread of this method is not on the UI thread.
            if (InvokeRequired)
            {
                // Invoke the delegate async.
                BeginInvoke(new UpdateSensorUi(UpdateSensorFourUI), new object[] { args });
            }
            else
            {
                // Already on the UI thread then just invoke the UpdateMethod.
                UpdateSensorFourUI(args);
            }
        }

        private void UpdateSensorFourUI(SensorUpdatedEventArgs args)
        {
            if (!Model.Instance.TargetEcu.SensorMonitorFourEnable) return;

            oScopeFour.Channel.AppendPoint(args.Current.Measurement);

            if (!args.Current.IsStatus)
            {
                lblSensorFourMaxValue.Text = String.Format(CultureInfo.InvariantCulture, "{0:0.00}", args.Maximum.Measurement);
                lblSensorFourMinValue.Text = String.Format(CultureInfo.InvariantCulture, "{0:0.00}", args.Minimum.Measurement);
                lblSensorFourCurrent.Text = String.Format(CultureInfo.InvariantCulture, "{0:0.00}", args.Current.Measurement);
            }
            else
            {
                lblSensorFourMaxValue.Text = String.Empty;
                lblSensorFourMinValue.Text = String.Empty;
                lblSensorFourCurrent.Text = args.Current.Status;
            }
        }

        private void OnEcuDisconnected(object sender, EcuDisconnectEventArgs args)
        {
            if (InvokeRequired)
            {
                Invoke(new VoidDelegate(HandleDisconnect));
            }
            else
            {
                HandleDisconnect();
            }
            
            connectedEcuBindingSource.ResetBindings(false);
            faultCodesBindingSource.ResetBindings(false);

            if (!Settings.Default.DisablePopups)
            {
                if (Common.ShowYesNoError(DnEcuDiag.WinFormStrings.ConnectionToTheEcuHasBeenLost) == DialogResult.Yes)
                {
                    TraceExceptionForm.Display(args.Exception);
                }
            }
        }

        /// <summary>
        /// Sets the UI busy.
        /// </summary>
        private void SetUIBusy()
        {
            disableTabbing = true;
            Cursor = Cursors.WaitCursor;
        }

        /// <summary>
        /// Sets the UI active.
        /// </summary>
        private void SetUIActive()
        {
            disableTabbing = false;
            Cursor = Cursors.Default;
        }

        #endregion Methods

        private static Bitmap LoadResourcedBitmap(string bitmapName)
        {
            Bitmap bmp = null;

            Assembly a = Assembly.GetExecutingAssembly();

            // get a list of resource names from the manifest
            string[] resNames = a.GetManifestResourceNames();

            foreach (string s in resNames)
            {
                if (String.Compare(s, bitmapName, StringComparison.OrdinalIgnoreCase) == 0)
                {
                    using (var imgStream = a.GetManifestResourceStream(s))
                    {
                        if (!(null == imgStream))
                        {
                            bmp = Image.FromStream(imgStream) as Bitmap;
                        }
                    }
                }
            }

            return bmp;
        }

        private void HandleSensorError(SensorReadErrorEventArgs e, String sensorNumber, String sensorName, VoidDelegate resetSensorMenuDelegate)
        {
            if (!disconnecting)
            {
                if (e.Error is SensorNotSupportedException)
                {
                    Common.ShowMessageExclamation(
                        String.Format(CultureInfo.InvariantCulture, DnEcuDiag.WinFormStrings.SensorIsNotSupportedByThisEcuRevision, sensorNumber,
                                      sensorName));

                    if (InvokeRequired)
                    {
                        BeginInvoke(resetSensorMenuDelegate);
                    }
                    else
                    {
                        resetSensorMenuDelegate.Invoke();
                    }
                }
                else
                {
#warning TODO
                }
            }
        }

        #endregion Boundary Artifacts

        private void menuItemAscending_Click(object sender, EventArgs e)
        {
            SetSensorMenuItems(true, false, false);
            ApplySensorSort();
        }

        private void menuItemDescending_Click(object sender, EventArgs e)
        {
            SetSensorMenuItems(false, true, false);
            ApplySensorSort();
        }

        private void menuItemNone_Click(object sender, EventArgs e)
        {
            SetSensorMenuItems(false, false, true);
            ApplySensorSort();
        }

        private void SetSensorMenuItems(bool ascending, bool descending, bool none)
        {
            Settings.Default.SortSensorsAscending = menuItemAscending.Checked = ascending;
            Settings.Default.SortSensorsDescending = menuItemDescending.Checked = descending;
            Settings.Default.SortSensorsNone = menuItemNone.Checked = none;
        }

        private void ApplySensorSort()
        {
            if (Settings.Default.SortSensorsAscending)
            {
                sensorOneBindingSource.Sort = "Name ASC";
                sensorTwoBindingSource.Sort = "Name ASC";
                sensorThreeBindingSource.Sort = "Name ASC";
                sensorFourBindingSource.Sort = "Name ASC";
            }

            if (Settings.Default.SortSensorsDescending)
            {
                sensorOneBindingSource.Sort = "Name DESC";
                sensorTwoBindingSource.Sort = "Name DESC";
                sensorThreeBindingSource.Sort = "Name DESC";
                sensorFourBindingSource.Sort = "Name DESC";
            }

            if (Settings.Default.SortSensorsNone)
            {
                sensorOneBindingSource.RemoveSort();
                sensorTwoBindingSource.RemoveSort();
                sensorThreeBindingSource.RemoveSort();
                sensorFourBindingSource.RemoveSort();
            }

            cboSensorOne.SelectedItem = Model.Instance.TargetEcu.SensorMonitorOne;
            cboSensorTwo.SelectedItem = Model.Instance.TargetEcu.SensorMonitorTwo;
            cboSensorThree.SelectedItem = Model.Instance.TargetEcu.SensorMonitorThree;
            cboSensorFour.SelectedItem = Model.Instance.TargetEcu.SensorMonitorFour;
        }

        private void cboEcuList_SelectedValueChanged(object sender, EventArgs e)
        {
            Common.ExecuteExceptionHandledDelegate(() => HandleSelectEcu((EcuDescriptor)cboEcuList.SelectedItem));
        }
    }
}